@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2020 Canonical Ltd
+ * Copyright (C) 2015-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -22,6 +22,7 @@
 package asserts
 
 import (
+	"errors"
 	"fmt"
 	"regexp"
 	"time"
@@ -43,6 +44,11 @@ func (e *NotFoundError) Error() string {
 	return fmt.Sprintf("%v not found", &Ref{Type: e.Type, PrimaryKey: pk})
 }
 
+func (e *NotFoundError) Is(err error) bool {
+	// TODO: replace IsNotFound usages for errors.Is(err, &NotFoundError{})
+	return IsNotFound(err)
+}
+
 // IsNotFound returns whether err is an assertion not found error.
 func IsNotFound(err error) bool {
 	_, ok := err.(*NotFoundError)
@@ -59,7 +65,10 @@ type Backstore interface {
 	// previously stored revision with the same primary key headers.
 	Put(assertType *AssertionType, assert Assertion) error
 	// Get returns the assertion with the given unique key for its
-	// primary key headers.  If none is present it returns a
+	// primary key headers.
+	// A suffix of optional primary keys can be left out from key
+	// in which case their default values are implied.
+	// If the assertion is not present it returns a
 	// NotFoundError, usually with omitted Headers.
 	Get(assertType *AssertionType, key []string, maxFormat int) (Assertion, error)
 	// Search returns assertions matching the given headers.
@@ -93,6 +102,24 @@ func (nbs nullBackstore) SequenceMemberAfter(t *AssertionType, kp []string, afte
 	return nil, &NotFoundError{Type: t}
 }
 
+// keyNotFoundError is returned when the key with a given ID cannot be found.
+type keyNotFoundError struct {
+	msg string
+}
+
+func (e *keyNotFoundError) Error() string { return e.msg }
+
+func (e *keyNotFoundError) Is(target error) bool {
+	_, ok := target.(*keyNotFoundError)
+	return ok
+}
+
+// IsKeyNotFound returns true when the error indicates that a given key was not
+// found.
+func IsKeyNotFound(err error) bool {
+	return errors.Is(err, &keyNotFoundError{})
+}
+
 // A KeypairManager is a manager and backstore for private/public key pairs.
 type KeypairManager interface {
 	// Put stores the given private/public key pair,
@@ -100,8 +127,12 @@ type KeypairManager interface {
 	// Trying to store a key with an already present key id should
 	// result in an error.
 	Put(privKey PrivateKey) error
-	// Get returns the private/public key pair with the given key id.
+	// Get returns the private/public key pair with the given key id. The
+	// error can be tested with IsKeyNotFound to check whether the given key
+	// was not found, or other error occurred.
 	Get(keyID string) (PrivateKey, error)
+	// Delete deletes the private/public key pair with the given key id.
+	Delete(keyID string) error
 }
 
 // DatabaseConfig for an assertion database.
@@ -127,7 +158,7 @@ type RevisionError struct {
 func (e *RevisionError) Error() string {
 	if e.Used < 0 || e.Current < 0 {
 		// TODO: message may need tweaking once there's a use.
-		return fmt.Sprintf("assertion revision is unknown")
+		return "assertion revision is unknown"
 	}
 	if e.Used == e.Current {
 		return fmt.Sprintf("revision %d is already the current revision", e.Used)
@@ -173,18 +204,26 @@ type RODatabase interface {
 	IsTrustedAccount(accountID string) bool
 	// Find an assertion based on arbitrary headers.
 	// Provided headers must contain the primary key for the assertion type.
+	// Optional primary key headers can be omitted in which case
+	// their default values will be used.
 	// It returns a NotFoundError if the assertion cannot be found.
 	Find(assertionType *AssertionType, headers map[string]string) (Assertion, error)
 	// FindPredefined finds an assertion in the predefined sets
 	// (trusted or not) based on arbitrary headers.  Provided
 	// headers must contain the primary key for the assertion
-	// type.  It returns a NotFoundError if the assertion cannot
+	// type.
+	// Optional primary key headers can be omitted in which case
+	// their default values will be used.
+	// It returns a NotFoundError if the assertion cannot
 	// be found.
 	FindPredefined(assertionType *AssertionType, headers map[string]string) (Assertion, error)
 	// FindTrusted finds an assertion in the trusted set based on
 	// arbitrary headers.  Provided headers must contain the
-	// primary key for the assertion type.  It returns a
-	// NotFoundError if the assertion cannot be found.
+	// primary key for the assertion type.
+	// Optional primary key headers can be omitted in which case
+	// their default values will be used.
+	// It returns a NotFoundError if the assertion cannot be
+	// found.
 	FindTrusted(assertionType *AssertionType, headers map[string]string) (Assertion, error)
 	// FindMany finds assertions based on arbitrary headers.
 	// It returns a NotFoundError if no assertion can be found.
@@ -193,6 +232,19 @@ type RODatabase interface {
 	// (trusted or not) based on arbitrary headers.  It returns a
 	// NotFoundError if no assertion can be found.
 	FindManyPredefined(assertionType *AssertionType, headers map[string]string) ([]Assertion, error)
+	// FindSequence finds an assertion for the given headers and after for
+	// a sequence-forming type.
+	// The provided headers must contain a sequence key, i.e. a prefix of
+	// the primary key for the assertion type except for the sequence
+	// number header.
+	// The assertion is the first in the sequence under the sequence key
+	// with sequence number > after.
+	// If after is -1 it returns instead the assertion with the largest
+	// sequence number.
+	// It will constraint itself to assertions with format <= maxFormat
+	// unless maxFormat is -1.
+	// It returns a NotFoundError if the assertion cannot be found.
+	FindSequence(assertType *AssertionType, sequenceHeaders map[string]string, after, maxFormat int) (SequenceMember, error)
 	// Check tests whether the assertion is properly signed and consistent with all the stored knowledge.
 	Check(assert Assertion) error
 }
@@ -200,7 +252,7 @@ type RODatabase interface {
 // A Checker defines a check on an assertion considering aspects such as
 // the signing key, and consistency with other
 // assertions in the database.
-type Checker func(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTime time.Time) error
+type Checker func(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTimeEarliest, checkTimeLatest time.Time) error
 
 // Database holds assertions and can be used to sign or check
 // further assertions.
@@ -215,7 +267,8 @@ type Database struct {
 	// backstores of dbs this was built on by stacking
 	stackedOn []Backstore
 
-	checkers []Checker
+	checkers     []Checker
+	earliestTime time.Time
 }
 
 // OpenDatabase opens the assertion database based on the configuration.
@@ -311,7 +364,7 @@ func (db *Database) ImportKey(privKey PrivateKey) error {
 }
 
 var (
-	// for sanity checking of base64 hash strings
+	// for validity checking of base64 hash strings
 	base64HashLike = regexp.MustCompile("^[[:alnum:]_-]*$")
 )
 
@@ -373,6 +426,15 @@ func (db *Database) IsTrustedAccount(accountID string) bool {
 	return err == nil
 }
 
+var timeNow = time.Now
+
+// SetEarliestTime affects how key expiration is checked.
+// Instead of considering current system time, only assume that current time
+// is >= earliest. If earliest is zero reset to considering current system time.
+func (db *Database) SetEarliestTime(earliest time.Time) {
+	db.earliestTime = earliest
+}
+
 // Check tests whether the assertion is properly signed and consistent with all the stored knowledge.
 func (db *Database) Check(assert Assertion) error {
 	if !assert.SupportedFormat() {
@@ -380,7 +442,14 @@ func (db *Database) Check(assert Assertion) error {
 	}
 
 	typ := assert.Type()
-	now := time.Now()
+	// assume current time is >= earliestTime and <= latestTime
+	earliestTime := db.earliestTime
+	var latestTime time.Time
+	if earliestTime.IsZero() {
+		// use the current system time by setting both to it
+		earliestTime = timeNow()
+		latestTime = earliestTime
+	}
 
 	var accKey *AccountKey
 	var err error
@@ -400,7 +469,7 @@ func (db *Database) Check(assert Assertion) error {
 	}
 
 	for _, checker := range db.checkers {
-		err := checker(assert, accKey, db, now)
+		err := checker(assert, accKey, db, earliestTime, latestTime)
 		if err != nil {
 			return err
 		}
@@ -520,6 +589,8 @@ func find(backstores []Backstore, assertionType *AssertionType, headers map[stri
 
 // Find an assertion based on arbitrary headers.
 // Provided headers must contain the primary key for the assertion type.
+// Optional primary key headers can be omitted in which case
+// their default values will be used.
 // It returns a NotFoundError if the assertion cannot be found.
 func (db *Database) Find(assertionType *AssertionType, headers map[string]string) (Assertion, error) {
 	return find(db.backstores, assertionType, headers, -1)
@@ -534,14 +605,18 @@ func (db *Database) FindMaxFormat(assertionType *AssertionType, headers map[stri
 
 // FindPredefined finds an assertion in the predefined sets (trusted
 // or not) based on arbitrary headers.  Provided headers must contain
-// the primary key for the assertion type.  It returns a NotFoundError
-// if the assertion cannot be found.
+// the primary key for the assertion type.
+// Optional primary key headers can be omitted in which case
+// their default values will be used.
+// It returns a NotFoundError if the assertion cannot be found.
 func (db *Database) FindPredefined(assertionType *AssertionType, headers map[string]string) (Assertion, error) {
 	return find([]Backstore{db.trusted, db.predefined}, assertionType, headers, -1)
 }
 
 // FindTrusted finds an assertion in the trusted set based on arbitrary headers.
 // Provided headers must contain the primary key for the assertion type.
+// Optional primary key headers can be omitted in which case
+// their default values will be used.
 // It returns a NotFoundError if the assertion cannot be found.
 func (db *Database) FindTrusted(assertionType *AssertionType, headers map[string]string) (Assertion, error) {
 	return find([]Backstore{db.trusted}, assertionType, headers, -1)
@@ -617,7 +692,7 @@ func (db *Database) FindSequence(assertType *AssertionType, sequenceHeaders map[
 
 	// form the sequence key using all keys but the last one which
 	// is the sequence number
-	seqKey, err := keysFromHeaders(assertType.PrimaryKey[:len(assertType.PrimaryKey)-1], sequenceHeaders)
+	seqKey, err := keysFromHeaders(assertType.PrimaryKey[:len(assertType.PrimaryKey)-1], sequenceHeaders, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -663,7 +738,7 @@ func (db *Database) FindSequence(assertType *AssertionType, sequenceHeaders map[
 // assertion checkers
 
 // CheckSigningKeyIsNotExpired checks that the signing key is not expired.
-func CheckSigningKeyIsNotExpired(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTime time.Time) error {
+func CheckSigningKeyIsNotExpired(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTimeEarliest, checkTimeLatest time.Time) error {
 	if signingKey == nil {
 		// assert isn't signed with an account-key key, CheckSignature
 		// will fail anyway unless we teach it more stuff,
@@ -671,17 +746,41 @@ func CheckSigningKeyIsNotExpired(assert Assertion, signingKey *AccountKey, roDB
 		// (e.g. account-key-request)
 		return nil
 	}
-	if !signingKey.isKeyValidAt(checkTime) {
-		return fmt.Errorf("assertion is signed with expired public key %q from %q", assert.SignKeyID(), assert.AuthorityID())
+	if !signingKey.isValidAssumingCurTimeWithin(checkTimeEarliest, checkTimeLatest) {
+		mismatchReason := timeMismatchMsg(checkTimeEarliest, checkTimeLatest, signingKey.since, signingKey.until)
+		return fmt.Errorf("assertion is signed with expired public key %q from %q: %s", assert.SignKeyID(), assert.AuthorityID(), mismatchReason)
 	}
 	return nil
 }
 
+func timeMismatchMsg(earliest, latest, keySince, keyUntil time.Time) string {
+	var msg string
+
+	validFrom := earliest.Format(time.RFC3339)
+	if !latest.IsZero() && !latest.Equal(earliest) {
+		validTo := latest.Format(time.RFC3339)
+		msg = fmt.Sprintf("current time range is [%s, %s]", validFrom, validTo)
+	} else {
+		msg = fmt.Sprintf("current time is %s", validFrom)
+	}
+
+	keyFrom := keySince.Format(time.RFC3339)
+	if !keyUntil.IsZero() {
+		keyTo := keyUntil.Format(time.RFC3339)
+		return msg + fmt.Sprintf(" but key is valid during [%s, %s)", keyFrom, keyTo)
+	}
+
+	return msg + fmt.Sprintf(" but key is valid from %s", keyFrom)
+}
+
 // CheckSignature checks that the signature is valid.
-func CheckSignature(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTime time.Time) error {
+func CheckSignature(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTimeEarliest, checkTimeLatest time.Time) (err error) {
 	var pubKey PublicKey
 	if signingKey != nil {
 		pubKey = signingKey.publicKey()
+		if assert.AuthorityID() != signingKey.AccountID() {
+			return fmt.Errorf("assertion authority %q does not match public key from %q", assert.AuthorityID(), signingKey.AccountID())
+		}
 	} else {
 		custom, ok := assert.(customSigner)
 		if !ok {
@@ -707,7 +806,7 @@ type timestamped interface {
 
 // CheckTimestampVsSigningKeyValidity verifies that the timestamp of
 // the assertion is within the signing key validity.
-func CheckTimestampVsSigningKeyValidity(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTime time.Time) error {
+func CheckTimestampVsSigningKeyValidity(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTimeEarliest, checkTimeLatest time.Time) error {
 	if signingKey == nil {
 		// assert isn't signed with an account-key key, CheckSignature
 		// will fail anyway unless we teach it more stuff.
@@ -717,19 +816,18 @@ func CheckTimestampVsSigningKeyValidity(assert Assertion, signingKey *AccountKey
 	}
 	if tstamped, ok := assert.(timestamped); ok {
 		checkTime := tstamped.Timestamp()
-		if !signingKey.isKeyValidAt(checkTime) {
+		if !signingKey.isValidAt(checkTime) {
 			until := ""
 			if !signingKey.Until().IsZero() {
 				until = fmt.Sprintf(" until %q", signingKey.Until())
 			}
-			return fmt.Errorf("%s assertion timestamp outside of signing key validity (key valid since %q%s)", assert.Type().Name, signingKey.Since(), until)
+			return fmt.Errorf("%s assertion timestamp %q outside of signing key validity (key valid since %q%s)",
+				assert.Type().Name, checkTime, signingKey.Since(), until)
 		}
 	}
 	return nil
 }
 
-// XXX: keeping these in this form until we know better
-
 // A consistencyChecker performs further checks based on the full
 // assertion database knowledge and its own signing key.
 type consistencyChecker interface {
@@ -737,7 +835,7 @@ type consistencyChecker interface {
 }
 
 // CheckCrossConsistency verifies that the assertion is consistent with the other statements in the database.
-func CheckCrossConsistency(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTime time.Time) error {
+func CheckCrossConsistency(assert Assertion, signingKey *AccountKey, roDB RODatabase, checkTimeEarliest, checkTimeLatest time.Time) error {
 	// see if the assertion requires further checks
 	if checker, ok := assert.(consistencyChecker); ok {
 		return checker.checkConsistency(roDB, signingKey)
