@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2019-2020 Canonical Ltd
+ * Copyright (C) 2019-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -23,7 +23,6 @@ import (
 	"errors"
 	"fmt"
 
-	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/bootloader"
 	"github.com/snapcore/snapd/snap"
 )
@@ -41,13 +40,33 @@ const (
 	TryingStatus = "trying"
 )
 
+// RebootInfo contains information about how to perform a reboot if
+// required
+type RebootInfo struct {
+	// RebootRequired is true if we need to reboot after an update
+	RebootRequired bool
+	// RebootBootloader will not be nil if the bootloader has something to say on
+	// how to perform the reboot
+	RebootBootloader bootloader.RebootBootloader
+}
+
+// NextBootContext carries additional significative information used when
+// setting the next boot.
+type NextBootContext struct {
+	// BootWithoutTry is sets if we don't want to use the "try" logic. This
+	// is useful if the next boot is part of an installation undo.
+	BootWithoutTry bool
+}
+
 // A BootParticipant handles the boot process details for a snap involved in it.
 type BootParticipant interface {
-	// SetNextBoot will schedule the snap to be used in the next boot. For
-	// base snaps it is up to the caller to select the right bootable base
-	// (from the model assertion). It is a noop for not relevant snaps.
-	// Otherwise it returns whether a reboot is required.
-	SetNextBoot() (rebootRequired bool, err error)
+	// SetNextBoot will schedule the snap to be used in the next
+	// boot. bootCtx contains context information that influences how the
+	// next boot is performed. For base snaps it is up to the caller to
+	// select the right bootable base (from the model assertion). It is a
+	// noop for not relevant snaps.  Otherwise it returns whether a reboot
+	// is required.
+	SetNextBoot(bootCtx NextBootContext) (rebootInfo RebootInfo, err error)
 
 	// Is this a trivial implementation of the interface?
 	IsTrivial() bool
@@ -68,7 +87,9 @@ type BootKernel interface {
 
 type trivial struct{}
 
-func (trivial) SetNextBoot() (bool, error)               { return false, nil }
+func (trivial) SetNextBoot(bootCtx NextBootContext) (RebootInfo, error) {
+	return RebootInfo{RebootRequired: false}, nil
+}
 func (trivial) IsTrivial() bool                          { return true }
 func (trivial) RemoveKernelAssets() error                { return nil }
 func (trivial) ExtractKernelAssets(snap.Container) error { return nil }
@@ -79,21 +100,6 @@ var _ BootParticipant = trivial{}
 // ensure trivial is a Kernel
 var _ BootKernel = trivial{}
 
-// Device carries information about the device model and mode that is
-// relevant to boot. Note snapstate.DeviceContext implements this, and that's
-// the expected use case.
-type Device interface {
-	RunMode() bool
-	Classic() bool
-
-	Kernel() string
-	Base() string
-
-	HasModeenv() bool
-
-	Model() *asserts.Model
-}
-
 // Participant figures out what the BootParticipant is for the given
 // arguments, and returns it. If the snap does _not_ participate in
 // the boot process, the returned object will be a NOP, so it's safe
@@ -101,7 +107,7 @@ type Device interface {
 //
 // Currently, on classic, nothing is a boot participant (returned will
 // always be NOP).
-func Participant(s snap.PlaceInfo, t snap.Type, dev Device) BootParticipant {
+func Participant(s snap.PlaceInfo, t snap.Type, dev snap.Device) BootParticipant {
 	if applicable(s, t, dev) {
 		bs, err := bootStateFor(t, dev)
 		if err != nil {
@@ -115,7 +121,7 @@ func Participant(s snap.PlaceInfo, t snap.Type, dev Device) BootParticipant {
 
 // bootloaderOptionsForDeviceKernel returns a set of bootloader options that
 // enable correct kernel extraction and removal for given device
-func bootloaderOptionsForDeviceKernel(dev Device) *bootloader.Options {
+func bootloaderOptionsForDeviceKernel(dev snap.Device) *bootloader.Options {
 	if !dev.HasModeenv() {
 		return nil
 	}
@@ -128,15 +134,32 @@ func bootloaderOptionsForDeviceKernel(dev Device) *bootloader.Options {
 // Kernel checks that the given arguments refer to a kernel snap
 // that participates in the boot process, and returns the associated
 // BootKernel, or a trivial implementation otherwise.
-func Kernel(s snap.PlaceInfo, t snap.Type, dev Device) BootKernel {
+func Kernel(s snap.PlaceInfo, t snap.Type, dev snap.Device) BootKernel {
 	if t == snap.TypeKernel && applicable(s, t, dev) {
 		return &coreKernel{s: s, bopts: bootloaderOptionsForDeviceKernel(dev)}
 	}
 	return trivial{}
 }
 
-func applicable(s snap.PlaceInfo, t snap.Type, dev Device) bool {
-	if dev.Classic() {
+// SnapTypeParticipatesInBoot returns whether a snap type participates in the
+// boot for a given device.
+func SnapTypeParticipatesInBoot(t snap.Type, dev snap.Device) bool {
+	if dev.IsClassicBoot() {
+		return false
+	}
+	switch t {
+	case snap.TypeBase, snap.TypeOS:
+		// Bases are not boot participants for classic with modes
+		return !dev.Classic()
+	case snap.TypeKernel, snap.TypeGadget:
+		return true
+	}
+
+	return false
+}
+
+func applicable(s snap.PlaceInfo, t snap.Type, dev snap.Device) bool {
+	if !SnapTypeParticipatesInBoot(t, dev) {
 		return false
 	}
 	// In ephemeral modes we never need to care about updating the boot
@@ -145,15 +168,12 @@ func applicable(s snap.PlaceInfo, t snap.Type, dev Device) bool {
 		return false
 	}
 
-	if t != snap.TypeOS && t != snap.TypeKernel && t != snap.TypeBase {
-		// note we don't currently have anything useful to do with gadgets
-		return false
-	}
-
 	switch t {
 	case snap.TypeKernel:
 		if s.InstanceName() != dev.Kernel() {
-			// a remodel might leave you in this state
+			// a remodel might leave behind installed a kernel that
+			// is not the device kernel anymore, ignore such a
+			// kernel by checking the name
 			return false
 		}
 	case snap.TypeBase, snap.TypeOS:
@@ -164,6 +184,16 @@ func applicable(s snap.PlaceInfo, t snap.Type, dev Device) bool {
 		if s.InstanceName() != base {
 			return false
 		}
+	case snap.TypeGadget:
+		// First condition: gadget is not a boot participant for UC16/18
+		// Second condition: a remodel might leave behind installed a
+		// gadget that is not the device gadget anymore, ignore such a
+		// gadget by checking the name
+		if !dev.HasModeenv() || s.InstanceName() != dev.Gadget() {
+			return false
+		}
+	default:
+		return false
 	}
 
 	return true
@@ -181,10 +211,12 @@ type bootState interface {
 	// curSnap instead if the error is only for the trySnap or tryingStatus.
 	revisions() (curSnap, trySnap snap.PlaceInfo, tryingStatus string, err error)
 
-	// setNext lazily implements setting the next boot target for
-	// the type's boot snap. actually committing the update
-	// is done via the returned bootStateUpdate's commit method.
-	setNext(s snap.PlaceInfo) (rebootRequired bool, u bootStateUpdate, err error)
+	// setNext lazily implements setting the next boot target for the type's
+	// boot snap. bootCtx specifies additional information bits we might
+	// need. Actually committing the update is done via the returned
+	// bootStateUpdate's commit method. It will return information for
+	// rebooting if necessary.
+	setNext(s snap.PlaceInfo, bootCtx NextBootContext) (rbi RebootInfo, u bootStateUpdate, err error)
 
 	// markSuccessful lazily implements marking the boot
 	// successful for the type's boot snap. The actual committing
@@ -203,22 +235,25 @@ type successfulBootState interface {
 
 // bootStateFor finds the right bootState implementation of the given
 // snap type and Device, if applicable.
-func bootStateFor(typ snap.Type, dev Device) (s bootState, err error) {
+func bootStateFor(typ snap.Type, dev snap.Device) (s bootState, err error) {
 	if !dev.RunMode() {
 		return nil, fmt.Errorf("internal error: no boot state handling for ephemeral modes")
 	}
+	if typ == snap.TypeOS {
+		typ = snap.TypeBase
+	}
 	newBootState := newBootState16
+	participantTypes := []snap.Type{snap.TypeBase, snap.TypeKernel}
 	if dev.HasModeenv() {
 		newBootState = newBootState20
+		participantTypes = append(participantTypes, snap.TypeGadget)
 	}
-	switch typ {
-	case snap.TypeOS, snap.TypeBase:
-		return newBootState(snap.TypeBase, dev), nil
-	case snap.TypeKernel:
-		return newBootState(snap.TypeKernel, dev), nil
-	default:
-		return nil, fmt.Errorf("internal error: no boot state handling for snap type %q", typ)
+	for _, partTyp := range participantTypes {
+		if typ == partTyp {
+			return newBootState(typ, dev), nil
+		}
 	}
+	return nil, fmt.Errorf("internal error: no boot state handling for snap type %q", typ)
 }
 
 // InUseFunc is a function to check if the snap is in use or not.
@@ -232,7 +267,7 @@ func fixedInUse(inUse bool) InUseFunc {
 
 // InUse returns a checker for whether a given name/revision is used in the
 // boot environment for snaps of the relevant snap type.
-func InUse(typ snap.Type, dev Device) (InUseFunc, error) {
+func InUse(typ snap.Type, dev snap.Device) (InUseFunc, error) {
 	if dev.Classic() {
 		// no boot state on classic
 		return fixedInUse(false), nil
@@ -280,7 +315,7 @@ var (
 // GetCurrentBoot returns the currently set name and revision for boot for the given
 // type of snap, which can be snap.TypeBase (or snap.TypeOS), or snap.TypeKernel.
 // Returns ErrBootNameAndRevisionNotReady if the values are temporarily not established.
-func GetCurrentBoot(t snap.Type, dev Device) (snap.PlaceInfo, error) {
+func GetCurrentBoot(t snap.Type, dev snap.Device) (snap.PlaceInfo, error) {
 	s, err := bootStateFor(t, dev)
 	if err != nil {
 		return nil, err
@@ -324,7 +359,7 @@ type bootStateUpdate interface {
 //   means snapd did not start successfully. In this case the bootloader
 //   will set snap_mode="" and the system will boot with the known good
 //   values from snap_{core,kernel}
-func MarkBootSuccessful(dev Device) error {
+func MarkBootSuccessful(dev snap.Device) error {
 	const errPrefix = "cannot mark boot successful: %s"
 
 	var u bootStateUpdate
@@ -344,6 +379,7 @@ func MarkBootSuccessful(dev Device) error {
 			trustedAssetsBootState(dev),
 			trustedCommandLineBootState(dev),
 			recoverySystemsBootState(dev),
+			modelBootState(dev),
 		} {
 			var err error
 			u, err = bs.markSuccessful(u)
@@ -367,7 +403,7 @@ var ErrUnsupportedSystemMode = errors.New("system mode is unsupported")
 // the given recovery system in a particular mode. Returns
 // ErrUnsupportedSystemMode when booting into a recovery system is not supported
 // by the device.
-func SetRecoveryBootSystemAndMode(dev Device, systemLabel, mode string) error {
+func SetRecoveryBootSystemAndMode(dev snap.Device, systemLabel, mode string) error {
 	if !dev.HasModeenv() {
 		// only UC20 devices are supported
 		return ErrUnsupportedSystemMode
@@ -400,7 +436,7 @@ func SetRecoveryBootSystemAndMode(dev Device, systemLabel, mode string) error {
 // UpdateManagedBootConfigs updates managed boot config assets if those are
 // present for the ubuntu-boot bootloader. Returns true when an update was
 // carried out.
-func UpdateManagedBootConfigs(dev Device, gadgetSnapOrDir string) (updated bool, err error) {
+func UpdateManagedBootConfigs(dev snap.Device, gadgetSnapOrDir string) (updated bool, err error) {
 	if !dev.HasModeenv() {
 		// only UC20 devices use managed boot config
 		return false, nil
@@ -411,7 +447,7 @@ func UpdateManagedBootConfigs(dev Device, gadgetSnapOrDir string) (updated bool,
 	return updateManagedBootConfigForBootloader(dev, ModeRun, gadgetSnapOrDir)
 }
 
-func updateManagedBootConfigForBootloader(dev Device, mode, gadgetSnapOrDir string) (updated bool, err error) {
+func updateManagedBootConfigForBootloader(dev snap.Device, mode, gadgetSnapOrDir string) (updated bool, err error) {
 	if mode != ModeRun {
 		return false, fmt.Errorf("internal error: updating boot config of recovery bootloader is not supported yet")
 	}
@@ -440,10 +476,10 @@ func updateManagedBootConfigForBootloader(dev Device, mode, gadgetSnapOrDir stri
 // contributes to the kernel command line of the run system. Returns true when a
 // change in command line has been observed and a reboot is needed. The reboot,
 // if needed, should be requested at the the earliest possible occasion.
-func UpdateCommandLineForGadgetComponent(dev Device, gadgetSnapOrDir string) (needsReboot bool, err error) {
+func UpdateCommandLineForGadgetComponent(dev snap.Device, gadgetSnapOrDir string) (needsReboot bool, err error) {
 	if !dev.HasModeenv() {
 		// only UC20 devices are supported
-		return false, fmt.Errorf("internal error: command line component cannot be updated on non UC20 devices")
+		return false, fmt.Errorf("internal error: command line component cannot be updated on pre-UC20 devices")
 	}
 	opts := &bootloader.Options{
 		Role: bootloader.RoleRunMode,
@@ -477,3 +513,16 @@ func UpdateCommandLineForGadgetComponent(dev Device, gadgetSnapOrDir string) (ne
 	}
 	return cmdlineChange, nil
 }
+
+// MarkFactoryResetComplete runs a series of steps in a run system that complete a
+// factory reset process.
+func MarkFactoryResetComplete(encrypted bool) error {
+	if !encrypted {
+		// there is nothing to do on an unencrypted system
+		return nil
+	}
+	if err := postFactoryResetCleanup(); err != nil {
+		return fmt.Errorf("cannot perform post factory reset boot cleanup: %v", err)
+	}
+	return nil
+}
