@@ -41,10 +41,15 @@ const (
 	// ModeRecover is a mode in which the device boots into the recovery
 	// system.
 	ModeRecover = "recover"
+	// ModeFactoryReset is a mode in which the device performs a factory
+	// reset.
+	ModeFactoryReset = "factory-reset"
+	// ModeRunCVM is Azure CVM specific run mode fde + classic debs
+	ModeRunCVM = "cloudimg-rootfs"
 )
 
 var (
-	validModes = []string{ModeInstall, ModeRecover, ModeRun}
+	validModes = []string{ModeInstall, ModeRecover, ModeFactoryReset, ModeRun, ModeRunCVM}
 )
 
 // ModeAndRecoverySystemFromKernelCommandLine returns the current system mode
@@ -134,11 +139,8 @@ const (
 	candidateEdition
 )
 
-func composeCommandLine(model *asserts.Model, currentOrCandidate int, mode, system, gadgetDirOrSnapPath string) (string, error) {
-	if model.Grade() == asserts.ModelGradeUnset {
-		return "", nil
-	}
-	if mode != ModeRun && mode != ModeRecover {
+func composeCommandLine(currentOrCandidate int, mode, system, gadgetDirOrSnapPath string) (string, error) {
+	if mode != ModeRun && mode != ModeRecover && mode != ModeFactoryReset {
 		return "", fmt.Errorf("internal error: unsupported command line mode %q", mode)
 	}
 	// get the run mode bootloader under the native run partition layout
@@ -150,7 +152,7 @@ func composeCommandLine(model *asserts.Model, currentOrCandidate int, mode, syst
 	components := bootloader.CommandLineComponents{
 		ModeArg: "snapd_recovery_mode=run",
 	}
-	if mode == ModeRecover {
+	if mode == ModeRecover || mode == ModeFactoryReset {
 		if system == "" {
 			return "", fmt.Errorf("internal error: system is unset")
 		}
@@ -158,8 +160,12 @@ func composeCommandLine(model *asserts.Model, currentOrCandidate int, mode, syst
 		opts.Role = bootloader.RoleRecovery
 		bootloaderRootDir = InitramfsUbuntuSeedDir
 		// recovery mode & system command line arguments
+		modeArg := "snapd_recovery_mode=recover"
+		if mode == ModeFactoryReset {
+			modeArg = "snapd_recovery_mode=factory-reset"
+		}
 		components = bootloader.CommandLineComponents{
-			ModeArg:   "snapd_recovery_mode=recover",
+			ModeArg:   modeArg,
 			SystemArg: fmt.Sprintf("snapd_recovery_system=%v", system),
 		}
 	}
@@ -194,27 +200,39 @@ func composeCommandLine(model *asserts.Model, currentOrCandidate int, mode, syst
 // ComposeRecoveryCommandLine composes the kernel command line used when booting
 // a given system in recover mode.
 func ComposeRecoveryCommandLine(model *asserts.Model, system, gadgetDirOrSnapPath string) (string, error) {
-	return composeCommandLine(model, currentEdition, ModeRecover, system, gadgetDirOrSnapPath)
+	if model.Grade() == asserts.ModelGradeUnset {
+		return "", nil
+	}
+	return composeCommandLine(currentEdition, ModeRecover, system, gadgetDirOrSnapPath)
 }
 
 // ComposeCommandLine composes the kernel command line used when booting the
 // system in run mode.
 func ComposeCommandLine(model *asserts.Model, gadgetDirOrSnapPath string) (string, error) {
-	return composeCommandLine(model, currentEdition, ModeRun, "", gadgetDirOrSnapPath)
+	if model.Grade() == asserts.ModelGradeUnset {
+		return "", nil
+	}
+	return composeCommandLine(currentEdition, ModeRun, "", gadgetDirOrSnapPath)
 }
 
 // ComposeCandidateCommandLine composes the kernel command line used when
 // booting the system in run mode with the current built-in edition of managed
 // boot assets.
 func ComposeCandidateCommandLine(model *asserts.Model, gadgetDirOrSnapPath string) (string, error) {
-	return composeCommandLine(model, candidateEdition, ModeRun, "", gadgetDirOrSnapPath)
+	if model.Grade() == asserts.ModelGradeUnset {
+		return "", nil
+	}
+	return composeCommandLine(candidateEdition, ModeRun, "", gadgetDirOrSnapPath)
 }
 
 // ComposeCandidateRecoveryCommandLine composes the kernel command line used
 // when booting the given system in recover mode with the current built-in
 // edition of managed boot assets.
 func ComposeCandidateRecoveryCommandLine(model *asserts.Model, system, gadgetDirOrSnapPath string) (string, error) {
-	return composeCommandLine(model, candidateEdition, ModeRecover, system, gadgetDirOrSnapPath)
+	if model.Grade() == asserts.ModelGradeUnset {
+		return "", nil
+	}
+	return composeCommandLine(candidateEdition, ModeRecover, system, gadgetDirOrSnapPath)
 }
 
 // observeSuccessfulCommandLine observes a successful boot with a command line
@@ -341,6 +359,7 @@ func observeCommandLineUpdate(model *asserts.Model, reason commandLineUpdateReas
 		// command line is the same or no actual change in modeenv
 		return false, nil
 	}
+	logger.Debugf("kernel commandline changes from %q to %q", cmdline, candidateCmdline)
 	// actual change of the command line content
 	m.CurrentKernelCommandLines = bootCommandLines{cmdline, candidateCmdline}
 
@@ -349,7 +368,7 @@ func observeCommandLineUpdate(model *asserts.Model, reason commandLineUpdateReas
 	}
 
 	expectReseal := true
-	if err := resealKeyToModeenv(dirs.GlobalRootDir, model, m, expectReseal); err != nil {
+	if err := resealKeyToModeenv(dirs.GlobalRootDir, m, expectReseal); err != nil {
 		return false, err
 	}
 	return true, nil
@@ -358,7 +377,7 @@ func observeCommandLineUpdate(model *asserts.Model, reason commandLineUpdateReas
 // kernelCommandLinesForResealWithFallback provides the list of kernel command
 // lines for use during reseal. During normal operation, the command lines will
 // be listed in the modeenv.
-func kernelCommandLinesForResealWithFallback(model *asserts.Model, modeenv *Modeenv) (cmdlines []string, err error) {
+func kernelCommandLinesForResealWithFallback(modeenv *Modeenv) (cmdlines []string, err error) {
 	if len(modeenv.CurrentKernelCommandLines) > 0 {
 		return modeenv.CurrentKernelCommandLines, nil
 	}
@@ -366,7 +385,8 @@ func kernelCommandLinesForResealWithFallback(model *asserts.Model, modeenv *Mode
 	// default during snapd update, since this is a compatibility scenario
 	// there would be no kernel command lines arguments coming from the
 	// gadget either
-	cmdline, err := ComposeCommandLine(model, "")
+	gadgetDir := ""
+	cmdline, err := composeCommandLine(currentEdition, ModeRun, "", gadgetDir)
 	if err != nil {
 		return nil, err
 	}
