@@ -32,6 +32,8 @@ import (
 	"github.com/snapcore/snapd/bootloader"
 	"github.com/snapcore/snapd/bootloader/bootloadertest"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/gadget"
+	"github.com/snapcore/snapd/gadget/gadgettest"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 )
@@ -96,9 +98,9 @@ func (s *initramfsSuite) TestEnsureNextBootToRunModeRealBootloader(c *C) {
 	})
 }
 
-func makeSnapFilesOnInitramfsUbuntuData(c *C, comment CommentInterface, snaps ...snap.PlaceInfo) (restore func()) {
+func makeSnapFilesOnInitramfsUbuntuData(c *C, rootfsDir string, comment CommentInterface, snaps ...snap.PlaceInfo) (restore func()) {
 	// also make sure the snaps also exist on ubuntu-data
-	snapDir := dirs.SnapBlobDirUnder(boot.InitramfsWritableDir)
+	snapDir := dirs.SnapBlobDirUnder(rootfsDir)
 	err := os.MkdirAll(snapDir, 0755)
 	c.Assert(err, IsNil, comment)
 	paths := make([]string, 0, len(snaps))
@@ -130,8 +132,12 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 	base2, err := snap.ParsePlaceInfoFromSnapFileName("core20_2.snap")
 	c.Assert(err, IsNil)
 
+	gadget, err := snap.ParsePlaceInfoFromSnapFileName("pc_1.snap")
+	c.Assert(err, IsNil)
+
 	baseT := snap.TypeBase
 	kernelT := snap.TypeKernel
+	gadgetT := snap.TypeGadget
 
 	tt := []struct {
 		m              *boot.Modeenv
@@ -143,8 +149,9 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 		snapsToMake    []snap.PlaceInfo
 		expected       map[snap.Type]snap.PlaceInfo
 		errPattern     string
-		comment        string
 		expRebootPanic string
+		rootfsDir      string
+		comment        string
 	}{
 		//
 		// default paths
@@ -156,8 +163,27 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{baseT},
 			snapsToMake: []snap.PlaceInfo{base1},
 			expected:    map[snap.Type]snap.PlaceInfo{baseT: base1},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "default base path",
 		},
+		// gadget base path
+		{
+			m:           &boot.Modeenv{Mode: "run", Gadget: gadget.Filename()},
+			typs:        []snap.Type{gadgetT},
+			snapsToMake: []snap.PlaceInfo{gadget},
+			expected:    map[snap.Type]snap.PlaceInfo{gadgetT: gadget},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
+			comment:     "default gadget path",
+		},
+		// gadget base path, but not in modeenv, so it is not selected
+		{
+			m:           &boot.Modeenv{Mode: "run"},
+			typs:        []snap.Type{gadgetT},
+			snapsToMake: []snap.PlaceInfo{gadget},
+			expected:    map[snap.Type]snap.PlaceInfo{},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
+			comment:     "default gadget path",
+		},
 		// default kernel path
 		{
 			m:           &boot.Modeenv{Mode: "run", CurrentKernels: []string{kernel1.Filename()}},
@@ -165,8 +191,28 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{kernelT},
 			snapsToMake: []snap.PlaceInfo{kernel1},
 			expected:    map[snap.Type]snap.PlaceInfo{kernelT: kernel1},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "default kernel path",
 		},
+		// gadget base path for classic with modes
+		{
+			m:           &boot.Modeenv{Mode: "run", Gadget: gadget.Filename()},
+			typs:        []snap.Type{gadgetT},
+			snapsToMake: []snap.PlaceInfo{gadget},
+			expected:    map[snap.Type]snap.PlaceInfo{gadgetT: gadget},
+			rootfsDir:   boot.InitramfsDataDir,
+			comment:     "default gadget path for classic with modes",
+		},
+		// default kernel path for classic with modes
+		{
+			m:           &boot.Modeenv{Mode: "run", CurrentKernels: []string{kernel1.Filename()}},
+			kernel:      kernel1,
+			typs:        []snap.Type{kernelT},
+			snapsToMake: []snap.PlaceInfo{kernel1},
+			expected:    map[snap.Type]snap.PlaceInfo{kernelT: kernel1},
+			rootfsDir:   boot.InitramfsDataDir,
+			comment:     "default kernel path for classic with modes",
+		},
 
 		//
 		// happy kernel upgrade paths
@@ -181,6 +227,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			blvars:      map[string]string{"kernel_status": boot.TryingStatus},
 			snapsToMake: []snap.PlaceInfo{kernel1, kernel2},
 			expected:    map[snap.Type]snap.PlaceInfo{kernelT: kernel2},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "successful kernel upgrade path",
 		},
 		// extraneous kernel extracted/set, but kernel_status is default,
@@ -198,6 +245,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			blvars:      map[string]string{"kernel_status": boot.DefaultStatus},
 			snapsToMake: []snap.PlaceInfo{kernel1, kernel2},
 			expected:    map[snap.Type]snap.PlaceInfo{kernelT: kernel1},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "fallback kernel upgrade path, due to kernel_status empty (default)",
 		},
 
@@ -214,6 +262,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			blvars:         map[string]string{"kernel_status": boot.TryingStatus},
 			snapsToMake:    []snap.PlaceInfo{kernel1, kernel2},
 			expRebootPanic: "reboot due to modeenv untrusted try kernel",
+			rootfsDir:      filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:        "fallback kernel upgrade path, due to modeenv untrusted try kernel",
 		},
 		// kernel upgrade path, but reboots to fallback due to try kernel file not existing
@@ -225,6 +274,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			blvars:         map[string]string{"kernel_status": boot.TryingStatus},
 			snapsToMake:    []snap.PlaceInfo{kernel1},
 			expRebootPanic: "reboot due to try kernel file not existing",
+			rootfsDir:      filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:        "fallback kernel upgrade path, due to try kernel file not existing",
 		},
 		// kernel upgrade path, but reboots to fallback due to invalid kernel_status
@@ -236,6 +286,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			blvars:         map[string]string{"kernel_status": boot.TryStatus},
 			snapsToMake:    []snap.PlaceInfo{kernel1, kernel2},
 			expRebootPanic: "reboot due to kernel_status wrong",
+			rootfsDir:      filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:        "fallback kernel upgrade path, due to kernel_status wrong",
 		},
 
@@ -250,6 +301,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{kernelT},
 			snapsToMake: []snap.PlaceInfo{kernel1},
 			errPattern:  fmt.Sprintf("fallback kernel snap %q is not trusted in the modeenv", kernel1.Filename()),
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "fallback kernel not trusted in modeenv",
 		},
 		// fallback kernel file doesn't exist
@@ -258,6 +310,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			kernel:     kernel1,
 			typs:       []snap.Type{kernelT},
 			errPattern: fmt.Sprintf("kernel snap %q does not exist on ubuntu-data", kernel1.Filename()),
+			rootfsDir:  filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:    "fallback kernel file doesn't exist",
 		},
 
@@ -282,6 +335,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{baseT},
 			snapsToMake: []snap.PlaceInfo{base1, base2},
 			expected:    map[snap.Type]snap.PlaceInfo{baseT: base2},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "successful base upgrade path",
 		},
 		// base upgrade path, but uses fallback due to try base file not existing
@@ -301,6 +355,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{baseT},
 			snapsToMake: []snap.PlaceInfo{base1},
 			expected:    map[snap.Type]snap.PlaceInfo{baseT: base1},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "fallback base upgrade path, due to missing try base file",
 		},
 		// base upgrade path, but uses fallback due to base_status trying
@@ -320,6 +375,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{baseT},
 			snapsToMake: []snap.PlaceInfo{base1, base2},
 			expected:    map[snap.Type]snap.PlaceInfo{baseT: base1},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "fallback base upgrade path, due to base_status trying",
 		},
 		// base upgrade path, but uses fallback due to base_status default
@@ -339,6 +395,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{baseT},
 			snapsToMake: []snap.PlaceInfo{base1, base2},
 			expected:    map[snap.Type]snap.PlaceInfo{baseT: base1},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "fallback base upgrade path, due to missing base_status",
 		},
 
@@ -352,6 +409,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{baseT},
 			snapsToMake: []snap.PlaceInfo{base1},
 			errPattern:  "fallback base snap unusable: cannot get snap revision: modeenv base boot variable is empty",
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "base snap unset in modeenv",
 		},
 		// base snap file doesn't exist
@@ -359,6 +417,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			m:          &boot.Modeenv{Mode: "run", Base: base1.Filename()},
 			typs:       []snap.Type{baseT},
 			errPattern: fmt.Sprintf("base snap %q does not exist on ubuntu-data", base1.Filename()),
+			rootfsDir:  filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:    "base snap unset in modeenv",
 		},
 		// unhappy, but silent path with fallback, due to invalid try base snap name
@@ -372,6 +431,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			typs:        []snap.Type{baseT},
 			snapsToMake: []snap.PlaceInfo{base1},
 			expected:    map[snap.Type]snap.PlaceInfo{baseT: base1},
+			rootfsDir:   filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
 			comment:     "corrupted base snap name",
 		},
 
@@ -398,7 +458,8 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 				baseT:   base1,
 				kernelT: kernel1,
 			},
-			comment: "default combined kernel + base",
+			rootfsDir: filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
+			comment:   "default combined kernel + base",
 		},
 		// combined, upgrade only the kernel
 		{
@@ -421,7 +482,8 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 				baseT:   base1,
 				kernelT: kernel2,
 			},
-			comment: "combined kernel + base, successful kernel upgrade",
+			rootfsDir: filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
+			comment:   "combined kernel + base, successful kernel upgrade",
 		},
 		// combined, upgrade only the base
 		{
@@ -446,7 +508,8 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 				baseT:   base2,
 				kernelT: kernel1,
 			},
-			comment: "combined kernel + base, successful base upgrade",
+			rootfsDir: filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
+			comment:   "combined kernel + base, successful base upgrade",
 		},
 		// bonus points: combined upgrade kernel and base
 		{
@@ -473,7 +536,8 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 				baseT:   base2,
 				kernelT: kernel2,
 			},
-			comment: "combined kernel + base, successful base + kernel upgrade",
+			rootfsDir: filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
+			comment:   "combined kernel + base, successful base + kernel upgrade",
 		},
 		// combined, fallback upgrade on kernel
 		{
@@ -496,7 +560,8 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 				baseT:   base1,
 				kernelT: kernel1,
 			},
-			comment: "combined kernel + base, fallback kernel upgrade, due to missing boot var",
+			rootfsDir: filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
+			comment:   "combined kernel + base, fallback kernel upgrade, due to missing boot var",
 		},
 		// combined, fallback upgrade on base
 		{
@@ -521,7 +586,8 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 				baseT:   base1,
 				kernelT: kernel1,
 			},
-			comment: "combined kernel + base, fallback base upgrade, due to base_status trying",
+			rootfsDir: filepath.Join(dirs.GlobalRootDir, "/run/mnt/data/system-data"),
+			comment:   "combined kernel + base, fallback base upgrade, due to base_status trying",
 		},
 	}
 
@@ -583,27 +649,27 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 			}
 
 			if len(t.snapsToMake) != 0 {
-				r := makeSnapFilesOnInitramfsUbuntuData(c, comment, t.snapsToMake...)
+				r := makeSnapFilesOnInitramfsUbuntuData(c, t.rootfsDir, comment, t.snapsToMake...)
 				cleanups = append(cleanups, r)
 			}
 
 			// write the modeenv to somewhere so we can read it and pass that to
 			// InitramfsRunModeChooseSnapsToMount
-			err := t.m.WriteTo(boot.InitramfsWritableDir)
+			err := t.m.WriteTo(t.rootfsDir)
 			// remove it because we are writing many modeenvs in this single test
 			cleanups = append(cleanups, func() {
-				c.Assert(os.Remove(dirs.SnapModeenvFileUnder(boot.InitramfsWritableDir)), IsNil, Commentf(t.comment))
+				c.Assert(os.Remove(dirs.SnapModeenvFileUnder(t.rootfsDir)), IsNil, Commentf(t.comment))
 			})
 			c.Assert(err, IsNil, comment)
 
-			m, err := boot.ReadModeenv(boot.InitramfsWritableDir)
+			m, err := boot.ReadModeenv(t.rootfsDir)
 			c.Assert(err, IsNil, comment)
 
 			if t.expRebootPanic != "" {
-				f := func() { boot.InitramfsRunModeSelectSnapsToMount(t.typs, m) }
+				f := func() { boot.InitramfsRunModeSelectSnapsToMount(t.typs, m, t.rootfsDir) }
 				c.Assert(f, PanicMatches, t.expRebootPanic, comment)
 			} else {
-				mountSnaps, err := boot.InitramfsRunModeSelectSnapsToMount(t.typs, m)
+				mountSnaps, err := boot.InitramfsRunModeSelectSnapsToMount(t.typs, m, t.rootfsDir)
 				if t.errPattern != "" {
 					c.Assert(err, ErrorMatches, t.errPattern, comment)
 				} else {
@@ -614,7 +680,7 @@ func (s *initramfsSuite) TestInitramfsRunModeSelectSnapsToMount(c *C) {
 
 			// check that the modeenv changed as expected
 			if t.expectedM != nil {
-				newM, err := boot.ReadModeenv(boot.InitramfsWritableDir)
+				newM, err := boot.ReadModeenv(t.rootfsDir)
 				c.Assert(err, IsNil, comment)
 				c.Assert(newM.Base, Equals, t.expectedM.Base, comment)
 				c.Assert(newM.BaseStatus, Equals, t.expectedM.BaseStatus, comment)
@@ -747,3 +813,38 @@ func (s *initramfsSuite) TestInitramfsRunModeUpdateBootloaderVarsNoBootloaderHap
 	err := boot.InitramfsRunModeUpdateBootloaderVars()
 	c.Assert(err, IsNil)
 }
+
+var classicModel = &gadgettest.ModelCharacteristics{
+	IsClassic: true,
+	HasModes:  true,
+}
+
+var coreModel = &gadgettest.ModelCharacteristics{
+	IsClassic: false,
+	HasModes:  true,
+}
+
+func (s *initramfsSuite) TestInstallHostWritableDir(c *C) {
+	c.Check(boot.InstallHostWritableDir(classicModel), Equals, filepath.Join(dirs.GlobalRootDir, "/run/mnt/ubuntu-data"))
+	c.Check(boot.InstallHostWritableDir(coreModel), Equals, filepath.Join(dirs.GlobalRootDir, "/run/mnt/ubuntu-data/system-data"))
+}
+
+func (s *initramfsSuite) TestInitramfsHostWritableDir(c *C) {
+	c.Check(boot.InitramfsHostWritableDir(classicModel), Equals, filepath.Join(dirs.GlobalRootDir, "/run/mnt/host/ubuntu-data"))
+	c.Check(boot.InitramfsHostWritableDir(coreModel), Equals, filepath.Join(dirs.GlobalRootDir, "/run/mnt/host/ubuntu-data/system-data"))
+}
+
+func (s *initramfsSuite) TestInitramfsWritableDir(c *C) {
+	for _, tc := range []struct {
+		model       gadget.Model
+		runMode     bool
+		expectedDir string
+	}{
+		{classicModel, true, "/run/mnt/data"},
+		{classicModel, false, "/run/mnt/data/system-data"},
+		{coreModel, true, "/run/mnt/data/system-data"},
+		{coreModel, false, "/run/mnt/data/system-data"},
+	} {
+		c.Check(boot.InitramfsWritableDir(tc.model, tc.runMode), Equals, filepath.Join(dirs.GlobalRootDir, tc.expectedDir))
+	}
+}
