@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2019 Canonical Ltd
+ * Copyright (C) 2014-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -38,6 +38,8 @@ type BootableSet struct {
 	BasePath   string
 	Kernel     *snap.Info
 	KernelPath string
+	Gadget     *snap.Info
+	GadgetPath string
 
 	RecoverySystemLabel string
 	// RecoverySystemDir is a path to a directory with recovery system
@@ -47,7 +49,7 @@ type BootableSet struct {
 
 	UnpackedGadgetDir string
 
-	// Recover is set when making the recovery partition bootable.
+	// Recovery is set when making the recovery partition bootable.
 	Recovery bool
 }
 
@@ -73,7 +75,17 @@ func MakeBootableImage(model *asserts.Model, rootdir string, bootWith *BootableS
 	if !bootWith.Recovery {
 		return fmt.Errorf("internal error: MakeBootableImage called at runtime, use MakeRunnableSystem instead")
 	}
-	return makeBootable20(model, rootdir, bootWith, bootFlags)
+	return makeBootable20(rootdir, bootWith, bootFlags)
+}
+
+// MakeBootablePartition configures a partition mounted on rootdir
+// using information from bootWith and bootFlags. Contrarily to
+// MakeBootableImage this happens in a live system.
+func MakeBootablePartition(partDir string, opts *bootloader.Options, bootWith *BootableSet, bootMode string, bootFlags []string) error {
+	if bootWith.RecoverySystemDir != "" {
+		return fmt.Errorf("internal error: RecoverySystemDir unexpectedly set for MakeBootablePartition")
+	}
+	return configureBootloader(partDir, opts, bootWith, bootMode, bootFlags)
 }
 
 // makeBootable16 setups the image filesystem for boot with UC16
@@ -153,20 +165,7 @@ func makeBootable16(model *asserts.Model, rootdir string, bootWith *BootableSet)
 	return nil
 }
 
-func makeBootable20(model *asserts.Model, rootdir string, bootWith *BootableSet, bootFlags []string) error {
-	// we can only make a single recovery system bootable right now
-	recoverySystems, err := filepath.Glob(filepath.Join(rootdir, "systems/*"))
-	if err != nil {
-		return fmt.Errorf("cannot validate recovery systems: %v", err)
-	}
-	if len(recoverySystems) > 1 {
-		return fmt.Errorf("cannot make multiple recovery systems bootable yet")
-	}
-
-	if bootWith.RecoverySystemLabel == "" {
-		return fmt.Errorf("internal error: recovery system label unset")
-	}
-
+func configureBootloader(rootdir string, opts *bootloader.Options, bootWith *BootableSet, bootMode string, bootFlags []string) error {
 	blVars := make(map[string]string, 3)
 	if len(bootFlags) != 0 {
 		if err := setImageBootFlags(bootFlags, blVars); err != nil {
@@ -174,12 +173,6 @@ func makeBootable20(model *asserts.Model, rootdir string, bootWith *BootableSet,
 		}
 	}
 
-	opts := &bootloader.Options{
-		PrepareImageTime: true,
-		// setup the recovery bootloader
-		Role: bootloader.RoleRecovery,
-	}
-
 	// install the bootloader configuration from the gadget
 	if err := bootloader.InstallBootConfig(bootWith.UnpackedGadgetDir, rootdir, opts); err != nil {
 		return err
@@ -191,19 +184,47 @@ func makeBootable20(model *asserts.Model, rootdir string, bootWith *BootableSet,
 		return fmt.Errorf("internal error: cannot find bootloader: %v", err)
 	}
 
-	// record which recovery system is to be used on the bootloader, note
-	// that this goes on the main bootloader environment, and not on the
-	// recovery system bootloader environment, for example for grub
-	// bootloader, this env var is set on the ubuntu-seed root grubenv, and
-	// not on the recovery system grubenv in the systems/20200314/ subdir on
-	// ubuntu-seed
-	blVars["snapd_recovery_system"] = bootWith.RecoverySystemLabel
-	// always set the mode as install
-	blVars["snapd_recovery_mode"] = ModeInstall
+	blVars["snapd_recovery_mode"] = bootMode
+	if bootWith.RecoverySystemLabel != "" {
+		// record which recovery system is to be used on the bootloader, note
+		// that this goes on the main bootloader environment, and not on the
+		// recovery system bootloader environment, for example for grub
+		// bootloader, this env var is set on the ubuntu-seed root grubenv, and
+		// not on the recovery system grubenv in the systems/20200314/ subdir on
+		// ubuntu-seed
+		blVars["snapd_recovery_system"] = bootWith.RecoverySystemLabel
+	}
+
 	if err := bl.SetBootVars(blVars); err != nil {
 		return fmt.Errorf("cannot set recovery environment: %v", err)
 	}
 
+	return nil
+}
+
+func makeBootable20(rootdir string, bootWith *BootableSet, bootFlags []string) error {
+	// we can only make a single recovery system bootable right now
+	recoverySystems, err := filepath.Glob(filepath.Join(rootdir, "systems/*"))
+	if err != nil {
+		return fmt.Errorf("cannot validate recovery systems: %v", err)
+	}
+	if len(recoverySystems) > 1 {
+		return fmt.Errorf("cannot make multiple recovery systems bootable yet")
+	}
+
+	if bootWith.RecoverySystemLabel == "" {
+		return fmt.Errorf("internal error: recovery system label unset")
+	}
+
+	opts := &bootloader.Options{
+		PrepareImageTime: true,
+		// setup the recovery bootloader
+		Role: bootloader.RoleRecovery,
+	}
+	if err := configureBootloader(rootdir, opts, bootWith, ModeInstall, bootFlags); err != nil {
+		return fmt.Errorf("cannot install bootloader: %v", err)
+	}
+
 	return MakeRecoverySystemBootable(rootdir, bootWith.RecoverySystemDir, &RecoverySystemBootableSet{
 		Kernel:           bootWith.Kernel,
 		KernelPath:       bootWith.KernelPath,
@@ -290,50 +311,66 @@ func MakeRecoverySystemBootable(rootdir string, relativeRecoverySystemDir string
 	return nil
 }
 
-// MakeRunnableSystem is like MakeBootableImage in that it sets up a system to
-// be able to boot, but is unique in that it is intended to be called from UC20
-// install mode and makes the run system bootable (hence it is called
-// "runnable").
-// Note that this function does not update the recovery bootloader env to
-// actually transition to run mode here, that is left to the caller via
-// something like boot.EnsureNextBootToRunMode(). This is to enable separately
-// setting up a run system and actually transitioning to it, with hooks, etc.
-// running in between.
-func MakeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver) error {
-	if model.Grade() == asserts.ModelGradeUnset {
-		return fmt.Errorf("internal error: cannot make non-uc20 system runnable")
+type makeRunnableOptions struct {
+	Standalone     bool
+	AfterDataReset bool
+}
+
+func copyBootSnap(orig string, dstInfo *snap.Info, dstSnapBlobDir string) error {
+	// if the source path is a symlink, don't copy the symlink, copy the
+	// target file instead of copying the symlink, as the initramfs won't
+	// follow the symlink when it goes to mount the base and kernel snaps by
+	// design as the initramfs should only be using trusted things from
+	// ubuntu-data to boot in run mode
+	if osutil.IsSymlink(orig) {
+		link, err := os.Readlink(orig)
+		if err != nil {
+			return err
+		}
+		orig = link
 	}
+	// note that we need to use the "Filename()" here because unasserted
+	// snaps will have names like pc-kernel_5.19.4.snap but snapd expects
+	// "pc-kernel_x1.snap"
+	dst := filepath.Join(dstSnapBlobDir, dstInfo.Filename())
+	if err := osutil.CopyFile(orig, dst, osutil.CopyFlagPreserveAll|osutil.CopyFlagSync); err != nil {
+		return err
+	}
+	return nil
+}
+
+func makeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver, makeOpts makeRunnableOptions) error {
+	if model.Grade() == asserts.ModelGradeUnset {
+		return fmt.Errorf("internal error: cannot make pre-UC20 system runnable")
+	}
+	if bootWith.RecoverySystemDir != "" {
+		return fmt.Errorf("internal error: RecoverySystemDir unexpectedly set for MakeRunnableSystem")
+	}
+
 	// TODO:UC20:
 	// - figure out what to do for uboot gadgets, currently we require them to
 	//   install the boot.sel onto ubuntu-boot directly, but the file should be
 	//   managed by snapd instead
 
-	// copy kernel/base into the ubuntu-data partition
-	snapBlobDir := dirs.SnapBlobDirUnder(InstallHostWritableDir)
+	// copy kernel/base/gadget into the ubuntu-data partition
+	snapBlobDir := dirs.SnapBlobDirUnder(InstallHostWritableDir(model))
 	if err := os.MkdirAll(snapBlobDir, 0755); err != nil {
 		return err
 	}
-	for _, fn := range []string{bootWith.BasePath, bootWith.KernelPath} {
-		dst := filepath.Join(snapBlobDir, filepath.Base(fn))
-		// if the source filename is a symlink, don't copy the symlink, copy the
-		// target file instead of copying the symlink, as the initramfs won't
-		// follow the symlink when it goes to mount the base and kernel snaps by
-		// design as the initramfs should only be using trusted things from
-		// ubuntu-data to boot in run mode
-		if osutil.IsSymlink(fn) {
-			link, err := os.Readlink(fn)
-			if err != nil {
-				return err
-			}
-			fn = link
-		}
-		if err := osutil.CopyFile(fn, dst, osutil.CopyFlagPreserveAll|osutil.CopyFlagSync); err != nil {
+	for _, origDest := range []struct {
+		orig     string
+		destInfo *snap.Info
+	}{
+		{orig: bootWith.BasePath, destInfo: bootWith.Base},
+		{orig: bootWith.KernelPath, destInfo: bootWith.Kernel},
+		{orig: bootWith.GadgetPath, destInfo: bootWith.Gadget}} {
+		if err := copyBootSnap(origDest.orig, origDest.destInfo, snapBlobDir); err != nil {
 			return err
 		}
 	}
 
 	// replicate the boot assets cache in host's writable
-	if err := CopyBootAssetsCacheToRoot(InstallHostWritableDir); err != nil {
+	if err := CopyBootAssetsCacheToRoot(InstallHostWritableDir(model)); err != nil {
 		return fmt.Errorf("cannot replicate boot assets cache: %v", err)
 	}
 
@@ -343,7 +380,7 @@ func MakeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 		currentTrustedBootAssets = sealer.currentTrustedBootAssetsMap()
 		currentTrustedRecoveryBootAssets = sealer.currentTrustedRecoveryBootAssetsMap()
 	}
-	recoverySystemLabel := filepath.Base(bootWith.RecoverySystemDir)
+	recoverySystemLabel := bootWith.RecoverySystemLabel
 	// write modeenv on the ubuntu-data partition
 	modeenv := &Modeenv{
 		Mode:           "run",
@@ -358,11 +395,15 @@ func MakeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 		// installed
 		CurrentKernelCommandLines: nil,
 		// keep this comment to make gofmt 1.9 happy
-		Base:           filepath.Base(bootWith.BasePath),
+		Base:           bootWith.Base.Filename(),
+		Gadget:         bootWith.Gadget.Filename(),
 		CurrentKernels: []string{bootWith.Kernel.Filename()},
 		BrandID:        model.BrandID(),
 		Model:          model.Model(),
+		// TODO: test this
+		Classic:        model.Classic(),
 		Grade:          string(model.Grade()),
+		ModelSignKeyID: model.SignKeyID(),
 	}
 
 	// get the ubuntu-boot bootloader and extract the kernel there
@@ -451,16 +492,66 @@ func MakeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *Tru
 
 	// all fields that needed to be set in the modeenv must have been set by
 	// now, write modeenv to disk
-	if err := modeenv.WriteTo(InstallHostWritableDir); err != nil {
+	if err := modeenv.WriteTo(InstallHostWritableDir(model)); err != nil {
 		return fmt.Errorf("cannot write modeenv: %v", err)
 	}
 
 	if sealer != nil {
+		hasHook, err := HasFDESetupHook(bootWith.Kernel)
+		if err != nil {
+			return fmt.Errorf("cannot check for fde-setup hook: %v", err)
+		}
+
+		flags := sealKeyToModeenvFlags{
+			HasFDESetupHook: hasHook,
+			FactoryReset:    makeOpts.AfterDataReset,
+		}
+		if makeOpts.Standalone {
+			flags.SnapsDir = snapBlobDir
+		}
 		// seal the encryption key to the parameters specified in modeenv
-		if err := sealKeyToModeenv(sealer.dataEncryptionKey, sealer.saveEncryptionKey, model, modeenv); err != nil {
+		if err := sealKeyToModeenv(sealer.dataEncryptionKey, sealer.saveEncryptionKey, model, modeenv, flags); err != nil {
 			return err
 		}
 	}
 
+	// so far so good, we managed to install the system, so it can be used
+	// for recovery as well
+	if err := MarkRecoveryCapableSystem(recoverySystemLabel); err != nil {
+		return fmt.Errorf("cannot record %q as a recovery capable system: %v", recoverySystemLabel, err)
+	}
 	return nil
 }
+
+// MakeRunnableSystem is like MakeBootableImage in that it sets up a system to
+// be able to boot, but is unique in that it is intended to be called from UC20
+// install mode and makes the run system bootable (hence it is called
+// "runnable").
+// Note that this function does not update the recovery bootloader env to
+// actually transition to run mode here, that is left to the caller via
+// something like boot.EnsureNextBootToRunMode(). This is to enable separately
+// setting up a run system and actually transitioning to it, with hooks, etc.
+// running in between.
+func MakeRunnableSystem(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver) error {
+	return makeRunnableSystem(model, bootWith, sealer, makeRunnableOptions{})
+}
+
+// MakeRunnableStandaloneSystem operates like MakeRunnableSystem but does
+// assume that the run system being set up is related to the current
+// system. This is appropriate e.g when installing from a classic installer.
+func MakeRunnableStandaloneSystem(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver) error {
+	// TODO consider merging this back into MakeRunnableSystem but need
+	// to consider the properties of the different input used for sealing
+	return makeRunnableSystem(model, bootWith, sealer, makeRunnableOptions{
+		Standalone: true,
+	})
+}
+
+// MakeRunnableSystemAfterDataReset sets up the system to be able to boot, but it is
+// intended to be called from UC20 factory reset mode right before switching
+// back to the new run system.
+func MakeRunnableSystemAfterDataReset(model *asserts.Model, bootWith *BootableSet, sealer *TrustedAssetsInstallObserver) error {
+	return makeRunnableSystem(model, bootWith, sealer, makeRunnableOptions{
+		AfterDataReset: true,
+	})
+}
