@@ -49,12 +49,13 @@ func (s *grubAssetsTestSuite) SetUpTest(c *C) {
 	s.AddCleanup(archtest.MockArchitecture("amd64"))
 	snippets := []assets.ForEditions{
 		{FirstEdition: 1, Snippet: []byte("console=ttyS0 console=tty1 panic=-1")},
+		{FirstEdition: 3, Snippet: []byte("console=ttyS0,115200n8 console=tty1 panic=-1")},
 	}
 	s.AddCleanup(assets.MockSnippetsForEdition("grub.cfg:static-cmdline", snippets))
 	s.AddCleanup(assets.MockSnippetsForEdition("grub-recovery.cfg:static-cmdline", snippets))
 }
 
-func (s *grubAssetsTestSuite) testGrubConfigContains(c *C, name string, keys ...string) {
+func (s *grubAssetsTestSuite) testGrubConfigContains(c *C, name string, edition int, keys ...string) {
 	a := assets.Internal(name)
 	c.Assert(a, NotNil)
 	as := string(a)
@@ -63,18 +64,19 @@ func (s *grubAssetsTestSuite) testGrubConfigContains(c *C, name string, keys ...
 	}
 	idx := bytes.IndexRune(a, '\n')
 	c.Assert(idx, Not(Equals), -1)
-	c.Assert(string(a[:idx]), Equals, "# Snapd-Boot-Config-Edition: 1")
+	prefix := fmt.Sprintf("# Snapd-Boot-Config-Edition: %d", edition)
+	c.Assert(string(a[:idx]), Equals, prefix)
 }
 
 func (s *grubAssetsTestSuite) TestGrubConf(c *C) {
-	s.testGrubConfigContains(c, "grub.cfg",
+	s.testGrubConfigContains(c, "grub.cfg", 3,
 		"snapd_recovery_mode",
-		"set snapd_static_cmdline_args='console=ttyS0 console=tty1 panic=-1'",
+		"set snapd_static_cmdline_args='console=ttyS0,115200n8 console=tty1 panic=-1'",
 	)
 }
 
 func (s *grubAssetsTestSuite) TestGrubRecoveryConf(c *C) {
-	s.testGrubConfigContains(c, "grub-recovery.cfg",
+	s.testGrubConfigContains(c, "grub-recovery.cfg", 1,
 		"snapd_recovery_mode",
 		"snapd_recovery_system",
 		"set snapd_static_cmdline_args='console=ttyS0 console=tty1 panic=-1'",
@@ -126,8 +128,8 @@ func (s *grubAssetsTestSuite) TestGrubCmdlineSnippetCrossCheck(c *C) {
 		pattern string
 	}{
 		{
-			asset: "grub.cfg", snippet: "grub.cfg:static-cmdline", edition: 1,
-			content: []byte("console=ttyS0 console=tty1 panic=-1"),
+			asset: "grub.cfg", snippet: "grub.cfg:static-cmdline", edition: 3,
+			content: []byte("console=ttyS0,115200n8 console=tty1 panic=-1"),
 			pattern: "set snapd_static_cmdline_args='%s'\n",
 		},
 		{
@@ -143,7 +145,7 @@ func (s *grubAssetsTestSuite) TestGrubCmdlineSnippetCrossCheck(c *C) {
 		// get a matching snippet
 		snip := assets.SnippetForEdition(tc.snippet, tc.edition)
 		c.Assert(snip, NotNil)
-		c.Assert(snip, DeepEquals, tc.content)
+		c.Assert(snip, DeepEquals, tc.content, Commentf("%s: '%s' != '%s'", tc.asset, snip, tc.content))
 		c.Assert(string(grubCfg), testutil.Contains, fmt.Sprintf(tc.pattern, string(snip)))
 	}
 }
