@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2020 Canonical Ltd
+ * Copyright (C) 2014-2021 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -77,6 +77,9 @@ func (o *Options) validate() error {
 	if o.NoSlashBoot && o.Role == RoleSole {
 		return fmt.Errorf("internal error: bootloader.RoleSole doesn't expect NoSlashBoot set")
 	}
+	if o.PrepareImageTime && o.Role == RoleRunMode {
+		return fmt.Errorf("internal error: cannot use run mode bootloader at prepare-image time")
+	}
 	return nil
 }
 
@@ -92,8 +95,12 @@ type Bootloader interface {
 	// Name returns the bootloader name.
 	Name() string
 
-	// ConfigFile returns the name of the config file.
-	ConfigFile() string
+	// Present returns whether the bootloader is currently present on the
+	// system - in other words whether this bootloader has been installed to the
+	// current system. Implementations should only return non-nil error if they
+	// can positively identify that the bootloader is installed, but there is
+	// actually an error with the installation.
+	Present() (bool, error)
 
 	// InstallBootConfig will try to install the boot config in the
 	// given gadgetDir to rootdir.
@@ -106,11 +113,6 @@ type Bootloader interface {
 	RemoveKernelAssets(s snap.PlaceInfo) error
 }
 
-type installableBootloader interface {
-	Bootloader
-	setRootDir(string)
-}
-
 type RecoveryAwareBootloader interface {
 	Bootloader
 	SetRecoverySystemEnv(recoverySystemDir string, values map[string]string) error
@@ -161,6 +163,31 @@ type ExtractedRunKernelImageBootloader interface {
 	DisableTryKernel() error
 }
 
+// ComamndLineComponents carries the components of the kernel command line. The
+// bootloader is expected to combine the provided components, optionally
+// including its built-in static set of arguments, and produce a command line
+// that will be passed to the kernel during boot.
+type CommandLineComponents struct {
+	// Argument related to mode selection.
+	ModeArg string
+	// Argument related to recovery system selection, relevant for given
+	// mode argument.
+	SystemArg string
+	// Extra arguments requested by the system.
+	ExtraArgs string
+	// A complete set of arguments that overrides both the built-in static
+	// set and ExtraArgs. Note that, it is an error if extra and full
+	// arguments are non-empty.
+	FullArgs string
+}
+
+func (c *CommandLineComponents) Validate() error {
+	if c.ExtraArgs != "" && c.FullArgs != "" {
+		return fmt.Errorf("cannot use both full and extra components of command line")
+	}
+	return nil
+}
+
 // TrustedAssetsBootloader has boot assets that take part in the secure boot
 // process and need to be tracked, while other boot assets (typically boot
 // config) are managed by snapd.
@@ -170,17 +197,19 @@ type TrustedAssetsBootloader interface {
 	// ManagedAssets returns a list of boot assets managed by the bootloader
 	// in the boot filesystem. Does not require rootdir to be set.
 	ManagedAssets() []string
-	// UpdateBootConfig updates the boot config assets used by the bootloader.
-	UpdateBootConfig(*Options) error
+	// UpdateBootConfig attempts to update the boot config assets used by
+	// the bootloader. Returns true when assets were updated.
+	UpdateBootConfig() (bool, error)
 	// CommandLine returns the kernel command line composed of mode and
-	// system arguments, built-in bootloader specific static arguments
-	// corresponding to the on-disk boot asset edition, followed by any
-	// extra arguments. The command line may be different when using a
-	// recovery bootloader.
-	CommandLine(modeArg, systemArg, extraArgs string) (string, error)
+	// system arguments, followed by either a built-in bootloader specific
+	// static arguments corresponding to the on-disk boot asset edition, and
+	// any extra arguments or a separate set of arguments provided in the
+	// components. The command line may be different when using a recovery
+	// bootloader.
+	CommandLine(pieces CommandLineComponents) (string, error)
 	// CandidateCommandLine is similar to CommandLine, but uses the current
 	// edition of managed built-in boot assets as reference.
-	CandidateCommandLine(modeArg, systemArg, extraArgs string) (string, error)
+	CandidateCommandLine(pieces CommandLineComponents) (string, error)
 
 	// TrustedAssets returns the list of relative paths to assets inside the
 	// bootloader's rootdir that are measured in the boot process in the
@@ -197,6 +226,27 @@ type TrustedAssetsBootloader interface {
 	BootChain(runBl Bootloader, kernelPath string) ([]BootFile, error)
 }
 
+// NotScriptableBootloader cannot change the bootloader environment
+// because it supports no scripting or cannot do any writes. This
+// applies to piboot for the moment.
+type NotScriptableBootloader interface {
+	Bootloader
+
+	// Sets boot variables from initramfs - this is needed in
+	// addition to SetBootVars() to prevent side effects like
+	// re-writing the bootloader configuration.
+	SetBootVarsFromInitramfs(values map[string]string) error
+}
+
+// RebootBootloader needs arguments to the reboot syscall when snaps
+// are being updated.
+type RebootBootloader interface {
+	Bootloader
+
+	// GetRebootArguments returns the needed reboot arguments
+	GetRebootArguments() (string, error)
+}
+
 func genericInstallBootConfig(gadgetFile, systemFile string) error {
 	if err := os.MkdirAll(filepath.Dir(systemFile), 0755); err != nil {
 		return err
@@ -215,28 +265,31 @@ func genericSetBootConfigFromAsset(systemFile, assetName string) error {
 	return osutil.AtomicWriteFile(systemFile, bootConfig, 0644, 0)
 }
 
-func genericUpdateBootConfigFromAssets(systemFile string, assetName string) error {
+func genericUpdateBootConfigFromAssets(systemFile string, assetName string) (updated bool, err error) {
 	currentBootConfigEdition, err := editionFromDiskConfigAsset(systemFile)
 	if err != nil && err != errNoEdition {
-		return err
+		return false, err
 	}
 	if err == errNoEdition {
-		return nil
+		return false, nil
 	}
 	newBootConfig := assets.Internal(assetName)
 	if len(newBootConfig) == 0 {
-		return fmt.Errorf("no boot config asset with name %q", assetName)
+		return false, fmt.Errorf("no boot config asset with name %q", assetName)
 	}
 	bc, err := configAssetFrom(newBootConfig)
 	if err != nil {
-		return err
+		return false, err
 	}
 	if bc.Edition() <= currentBootConfigEdition {
 		// edition of the candidate boot config is lower than or equal
 		// to one currently installed
-		return nil
+		return false, nil
 	}
-	return osutil.AtomicWriteFile(systemFile, bc.Raw(), 0644, 0)
+	if err := osutil.AtomicWriteFile(systemFile, bc.Raw(), 0644, 0); err != nil {
+		return false, err
+	}
+	return true, nil
 }
 
 // InstallBootConfig installs the bootloader config from the gadget
@@ -262,6 +315,7 @@ var (
 		newGrub,
 		newAndroidBoot,
 		newLk,
+		newPiboot,
 	}
 )
 
@@ -291,9 +345,14 @@ func Find(rootdir string, opts *Options) (Bootloader, error) {
 		opts = &Options{}
 	}
 
+	// note that the order of this is not deterministic
 	for _, blNew := range bootloaders {
 		bl := blNew(rootdir, opts)
-		if osutil.FileExists(bl.ConfigFile()) {
+		present, err := bl.Present()
+		if err != nil {
+			return nil, fmt.Errorf("bootloader %q found but not usable: %v", bl.Name(), err)
+		}
+		if present {
 			return bl, nil
 		}
 	}
