@@ -21,10 +21,14 @@ package client_test
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
+	"io/ioutil"
 	"net/url"
+	"os"
 	"time"
 
+	"golang.org/x/xerrors"
 	"gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/client"
@@ -258,7 +262,12 @@ func (cs *clientSuite) TestClientSnap(c *check.C) {
                             {"type": "screenshot", "url":"http://example.com/shot2.png"}
                         ],
                         "cohort-key": "some-long-cohort-key",
-                        "common-ids": ["org.funky.snap"]
+                        "links": {
+                            "website": ["http://example.com/funky"]
+                        },
+                        "website": "http://example.com/funky",
+                        "common-ids": ["org.funky.snap"],
+                        "store-url": "https://snapcraft.io/chatroom"
 		}
 	}`
 	pkg, _, err := cs.cli.Snap(pkgName)
@@ -302,6 +311,11 @@ func (cs *clientSuite) TestClientSnap(c *check.C) {
 		},
 		CommonIDs: []string{"org.funky.snap"},
 		CohortKey: "some-long-cohort-key",
+		Links: map[string][]string{
+			"website": {"http://example.com/funky"},
+		},
+		Website:  "http://example.com/funky",
+		StoreURL: "https://snapcraft.io/chatroom",
 	})
 }
 
@@ -356,3 +370,44 @@ func (cs *clientSuite) TestAppInfoDaemonIsService(c *check.C) {
 	c.Check(app.Name, check.Equals, "hello")
 	c.Check(app.IsService(), check.Equals, true)
 }
+
+func (cs *clientSuite) TestClientSectionsErrIsWrapped(c *check.C) {
+	cs.err = errors.New("boom")
+	_, err := cs.cli.Sections()
+	var e xerrors.Wrapper
+	c.Assert(err, check.Implements, &e)
+}
+
+func (cs *clientSuite) TestClientFindOneErrIsWrapped(c *check.C) {
+	cs.err = errors.New("boom")
+	_, _, err := cs.cli.FindOne("snap")
+	var e xerrors.Wrapper
+	c.Assert(err, check.Implements, &e)
+}
+
+func (cs *clientSuite) TestClientSnapErrIsWrapped(c *check.C) {
+	// setting cs.err will trigger a "client.ClientError"
+	cs.err = errors.New("boom")
+	_, _, err := cs.cli.Snap("snap")
+	var e xerrors.Wrapper
+	c.Assert(err, check.Implements, &e)
+}
+
+func (cs *clientSuite) TestClientFindFromPathErrIsWrapped(c *check.C) {
+	var e client.AuthorizationError
+
+	// this will trigger a "client.AuthorizationError"
+	err := ioutil.WriteFile(client.TestStoreAuthFilename(os.Getenv("HOME")), []byte("rubbish"), 0644)
+	c.Assert(err, check.IsNil)
+
+	// check that all the functions that use snapsFromPath() get a
+	// wrapped error
+	_, _, err = cs.cli.FindOne("snap")
+	c.Assert(xerrors.As(err, &e), check.Equals, true)
+
+	_, _, err = cs.cli.Find(nil)
+	c.Assert(xerrors.As(err, &e), check.Equals, true)
+
+	_, err = cs.cli.List([]string{"snap"}, nil)
+	c.Assert(xerrors.As(err, &e), check.Equals, true)
+}
