@@ -24,9 +24,11 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"runtime"
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/boot"
+	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/seed"
@@ -36,6 +38,7 @@ import (
 
 var (
 	osutilSetTime = osutil.SetTime
+	runtimeNumCPU = runtime.NumCPU
 )
 
 // initramfsMountsState helps tracking the state and progress
@@ -43,6 +46,8 @@ var (
 type initramfsMountsState struct {
 	mode           string
 	recoverySystem string
+
+	verifiedModel gadget.Model
 }
 
 var errRunModeNoImpliedRecoverySystem = errors.New("internal error: no implied recovery system in run mode")
@@ -70,7 +75,11 @@ func (mst *initramfsMountsState) ReadEssential(recoverySystem string, essentialT
 	//   the RTC does not have a battery or is otherwise unreliable, etc.
 	now := timeNow()
 
-	model, snaps, newTrustedEarliestTime, err := seed.ReadSystemEssentialAndBetterEarliestTime(boot.InitramfsUbuntuSeedDir, recoverySystem, essentialTypes, now, perf)
+	jobs := 1
+	if runtimeNumCPU() > 1 {
+		jobs = 2
+	}
+	model, snaps, newTrustedEarliestTime, err := seed.ReadSystemEssentialAndBetterEarliestTime(boot.InitramfsUbuntuSeedDir, recoverySystem, essentialTypes, now, jobs, perf)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -88,6 +97,14 @@ func (mst *initramfsMountsState) ReadEssential(recoverySystem string, essentialT
 	return model, snaps, nil
 }
 
+// SetVerifiedBootModel sets the "verifiedModel" field. It should only
+// be called after the model is verified. Either via a successful unlock
+// of the encrypted data or after validating the seed in install/recover
+// mode.
+func (mst *initramfsMountsState) SetVerifiedBootModel(m gadget.Model) {
+	mst.verifiedModel = m
+}
+
 // UnverifiedBootModel returns the unverified model from the
 // boot partition for run mode. The current and only use case
 // is measuring the model for run mode. Otherwise no decisions
@@ -123,6 +140,7 @@ func (mst *initramfsMountsState) EphemeralModeenvForModel(model *asserts.Model,
 		Mode:           mst.mode,
 		RecoverySystem: mst.recoverySystem,
 		Base:           snaps[snap.TypeBase].Filename(),
+		Gadget:         snaps[snap.TypeGadget].Filename(),
 		Model:          model.Model(),
 		BrandID:        model.BrandID(),
 		Grade:          string(model.Grade()),
