@@ -125,6 +125,7 @@ var SeccompSyscalls = []string{
 	"ftruncate64",
 	"futex",
 	"futex_time64",
+	"futex_waitv",
 	"futimesat",
 	"get_kernel_syms",
 	"get_mempolicy",
@@ -372,6 +373,7 @@ var SeccompSyscalls = []string{
 	"sendmsg",
 	"sendto",
 	"set_mempolicy",
+	"set_mempolicy_home_node",
 	"set_robust_list",
 	"set_thread_area",
 	"set_tid_address",
