@@ -23,10 +23,9 @@ import (
 	"os"
 	"syscall"
 
-	. "gopkg.in/check.v1"
-
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/sys"
+	"github.com/snapcore/snapd/testutil"
 )
 
 var (
@@ -34,10 +33,6 @@ var (
 	ValidateInstanceName = validateInstanceName
 	ProcessArguments     = processArguments
 
-	// freezer
-	FreezeSnapProcesses = freezeSnapProcesses
-	ThawSnapProcesses   = thawSnapProcesses
-
 	// utils
 	PlanWritableMimic = planWritableMimic
 	ExecWritableMimic = execWritableMimic
@@ -149,31 +144,6 @@ func MockSystemCalls(sc SystemCalls) (restore func()) {
 	}
 }
 
-func MockFreezerCgroupDir(c *C) (restore func()) {
-	old := freezerCgroupDir
-	freezerCgroupDir = c.MkDir()
-	return func() {
-		freezerCgroupDir = old
-	}
-}
-
-func FreezerCgroupDir() string {
-	return freezerCgroupDir
-}
-
-func MockFreezing(freeze, thaw func(snapName string) error) (restore func()) {
-	oldFreeze := freezeSnapProcesses
-	oldThaw := thawSnapProcesses
-
-	freezeSnapProcesses = freeze
-	thawSnapProcesses = thaw
-
-	return func() {
-		freezeSnapProcesses = oldFreeze
-		thawSnapProcesses = oldThaw
-	}
-}
-
 func MockChangePerform(f func(chg *Change, as *Assumptions) ([]*Change, error)) func() {
 	origChangePerform := changePerform
 	changePerform = f
@@ -182,6 +152,12 @@ func MockChangePerform(f func(chg *Change, as *Assumptions) ([]*Change, error))
 	}
 }
 
+func MockIsDirectory(fn func(string) bool) (restore func()) {
+	r := testutil.Backup(&osutilIsDirectory)
+	osutilIsDirectory = fn
+	return r
+}
+
 func MockNeededChanges(f func(old, new *osutil.MountProfile) []*Change) (restore func()) {
 	origNeededChanges := NeededChanges
 	NeededChanges = f
