@@ -173,3 +173,31 @@ func (s *SnapPrepareImageSuite) TestPrepareImageCustomize(c *C) {
 		},
 	})
 }
+
+func (s *SnapPrepareImageSuite) TestPrepareImagePreseedArgError(c *C) {
+	_, err := snap.Parser(snap.Client()).ParseArgs([]string{"prepare-image", "--preseed-sign-key", "key", "model", "prepare-dir"})
+	c.Assert(err, ErrorMatches, `--preseed-sign-key cannot be used without --preseed`)
+}
+
+func (s *SnapPrepareImageSuite) TestPrepareImagePreseed(c *C) {
+	var opts *image.Options
+	prep := func(o *image.Options) error {
+		opts = o
+		return nil
+	}
+	r := snap.MockImagePrepare(prep)
+	defer r()
+
+	rest, err := snap.Parser(snap.Client()).ParseArgs([]string{"prepare-image", "--preseed", "--preseed-sign-key", "key", "--apparmor-features-dir", "aafeatures-dir", "--sysfs-overlay", "sys-overlay", "model", "prepare-dir"})
+	c.Assert(err, IsNil)
+	c.Assert(rest, DeepEquals, []string{})
+
+	c.Check(opts, DeepEquals, &image.Options{
+		ModelFile:                 "model",
+		PrepareDir:                "prepare-dir",
+		Preseed:                   true,
+		PreseedSignKey:            "key",
+		SysfsOverlay:              "sys-overlay",
+		AppArmorKernelFeaturesDir: "aafeatures-dir",
+	})
+}
