@@ -33,6 +33,7 @@ import (
 	"github.com/snapcore/snapd/sandbox/selinux"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/store"
+	"github.com/snapcore/snapd/store/tooling"
 	usersessionclient "github.com/snapcore/snapd/usersession/client"
 )
 
@@ -51,7 +52,6 @@ var (
 	Antialias           = antialias
 	FormatChannel       = fmtChannel
 	PrintDescr          = printDescr
-	WrapFlow            = wrapFlow
 	TrueishJSON         = trueishJSON
 	CompletionHandler   = completionHandler
 	MarkForNoCompletion = markForNoCompletion
@@ -94,12 +94,7 @@ var (
 
 	IsStopping = isStopping
 
-	GetKeypairManager = getKeypairManager
-	GenerateKey       = generateKey
-
 	GetSnapDirOptions = getSnapDirOptions
-
-	ParseQuotas = parseQuotas
 )
 
 func HiddenCmd(descr string, completeHidden bool) *cmdInfo {
@@ -137,7 +132,7 @@ var (
 	MaybePrintNotes             = (*infoWriter).maybePrintNotes
 	MaybePrintStandaloneVersion = (*infoWriter).maybePrintStandaloneVersion
 	MaybePrintBuildDate         = (*infoWriter).maybePrintBuildDate
-	MaybePrintContact           = (*infoWriter).maybePrintContact
+	MaybePrintLinks             = (*infoWriter).maybePrintLinks
 	MaybePrintBase              = (*infoWriter).maybePrintBase
 	MaybePrintPath              = (*infoWriter).maybePrintPath
 	MaybePrintSum               = (*infoWriter).maybePrintSum
@@ -381,7 +376,7 @@ func MockIoutilTempDir(f func(string, string) (string, error)) (restore func())
 	}
 }
 
-func MockDownloadDirect(f func(snapName string, revision snap.Revision, dlOpts image.DownloadSnapOptions) error) (restore func()) {
+func MockDownloadDirect(f func(snapName string, revision snap.Revision, dlOpts tooling.DownloadSnapOptions) error) (restore func()) {
 	old := downloadDirect
 	downloadDirect = f
 	return func() {
@@ -462,3 +457,16 @@ func MockAutostartSessionApps(f func(string) error) func() {
 		autostartSessionApps = old
 	}
 }
+
+func ParseQuotaValues(maxMemory, cpuMax, cpuSet, threadsMax, journalSizeMax, journalRateLimit string) (*client.QuotaValues, error) {
+	var quotas cmdSetQuota
+
+	quotas.MemoryMax = maxMemory
+	quotas.CPUMax = cpuMax
+	quotas.CPUSet = cpuSet
+	quotas.ThreadsMax = threadsMax
+	quotas.JournalSizeMax = journalSizeMax
+	quotas.JournalRateLimit = journalRateLimit
+
+	return quotas.parseQuotas()
+}
