@@ -102,7 +102,8 @@ func (s *apiValidationSetsSuite) mockValidationSetsTracking(st *state.State) {
 			"name":       "foo",
 			"mode":       assertstate.Enforce,
 			"pinned-at":  9,
-			"current":    99,
+			// Current should equal pinned-at if pinned-at != 0 but let's check api_validate is robust
+			"current": 99,
 		},
 		fmt.Sprintf("%s/baz", s.dev1acct.AccountID()): map[string]interface{}{
 			"account-id": s.dev1acct.AccountID(),
@@ -247,7 +248,7 @@ func (s *apiValidationSetsSuite) TestListValidationSets(c *check.C) {
 			Name:      "foo",
 			PinnedAt:  9,
 			Mode:      "enforce",
-			Sequence:  99,
+			Sequence:  9,
 			Valid:     false,
 		},
 	})
@@ -304,7 +305,7 @@ func (s *apiValidationSetsSuite) TestGetValidationSetPinned(c *check.C) {
 		Name:      "foo",
 		PinnedAt:  9,
 		Mode:      "enforce",
-		Sequence:  99,
+		Sequence:  9,
 		Valid:     false,
 	})
 }
@@ -360,7 +361,7 @@ func (s *apiValidationSetsSuite) TestGetValidationSetLatestFromRemote(c *check.C
 		c.Assert(sequence, check.Equals, 0)
 		as, err := asserts.Decode(validationSetAssertion)
 		c.Assert(err, check.IsNil)
-		// sanity
+		// validity
 		c.Assert(as.Type().Name, check.Equals, "validation-set")
 		return as, nil
 	}
@@ -599,12 +600,28 @@ func (s *apiValidationSetsSuite) TestGetValidationSetPinnedNotFound(c *check.C)
 }
 
 func (s *apiValidationSetsSuite) TestApplyValidationSetMonitorModePinnedLocalOnly(c *check.C) {
+	st := s.d.Overlord().State()
+	st.Lock()
+	s.mockValidationSetsTracking(st)
+	assertstatetest.AddMany(st, s.dev1acct, s.acct1Key)
+	as := s.mockAssert(c, "bar", "99")
+	err := assertstate.Add(st, as)
+	st.Unlock()
+	c.Assert(err, check.IsNil)
+
 	var called int
-	restore := daemon.MockAssertstateMonitorValidationSet(func(st *state.State, accountID, name string, sequence, userID int) error {
+	restore := daemon.MockAssertstateMonitorValidationSet(func(st *state.State, accountID, name string, sequence, userID int) (*assertstate.ValidationSetTracking, error) {
 		c.Assert(accountID, check.Equals, s.dev1acct.AccountID())
 		c.Assert(name, check.Equals, "bar")
 		c.Assert(sequence, check.Equals, 99)
 		called++
+		// Current should be the same as PinnedAt when PinnedAt != 0 but let's check api_validate is robust
+		return &assertstate.ValidationSetTracking{AccountID: accountID, Name: name, PinnedAt: 99, Current: 99999}, nil
+	})
+	defer restore()
+
+	restore = daemon.MockCheckInstalledSnaps(func(vsets *snapasserts.ValidationSets, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) error {
+		// nil indicates successful validation
 		return nil
 	})
 	defer restore()
@@ -615,12 +632,21 @@ func (s *apiValidationSetsSuite) TestApplyValidationSetMonitorModePinnedLocalOnl
 
 	rsp := s.syncReq(c, req, nil)
 	c.Assert(rsp.Status, check.Equals, 200)
+	res := rsp.Result.(daemon.ValidationSetResult)
+	c.Check(res, check.DeepEquals, daemon.ValidationSetResult{
+		AccountID: s.dev1acct.AccountID(),
+		Name:      "bar",
+		Mode:      "monitor",
+		PinnedAt:  99,
+		Sequence:  99,
+		Valid:     true,
+	})
 	c.Check(called, check.Equals, 1)
 }
 
 func (s *apiValidationSetsSuite) TestApplyValidationSetMonitorModeError(c *check.C) {
-	restore := daemon.MockAssertstateMonitorValidationSet(func(st *state.State, accountID, name string, sequence, userID int) error {
-		return fmt.Errorf("boom")
+	restore := daemon.MockAssertstateMonitorValidationSet(func(st *state.State, accountID, name string, sequence, userID int) (*assertstate.ValidationSetTracking, error) {
+		return nil, fmt.Errorf("boom")
 	})
 	defer restore()
 
@@ -651,7 +677,7 @@ func (s *apiValidationSetsSuite) TestForgetValidationSet(c *check.C) {
 		var tr assertstate.ValidationSetTracking
 
 		st.Lock()
-		// sanity, it exists before removing
+		// validity, it exists before removing
 		err := assertstate.GetValidationSet(st, s.dev1acct.AccountID(), "foo", &tr)
 		st.Unlock()
 		c.Assert(err, check.IsNil)
@@ -667,7 +693,7 @@ func (s *apiValidationSetsSuite) TestForgetValidationSet(c *check.C) {
 		st.Lock()
 		err = assertstate.GetValidationSet(st, s.dev1acct.AccountID(), "foo", &tr)
 		st.Unlock()
-		c.Assert(err, check.Equals, state.ErrNoState)
+		c.Assert(err, testutil.ErrorIs, state.ErrNoState)
 
 		// and forget again fails
 		req, err = http.NewRequest("POST", fmt.Sprintf("/v2/validation-sets/%s/foo", s.dev1acct.AccountID()), strings.NewReader(body))
@@ -755,22 +781,28 @@ func (s *apiValidationSetsSuite) TestApplyValidationSetUnsupportedAction(c *chec
 }
 
 func (s *apiValidationSetsSuite) TestApplyValidationSetEnforceMode(c *check.C) {
+	st := s.d.Overlord().State()
+	st.Lock()
+	defer st.Unlock()
+
+	s.mockValidationSetsTracking(st)
+	assertstatetest.AddMany(st, s.dev1acct, s.acct1Key)
+	as := s.mockAssert(c, "bar", "99")
+	err := assertstate.Add(st, as)
+	c.Assert(err, check.IsNil)
+
 	var called int
-	restore := daemon.MockAssertstateEnforceValidationSet(func(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) error {
+	restore := daemon.MockAssertstateFetchEnforceValidationSet(func(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) (*assertstate.ValidationSetTracking, error) {
 		c.Check(ignoreValidation, check.HasLen, 0)
 		c.Assert(accountID, check.Equals, s.dev1acct.AccountID())
 		c.Assert(name, check.Equals, "bar")
 		c.Assert(sequence, check.Equals, 0)
 		c.Check(userID, check.Equals, 0)
 		called++
-		return nil
+		return &assertstate.ValidationSetTracking{AccountID: accountID, Name: name, Mode: assertstate.Enforce, Current: 99}, nil
 	})
 	defer restore()
 
-	st := s.d.Overlord().State()
-	st.Lock()
-	defer st.Unlock()
-
 	snapstate.Set(st, "snap-b", &snapstate.SnapState{
 		Active:   true,
 		Sequence: []*snap.SideInfo{{RealName: "snap-b", Revision: snap.R(1), SnapID: "yOqKhntON3vR7kwEbVPsILm7bUViPDzz"}},
@@ -787,12 +819,30 @@ func (s *apiValidationSetsSuite) TestApplyValidationSetEnforceMode(c *check.C) {
 
 	rsp := s.syncReq(c, req, nil)
 	c.Assert(rsp.Status, check.Equals, 200)
+	res := rsp.Result.(daemon.ValidationSetResult)
+	c.Check(res, check.DeepEquals, daemon.ValidationSetResult{
+		AccountID: s.dev1acct.AccountID(),
+		Name:      "bar",
+		Mode:      "enforce",
+		Sequence:  99,
+		Valid:     true,
+	})
 	c.Check(called, check.Equals, 1)
 }
 
 func (s *apiValidationSetsSuite) TestApplyValidationSetEnforceModeIgnoreValidationOK(c *check.C) {
+	st := s.d.Overlord().State()
+	st.Lock()
+	defer st.Unlock()
+
+	s.mockValidationSetsTracking(st)
+	assertstatetest.AddMany(st, s.dev1acct, s.acct1Key)
+	as := s.mockAssert(c, "bar", "99")
+	err := assertstate.Add(st, as)
+	c.Assert(err, check.IsNil)
+
 	var called int
-	restore := daemon.MockAssertstateEnforceValidationSet(func(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) error {
+	restore := daemon.MockAssertstateFetchEnforceValidationSet(func(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) (*assertstate.ValidationSetTracking, error) {
 		c.Check(ignoreValidation, check.DeepEquals, map[string]bool{"snap-b": true})
 		c.Check(snaps, testutil.DeepUnsortedMatches, []*snapasserts.InstalledSnap{
 			snapasserts.NewInstalledSnap("snap-b", "yOqKhntON3vR7kwEbVPsILm7bUViPDzz", snap.R("1"))})
@@ -801,14 +851,10 @@ func (s *apiValidationSetsSuite) TestApplyValidationSetEnforceModeIgnoreValidati
 		c.Assert(sequence, check.Equals, 0)
 		c.Check(userID, check.Equals, 0)
 		called++
-		return nil
+		return &assertstate.ValidationSetTracking{AccountID: accountID, Name: name, Mode: assertstate.Enforce, Current: 99}, nil
 	})
 	defer restore()
 
-	st := s.d.Overlord().State()
-	st.Lock()
-	defer st.Unlock()
-
 	snapstate.Set(st, "snap-b", &snapstate.SnapState{
 		Active:   true,
 		Sequence: []*snap.SideInfo{{RealName: "snap-b", Revision: snap.R(1), SnapID: "yOqKhntON3vR7kwEbVPsILm7bUViPDzz"}},
@@ -826,25 +872,39 @@ func (s *apiValidationSetsSuite) TestApplyValidationSetEnforceModeIgnoreValidati
 
 	rsp := s.syncReq(c, req, nil)
 	c.Assert(rsp.Status, check.Equals, 200)
+	res := rsp.Result.(daemon.ValidationSetResult)
+	c.Check(res, check.DeepEquals, daemon.ValidationSetResult{
+		AccountID: s.dev1acct.AccountID(),
+		Name:      "bar",
+		Mode:      "enforce",
+		Sequence:  99,
+		Valid:     true,
+	})
 	c.Check(called, check.Equals, 1)
 }
 
 func (s *apiValidationSetsSuite) TestApplyValidationSetEnforceModeSpecificSequence(c *check.C) {
+	st := s.d.Overlord().State()
+	st.Lock()
+	defer st.Unlock()
+
+	s.mockValidationSetsTracking(st)
+	assertstatetest.AddMany(st, s.dev1acct, s.acct1Key)
+	as := s.mockAssert(c, "bar", "5")
+	err := assertstate.Add(st, as)
+	c.Assert(err, check.IsNil)
+
 	var called int
-	restore := daemon.MockAssertstateEnforceValidationSet(func(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) error {
+	restore := daemon.MockAssertstateFetchEnforceValidationSet(func(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) (*assertstate.ValidationSetTracking, error) {
 		c.Assert(accountID, check.Equals, s.dev1acct.AccountID())
 		c.Assert(name, check.Equals, "bar")
 		c.Assert(sequence, check.Equals, 5)
 		c.Check(userID, check.Equals, 0)
 		called++
-		return nil
+		return &assertstate.ValidationSetTracking{AccountID: accountID, Name: name, Mode: assertstate.Enforce, PinnedAt: 5, Current: 5}, nil
 	})
 	defer restore()
 
-	st := s.d.Overlord().State()
-	st.Lock()
-	defer st.Unlock()
-
 	snapstate.Set(st, "snap-b", &snapstate.SnapState{
 		Active:   true,
 		Sequence: []*snap.SideInfo{{RealName: "snap-b", Revision: snap.R(1), SnapID: "yOqKhntON3vR7kwEbVPsILm7bUViPDzz"}},
@@ -861,12 +921,21 @@ func (s *apiValidationSetsSuite) TestApplyValidationSetEnforceModeSpecificSequen
 
 	rsp := s.syncReq(c, req, nil)
 	c.Assert(rsp.Status, check.Equals, 200)
+	res := rsp.Result.(daemon.ValidationSetResult)
+	c.Check(res, check.DeepEquals, daemon.ValidationSetResult{
+		AccountID: s.dev1acct.AccountID(),
+		Name:      "bar",
+		Mode:      "enforce",
+		PinnedAt:  5,
+		Sequence:  5,
+		Valid:     true,
+	})
 	c.Check(called, check.Equals, 1)
 }
 
 func (s *apiValidationSetsSuite) TestApplyValidationSetEnforceModeError(c *check.C) {
-	restore := daemon.MockAssertstateEnforceValidationSet(func(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) error {
-		return fmt.Errorf("boom")
+	restore := daemon.MockAssertstateFetchEnforceValidationSet(func(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) (*assertstate.ValidationSetTracking, error) {
+		return nil, fmt.Errorf("boom")
 	})
 	defer restore()
 
