@@ -32,15 +32,15 @@ import (
 	"testing"
 	"time"
 
-	"gopkg.in/mgo.v2/bson"
-
 	. "gopkg.in/check.v1"
+	"gopkg.in/mgo.v2/bson"
 
 	"github.com/snapcore/snapd/arch"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/errtracker"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/snapdenv"
 	"github.com/snapcore/snapd/testutil"
 )
 
@@ -123,6 +123,9 @@ bugs		: very yes
 	s.AddCleanup(errtracker.MockProcSelfExe(mockSelfExe))
 	s.AddCleanup(errtracker.MockProcSelfCwd(mockSelfCwd))
 	s.AddCleanup(testutil.MockCommand(c, "journalctl", "echo "+someJournalEntry).Restore)
+
+	mockCmd := testutil.MockCommand(c, "systemctl", "echo enabled; exit 0")
+	s.AddCleanup(mockCmd.Restore)
 }
 
 func (s *ErrtrackerTestSuite) TestReport(c *C) {
@@ -232,8 +235,7 @@ func (s *ErrtrackerTestSuite) TestReport(c *C) {
 }
 
 func (s *ErrtrackerTestSuite) TestReportUnderTesting(c *C) {
-	os.Setenv("SNAPPY_TESTING", "1")
-	defer os.Unsetenv("SNAPPY_TESTING")
+	defer snapdenv.MockTesting(true)()
 
 	n := 0
 	prev := errtracker.SnapdVersion
@@ -393,7 +395,7 @@ func (s *ErrtrackerTestSuite) TestReportWithNoWhoopsieInstalled(c *C) {
 
 func (s *ErrtrackerTestSuite) TestProcCpuinfo(c *C) {
 	fn := filepath.Join(s.tmpdir, "cpuinfo")
-	// sanity check
+	// validity check
 	buf, err := ioutil.ReadFile(fn)
 	c.Assert(err, IsNil)
 	c.Check(string(buf), Equals, `
