@@ -21,8 +21,6 @@ package gadget
 
 import (
 	"errors"
-
-	"github.com/snapcore/snapd/gadget/quantity"
 )
 
 var errNotImplemented = errors.New("not implemented")
@@ -30,11 +28,3 @@ var errNotImplemented = errors.New("not implemented")
 func FindDeviceForStructure(ps *LaidOutStructure) (string, error) {
 	return "", errNotImplemented
 }
-
-func findDeviceForStructureWithFallback(ps *LaidOutStructure) (string, quantity.Size, error) {
-	return "", 0, errNotImplemented
-}
-
-func findMountPointForStructure(ps *LaidOutStructure) (string, error) {
-	return "", errNotImplemented
-}
