@@ -32,67 +32,53 @@ import (
 	"github.com/snapcore/snapd/osutil/mkfs"
 )
 
-var contentMountpoint string
-
 var mkfsImpl = mkfs.Make
 
-func init() {
-	contentMountpoint = filepath.Join(dirs.SnapRunDir, "gadget-install")
+type mkfsParams struct {
+	Type       string
+	Device     string
+	Label      string
+	Size       quantity.Size
+	SectorSize quantity.Size
 }
 
 // makeFilesystem creates a filesystem on the on-disk structure, according
 // to the filesystem type defined in the gadget. If sectorSize is specified,
 // that sector size is used when creating the filesystem, otherwise if it is
 // zero, automatic values are used instead.
-func makeFilesystem(ds *gadget.OnDiskStructure, sectorSize quantity.Size) error {
-	if !ds.HasFilesystem() {
-		return fmt.Errorf("internal error: on disk structure for partition %s has no filesystem", ds.Node)
-	}
-	logger.Debugf("create %s filesystem on %s with label %q", ds.VolumeStructure.Filesystem, ds.Node, ds.VolumeStructure.Label)
-	if err := mkfsImpl(ds.VolumeStructure.Filesystem, ds.Node, ds.VolumeStructure.Label, ds.Size, sectorSize); err != nil {
+func makeFilesystem(params mkfsParams) error {
+	logger.Debugf("create %s filesystem on %s with label %q", params.Type, params.Device, params.Label)
+	if err := mkfsImpl(params.Type, params.Device, params.Label, params.Size, params.SectorSize); err != nil {
 		return err
 	}
-	return udevTrigger(ds.Node)
+	return udevTrigger(params.Device)
 }
 
-// writeContent populates the given on-disk structure, according to the contents
-// defined in the gadget.
-func writeContent(ds *gadget.OnDiskStructure, observer gadget.ContentObserver) error {
-	if ds.HasFilesystem() {
-		return writeFilesystemContent(ds, observer)
-	}
-	return fmt.Errorf("cannot write non-filesystem structures during install")
-}
-
-// mountFilesystem mounts the on-disk structure filesystem under the given base
-// directory, using the label defined in the gadget as the mount point name.
-func mountFilesystem(ds *gadget.OnDiskStructure, baseMntPoint string) error {
-	if !ds.HasFilesystem() {
-		return fmt.Errorf("cannot mount a partition with no filesystem")
-	}
-	if ds.Label == "" {
-		return fmt.Errorf("cannot mount a filesystem with no label")
-	}
-
-	mountpoint := filepath.Join(baseMntPoint, ds.Label)
+// mountFilesystem mounts the filesystem on a given device with
+// filesystem type fs under the provided mount point directory.
+func mountFilesystem(fsDevice, fs, mountpoint string) error {
 	if err := os.MkdirAll(mountpoint, 0755); err != nil {
 		return fmt.Errorf("cannot create mountpoint: %v", err)
 	}
-	if err := sysMount(ds.Node, mountpoint, ds.Filesystem, 0, ""); err != nil {
-		return fmt.Errorf("cannot mount filesystem %q at %q: %v", ds.Node, mountpoint, err)
+	if err := sysMount(fsDevice, mountpoint, fs, 0, ""); err != nil {
+		return fmt.Errorf("cannot mount filesystem %q at %q: %v", fsDevice, mountpoint, err)
 	}
 
 	return nil
 }
 
-func writeFilesystemContent(ds *gadget.OnDiskStructure, observer gadget.ContentObserver) (err error) {
-	mountpoint := filepath.Join(contentMountpoint, strconv.Itoa(ds.DiskIndex))
+// writeContent populates the given on-disk filesystem structure with a
+// corresponding filesystem device, according to the contents defined in the
+// gadget.
+func writeFilesystemContent(ds *gadget.OnDiskStructure, fsDevice string, observer gadget.ContentObserver) (err error) {
+	mountpoint := filepath.Join(dirs.SnapRunDir, "gadget-install", strconv.Itoa(ds.DiskIndex))
 	if err := os.MkdirAll(mountpoint, 0755); err != nil {
 		return err
 	}
 
 	// temporarily mount the filesystem
-	if err := sysMount(ds.Node, mountpoint, ds.Filesystem, 0, ""); err != nil {
+	logger.Debugf("mounting %q in %q (fs type %q)", fsDevice, mountpoint, ds.Filesystem)
+	if err := sysMount(fsDevice, mountpoint, ds.Filesystem, 0, ""); err != nil {
 		return fmt.Errorf("cannot mount filesystem %q at %q: %v", ds.Node, mountpoint, err)
 	}
 	defer func() {
