@@ -22,26 +22,27 @@
 package install
 
 import (
-	"github.com/snapcore/snapd/secboot"
+	"github.com/snapcore/snapd/boot"
+	"github.com/snapcore/snapd/kernel/fde"
+	"github.com/snapcore/snapd/secboot/keys"
+	"github.com/snapcore/snapd/testutil"
 )
 
 var (
-	DiskWithSystemSeed     = diskWithSystemSeed
-	NewEncryptedDeviceLUKS = newEncryptedDeviceLUKS
+	DiskWithSystemSeed                 = diskWithSystemSeed
+	NewEncryptedDeviceLUKS             = newEncryptedDeviceLUKS
+	CreateEncryptedDeviceWithSetupHook = createEncryptedDeviceWithSetupHook
 )
 
-func MockSecbootFormatEncryptedDevice(f func(key secboot.EncryptionKey, label, node string) error) (restore func()) {
-	old := secbootFormatEncryptedDevice
+func MockSecbootFormatEncryptedDevice(f func(key keys.EncryptionKey, label, node string) error) (restore func()) {
+	r := testutil.Backup(&secbootFormatEncryptedDevice)
 	secbootFormatEncryptedDevice = f
-	return func() {
-		secbootFormatEncryptedDevice = old
-	}
+	return r
+
 }
 
-func MockSecbootAddRecoveryKey(f func(key secboot.EncryptionKey, rkey secboot.RecoveryKey, node string) error) (restore func()) {
-	old := secbootAddRecoveryKey
-	secbootAddRecoveryKey = f
-	return func() {
-		secbootAddRecoveryKey = old
-	}
+func MockBootRunFDESetupHook(f func(req *fde.SetupRequest) ([]byte, error)) (restore func()) {
+	r := testutil.Backup(&boot.RunFDESetupHook)
+	boot.RunFDESetupHook = f
+	return r
 }
