@@ -55,7 +55,8 @@ func (p *layoutTestSuite) TestVolumeSize(c *C) {
 			{Size: 2 * quantity.SizeMiB},
 		},
 	}
-	v, err := gadget.LayoutVolume(p.dir, "", &vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(&vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
@@ -96,7 +97,8 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first-image")
 	c.Assert(vol.Structure, HasLen, 2)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
@@ -138,7 +140,8 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 4)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
@@ -194,7 +197,8 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 4)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
@@ -249,7 +253,8 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 4)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
@@ -295,11 +300,51 @@ volumes:
               - image: foo.img
 `
 	vol := mustParseVolume(c, gadgetYaml, "first")
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img":.*no such file or directory`)
 }
 
+func (p *layoutTestSuite) TestLayoutVolumeContentCanIgnoreContent(c *C) {
+	gadgetYaml := `
+volumes:
+  first:
+    schema: gpt
+    bootloader: grub
+    structure:
+        - type: 00000000-0000-0000-0000-dd00deadbeef
+          size: 400M
+          offset: 800M
+          content:
+              - image: foo.img
+`
+	vol := mustParseVolume(c, gadgetYaml, "first")
+	opts := &gadget.LayoutOptions{
+		GadgetRootDir: p.dir,
+	}
+	// LayoutVolume fails with default constraints because the foo.img
+	// file is missing
+	_, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
+	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img":.*no such file or directory`)
+
+	// But LayoutVolume works with the IgnoreContent works
+	opts.IgnoreContent = true
+	v, err := gadget.LayoutVolume(vol, gadget.DefaultConstraints, opts)
+	c.Assert(err, IsNil)
+	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
+		Volume:  vol,
+		Size:    1200 * quantity.SizeMiB,
+		RootDir: p.dir,
+		LaidOutStructure: []gadget.LaidOutStructure{
+			{
+				StartOffset:     800 * quantity.OffsetMiB,
+				VolumeStructure: &vol.Structure[0],
+			},
+		},
+	})
+}
+
 func makeSizedFile(c *C, path string, size quantity.Size, content []byte) {
 	err := os.MkdirAll(filepath.Dir(path), 0755)
 	c.Assert(err, IsNil)
@@ -333,7 +378,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img" does not fit in the structure`)
 }
@@ -359,7 +405,8 @@ volumes:
 	constraints := gadget.LayoutConstraints{
 		NonMBRStartOffset: 1 * quantity.OffsetMiB,
 	}
-	v, err := gadget.LayoutVolume(p.dir, "", vol, constraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, constraints, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "bar.img" does not fit in the structure`)
 }
@@ -382,7 +429,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img" does not fit in the structure`)
 }
@@ -404,7 +452,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, fmt.Sprintf(`cannot lay out structure #0: content "foo.img" size %v is larger than declared %v`, quantity.SizeMiB+1, quantity.SizeMiB))
 }
@@ -432,7 +481,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYaml, "first")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(v, IsNil)
 	c.Assert(err, ErrorMatches, `cannot lay out structure #0: content "foo.img" overlaps with preceding image "bar.img"`)
 }
@@ -461,7 +511,8 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 1)
 	c.Assert(vol.Structure[0].Content, HasLen, 2)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -512,7 +563,8 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 1)
 	c.Assert(vol.Structure[0].Content, HasLen, 2)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -560,7 +612,8 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 1)
 	c.Assert(vol.Structure[0].Content, HasLen, 1)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -602,7 +655,8 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 1)
 	c.Assert(vol.Structure[0].Content, HasLen, 1)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -660,7 +714,8 @@ volumes:
 	c.Assert(vol.Structure, HasLen, 2)
 	c.Assert(vol.Structure[1].Content, HasLen, 1)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -686,7 +741,7 @@ volumes:
 		// 512kiB
 		NonMBRStartOffset: 512 * quantity.OffsetKiB,
 	}
-	v, err = gadget.LayoutVolume(p.dir, "", vol, constraints)
+	v, err = gadget.LayoutVolume(vol, constraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -713,7 +768,7 @@ volumes:
 	constraintsBad := gadget.LayoutConstraints{
 		NonMBRStartOffset: 400,
 	}
-	v, err = gadget.LayoutVolume(p.dir, "", vol, constraintsBad)
+	v, err = gadget.LayoutVolume(vol, constraintsBad, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -752,7 +807,8 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "first")
 	c.Assert(vol.Structure, HasLen, 2)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -804,7 +860,8 @@ volumes:
 	vol := mustParseVolume(c, gadgetYaml, "pc")
 	c.Assert(vol.Structure, HasLen, 3)
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v, DeepEquals, &gadget.LaidOutVolume{
 		Volume:  vol,
@@ -889,11 +946,12 @@ func (p *layoutTestSuite) TestLayoutVolumeOffsetWriteBadRelativeTo(c *C) {
 
 	makeSizedFile(c, filepath.Join(p.dir, "foo.img"), 200*quantity.SizeKiB, []byte(""))
 
-	v, err := gadget.LayoutVolume(p.dir, "", &volBadStructure, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(&volBadStructure, defaultConstraints, opts)
 	c.Check(v, IsNil)
 	c.Check(err, ErrorMatches, `cannot resolve offset-write of structure #0 \("foo"\): refers to an unknown structure "bar"`)
 
-	v, err = gadget.LayoutVolume(p.dir, "", &volBadContent, defaultConstraints)
+	v, err = gadget.LayoutVolume(&volBadContent, defaultConstraints, opts)
 	c.Check(v, IsNil)
 	c.Check(err, ErrorMatches, `cannot resolve offset-write of structure #0 \("foo"\) content "foo.img": refers to an unknown structure "bar"`)
 }
@@ -917,7 +975,8 @@ volumes:
 `
 	vol := mustParseVolume(c, gadgetYamlStructure, "pc")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	// offset-write is at 1GB
 	c.Check(v.Size, Equals, 1*quantity.SizeGiB+gadget.SizeLBA48Pointer)
@@ -952,7 +1011,7 @@ volumes:
 
 	vol = mustParseVolume(c, gadgetYamlContent, "pc")
 
-	v, err = gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	v, err = gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	// foo.img offset-write is at 3GB
 	c.Check(v.Size, Equals, 3*quantity.SizeGiB+gadget.SizeLBA48Pointer)
@@ -1010,7 +1069,8 @@ volumes:
 
 	vol := mustParseVolume(c, gadgetYamlContent, "pc")
 
-	v, err := gadget.LayoutVolume(p.dir, "", vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir}
+	v, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	c.Assert(v.LaidOutStructure, HasLen, 1)
 	c.Assert(v.LaidOutStructure[0].LaidOutContent, HasLen, 2)
@@ -1081,7 +1141,7 @@ volumes:
 }
 
 func mockKernel(c *C, kernelYaml string, filesWithContent map[string]string) string {
-	// sanity
+	// precondition
 	_, err := kernel.InfoFromKernelYaml([]byte(kernelYaml))
 	c.Assert(err, IsNil)
 
@@ -1131,10 +1191,39 @@ func (p *layoutTestSuite) TestResolveContentPathsNotInWantedAssets(c *C) {
 	c.Assert(vol.Structure, HasLen, 1)
 
 	kernelSnapDir := c.MkDir()
-	_, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	_, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, ErrorMatches, `cannot resolve content for structure #0 at index 0: cannot find "dtbs" in kernel info from "/.*"`)
 }
 
+func (p *layoutTestSuite) TestResolveContentPathsSkipResolveContent(c *C) {
+	vol := mustParseVolume(c, gadgetYamlWithKernelRef, "pi")
+	c.Assert(vol.Structure, HasLen, 1)
+
+	kernelSnapDir := c.MkDir()
+	defaultOpts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+
+	opts := defaultOpts
+	_, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
+	c.Assert(err, ErrorMatches, `cannot resolve content for structure #0 at index 0: cannot find "dtbs" in kernel info from "/.*"`)
+
+	// SkipResolveContent will allow to layout the volume even if
+	// files are missing
+	opts = defaultOpts
+	opts.SkipResolveContent = true
+	v, err := gadget.LayoutVolume(vol, gadget.DefaultConstraints, opts)
+	c.Assert(err, IsNil)
+	c.Assert(v.Structure, HasLen, 1)
+
+	// As does IgnoreContent will allow to layout the volume even if
+	// files are missing
+	opts = defaultOpts
+	opts.IgnoreContent = true
+	v, err = gadget.LayoutVolume(vol, gadget.DefaultConstraints, opts)
+	c.Assert(err, IsNil)
+	c.Assert(v.Structure, HasLen, 1)
+}
+
 func (p *layoutTestSuite) TestResolveContentPathsErrorInKernelRef(c *C) {
 	// create invalid kernel ref
 	s := strings.Replace(gadgetYamlWithKernelRef, "$kernel:dtbs", "$kernel:-invalid-kernel-ref", -1)
@@ -1144,7 +1233,8 @@ func (p *layoutTestSuite) TestResolveContentPathsErrorInKernelRef(c *C) {
 	c.Assert(vol.Structure, HasLen, 1)
 
 	kernelSnapDir := c.MkDir()
-	_, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	_, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, ErrorMatches, `cannot resolve content for structure #0 at index 0: cannot parse kernel ref: invalid asset name in kernel ref "\$kernel:-invalid-kernel-ref/boot-assets/"`)
 }
 
@@ -1163,7 +1253,8 @@ assets:
 	kernelSnapDir := mockKernel(c, kernelYaml, map[string]string{
 		"dtbs/foo.dtb": "foo.dtb content",
 	})
-	_, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	_, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, ErrorMatches, `cannot resolve content for structure #0 at index 0: cannot find wanted kernel content "boot-assets/" in "/.*"`)
 }
 
@@ -1184,7 +1275,8 @@ assets:
 		"some-file":       "some-file content",
 	}
 	kernelSnapDir := mockKernel(c, kernelYaml, kernelSnapFiles)
-	lv, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	lv, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	// Volume.Content is unchanged
 	c.Assert(lv.Structure, HasLen, 1)
@@ -1264,7 +1356,8 @@ assets:
 	kernelSnapDir := mockKernel(c, kernelYaml, map[string]string{
 		"dtbs/foo.dtb": "foo.dtb content",
 	})
-	lv, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	lv, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, IsNil)
 	// Volume.Content is unchanged
 	c.Assert(lv.Structure, HasLen, 1)
@@ -1333,6 +1426,7 @@ assets:
 	kernelSnapDir := mockKernel(c, kernelYaml, map[string]string{
 		"a/foo.dtb": "foo.dtb content",
 	})
-	_, err := gadget.LayoutVolume(p.dir, kernelSnapDir, vol, defaultConstraints)
+	opts := &gadget.LayoutOptions{GadgetRootDir: p.dir, KernelRootDir: kernelSnapDir}
+	_, err := gadget.LayoutVolume(vol, defaultConstraints, opts)
 	c.Assert(err, ErrorMatches, `.*: cannot find wanted kernel content "ab" in.*`)
 }
