@@ -22,16 +22,17 @@ package gadget
 import (
 	"errors"
 	"fmt"
-	"path/filepath"
+	"sort"
 	"strings"
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/gadget/device"
 	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/kernel"
 	"github.com/snapcore/snapd/logger"
-	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/disks"
+	"github.com/snapcore/snapd/strutil"
 )
 
 var (
@@ -292,7 +293,7 @@ func onDiskStructureIsLikelyImplicitSystemDataRole(gadgetLayout *LaidOutVolume,
 	numPartsOnDisk := len(diskLayout.Structure)
 
 	return s.Filesystem == "ext4" &&
-		s.Type == "0FC63DAF-8483-4772-8E79-3D69D8477DE4" && // TODO: check hybrid and on MBR/DOS too
+		(s.Type == "0FC63DAF-8483-4772-8E79-3D69D8477DE4" || s.Type == "83") &&
 		s.Label == "writable" &&
 		// DiskIndex is 1-based
 		s.DiskIndex == numPartsOnDisk &&
@@ -881,8 +882,7 @@ func buildNewVolumeToDeviceMapping(mod Model, old GadgetData, laidOutVols map[st
 
 		// check if there is a marker file written, that will indicate if
 		// encryption was turned on
-		encryptionMarkerFile := filepath.Join(dirs.SnapFDEDir, "marker")
-		if osutil.FileExists(encryptionMarkerFile) {
+		if device.HasEncryptedMarkerUnder(dirs.SnapFDEDir) {
 			// then we have the crypto marker file for encryption
 			// cross-validation between ubuntu-data and ubuntu-save stored from
 			// install mode, so mark ubuntu-save and data as expected to be
@@ -946,9 +946,6 @@ func buildVolumeStructureToLocation(mod Model,
 	}
 
 	volumeStructureToLocation := make(map[string]map[int]StructureLocation, len(old.Info.Volumes))
-	for volName := range old.Info.Volumes {
-		volumeStructureToLocation[volName] = make(map[int]StructureLocation)
-	}
 
 	// now for each volume, iterate over the structures, putting the
 	// necessary info into the map for that volume as we iterate
@@ -958,6 +955,7 @@ func buildVolumeStructureToLocation(mod Model,
 	// unsupported structure change is present in the new one, but we check that
 	// situation after we have built the mapping
 	for volName, diskDeviceTraits := range volToDeviceMapping {
+		volumeStructureToLocation[volName] = make(map[int]StructureLocation)
 		vol, ok := old.Info.Volumes[volName]
 		if !ok {
 			return nil, fmt.Errorf("internal error: volume %s not present in gadget.yaml but present in traits mapping", volName)
@@ -1071,7 +1069,15 @@ func buildVolumeStructureToLocation(mod Model,
 	return volumeStructureToLocation, nil
 }
 
-// exposed for mocking later on
+func MockVolumeStructureToLocationMap(f func(_ GadgetData, _ Model, _ map[string]*LaidOutVolume) (map[string]map[int]StructureLocation, error)) (restore func()) {
+	old := volumeStructureToLocationMap
+	volumeStructureToLocationMap = f
+	return func() {
+		volumeStructureToLocationMap = old
+	}
+}
+
+// use indirection to allow mocking
 var volumeStructureToLocationMap = volumeStructureToLocationMapImpl
 
 func volumeStructureToLocationMapImpl(old GadgetData, mod Model, laidOutVols map[string]*LaidOutVolume) (map[string]map[int]StructureLocation, error) {
@@ -1174,74 +1180,189 @@ func volumeStructureToLocationMapImpl(old GadgetData, mod Model, laidOutVols map
 // d. After step (c) is completed the kernel refresh will now also
 //    work (no more violation of rule 1)
 func Update(model Model, old, new GadgetData, rollbackDirPath string, updatePolicy UpdatePolicyFunc, observer ContentUpdateObserver) error {
-	// TODO: support multi-volume gadgets. But for now we simply
-	//       do not do any gadget updates on those. We cannot error
-	//       here because this would break refreshes of gadgets even
-	//       when they don't require any updates.
-	if len(new.Info.Volumes) != 1 || len(old.Info.Volumes) != 1 {
-		logger.Noticef("WARNING: gadget assests cannot be updated yet when multiple volumes are used")
-		return nil
+	// if the volumes from the old and the new gadgets do not match, then fail -
+	// we don't support adding or removing volumes from the gadget.yaml
+	newVolumes := make([]string, 0, len(new.Info.Volumes))
+	oldVolumes := make([]string, 0, len(old.Info.Volumes))
+	for newVol := range new.Info.Volumes {
+		newVolumes = append(newVolumes, newVol)
 	}
-
-	oldVol, newVol, err := resolveVolume(old.Info, new.Info)
-	if err != nil {
-		return err
+	for oldVol := range old.Info.Volumes {
+		oldVolumes = append(oldVolumes, oldVol)
 	}
-
-	if oldVol.Schema == "" || newVol.Schema == "" {
-		return fmt.Errorf("internal error: unset volume schemas: old: %q new: %q", oldVol.Schema, newVol.Schema)
-	}
-
-	// layout old partially, without going deep into the layout of structure
-	// content
-	pOld, err := LayoutVolumePartially(oldVol, DefaultConstraints)
-	if err != nil {
-		return fmt.Errorf("cannot lay out the old volume: %v", err)
-	}
-
-	// Layout new volume, delay resolving of filesystem content
-	constraints := DefaultConstraints
-	constraints.SkipResolveContent = true
-	pNew, err := LayoutVolume(new.RootDir, new.KernelRootDir, newVol, constraints)
-	if err != nil {
-		return fmt.Errorf("cannot lay out the new volume: %v", err)
-	}
-
-	if err := canUpdateVolume(pOld, pNew); err != nil {
-		return fmt.Errorf("cannot apply update to volume: %v", err)
+	common := strutil.Intersection(newVolumes, oldVolumes)
+	// check dissimilar cases between common, new and old
+	switch {
+	case len(common) != len(newVolumes) && len(common) != len(oldVolumes):
+		// there are both volumes removed from old and volumes added to new
+		return fmt.Errorf("cannot update gadget assets: volumes were both added and removed")
+	case len(common) != len(newVolumes):
+		// then there are volumes in old that are not in new, i.e. a volume
+		// was removed
+		return fmt.Errorf("cannot update gadget assets: volumes were removed")
+	case len(common) != len(oldVolumes):
+		// then there are volumes in new that are not in old, i.e. a volume
+		// was added
+		return fmt.Errorf("cannot update gadget assets: volumes were added")
 	}
 
 	if updatePolicy == nil {
 		updatePolicy = defaultPolicy
 	}
 
+	// collect the updates and validate that they are doable from an abstract
+	// sense first
+
+	// note that this code is written such that before we perform any update, we
+	// validate that all updates are valid and that all volumes are compatible
+	// between the old and the new state, this is to prevent applying valid
+	// updates on one volume when another volume is invalid, if that's the case
+	// we treat the whole gadget as invalid and return an error blocking the
+	// refresh
+
+	// TODO: should we handle the updates on multiple volumes in a
+	// deterministic order? iterating over maps is not deterministic, but we
+	// perform all updates at the end together in one call
+
 	// ensure all required kernel assets are found in the gadget
 	kernelInfo, err := kernel.ReadInfo(new.KernelRootDir)
 	if err != nil {
 		return err
 	}
-	if err := gadgetVolumeConsumesOneKernelUpdateAsset(pNew.Volume, kernelInfo); err != nil {
-		return err
+
+	allKernelAssets := []string{}
+	for assetName, asset := range kernelInfo.Assets {
+		if !asset.Update {
+			continue
+		}
+		allKernelAssets = append(allKernelAssets, assetName)
 	}
 
-	// now we know which structure is which, find which ones need an update
-	updates, err := resolveUpdate(pOld, pNew, updatePolicy, new.RootDir, new.KernelRootDir, kernelInfo)
-	if err != nil {
-		return err
+	atLeastOneKernelAssetConsumed := false
+
+	// Layout new volume, delay resolving of filesystem content
+	opts := &LayoutOptions{
+		SkipResolveContent: true,
+		GadgetRootDir:      new.RootDir,
+		KernelRootDir:      new.KernelRootDir,
 	}
-	if len(updates) == 0 {
+
+	allUpdates := []updatePair{}
+	laidOutVols := map[string]*LaidOutVolume{}
+	for volName, oldVol := range old.Info.Volumes {
+		newVol := new.Info.Volumes[volName]
+
+		if oldVol.Schema == "" || newVol.Schema == "" {
+			return fmt.Errorf("internal error: unset volume schemas: old: %q new: %q", oldVol.Schema, newVol.Schema)
+		}
+
+		// layout old partially, without going deep into the layout of structure
+		// content
+		pOld, err := LayoutVolumePartially(oldVol, DefaultConstraints)
+		if err != nil {
+			return fmt.Errorf("cannot lay out the old volume %s: %v", volName, err)
+		}
+
+		pNew, err := LayoutVolume(newVol, DefaultConstraints, opts)
+		if err != nil {
+			return fmt.Errorf("cannot lay out the new volume %s: %v", volName, err)
+		}
+
+		laidOutVols[volName] = pNew
+
+		if err := canUpdateVolume(pOld, pNew); err != nil {
+			return fmt.Errorf("cannot apply update to volume %s: %v", volName, err)
+		}
+
+		// if we haven't consumed any kernel assets yet check if this volume
+		// consumes at least one - we require at least one asset to be consumed
+		// by some volume in the gadget
+		if !atLeastOneKernelAssetConsumed {
+			consumed, err := gadgetVolumeKernelUpdateAssetsConsumed(pNew.Volume, kernelInfo)
+			if err != nil {
+				return err
+			}
+			atLeastOneKernelAssetConsumed = consumed
+		}
+
+		// now we know which structure is which, find which ones need an update
+		updates, err := resolveUpdate(pOld, pNew, updatePolicy, new.RootDir, new.KernelRootDir, kernelInfo)
+		if err != nil {
+			return err
+		}
+
+		// can update old layout to new layout
+		for _, update := range updates {
+			if err := canUpdateStructure(update.from, update.to, pNew.Schema); err != nil {
+				return fmt.Errorf("cannot update volume structure %v for volume %s: %v", update.to, volName, err)
+			}
+		}
+
+		// collect updates per volume into a single set of updates to perform
+		// at once
+		allUpdates = append(allUpdates, updates...)
+	}
+
+	// check if there were kernel assets that at least one was consumed across
+	// any of the volumes
+	if len(allKernelAssets) != 0 && !atLeastOneKernelAssetConsumed {
+		sort.Strings(allKernelAssets)
+		return fmt.Errorf("gadget does not consume any of the kernel assets needing synced update %s", strutil.Quoted(allKernelAssets))
+	}
+
+	if len(allUpdates) == 0 {
 		// nothing to update
 		return ErrNoUpdate
 	}
 
-	// can update old layout to new layout
-	for _, update := range updates {
-		if err := canUpdateStructure(update.from, update.to, pNew.Schema); err != nil {
-			return fmt.Errorf("cannot update volume structure %v: %v", update.to, err)
+	// build the map of volume structure locations where the first key is the
+	// volume name, and the second key is the structure's index in the list of
+	// structures on that volume, and the final value is the StructureLocation
+	// hat can actually be used to perform the lookup/update in applyUpdates
+	structureLocations, err := volumeStructureToLocationMap(old, model, laidOutVols)
+	if err != nil {
+		if err == errSkipUpdateProceedRefresh {
+			// we couldn't successfully build a map for the structure locations,
+			// but for various reasons this isn't considered a fatal error for
+			// the gadget refresh, so just return nil instead, a message should
+			// already have been logged
+			return nil
+		}
+		return err
+	}
+
+	if len(new.Info.Volumes) != 1 {
+		logger.Debugf("gadget asset update routine for multiple volumes")
+
+		// check if the structure location map has only one volume in it - this
+		// is the case in legacy update operations where we only support updates
+		// to the system-boot / main volume
+		if len(structureLocations) == 1 {
+			// log a message and drop all updates to structures not in the
+			// volume we have
+			supportedVolume := ""
+			for volName := range structureLocations {
+				supportedVolume = volName
+			}
+			keepUpdates := make([]updatePair, 0, len(allUpdates))
+			for _, update := range allUpdates {
+				if update.volume.Name != supportedVolume {
+					// TODO: or should we error here instead?
+					logger.Noticef("skipping update on non-supported volume %s to structure %s", update.volume.Name, update.to.Name)
+				} else {
+					keepUpdates = append(keepUpdates, update)
+				}
+			}
+			allUpdates = keepUpdates
 		}
 	}
 
-	return applyUpdates(new, updates, rollbackDirPath, observer)
+	// apply all updates at once
+	if err := applyUpdates(structureLocations, new, allUpdates, rollbackDirPath, observer); err != nil {
+		return err
+	}
+
+	return nil
 }
 
 func resolveVolume(old *Info, new *Info) (oldVol, newVol *Volume, err error) {
@@ -1355,8 +1476,9 @@ func canUpdateVolume(from *PartiallyLaidOutVolume, to *LaidOutVolume) error {
 }
 
 type updatePair struct {
-	from *LaidOutStructure
-	to   *LaidOutStructure
+	from   *LaidOutStructure
+	to     *LaidOutStructure
+	volume *Volume
 }
 
 func defaultPolicy(from, to *LaidOutStructure) (bool, ResolvedContentFilterFunc) {
@@ -1420,8 +1542,9 @@ func resolveUpdate(oldVol *PartiallyLaidOutVolume, newVol *LaidOutVolume, policy
 
 			// and add to updates
 			updates = append(updates, updatePair{
-				from: &oldVol.LaidOutStructure[j],
-				to:   &newVol.LaidOutStructure[j],
+				from:   &oldVol.LaidOutStructure[j],
+				to:     &newVol.LaidOutStructure[j],
+				volume: newVol.Volume,
 			})
 		}
 	}
@@ -1440,13 +1563,42 @@ type Updater interface {
 	Rollback() error
 }
 
-func applyUpdates(new GadgetData, updates []updatePair, rollbackDir string, observer ContentUpdateObserver) error {
+func updateLocationForStructure(structureLocations map[string]map[int]StructureLocation, ps *LaidOutStructure) (loc StructureLocation, err error) {
+	loc, ok := structureLocations[ps.VolumeName][ps.YamlIndex]
+	if !ok {
+		return loc, fmt.Errorf("structure with index %d on volume %s not found", ps.YamlIndex, ps.VolumeName)
+	}
+	if !ps.HasFilesystem() {
+		if loc.Device == "" {
+			return loc, fmt.Errorf("internal error: structure %d on volume %s should have had a device set but did not have one in an internal mapping", ps.YamlIndex, ps.VolumeName)
+		}
+		return loc, nil
+	} else {
+		if loc.RootMountPoint == "" {
+			// then we can't update this structure because it has a filesystem
+			// specified in the gadget.yaml, but that partition is not mounted
+			// anywhere writable for us to update the filesystem content
+			// there is a TODO in buildVolumeStructureToLocation above about
+			// possibly mounting it, we could also mount it here instead and
+			// then proceed with the update, but we should also have a way to
+			// unmount it when we are done updating it
+			return loc, fmt.Errorf("structure %d on volume %s does not have a writable mountpoint in order to update the filesystem content", ps.YamlIndex, ps.VolumeName)
+		}
+		return loc, nil
+	}
+}
+
+func applyUpdates(structureLocations map[string]map[int]StructureLocation, new GadgetData, updates []updatePair, rollbackDir string, observer ContentUpdateObserver) error {
 	updaters := make([]Updater, len(updates))
 
 	for i, one := range updates {
-		up, err := updaterForStructure(one.to, new.RootDir, rollbackDir, observer)
+		loc, err := updateLocationForStructure(structureLocations, one.to)
 		if err != nil {
-			return fmt.Errorf("cannot prepare update for volume structure %v: %v", one.to, err)
+			return fmt.Errorf("cannot prepare update for volume structure %v on volume %s: %v", one.to, one.volume.Name, err)
+		}
+		up, err := updaterForStructure(loc, one.to, new.RootDir, rollbackDir, observer)
+		if err != nil {
+			return fmt.Errorf("cannot prepare update for volume structure %v on volume %s: %v", one.to, one.volume.Name, err)
 		}
 		updaters[i] = up
 	}
@@ -1454,7 +1606,7 @@ func applyUpdates(new GadgetData, updates []updatePair, rollbackDir string, obse
 	var backupErr error
 	for i, one := range updaters {
 		if err := one.Backup(); err != nil {
-			backupErr = fmt.Errorf("cannot backup volume structure %v: %v", updates[i].to, err)
+			backupErr = fmt.Errorf("cannot backup volume structure %v on volume %s: %v", updates[i].to, updates[i].volume.Name, err)
 			break
 		}
 	}
@@ -1482,7 +1634,7 @@ func applyUpdates(new GadgetData, updates []updatePair, rollbackDir string, obse
 				skipped++
 				continue
 			}
-			updateErr = fmt.Errorf("cannot update volume structure %v: %v", updates[i].to, err)
+			updateErr = fmt.Errorf("cannot update volume structure %v on volume %s: %v", updates[i].to, updates[i].volume.Name, err)
 			break
 		}
 	}
@@ -1502,7 +1654,7 @@ func applyUpdates(new GadgetData, updates []updatePair, rollbackDir string, obse
 		one := updaters[i]
 		if err := one.Rollback(); err != nil {
 			// TODO: log errors to oplog
-			logger.Noticef("cannot rollback volume structure %v update: %v", updates[i].to, err)
+			logger.Noticef("cannot rollback volume structure %v update on volume %s: %v", updates[i].to, updates[i].volume.Name, err)
 		}
 	}
 
@@ -1517,19 +1669,24 @@ func applyUpdates(new GadgetData, updates []updatePair, rollbackDir string, obse
 
 var updaterForStructure = updaterForStructureImpl
 
-func updaterForStructureImpl(ps *LaidOutStructure, newRootDir, rollbackDir string, observer ContentUpdateObserver) (Updater, error) {
-	var updater Updater
-	var err error
+func updaterForStructureImpl(loc StructureLocation, ps *LaidOutStructure, newRootDir, rollbackDir string, observer ContentUpdateObserver) (Updater, error) {
+	// TODO: this is sort of clunky, we already did the lookup, but doing the
+	// lookup out of band from this function makes for easier mocking
 	if !ps.HasFilesystem() {
-		updater, err = newRawStructureUpdater(newRootDir, ps, rollbackDir, findDeviceForStructureWithFallback)
+		lookup := func(ps *LaidOutStructure) (device string, offs quantity.Offset, err error) {
+			return loc.Device, loc.Offset, nil
+		}
+		return newRawStructureUpdater(newRootDir, ps, rollbackDir, lookup)
 	} else {
-		updater, err = newMountedFilesystemUpdater(ps, rollbackDir, findMountPointForStructure, observer)
+		lookup := func(ps *LaidOutStructure) (string, error) {
+			return loc.RootMountPoint, nil
+		}
+		return newMountedFilesystemUpdater(ps, rollbackDir, lookup, observer)
 	}
-	return updater, err
 }
 
 // MockUpdaterForStructure replace internal call with a mocked one, for use in tests only
-func MockUpdaterForStructure(mock func(ps *LaidOutStructure, rootDir, rollbackDir string, observer ContentUpdateObserver) (Updater, error)) (restore func()) {
+func MockUpdaterForStructure(mock func(loc StructureLocation, ps *LaidOutStructure, rootDir, rollbackDir string, observer ContentUpdateObserver) (Updater, error)) (restore func()) {
 	old := updaterForStructure
 	updaterForStructure = mock
 	return func() {
