@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2021 Canonical Ltd
+ * Copyright (C) 2014-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -34,9 +34,11 @@ import (
 	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
-	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/store/tooling"
 
 	// to set sysconfig.ApplyFilesystemOnlyDefaults hook
+	"github.com/snapcore/snapd/image/preseed"
+	"github.com/snapcore/snapd/osutil"
 	_ "github.com/snapcore/snapd/overlord/configstate/configcore"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/seed/seedwriter"
@@ -50,6 +52,8 @@ import (
 var (
 	Stdout io.Writer = os.Stdout
 	Stderr io.Writer = os.Stderr
+
+	preseedCore20 = preseed.Core20
 )
 
 func (custo *Customizations) validate(model *asserts.Model) error {
@@ -69,7 +73,7 @@ func (custo *Customizations) validate(model *asserts.Model) error {
 	kind := "UC16/18"
 	switch {
 	case core20:
-		kind = "UC20"
+		kind = "UC20+"
 		// TODO:UC20: consider supporting these with grade dangerous?
 		unsupportedConsoleConfDisable()
 		if custo.CloudInitUserData != "" {
@@ -94,7 +98,7 @@ func classicHasSnaps(model *asserts.Model, opts *Options) bool {
 	return model.Gadget() != "" || len(model.RequiredNoEssentialSnaps()) != 0 || len(opts.Snaps) != 0
 }
 
-var newToolingStoreFromModel = NewToolingStoreFromModel
+var newToolingStoreFromModel = tooling.NewToolingStoreFromModel
 
 func Prepare(opts *Options) error {
 	var model *asserts.Model
@@ -134,6 +138,7 @@ func Prepare(opts *Options) error {
 	if err != nil {
 		return err
 	}
+	tsto.Stdout = Stdout
 
 	// FIXME: limitation until we can pass series parametrized much more
 	if model.Series() != release.Series {
@@ -144,7 +149,28 @@ func Prepare(opts *Options) error {
 		return err
 	}
 
-	return setupSeed(tsto, model, opts)
+	if err := setupSeed(tsto, model, opts); err != nil {
+		return err
+	}
+
+	if opts.Preseed {
+		// TODO: support UC22
+		if model.Classic() {
+			return fmt.Errorf("cannot preseed the image for a classic model")
+		}
+		if model.Base() != "core20" {
+			return fmt.Errorf("cannot preseed the image for a model other than core20")
+		}
+		coreOpts := &preseed.CoreOptions{
+			PrepareImageDir:           opts.PrepareDir,
+			PreseedSignKey:            opts.PreseedSignKey,
+			AppArmorKernelFeaturesDir: opts.AppArmorKernelFeaturesDir,
+			SysfsOverlay:              opts.SysfsOverlay,
+		}
+		return preseedCore20(coreOpts)
+	}
+
+	return nil
 }
 
 // these are postponed, not implemented or abandoned, not finalized,
@@ -245,7 +271,7 @@ func makeLabel(now time.Time) string {
 	return now.UTC().Format("20060102")
 }
 
-func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
+var setupSeed = func(tsto *tooling.ToolingStore, model *asserts.Model, opts *Options) error {
 	if model.Classic() != opts.Classic {
 		return fmt.Errorf("internal error: classic model but classic mode not set")
 	}
@@ -358,9 +384,9 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
 		return err
 	}
 
-	var curSnaps []*CurrentSnap
+	var curSnaps []*tooling.CurrentSnap
 	for _, sn := range localSnaps {
-		si, aRefs, err := seedwriter.DeriveSideInfo(sn.Path, f, db)
+		si, aRefs, err := seedwriter.DeriveSideInfo(sn.Path, model, f, db)
 		if err != nil && !asserts.IsNotFound(err) {
 			return err
 		}
@@ -380,7 +406,7 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
 		sn.ARefs = aRefs
 
 		if info.ID() != "" {
-			curSnaps = append(curSnaps, &CurrentSnap{
+			curSnaps = append(curSnaps, &tooling.CurrentSnap{
 				SnapName: info.SnapName(),
 				SnapID:   info.ID(),
 				Revision: info.Revision,
@@ -411,14 +437,14 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
 			}
 			return sn.Path, nil
 		}
-		snapToDownloadOptions := make([]SnapToDownload, len(toDownload))
+		snapToDownloadOptions := make([]tooling.SnapToDownload, len(toDownload))
 		for i, sn := range toDownload {
 			byName[sn.SnapName()] = sn
 			snapToDownloadOptions[i].Snap = sn
 			snapToDownloadOptions[i].Channel = sn.Channel
 			snapToDownloadOptions[i].CohortKey = opts.WideCohortKey
 		}
-		downloadedSnaps, err := tsto.DownloadMany(snapToDownloadOptions, curSnaps, DownloadManyOptions{
+		downloadedSnaps, err := tsto.DownloadMany(snapToDownloadOptions, curSnaps, tooling.DownloadManyOptions{
 			BeforeDownloadFunc: beforeDownload,
 			EnforceValidation:  opts.Customizations.Validation == "enforce",
 		})
@@ -435,13 +461,13 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
 
 			// fetch snap assertions
 			prev := len(f.Refs())
-			if _, err = FetchAndCheckSnapAssertions(dlsn.Path, dlsn.Info, f, db); err != nil {
+			if _, err = FetchAndCheckSnapAssertions(dlsn.Path, dlsn.Info, model, f, db); err != nil {
 				return err
 			}
 			aRefs := f.Refs()[prev:]
 			sn.ARefs = aRefs
 
-			curSnaps = append(curSnaps, &CurrentSnap{
+			curSnaps = append(curSnaps, &tooling.CurrentSnap{
 				SnapName: sn.Info.SnapName(),
 				SnapID:   sn.Info.ID(),
 				Revision: sn.Info.Revision,
@@ -487,17 +513,24 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
 		return err
 	}
 
+	// TODO: There will be classic UC20+ model based systems
+	//       that will have a bootable  ubuntu-seed partition.
+	//       This will need to be handled here eventually too.
 	if opts.Classic {
-		// TODO:UC20: consider Core 20 extended models vs classic
-		seedFn := filepath.Join(seedDir, "seed.yaml")
+		var fpath string
+		if core20 {
+			fpath = filepath.Join(seedDir, "systems")
+		} else {
+			fpath = filepath.Join(seedDir, "seed.yaml")
+		}
 		// warn about ownership if not root:root
-		fi, err := os.Stat(seedFn)
+		fi, err := os.Stat(fpath)
 		if err != nil {
-			return fmt.Errorf("cannot stat seed.yaml: %s", err)
+			return fmt.Errorf("cannot stat %q: %s", fpath, err)
 		}
 		if st, ok := fi.Sys().(*syscall.Stat_t); ok {
 			if st.Uid != 0 || st.Gid != 0 {
-				fmt.Fprintf(Stderr, "WARNING: ensure that the contents under %s are owned by root:root in the (final) image", seedDir)
+				fmt.Fprintf(Stderr, "WARNING: ensure that the contents under %s are owned by root:root in the (final) image\n", seedDir)
 			}
 		}
 		// done already
@@ -518,15 +551,14 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
 		bootWith.RecoverySystemLabel = label
 	}
 
-	// find the gadget file
-	// find the snap.Info/path for kernel/os/base so
+	// find the snap.Info/path for kernel/os/base/gadget so
 	// that boot.MakeBootable can DTRT
-	gadgetFname := ""
 	kernelFname := ""
 	for _, sn := range bootSnaps {
 		switch sn.Info.Type() {
 		case snap.TypeGadget:
-			gadgetFname = sn.Path
+			bootWith.Gadget = sn.Info
+			bootWith.GadgetPath = sn.Path
 		case snap.TypeOS, snap.TypeBase:
 			bootWith.Base = sn.Info
 			bootWith.BasePath = sn.Path
@@ -538,7 +570,7 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
 	}
 
 	// unpacking the gadget for core models
-	if err := unpackSnap(gadgetFname, gadgetUnpackDir); err != nil {
+	if err := unpackSnap(bootWith.GadgetPath, gadgetUnpackDir); err != nil {
 		return err
 	}
 	if err := unpackSnap(kernelFname, kernelUnpackDir); err != nil {
@@ -576,7 +608,9 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options) error {
 			if err := os.MkdirAll(sysconfig.WritableDefaultsDir(rootDir, "/etc"), 0755); err != nil {
 				return err
 			}
-			return sysconfig.ApplyFilesystemOnlyDefaults(model, defaultsDir, defaults)
+			if err := sysconfig.ApplyFilesystemOnlyDefaults(model, defaultsDir, defaults); err != nil {
+				return err
+			}
 		}
 
 		customizeImage(rootDir, defaultsDir, &opts.Customizations)
