@@ -30,6 +30,7 @@ import (
 	"github.com/snapcore/snapd/interfaces/mount"
 	"github.com/snapcore/snapd/interfaces/seccomp"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 )
 
@@ -76,6 +77,7 @@ const fwupdPermanentSlotAppArmor = `
   /boot/efi/EFI/boot/fwupd*.efi* rw,
   /boot/efi/EFI/boot/fw/** rw,
   # allow access to fwupd* and fw/ under ubuntu/ for classic systems
+  # but it should be deprecated once old uefi-fw-tools is no longer used
   /boot/efi/EFI/ubuntu/fwupd*.efi* rw,
   /boot/efi/EFI/ubuntu/fw/** rw,
 
@@ -111,6 +113,15 @@ const fwupdPermanentSlotAppArmor = `
       name="org.freedesktop.fwupd",
 `
 
+const fwupdPermanentSlotAppArmorClassic = `
+# Description: Allow operating as the fwupd service. This gives privileged
+# access to the Classic system.
+
+  # allow access to fwupd* and fw/ under any distro for classic systems
+  /boot/efi/EFI/*/fwupd*.efi* rw,
+  /boot/efi/EFI/*/fw/** rw,
+`
+
 const fwupdConnectedPlugAppArmor = `
 # Description: Allow using fwupd service. This gives # privileged access to the
 # fwupd service.
@@ -260,10 +271,13 @@ func (iface *fwupdInterface) AppArmorConnectedPlug(spec *apparmor.Specification,
 }
 
 func (iface *fwupdInterface) AppArmorPermanentSlot(spec *apparmor.Specification, slot *snap.SlotInfo) error {
-	// Only apply slot snippet when running as application snap on
-	// classic, slot side can be system or application
+	// Only apply slot snippet when running as application snap
 	if !implicitSystemPermanentSlot(slot) {
 		spec.AddSnippet(fwupdPermanentSlotAppArmor)
+		// An application snap on classic also should have these rules
+		if release.OnClassic {
+			spec.AddSnippet(fwupdPermanentSlotAppArmorClassic)
+		}
 
 		// Allow mounting boot partition to snap-update-ns
 		emit := spec.AddUpdateNSf
