@@ -62,6 +62,27 @@ slots:
       - read
       - write
 
+  test-path-array:
+    interface: posix-mq
+    path:
+      - /test-array-1
+      - /test-array-2
+      - /test-array-3
+
+  test-empty-path-array:
+    interface: posix-mq
+    path: []
+
+  test-one-empty-path-array:
+    interface: posix-mq
+    path:
+      - /test-array-1
+      - ""
+
+  test-empty-path:
+    interface: posix-mq
+    path: ""
+
   test-invalid-path-1:
     interface: posix-mq
     path: ../../test-invalid
@@ -73,7 +94,14 @@ slots:
   test-invalid-path-3:
     interface: posix-mq
     path:
-      - this-is-not-a-string
+      this-is-not-a-valid-path: true
+
+  test-invalid-path-4:
+    interface: posix-mq
+
+  test-invalid-path-5:
+    interface: posix-mq
+    path: /.
 
   test-invalid-perms-1:
     interface: posix-mq
@@ -88,6 +116,13 @@ slots:
       path: /test-invalid-perms-2
       permissions: not-a-list
 
+  test-invalid-perms-3:
+    interface: posix-mq
+    path: /test-invalid-perms-3
+    permissions:
+      - create
+      - [not-a-string]
+
   test-label:
       interface: posix-mq
       posix-mq: this-is-a-test-label
@@ -163,6 +198,19 @@ apps:
     plugs: [test-all-perms]
 `
 
+const pathArrayPlugSnapInfoYaml = `name: consumer
+version: 1.0
+
+plugs:
+  test-path-array:
+    interface: posix-mq
+
+apps:
+  app:
+    command: foo
+    plugs: [test-path-array]
+`
+
 const invalidPerms1PlugSnapInfoYaml = `name: consumer
 version: 1.0
 
@@ -190,6 +238,19 @@ apps:
     plugs: [test-label]
 `
 
+const invalidPerms3PlugSnapInfoYaml = `name: consumer
+version: 1.0
+
+plugs:
+  test-invalid-perms-3:
+    interface: posix-mq
+
+apps:
+  app:
+    command: foo
+    plugs: [test-invalid-perms-3]
+`
+
 const testInvalidLabelPlugSnapInfoYaml = `name: consumer
 version: 1.0
 
@@ -230,6 +291,20 @@ type PosixMQInterfaceSuite struct {
 	testAllPermsPlugInfo *snap.PlugInfo
 	testAllPermsPlug     *interfaces.ConnectedPlug
 
+	testPathArraySlotInfo *snap.SlotInfo
+	testPathArraySlot     *interfaces.ConnectedSlot
+	testPathArrayPlugInfo *snap.PlugInfo
+	testPathArrayPlug     *interfaces.ConnectedPlug
+
+	testEmptyPathArraySlotInfo *snap.SlotInfo
+	testEmptyPathArraySlot     *interfaces.ConnectedSlot
+
+	testOneEmptyPathArraySlotInfo *snap.SlotInfo
+	testOneEmptyPathArraySlot     *interfaces.ConnectedSlot
+
+	testEmptyPathSlotInfo *snap.SlotInfo
+	testEmptyPathSlot     *interfaces.ConnectedSlot
+
 	testInvalidPath1SlotInfo *snap.SlotInfo
 	testInvalidPath1Slot     *interfaces.ConnectedSlot
 
@@ -239,6 +314,12 @@ type PosixMQInterfaceSuite struct {
 	testInvalidPath3SlotInfo *snap.SlotInfo
 	testInvalidPath3Slot     *interfaces.ConnectedSlot
 
+	testInvalidPath4SlotInfo *snap.SlotInfo
+	testInvalidPath4Slot     *interfaces.ConnectedSlot
+
+	testInvalidPath5SlotInfo *snap.SlotInfo
+	testInvalidPath5Slot     *interfaces.ConnectedSlot
+
 	testInvalidPerms1SlotInfo *snap.SlotInfo
 	testInvalidPerms1Slot     *interfaces.ConnectedSlot
 	testInvalidPerms1PlugInfo *snap.PlugInfo
@@ -247,6 +328,11 @@ type PosixMQInterfaceSuite struct {
 	testInvalidPerms2SlotInfo *snap.SlotInfo
 	testInvalidPerms2Slot     *interfaces.ConnectedSlot
 
+	testInvalidPerms3SlotInfo *snap.SlotInfo
+	testInvalidPerms3Slot     *interfaces.ConnectedSlot
+	testInvalidPerms3PlugInfo *snap.PlugInfo
+	testInvalidPerms3Plug     *interfaces.ConnectedPlug
+
 	testLabelSlotInfo *snap.SlotInfo
 	testLabelSlot     *interfaces.ConnectedSlot
 	testLabelPlugInfo *snap.PlugInfo
@@ -279,6 +365,18 @@ func (s *PosixMQInterfaceSuite) SetUpTest(c *C) {
 	s.testAllPermsSlotInfo = slotSnap.Slots["test-all-perms"]
 	s.testAllPermsSlot = interfaces.NewConnectedSlot(s.testAllPermsSlotInfo, nil, nil)
 
+	s.testPathArraySlotInfo = slotSnap.Slots["test-path-array"]
+	s.testPathArraySlot = interfaces.NewConnectedSlot(s.testPathArraySlotInfo, nil, nil)
+
+	s.testEmptyPathArraySlotInfo = slotSnap.Slots["test-empty-path-array"]
+	s.testEmptyPathArraySlot = interfaces.NewConnectedSlot(s.testEmptyPathArraySlotInfo, nil, nil)
+
+	s.testOneEmptyPathArraySlotInfo = slotSnap.Slots["test-one-empty-path-array"]
+	s.testOneEmptyPathArraySlot = interfaces.NewConnectedSlot(s.testOneEmptyPathArraySlotInfo, nil, nil)
+
+	s.testEmptyPathSlotInfo = slotSnap.Slots["test-empty-path"]
+	s.testEmptyPathSlot = interfaces.NewConnectedSlot(s.testEmptyPathSlotInfo, nil, nil)
+
 	s.testInvalidPath1SlotInfo = slotSnap.Slots["test-invalid-path-1"]
 	s.testInvalidPath1Slot = interfaces.NewConnectedSlot(s.testInvalidPath1SlotInfo, nil, nil)
 
@@ -288,12 +386,21 @@ func (s *PosixMQInterfaceSuite) SetUpTest(c *C) {
 	s.testInvalidPath3SlotInfo = slotSnap.Slots["test-invalid-path-3"]
 	s.testInvalidPath3Slot = interfaces.NewConnectedSlot(s.testInvalidPath3SlotInfo, nil, nil)
 
+	s.testInvalidPath4SlotInfo = slotSnap.Slots["test-invalid-path-4"]
+	s.testInvalidPath4Slot = interfaces.NewConnectedSlot(s.testInvalidPath4SlotInfo, nil, nil)
+
+	s.testInvalidPath5SlotInfo = slotSnap.Slots["test-invalid-path-5"]
+	s.testInvalidPath5Slot = interfaces.NewConnectedSlot(s.testInvalidPath5SlotInfo, nil, nil)
+
 	s.testInvalidPerms1SlotInfo = slotSnap.Slots["test-invalid-perms-1"]
 	s.testInvalidPerms1Slot = interfaces.NewConnectedSlot(s.testInvalidPerms1SlotInfo, nil, nil)
 
 	s.testInvalidPerms2SlotInfo = slotSnap.Slots["test-invalid-perms-2"]
 	s.testInvalidPerms2Slot = interfaces.NewConnectedSlot(s.testInvalidPerms2SlotInfo, nil, nil)
 
+	s.testInvalidPerms3SlotInfo = slotSnap.Slots["test-invalid-perms-3"]
+	s.testInvalidPerms3Slot = interfaces.NewConnectedSlot(s.testInvalidPerms3SlotInfo, nil, nil)
+
 	s.testLabelSlotInfo = slotSnap.Slots["test-label"]
 	s.testLabelSlot = interfaces.NewConnectedSlot(s.testLabelSlotInfo, nil, nil)
 
@@ -327,6 +434,14 @@ func (s *PosixMQInterfaceSuite) SetUpTest(c *C) {
 	plugSnap6 := snaptest.MockInfo(c, testInvalidLabelPlugSnapInfoYaml, nil)
 	s.testInvalidLabelPlugInfo = plugSnap6.Plugs["test-invalid-label"]
 	s.testInvalidLabelPlug = interfaces.NewConnectedPlug(s.testInvalidLabelPlugInfo, nil, nil)
+
+	plugSnap7 := snaptest.MockInfo(c, invalidPerms3PlugSnapInfoYaml, nil)
+	s.testInvalidPerms3PlugInfo = plugSnap7.Plugs["test-invalid-perms-3"]
+	s.testInvalidPerms3Plug = interfaces.NewConnectedPlug(s.testInvalidPerms3PlugInfo, nil, nil)
+
+	plugSnap8 := snaptest.MockInfo(c, pathArrayPlugSnapInfoYaml, nil)
+	s.testPathArrayPlugInfo = plugSnap8.Plugs["test-path-array"]
+	s.testPathArrayPlug = interfaces.NewConnectedPlug(s.testPathArrayPlugInfo, nil, nil)
 }
 
 func (s *PosixMQInterfaceSuite) checkSlotSeccompSnippet(c *C, spec *seccomp.Specification) {
@@ -348,9 +463,11 @@ func (s *PosixMQInterfaceSuite) TestReadWriteMQAppArmor(c *C) {
 	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.consumer.app", "snap.producer.app"})
 
 	slotSnippet := spec.SnippetForTag("snap.producer.app")
+	c.Check(slotSnippet, testutil.Contains, `# POSIX Message Queue slot: test-rw`)
 	c.Check(slotSnippet, testutil.Contains, `mqueue (open read write create delete) "/test-rw",`)
 
 	plugSnippet := spec.SnippetForTag("snap.consumer.app")
+	c.Check(plugSnippet, testutil.Contains, `# POSIX Message Queue plug: test-rw`)
 	c.Check(plugSnippet, testutil.Contains, `mqueue (read write open) "/test-rw",`)
 }
 
@@ -381,9 +498,11 @@ func (s *PosixMQInterfaceSuite) TestDefaultReadWriteMQAppArmor(c *C) {
 	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.consumer.app", "snap.producer.app"})
 
 	slotSnippet := spec.SnippetForTag("snap.producer.app")
+	c.Check(slotSnippet, testutil.Contains, `# POSIX Message Queue slot: test-default`)
 	c.Check(slotSnippet, testutil.Contains, `mqueue (open read write create delete) "/test-default",`)
 
 	plugSnippet := spec.SnippetForTag("snap.consumer.app")
+	c.Check(plugSnippet, testutil.Contains, `# POSIX Message Queue plug: test-default`)
 	c.Check(plugSnippet, testutil.Contains, `mqueue (read write open) "/test-default",`)
 }
 
@@ -440,6 +559,46 @@ func (s *PosixMQInterfaceSuite) TestReadOnlyMQSeccomp(c *C) {
 	c.Check(plugSnippet, Not(testutil.Contains), "mq_unlink")
 }
 
+func (s *PosixMQInterfaceSuite) TestPathArrayMQAppArmor(c *C) {
+	spec := &apparmor.Specification{}
+	err := spec.AddPermanentSlot(s.iface, s.testPathArraySlotInfo)
+	c.Assert(err, IsNil)
+	err = spec.AddConnectedPlug(s.iface, s.testPathArrayPlug, s.testPathArraySlot)
+	c.Assert(err, IsNil)
+	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.consumer.app", "snap.producer.app"})
+
+	slotSnippet := spec.SnippetForTag("snap.producer.app")
+	c.Check(slotSnippet, testutil.Contains, `  mqueue (open read write create delete) "/test-array-1",
+  mqueue (open read write create delete) "/test-array-2",
+  mqueue (open read write create delete) "/test-array-3",
+`)
+
+	plugSnippet := spec.SnippetForTag("snap.consumer.app")
+	c.Check(plugSnippet, testutil.Contains, `  mqueue (read write open) "/test-array-1",
+  mqueue (read write open) "/test-array-2",
+  mqueue (read write open) "/test-array-3",
+`)
+}
+
+func (s *PosixMQInterfaceSuite) TestPathArrayMQSeccomp(c *C) {
+	spec := &seccomp.Specification{}
+	err := spec.AddPermanentSlot(s.iface, s.testPathArraySlotInfo)
+	c.Assert(err, IsNil)
+	err = spec.AddConnectedPlug(s.iface, s.testPathArrayPlug, s.testPathArraySlot)
+	c.Assert(err, IsNil)
+	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.consumer.app", "snap.producer.app"})
+
+	s.checkSlotSeccompSnippet(c, spec)
+
+	plugSnippet := spec.SnippetForTag("snap.consumer.app")
+	c.Check(plugSnippet, testutil.Contains, "mq_open")
+	c.Check(plugSnippet, testutil.Contains, "mq_notify")
+	c.Check(plugSnippet, testutil.Contains, "mq_timedreceive")
+	c.Check(plugSnippet, testutil.Contains, "mq_timedsend")
+	c.Check(plugSnippet, testutil.Contains, "mq_getsetattr")
+	c.Check(plugSnippet, Not(testutil.Contains), "mq_unlink")
+}
+
 func (s *PosixMQInterfaceSuite) TestAllPermsMQAppArmor(c *C) {
 	spec := &apparmor.Specification{}
 	err := spec.AddPermanentSlot(s.iface, s.testAllPermsSlotInfo)
@@ -490,7 +649,7 @@ func (s *PosixMQInterfaceSuite) TestPathValidationAppArmorRegex(c *C) {
 func (s *PosixMQInterfaceSuite) TestPathStringValidation(c *C) {
 	spec := &apparmor.Specification{}
 	err := spec.AddPermanentSlot(s.iface, s.testInvalidPath3SlotInfo)
-	c.Check(err, ErrorMatches, `posix-mq slot "path" attribute must be a string, not \[this-is-not-a-string\]`)
+	c.Check(err, ErrorMatches, `snap "producer" has interface "posix-mq" with invalid value type map\[string\]interface {} for "path" attribute: \*\[\]string`)
 }
 
 func (s *PosixMQInterfaceSuite) TestInvalidPerms1(c *C) {
@@ -506,6 +665,15 @@ func (s *PosixMQInterfaceSuite) TestInvalidPerms1(c *C) {
 		`posix-mq slot permission "break-everything" not valid, must be one of \[open read write create delete\]`)
 }
 
+func (s *PosixMQInterfaceSuite) TestInvalidPerms3(c *C) {
+	spec := &apparmor.Specification{}
+	err := spec.AddPermanentSlot(s.iface, s.testInvalidPerms3SlotInfo)
+	c.Assert(err, IsNil)
+	err = spec.AddConnectedPlug(s.iface, s.testInvalidPerms3Plug, s.testInvalidPerms3Slot)
+	c.Check(err, ErrorMatches,
+		`snap "producer" has interface "posix-mq" with invalid value type \[\]interface {} for "permissions" attribute: \*\[\]string`)
+}
+
 func (s *PosixMQInterfaceSuite) TestName(c *C) {
 	c.Assert(s.iface.Name(), Equals, "posix-mq")
 }
@@ -514,7 +682,9 @@ func (s *PosixMQInterfaceSuite) TestNoAppArmor(c *C) {
 	// Ensure that the interface does not fail if AppArmor is unsupported
 	restore := apparmor_sandbox.MockLevel(apparmor_sandbox.Unsupported)
 	defer restore()
-	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.testReadWriteSlotInfo), IsNil)
+
+	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testReadWriteSlotInfo), IsNil)
+	c.Check(interfaces.BeforePreparePlug(s.iface, s.testReadWritePlugInfo), IsNil)
 }
 
 func (s *PosixMQInterfaceSuite) TestFeatureDetection(c *C) {
@@ -546,6 +716,8 @@ func (s *PosixMQInterfaceSuite) TestSanitizeSlot(c *C) {
 	s.checkSlotPosixMQAttr(c, s.testReadOnlySlotInfo)
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.testAllPermsSlotInfo), IsNil)
 	s.checkSlotPosixMQAttr(c, s.testAllPermsSlotInfo)
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.testPathArraySlotInfo), IsNil)
+	s.checkSlotPosixMQAttr(c, s.testPathArraySlotInfo)
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.testLabelSlotInfo), IsNil)
 	c.Check(s.testLabelSlotInfo.Attrs["posix-mq"], Equals, "this-is-a-test-label")
 
@@ -555,13 +727,23 @@ func (s *PosixMQInterfaceSuite) TestSanitizeSlot(c *C) {
 	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testInvalidPath2SlotInfo), ErrorMatches,
 		`posix-mq "path" attribute is invalid: /test-invalid-2"\["`)
 	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testInvalidPath3SlotInfo), ErrorMatches,
-		`posix-mq slot "path" attribute must be a string, not \[this-is-not-a-string\]`)
+		`snap "producer" has interface "posix-mq" with invalid value type map\[string\]interface {} for "path" attribute: \*\[\]string`)
+	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testInvalidPath4SlotInfo), ErrorMatches,
+		`posix-mq slot requires the "path" attribute`)
+	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testInvalidPath5SlotInfo), ErrorMatches,
+		`posix-mq "path" attribute is not a clean path: "/."`)
 	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testInvalidPerms1SlotInfo), ErrorMatches,
 		`posix-mq slot permission "break-everything" not valid, must be one of \[open read write create delete\]`)
 	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testInvalidPerms2SlotInfo), ErrorMatches,
-		`posix-mq slot "permissions" attribute must be a list of strings, not not-a-list`)
+		`snap "producer" has interface "posix-mq" with invalid value type string for "permissions" attribute: \*\[\]string`)
 	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testInvalidLabelSlotInfo), ErrorMatches,
 		`posix-mq "posix-mq" attribute must be a string, not \[broken\]`)
+	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testEmptyPathArraySlotInfo), ErrorMatches,
+		`posix-mq slot requires at least one value in the "path" attribute`)
+	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testEmptyPathSlotInfo), ErrorMatches,
+		`posix-mq slot "path" attribute values cannot be empty`)
+	c.Check(interfaces.BeforePrepareSlot(s.iface, s.testOneEmptyPathArraySlotInfo), ErrorMatches,
+		`posix-mq slot "path" attribute values cannot be empty`)
 }
 
 func (s *PosixMQInterfaceSuite) TestSanitizePlug(c *C) {
@@ -577,6 +759,8 @@ func (s *PosixMQInterfaceSuite) TestSanitizePlug(c *C) {
 	s.checkPlugPosixMQAttr(c, s.testReadOnlyPlugInfo)
 	c.Assert(interfaces.BeforePreparePlug(s.iface, s.testAllPermsPlugInfo), IsNil)
 	s.checkPlugPosixMQAttr(c, s.testAllPermsPlugInfo)
+	c.Assert(interfaces.BeforePreparePlug(s.iface, s.testPathArrayPlugInfo), IsNil)
+	s.checkPlugPosixMQAttr(c, s.testPathArrayPlugInfo)
 	c.Assert(interfaces.BeforePreparePlug(s.iface, s.testInvalidPerms1PlugInfo), IsNil)
 	s.checkPlugPosixMQAttr(c, s.testInvalidPerms1PlugInfo)
 	c.Assert(interfaces.BeforePreparePlug(s.iface, s.testLabelPlugInfo), IsNil)
