@@ -86,6 +86,8 @@ func (s *systemPackagesDocSuite) TestAppArmorSpec(c *C) {
 	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.consumer.app"})
 	c.Assert(spec.SnippetForTag("snap.consumer.app"), testutil.Contains, "# Description: can access documentation of system packages.")
 	c.Assert(spec.SnippetForTag("snap.consumer.app"), testutil.Contains, "/usr/share/doc/{,**} r,")
+	c.Assert(spec.SnippetForTag("snap.consumer.app"), testutil.Contains, "/usr/share/cups/doc-root/{,**} r,")
+	c.Assert(spec.SnippetForTag("snap.consumer.app"), testutil.Contains, "/usr/share/gimp/2.0/help/{,**} r,")
 	c.Assert(spec.SnippetForTag("snap.consumer.app"), testutil.Contains, "/usr/share/libreoffice/help/{,**} r,")
 	c.Assert(spec.SnippetForTag("snap.consumer.app"), testutil.Contains, "/usr/share/xubuntu-docs/{,**} r,")
 
@@ -95,6 +97,14 @@ func (s *systemPackagesDocSuite) TestAppArmorSpec(c *C) {
 	c.Check(updateNS, testutil.Contains, "  remount options=(bind, ro) /usr/share/doc/,\n")
 	c.Check(updateNS, testutil.Contains, "  umount /usr/share/doc/,\n")
 
+	c.Check(updateNS, testutil.Contains, "  mount options=(bind) /var/lib/snapd/hostfs/usr/share/cups/doc-root/ -> /usr/share/cups/doc-root/,\n")
+	c.Check(updateNS, testutil.Contains, "  remount options=(bind, ro) /usr/share/cups/doc-root/,\n")
+	c.Check(updateNS, testutil.Contains, "  umount /usr/share/cups/doc-root/,\n")
+
+	c.Check(updateNS, testutil.Contains, "  mount options=(bind) /var/lib/snapd/hostfs/usr/share/gimp/2.0/help/ -> /usr/share/gimp/2.0/help/,\n")
+	c.Check(updateNS, testutil.Contains, "  remount options=(bind, ro) /usr/share/gimp/2.0/help/,\n")
+	c.Check(updateNS, testutil.Contains, "  umount /usr/share/gimp/2.0/help/,\n")
+
 	c.Check(updateNS, testutil.Contains, "  mount options=(bind) /var/lib/snapd/hostfs/usr/share/gtk-doc/ -> /usr/share/gtk-doc/,\n")
 	c.Check(updateNS, testutil.Contains, "  remount options=(bind, ro) /usr/share/gtk-doc/,\n")
 	c.Check(updateNS, testutil.Contains, "  umount /usr/share/gtk-doc/,\n")
@@ -107,10 +117,21 @@ func (s *systemPackagesDocSuite) TestAppArmorSpec(c *C) {
 	c.Check(updateNS, testutil.Contains, "  remount options=(bind, ro) /usr/share/xubuntu-docs/,\n")
 	c.Check(updateNS, testutil.Contains, "  umount /usr/share/xubuntu-docs/,\n")
 	// check mimic bits
-	c.Check(updateNS, testutil.Contains, "  # Writable mimic /usr/share\n")
+	c.Check(updateNS, testutil.Contains, "  # Writable mimic /usr/share/libreoffice\n")
 	c.Check(updateNS, testutil.Contains, "  mount fstype=tmpfs options=(rw) tmpfs -> \"/usr/share/\",\n")
 	c.Check(updateNS, testutil.Contains, "  \"/usr/share/\" r,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/usr/share/*/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/usr/share/cups/*/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/usr/share/gimp/*/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/usr/share/gimp/2.0/*/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/usr/share/libreoffice/*/\" rw,\n")
 	c.Check(updateNS, testutil.Contains, "  \"/tmp/.snap/usr/share/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/tmp/.snap/usr/share/cups/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/tmp/.snap/usr/share/cups/*/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/tmp/.snap/usr/share/gimp/2.0/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/tmp/.snap/usr/share/gimp/2.0/*/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/tmp/.snap/usr/share/libreoffice/\" rw,\n")
+	c.Check(updateNS, testutil.Contains, "  \"/tmp/.snap/usr/share/libreoffice/*/\" rw,\n")
 	c.Check(updateNS, testutil.Contains, "  mount options=(bind, rw) \"/tmp/.snap/usr/share/*\" -> \"/usr/share/*\",\n")
 
 }
@@ -123,19 +144,25 @@ func (s *systemPackagesDocSuite) TestMountSpec(c *C) {
 	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.coreSlot), IsNil)
 
 	entries := spec.MountEntries()
-	c.Assert(entries, HasLen, 4)
+	c.Assert(entries, HasLen, 6)
 	c.Check(entries[0].Name, Equals, "/var/lib/snapd/hostfs/usr/share/doc")
 	c.Check(entries[0].Dir, Equals, "/usr/share/doc")
 	c.Check(entries[0].Options, DeepEquals, []string{"bind", "ro"})
-	c.Check(entries[1].Name, Equals, "/var/lib/snapd/hostfs/usr/share/gtk-doc")
-	c.Check(entries[1].Dir, Equals, "/usr/share/gtk-doc")
+	c.Check(entries[1].Name, Equals, "/var/lib/snapd/hostfs/usr/share/cups/doc-root")
+	c.Check(entries[1].Dir, Equals, "/usr/share/cups/doc-root")
 	c.Check(entries[1].Options, DeepEquals, []string{"bind", "ro"})
-	c.Check(entries[2].Name, Equals, "/var/lib/snapd/hostfs/usr/share/libreoffice/help")
-	c.Check(entries[2].Dir, Equals, "/usr/share/libreoffice/help")
+	c.Check(entries[2].Name, Equals, "/var/lib/snapd/hostfs/usr/share/gimp/2.0/help")
+	c.Check(entries[2].Dir, Equals, "/usr/share/gimp/2.0/help")
 	c.Check(entries[2].Options, DeepEquals, []string{"bind", "ro"})
-	c.Check(entries[3].Name, Equals, "/var/lib/snapd/hostfs/usr/share/xubuntu-docs")
-	c.Check(entries[3].Dir, Equals, "/usr/share/xubuntu-docs")
+	c.Check(entries[3].Name, Equals, "/var/lib/snapd/hostfs/usr/share/gtk-doc")
+	c.Check(entries[3].Dir, Equals, "/usr/share/gtk-doc")
 	c.Check(entries[3].Options, DeepEquals, []string{"bind", "ro"})
+	c.Check(entries[4].Name, Equals, "/var/lib/snapd/hostfs/usr/share/libreoffice/help")
+	c.Check(entries[4].Dir, Equals, "/usr/share/libreoffice/help")
+	c.Check(entries[4].Options, DeepEquals, []string{"bind", "ro"})
+	c.Check(entries[5].Name, Equals, "/var/lib/snapd/hostfs/usr/share/xubuntu-docs")
+	c.Check(entries[5].Dir, Equals, "/usr/share/xubuntu-docs")
+	c.Check(entries[5].Options, DeepEquals, []string{"bind", "ro"})
 
 	entries = spec.UserMountEntries()
 	c.Assert(entries, HasLen, 0)
