@@ -149,7 +149,7 @@ func (s *baseDeclSuite) TestAutoConnection(c *C) {
 		"packagekit-control": true,
 		"pkcs11":             true,
 		"snapd-control":      true,
-		"dummy":              true,
+		"empty":              true,
 	}
 
 	// these simply auto-connect, anything else doesn't
@@ -781,7 +781,7 @@ var (
 		"docker-support":            {"core"},
 		"desktop-launch":            {"core"},
 		"dsp":                       {"core", "gadget"},
-		"dummy":                     {"app"},
+		"empty":                     {"app"},
 		"fwupd":                     {"app", "core"},
 		"gpio":                      {"core", "gadget"},
 		"gpio-control":              {"core"},
@@ -835,6 +835,7 @@ var (
 		"custom-device":   nil,
 		"docker":          nil,
 		"lxd":             nil,
+		"microceph":       nil,
 		"pkcs11":          nil,
 		"posix-mq":        nil,
 		"shared-memory":   nil,
@@ -890,6 +891,12 @@ func (s *baseDeclSuite) TestSlotInstallation(c *C) {
 	c.Assert(err, Not(IsNil))
 	c.Assert(err, ErrorMatches, "installation not allowed by \"lxd\" slot rule of interface \"lxd\"")
 
+	// test microceph specially
+	ic = s.installSlotCand(c, "microceph", snap.TypeApp, ``)
+	err = ic.Check()
+	c.Assert(err, Not(IsNil))
+	c.Assert(err, ErrorMatches, "installation not allowed by \"microceph\" slot rule of interface \"microceph\"")
+
 	// test shared-memory specially
 	ic = s.installSlotCand(c, "shared-memory", snap.TypeApp, ``)
 	err = ic.Check()
@@ -995,6 +1002,7 @@ func (s *baseDeclSuite) TestConnection(c *C) {
 		"location-observe":          true,
 		"lxd":                       true,
 		"maliit":                    true,
+		"microceph":                 true,
 		"mir":                       true,
 		"online-accounts-service":   true,
 		"posix-mq":                  true,
@@ -1159,7 +1167,7 @@ slots:
 	}
 }
 
-func (s *baseDeclSuite) TestSanity(c *C) {
+func (s *baseDeclSuite) TestValidity(c *C) {
 	all := builtin.Interfaces()
 
 	// these interfaces have rules both for the slots and plugs side
