@@ -41,15 +41,22 @@ type InstallCandidate struct {
 	Store *asserts.Store
 }
 
+func (ic *InstallCandidate) snapID() string {
+	if ic.SnapDeclaration != nil {
+		return ic.SnapDeclaration.SnapID()
+	}
+	return "" // never a valid snap-id
+}
+
 func (ic *InstallCandidate) checkSlotRule(slot *snap.SlotInfo, rule *asserts.SlotRule, snapRule bool) error {
 	context := ""
 	if snapRule {
 		context = fmt.Sprintf(" for %q snap", ic.SnapDeclaration.SnapName())
 	}
-	if checkSlotInstallationConstraints(ic, slot, rule.DenyInstallation) == nil {
+	if checkSlotInstallationAltConstraints(ic, slot, rule.DenyInstallation) == nil {
 		return fmt.Errorf("installation denied by %q slot rule of interface %q%s", slot.Name, slot.Interface, context)
 	}
-	if checkSlotInstallationConstraints(ic, slot, rule.AllowInstallation) != nil {
+	if checkSlotInstallationAltConstraints(ic, slot, rule.AllowInstallation) != nil {
 		return fmt.Errorf("installation not allowed by %q slot rule of interface %q%s", slot.Name, slot.Interface, context)
 	}
 	return nil
@@ -60,10 +67,10 @@ func (ic *InstallCandidate) checkPlugRule(plug *snap.PlugInfo, rule *asserts.Plu
 	if snapRule {
 		context = fmt.Sprintf(" for %q snap", ic.SnapDeclaration.SnapName())
 	}
-	if checkPlugInstallationConstraints(ic, plug, rule.DenyInstallation) == nil {
+	if checkPlugInstallationAltConstraints(ic, plug, rule.DenyInstallation) == nil {
 		return fmt.Errorf("installation denied by %q plug rule of interface %q%s", plug.Name, plug.Interface, context)
 	}
-	if checkPlugInstallationConstraints(ic, plug, rule.AllowInstallation) != nil {
+	if checkPlugInstallationAltConstraints(ic, plug, rule.AllowInstallation) != nil {
 		return fmt.Errorf("installation not allowed by %q plug rule of interface %q%s", plug.Name, plug.Interface, context)
 	}
 	return nil
@@ -176,7 +183,7 @@ func (connc *ConnectCandidate) slotPublisherID() string {
 	return "" // never a valid publisher-id
 }
 
-func (connc *ConnectCandidate) checkPlugRule(kind string, rule *asserts.PlugRule, snapRule bool) error {
+func (connc *ConnectCandidate) checkPlugRule(kind string, rule *asserts.PlugRule, snapRule bool) (interfaces.SideArity, error) {
 	context := ""
 	if snapRule {
 		context = fmt.Sprintf(" for %q snap", connc.PlugSnapDeclaration.SnapName())
@@ -187,16 +194,18 @@ func (connc *ConnectCandidate) checkPlugRule(kind string, rule *asserts.PlugRule
 		denyConst = rule.DenyAutoConnection
 		allowConst = rule.AllowAutoConnection
 	}
-	if checkPlugConnectionConstraints(connc, denyConst) == nil {
-		return fmt.Errorf("%s denied by plug rule of interface %q%s", kind, connc.Plug.Interface(), context)
+	if _, err := checkPlugConnectionAltConstraints(connc, denyConst); err == nil {
+		return nil, fmt.Errorf("%s denied by plug rule of interface %q%s", kind, connc.Plug.Interface(), context)
 	}
-	if checkPlugConnectionConstraints(connc, allowConst) != nil {
-		return fmt.Errorf("%s not allowed by plug rule of interface %q%s", kind, connc.Plug.Interface(), context)
+
+	allowedConstraints, err := checkPlugConnectionAltConstraints(connc, allowConst)
+	if err != nil {
+		return nil, fmt.Errorf("%s not allowed by plug rule of interface %q%s", kind, connc.Plug.Interface(), context)
 	}
-	return nil
+	return sideArity{allowedConstraints.SlotsPerPlug}, nil
 }
 
-func (connc *ConnectCandidate) checkSlotRule(kind string, rule *asserts.SlotRule, snapRule bool) error {
+func (connc *ConnectCandidate) checkSlotRule(kind string, rule *asserts.SlotRule, snapRule bool) (interfaces.SideArity, error) {
 	context := ""
 	if snapRule {
 		context = fmt.Sprintf(" for %q snap", connc.SlotSnapDeclaration.SnapName())
@@ -207,25 +216,27 @@ func (connc *ConnectCandidate) checkSlotRule(kind string, rule *asserts.SlotRule
 		denyConst = rule.DenyAutoConnection
 		allowConst = rule.AllowAutoConnection
 	}
-	if checkSlotConnectionConstraints(connc, denyConst) == nil {
-		return fmt.Errorf("%s denied by slot rule of interface %q%s", kind, connc.Plug.Interface(), context)
+	if _, err := checkSlotConnectionAltConstraints(connc, denyConst); err == nil {
+		return nil, fmt.Errorf("%s denied by slot rule of interface %q%s", kind, connc.Plug.Interface(), context)
 	}
-	if checkSlotConnectionConstraints(connc, allowConst) != nil {
-		return fmt.Errorf("%s not allowed by slot rule of interface %q%s", kind, connc.Plug.Interface(), context)
+
+	allowedConstraints, err := checkSlotConnectionAltConstraints(connc, allowConst)
+	if err != nil {
+		return nil, fmt.Errorf("%s not allowed by slot rule of interface %q%s", kind, connc.Plug.Interface(), context)
 	}
-	return nil
+	return sideArity{allowedConstraints.SlotsPerPlug}, nil
 }
 
-func (connc *ConnectCandidate) check(kind string) error {
+func (connc *ConnectCandidate) check(kind string) (interfaces.SideArity, error) {
 	baseDecl := connc.BaseDeclaration
 	if baseDecl == nil {
-		return fmt.Errorf("internal error: improperly initialized ConnectCandidate")
+		return nil, fmt.Errorf("internal error: improperly initialized ConnectCandidate")
 	}
 
 	iface := connc.Plug.Interface()
 
 	if connc.Slot.Interface() != iface {
-		return fmt.Errorf("cannot connect mismatched plug interface %q to slot interface %q", iface, connc.Slot.Interface())
+		return nil, fmt.Errorf("cannot connect mismatched plug interface %q to slot interface %q", iface, connc.Slot.Interface())
 	}
 
 	if plugDecl := connc.PlugSnapDeclaration; plugDecl != nil {
@@ -244,17 +255,27 @@ func (connc *ConnectCandidate) check(kind string) error {
 	if rule := baseDecl.SlotRule(iface); rule != nil {
 		return connc.checkSlotRule(kind, rule, false)
 	}
-	return nil
+	return nil, nil
 }
 
 // Check checks whether the connection is allowed.
 func (connc *ConnectCandidate) Check() error {
-	return connc.check("connection")
+	_, err := connc.check("connection")
+	return err
 }
 
 // CheckAutoConnect checks whether the connection is allowed to auto-connect.
-func (connc *ConnectCandidate) CheckAutoConnect() error {
-	return connc.check("auto-connection")
+func (connc *ConnectCandidate) CheckAutoConnect() (interfaces.SideArity, error) {
+	arity, err := connc.check("auto-connection")
+	if err != nil {
+		return nil, err
+	}
+	if arity == nil {
+		// shouldn't happen but be safe, the callers should be able
+		// to assume arity to be non nil
+		arity = sideArity{asserts.SideArityConstraint{N: 1}}
+	}
+	return arity, nil
 }
 
 // InstallCandidateMinimalCheck represents a candidate snap installed with --dangerous flag that should pass minimum checks
@@ -267,12 +288,12 @@ type InstallCandidateMinimalCheck struct {
 }
 
 func (ic *InstallCandidateMinimalCheck) checkSlotRule(slot *snap.SlotInfo, rule *asserts.SlotRule) error {
-	if hasConstraints, err := checkMinimalSlotInstallationConstraints(ic, slot, rule.DenyInstallation); hasConstraints && err == nil {
+	if hasConstraints, err := checkMinimalSlotInstallationAltConstraints(slot, rule.DenyInstallation); hasConstraints && err == nil {
 		return fmt.Errorf("installation denied by %q slot rule of interface %q", slot.Name, slot.Interface)
 	}
 
 	// TODO check that the snap is an app or gadget if allow-installation had no slot-snap-type constraints
-	if _, err := checkMinimalSlotInstallationConstraints(ic, slot, rule.AllowInstallation); err != nil {
+	if _, err := checkMinimalSlotInstallationAltConstraints(slot, rule.AllowInstallation); err != nil {
 		return fmt.Errorf("installation not allowed by %q slot rule of interface %q", slot.Name, slot.Interface)
 	}
 	return nil
