@@ -2121,7 +2121,7 @@ func (s *RepositorySuite) TestHotplugMethods(c *C) {
 
 	coreSlot := &snap.SlotInfo{
 		Snap:       s.coreSnap,
-		Name:       "dummy-slot",
+		Name:       "test-slot",
 		Interface:  "interface",
 		HotplugKey: "1234",
 	}
@@ -2153,7 +2153,7 @@ func (s *RepositorySuite) TestUpdateHotplugSlotAttrs(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	coreSlot := &snap.SlotInfo{
 		Snap:       s.coreSnap,
-		Name:       "dummy-slot",
+		Name:       "test-slot",
 		Interface:  "interface",
 		HotplugKey: "1234",
 		Attrs:      map[string]interface{}{"a": "b"},
@@ -2178,7 +2178,7 @@ func (s *RepositorySuite) TestUpdateHotplugSlotAttrsConnectedError(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	coreSlot := &snap.SlotInfo{
 		Snap:       s.coreSnap,
-		Name:       "dummy-slot",
+		Name:       "test-slot",
 		Interface:  "interface",
 		HotplugKey: "1234",
 	}
@@ -2188,6 +2188,6 @@ func (s *RepositorySuite) TestUpdateHotplugSlotAttrsConnectedError(c *C) {
 	c.Assert(err, IsNil)
 
 	slot, err := s.testRepo.UpdateHotplugSlotAttrs("interface", "1234", map[string]interface{}{"c": "d"})
-	c.Assert(err, ErrorMatches, `internal error: cannot update slot dummy-slot while connected`)
+	c.Assert(err, ErrorMatches, `internal error: cannot update slot test-slot while connected`)
 	c.Assert(slot, IsNil)
 }
