@@ -91,7 +91,7 @@ func (s *backendSuite) TestName(c *C) {
 func (s *backendSuite) TestInstallingSnapWritesAndLoadsRules(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -117,11 +117,11 @@ func (s *backendSuite) TestInstallingSnapWritesAndLoadsRules(c *C) {
 func (s *backendSuite) TestInstallingSnapWithHookWritesAndLoadsRules(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	s.Iface.UDevPermanentPlugCallback = func(spec *udev.Specification, slot *snap.PlugInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -149,7 +149,7 @@ func (s *backendSuite) TestInstallingSnapWithHookWritesAndLoadsRules(c *C) {
 func (s *backendSuite) TestSecurityIsStable(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -166,7 +166,7 @@ func (s *backendSuite) TestSecurityIsStable(c *C) {
 func (s *backendSuite) TestRemovingSnapRemovesAndReloadsRules(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -223,7 +223,7 @@ func (s *backendSuite) TestUpdatingSnapToOneWithMoreHooks(c *C) {
 		return nil
 	}
 	s.Iface.UDevPermanentPlugCallback = func(spec *udev.Specification, slot *snap.PlugInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -283,7 +283,7 @@ func (s *backendSuite) TestUpdatingSnapToOneWithFewerHooks(c *C) {
 		return nil
 	}
 	s.Iface.UDevPermanentPlugCallback = func(spec *udev.Specification, slot *snap.PlugInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -310,16 +310,16 @@ func (s *backendSuite) TestUpdatingSnapToOneWithFewerHooks(c *C) {
 func (s *backendSuite) TestCombineSnippetsWithActualSnippets(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
 		snapInfo := s.InstallSnap(c, opts, "", ifacetest.SambaYamlV1, 0)
 		fname := filepath.Join(dirs.SnapUdevRulesDir, "70-snap.samba.rules")
 		if opts.DevMode || opts.Classic {
-			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\n# udev tagging/device cgroups disabled with non-strict mode snaps\n#dummy\n")
+			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\n# udev tagging/device cgroups disabled with non-strict mode snaps\n#sample\n")
 		} else {
-			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\ndummy\n")
+			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\nsample\n")
 		}
 		stat, err := os.Stat(fname)
 		c.Assert(err, IsNil)
@@ -331,7 +331,7 @@ func (s *backendSuite) TestCombineSnippetsWithActualSnippets(c *C) {
 func (s *backendSuite) TestControlsDeviceCgroup(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		spec.SetControlsDeviceCgroup()
 		return nil
 	}
@@ -346,16 +346,16 @@ func (s *backendSuite) TestControlsDeviceCgroup(c *C) {
 func (s *backendSuite) TestCombineSnippetsWithActualSnippetsWithNewline(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy1\ndummy2")
+		spec.AddSnippet("sample1\nsample2")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
 		snapInfo := s.InstallSnap(c, opts, "", ifacetest.SambaYamlV1, 0)
 		fname := filepath.Join(dirs.SnapUdevRulesDir, "70-snap.samba.rules")
 		if opts.DevMode || opts.Classic {
-			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\n# udev tagging/device cgroups disabled with non-strict mode snaps\n#dummy1\n#dummy2\n")
+			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\n# udev tagging/device cgroups disabled with non-strict mode snaps\n#sample1\n#sample2\n")
 		} else {
-			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\ndummy1\ndummy2\n")
+			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\nsample1\nsample2\n")
 		}
 		stat, err := os.Stat(fname)
 		c.Assert(err, IsNil)
@@ -366,16 +366,16 @@ func (s *backendSuite) TestCombineSnippetsWithActualSnippetsWithNewline(c *C) {
 func (s *backendSuite) TestCombineSnippetsWithActualSnippetsWhenPlugNoApps(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentPlugCallback = func(spec *udev.Specification, slot *snap.PlugInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
 		snapInfo := s.InstallSnap(c, opts, "", ifacetest.PlugNoAppsYaml, 0)
 		fname := filepath.Join(dirs.SnapUdevRulesDir, "70-snap.foo.rules")
 		if opts.DevMode || opts.Classic {
-			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\n# udev tagging/device cgroups disabled with non-strict mode snaps\n#dummy\n")
+			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\n# udev tagging/device cgroups disabled with non-strict mode snaps\n#sample\n")
 		} else {
-			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\ndummy\n")
+			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\nsample\n")
 		}
 		stat, err := os.Stat(fname)
 		c.Assert(err, IsNil)
@@ -387,16 +387,16 @@ func (s *backendSuite) TestCombineSnippetsWithActualSnippetsWhenPlugNoApps(c *C)
 func (s *backendSuite) TestCombineSnippetsWithActualSnippetsWhenSlotNoApps(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
 		snapInfo := s.InstallSnap(c, opts, "", ifacetest.SlotNoAppsYaml, 0)
 		fname := filepath.Join(dirs.SnapUdevRulesDir, "70-snap.foo.rules")
 		if opts.DevMode || opts.Classic {
-			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\n# udev tagging/device cgroups disabled with non-strict mode snaps\n#dummy\n")
+			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\n# udev tagging/device cgroups disabled with non-strict mode snaps\n#sample\n")
 		} else {
-			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\ndummy\n")
+			c.Check(fname, testutil.FileEquals, "# This file is automatically generated.\nsample\n")
 		}
 		stat, err := os.Stat(fname)
 		c.Assert(err, IsNil)
@@ -419,7 +419,7 @@ func (s *backendSuite) TestCombineSnippetsWithoutAnySnippets(c *C) {
 func (s *backendSuite) TestUpdatingSnapToOneWithoutSlots(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -446,7 +446,7 @@ func (s *backendSuite) TestUpdatingSnapToOneWithoutSlots(c *C) {
 func (s *backendSuite) TestUpdatingSnapWithoutSlotsToOneWithoutSlots(c *C) {
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -471,7 +471,7 @@ func (s *backendSuite) TestInstallingSnapWritesAndLoadsRulesWithInputSubsystem(c
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
 		spec.TriggerSubsystem("input")
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
@@ -496,7 +496,7 @@ func (s *backendSuite) TestInstallingSnapWritesAndLoadsRulesWithInputJoystickSub
 	// NOTE: Hand out a permanent snippet so that .rules file is generated.
 	s.Iface.UDevPermanentSlotCallback = func(spec *udev.Specification, slot *snap.SlotInfo) error {
 		spec.TriggerSubsystem("input/joystick")
-		spec.AddSnippet("dummy")
+		spec.AddSnippet("sample")
 		return nil
 	}
 	for _, opts := range testedConfinementOpts {
