@@ -516,6 +516,7 @@ func (s *fdeSuite) TestRevealErr(c *C) {
 			"systemd-run", "--collect", "--service-type=exec", "--quiet",
 			"--property=RuntimeMaxSec=2m0s",
 			"--property=SystemCallFilter=~@mount",
+			"--property=DefaultDependencies=no",
 			fmt.Sprintf("--property=StandardInput=file:%s/run/fde-reveal-key/fde-reveal-key.stdin", root),
 			fmt.Sprintf("--property=StandardOutput=file:%s/run/fde-reveal-key/fde-reveal-key.stdout", root),
 			fmt.Sprintf("--property=StandardError=file:%s/run/fde-reveal-key/fde-reveal-key.stderr", root),
@@ -683,3 +684,14 @@ func (s *fdeSuite) TestIsEncryptedDeviceMapperName(c *C) {
 		c.Assert(fde.IsHardwareEncryptedDeviceMapperName(t), Equals, false)
 	}
 }
+
+func (s *fdeSuite) TestEncryptedDeviceMapperName(c *C) {
+	for _, str := range []string{
+		"ubuntu-data",
+		"ubuntu-save",
+		"foo",
+		"other",
+	} {
+		c.Assert(fde.EncryptedDeviceMapperName(str), Equals, str+"-device-locked")
+	}
+}
