@@ -23,14 +23,12 @@ import (
 	"bytes"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"log"
 	"os"
-	"strings"
 	"sync"
+	"time"
 
 	"github.com/snapcore/snapd/osutil"
-	"github.com/snapcore/snapd/strutil"
 )
 
 // A Logger is a fairly minimal logging tool.
@@ -163,15 +161,26 @@ func SimpleSetup() error {
 	return err
 }
 
-var procCmdline = "/proc/cmdline"
+// used to force testing of the kernel command line parsing
+var procCmdlineUseDefaultMockInTests = true
 
 // TODO: consider generalizing this to snapdenv and having it used by
 // other places that consider SNAPD_DEBUG
 func debugEnabledOnKernelCmdline() bool {
-	buf, err := ioutil.ReadFile(procCmdline)
-	if err != nil {
+	// if this is called during tests, always ignore it so we don't have to mock
+	// the /proc/cmdline for every test that ends up using a logger
+	if osutil.IsTestBinary() && procCmdlineUseDefaultMockInTests {
 		return false
 	}
-	l := strings.Split(string(buf), " ")
-	return strutil.ListContains(l, "snapd.debug=1")
+	m, _ := osutil.KernelCommandLineKeyValues("snapd.debug")
+	return m["snapd.debug"] == "1"
+}
+
+var timeNow = time.Now
+
+// StartupStageTimestamp produce snap startup timings message.
+func StartupStageTimestamp(stage string) {
+	now := timeNow()
+	Debugf(`-- snap startup {"stage":"%s", "time":"%v.%06d"}`,
+		stage, now.Unix(), (now.UnixNano()/1e3)%1e6)
 }
