@@ -36,14 +36,63 @@ type flockSuite struct{}
 
 var _ = Suite(&flockSuite{})
 
+// Test that an existing lock file can be opened.
+func (s *flockSuite) TestOpenExistingLockForReading(c *C) {
+	fname := filepath.Join(c.MkDir(), "name")
+	lock, err := osutil.OpenExistingLockForReading(fname)
+	c.Assert(err, ErrorMatches, ".* no such file or directory")
+	c.Assert(lock, IsNil)
+
+	lock, err = osutil.NewFileLockWithMode(fname, 0644)
+	c.Assert(err, IsNil)
+	lock.Close()
+
+	// Having created the lock above, we can now open it correctly.
+	lock, err = osutil.OpenExistingLockForReading(fname)
+	c.Assert(err, IsNil)
+	defer lock.Close()
+
+	// The lock file is read-only though.
+	file := lock.File()
+	defer file.Close()
+	n, err := file.Write([]byte{1, 2, 3})
+	// write(2) returns EBADF if the file descriptor is read only.
+	c.Assert(err, ErrorMatches, ".* bad file descriptor")
+	c.Assert(n, Equals, 0)
+}
+
+// Test that opening and closing a lock works as expected, and that the mode is right.
+func (s *flockSuite) TestNewFileLockWithMode(c *C) {
+	lock, err := osutil.NewFileLockWithMode(filepath.Join(c.MkDir(), "name"), 0644)
+	c.Assert(err, IsNil)
+	defer lock.Close()
+
+	fi, err := os.Stat(lock.Path())
+	c.Assert(err, IsNil)
+	c.Assert(fi.Mode().Perm(), Equals, os.FileMode(0644))
+}
+
 // Test that opening and closing a lock works as expected.
 func (s *flockSuite) TestNewFileLock(c *C) {
 	lock, err := osutil.NewFileLock(filepath.Join(c.MkDir(), "name"))
 	c.Assert(err, IsNil)
 	defer lock.Close()
 
-	_, err = os.Stat(lock.Path())
+	fi, err := os.Stat(lock.Path())
 	c.Assert(err, IsNil)
+	c.Assert(fi.Mode().Perm(), Equals, os.FileMode(0600))
+}
+
+// Test that we can access the underlying open file.
+func (s *flockSuite) TestFile(c *C) {
+	fname := filepath.Join(c.MkDir(), "name")
+	lock, err := osutil.NewFileLock(fname)
+	c.Assert(err, IsNil)
+	defer lock.Close()
+
+	f := lock.File()
+	c.Assert(f, NotNil)
+	c.Check(f.Name(), Equals, fname)
 }
 
 func flockSupportsConflictExitCodeSwitch(c *C) bool {
@@ -54,10 +103,6 @@ func flockSupportsConflictExitCodeSwitch(c *C) bool {
 
 // Test that Lock and Unlock work as expected.
 func (s *flockSuite) TestLockUnlockWorks(c *C) {
-	if os.Getenv("TRAVIS_BUILD_NUMBER") != "" {
-		c.Skip("Cannot use this under travis")
-		return
-	}
 	if !flockSupportsConflictExitCodeSwitch(c) {
 		c.Skip("flock too old for this test")
 	}
@@ -89,6 +134,46 @@ func (s *flockSuite) TestLockUnlockWorks(c *C) {
 	c.Assert(cmd.Run(), IsNil)
 }
 
+// Test that ReadLock and Unlock work as expected.
+func (s *flockSuite) TestReadLockUnlockWorks(c *C) {
+	if !flockSupportsConflictExitCodeSwitch(c) {
+		c.Skip("flock too old for this test")
+	}
+
+	lock, err := osutil.NewFileLock(filepath.Join(c.MkDir(), "name"))
+	c.Assert(err, IsNil)
+	defer lock.Close()
+
+	// Run a flock command in another process, it should succeed because it can
+	// lock the lock as we didn't do it yet.
+	cmd := exec.Command("flock", "--exclusive", "--nonblock", lock.Path(), "true")
+	c.Assert(cmd.Run(), IsNil)
+
+	// Grab a shared lock.
+	c.Assert(lock.ReadLock(), IsNil)
+
+	// Run a flock command in another process, it should fail with the distinct
+	// error code because we hold a shared lock already and we asked it not to block.
+	cmd = exec.Command("flock", "--exclusive", "--nonblock",
+		"--conflict-exit-code", "2", lock.Path(), "true")
+	c.Assert(cmd.Run(), ErrorMatches, "exit status 2")
+
+	// Run a flock command in another process, it should succeed because we
+	// hold a shared lock and those do not prevent others from acquiring a
+	// shared lock.
+	cmd = exec.Command("flock", "--shared", "--nonblock",
+		"--conflict-exit-code", "2", lock.Path(), "true")
+	c.Assert(cmd.Run(), IsNil)
+
+	// Unlock the lock.
+	c.Assert(lock.Unlock(), IsNil)
+
+	// Run a flock command in another process, it should succeed because it can
+	// grab the lock again now.
+	cmd = exec.Command("flock", "--exclusive", "--nonblock", lock.Path(), "true")
+	c.Assert(cmd.Run(), IsNil)
+}
+
 // Test that locking a locked lock does nothing.
 func (s *flockSuite) TestLockLocked(c *C) {
 	lock, err := osutil.NewFileLock(filepath.Join(c.MkDir(), "name"))
@@ -121,11 +206,6 @@ func (s *flockSuite) TestUsingClosedLock(c *C) {
 
 // Test that non-blocking locking reports error on pre-acquired lock.
 func (s *flockSuite) TestLockUnlockNonblockingWorks(c *C) {
-	if os.Getenv("TRAVIS_BUILD_NUMBER") != "" {
-		c.Skip("Cannot use this under travis")
-		return
-	}
-
 	// Use the "flock" command to grab a lock for 9999 seconds in another process.
 	lockPath := filepath.Join(c.MkDir(), "lock")
 	sleeperKillerPath := filepath.Join(c.MkDir(), "pid")
