@@ -23,6 +23,7 @@
 package assertstate
 
 import (
+	"errors"
 	"fmt"
 	"strings"
 
@@ -117,7 +118,7 @@ func RefreshSnapDeclarations(s *state.State, userID int, opts *RefreshAssertions
 
 		return nil
 	}
-	return doFetch(s, userID, deviceCtx, fetching)
+	return doFetch(s, userID, deviceCtx, nil, fetching)
 }
 
 type refreshControlError struct {
@@ -213,7 +214,7 @@ func ValidateRefreshes(s *state.State, snapInfos []*snap.Info, ignoreValidation
 			}
 			return nil
 		}
-		err := doFetch(s, userID, deviceCtx, fetching)
+		err := doFetch(s, userID, deviceCtx, nil, fetching)
 		if err != nil {
 			errs = append(errs, fmt.Errorf("cannot refresh %q to revision %s: %v", candInfo.InstanceName(), candInfo.Revision, err))
 			continue
@@ -289,6 +290,24 @@ func Publisher(s *state.State, snapID string) (*asserts.Account, error) {
 	return a.(*asserts.Account), nil
 }
 
+// PublisherStoreAccount returns the store account information from the publisher assertion.
+func PublisherStoreAccount(st *state.State, snapID string) (snap.StoreAccount, error) {
+	if snapID == "" {
+		return snap.StoreAccount{}, nil
+	}
+
+	pubAcct, err := Publisher(st, snapID)
+	if err != nil {
+		return snap.StoreAccount{}, fmt.Errorf("cannot find publisher details: %v", err)
+	}
+	return snap.StoreAccount{
+		ID:          pubAcct.AccountID(),
+		Username:    pubAcct.Username(),
+		DisplayName: pubAcct.DisplayName(),
+		Validation:  pubAcct.Validation(),
+	}, nil
+}
+
 // Store returns the store assertion with the given name/id if it is
 // present in the system assertion database.
 func Store(s *state.State, store string) (*asserts.Store, error) {
@@ -314,7 +333,13 @@ func AutoAliases(s *state.State, info *snap.Info) (map[string]string, error) {
 	}
 	explicitAliases := decl.Aliases()
 	if len(explicitAliases) != 0 {
-		return explicitAliases, nil
+		aliasesForApps := make(map[string]string, len(explicitAliases))
+		for alias, app := range explicitAliases {
+			if _, ok := info.Apps[app]; ok {
+				aliasesForApps[alias] = app
+			}
+		}
+		return aliasesForApps, nil
 	}
 	// XXX: old header fallback, just to keep edge working while we fix the
 	// store, to remove before next release!
@@ -343,11 +368,13 @@ func delayedCrossMgrInit() {
 	// hook retrieving auto-aliases into snapstate logic
 	snapstate.AutoAliases = AutoAliases
 	// hook the helper for getting enforced validation sets
-	snapstate.EnforcedValidationSets = EnforcedValidationSets
+	snapstate.EnforcedValidationSets = TrackedEnforcedValidationSets
 	// hook the helper for saving current validation sets to the stack
 	snapstate.AddCurrentTrackingToValidationSetsStack = addCurrentTrackingToValidationSetsHistory
 	// hook the helper for restoring validation sets tracking from the stack
 	snapstate.RestoreValidationSetsTracking = RestoreValidationSetsTracking
+	// hook helper for enforcing validation sets without fetching them
+	snapstate.EnforceValidationSets = ApplyEnforcedValidationSets
 }
 
 // AutoRefreshAssertions tries to refresh all assertions
@@ -595,37 +622,27 @@ func validationSetAssertionForMonitor(st *state.State, accountID, name string, s
 	return as, false, err
 }
 
+func getSpecificSequenceOrLatest(db *asserts.Database, headers map[string]string) (vs *asserts.ValidationSet, err error) {
+	var a asserts.Assertion
+	if _, ok := headers["sequence"]; ok {
+		a, err = db.Find(asserts.ValidationSetType, headers)
+	} else {
+		a, err = db.FindSequence(asserts.ValidationSetType, headers, -1, -1)
+	}
+	if err != nil {
+		return nil, err
+	}
+	vs = a.(*asserts.ValidationSet)
+	return vs, nil
+}
+
 // validationSetAssertionForEnforce tries to fetch the validation set assertion
 // with the given accountID/name/sequence (sequence is optional) using pool and
-// checks if it's not in conflict with existing validation sets in enforcing mode
-// (all currently tracked validation set assertions get refreshed), and if they
-// are valid for installed snaps.
-func validationSetAssertionForEnforce(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) (vs *asserts.ValidationSet, current int, err error) {
+// checks if it's not in conflict with existing validation sets in enforcing mode.
+func validationSetAssertionForEnforce(st *state.State, accountID, name string, sequence int, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) (vs *asserts.ValidationSet, err error) {
 	deviceCtx, err := snapstate.DevicePastSeeding(st, nil)
 	if err != nil {
-		return nil, 0, err
-	}
-
-	opts := &RefreshAssertionsOptions{IsAutoRefresh: false}
-
-	// refresh all currently tracked validation set assertions (this may or may not
-	// include the one requested by the caller).
-	if err = RefreshValidationSetAssertions(st, userID, opts); err != nil {
-		return nil, 0, err
-	}
-
-	getSpecificSequenceOrLatest := func(db *asserts.Database, headers map[string]string) (vs *asserts.ValidationSet, err error) {
-		var a asserts.Assertion
-		if _, ok := headers["sequence"]; ok {
-			a, err = db.Find(asserts.ValidationSetType, headers)
-		} else {
-			a, err = db.FindSequence(asserts.ValidationSetType, headers, -1, -1)
-		}
-		if err != nil {
-			return nil, err
-		}
-		vs = a.(*asserts.ValidationSet)
-		return vs, nil
+		return nil, err
 	}
 
 	// try to get existing from the db. It will be the latest one if it was
@@ -654,7 +671,7 @@ func validationSetAssertionForEnforce(st *state.State, accountID, name string, s
 	vs, err = getSpecificSequenceOrLatest(db, headers)
 
 	checkForConflicts := func() error {
-		valsets, err := EnforcedValidationSets(st, vs)
+		valsets, err := TrackedEnforcedValidationSets(st, vs)
 		if err != nil {
 			return err
 		}
@@ -667,19 +684,6 @@ func validationSetAssertionForEnforce(st *state.State, accountID, name string, s
 		return nil
 	}
 
-	getLatest := func() (int, error) {
-		headers := map[string]string{
-			"series":     release.Series,
-			"account-id": accountID,
-			"name":       name,
-		}
-		a, err := db.FindSequence(asserts.ValidationSetType, headers, -1, -1)
-		if err != nil {
-			return 0, fmt.Errorf("internal error: %v", err)
-		}
-		return a.(*asserts.ValidationSet).Sequence(), nil
-	}
-
 	// found locally
 	if err == nil {
 		// check if we were tracking it already; if not, that
@@ -688,36 +692,33 @@ func validationSetAssertionForEnforce(st *state.State, accountID, name string, s
 		// by RefreshValidationSetAssertions.
 		var tr ValidationSetTracking
 		trerr := GetValidationSet(st, accountID, name, &tr)
-		if trerr != nil && trerr != state.ErrNoState {
-			return nil, 0, trerr
+		if trerr != nil && !errors.Is(trerr, state.ErrNoState) {
+			return nil, trerr
 		}
 		// not tracked, update the assertion
-		if trerr == state.ErrNoState {
+		if errors.Is(trerr, state.ErrNoState) {
 			// update with pool
 			atSeq.Sequence = vs.Sequence()
 			atSeq.Revision = vs.Revision()
 			if err := pool.AddSequenceToUpdate(atSeq, atSeq.Unique()); err != nil {
-				return nil, 0, err
+				return nil, err
 			}
 		} else {
 			// was already tracked, add to validation sets and check
 			if err := checkForConflicts(); err != nil {
-				return nil, 0, err
+				return nil, err
 			}
-			latest, err := getLatest()
-			if err != nil {
-				return nil, 0, err
-			}
-			return vs, latest, nil
+
+			return vs, nil
 		}
 	} else {
 		if !asserts.IsNotFound(err) {
-			return nil, 0, err
+			return nil, err
 		}
 
 		// try to resolve with pool
 		if err := pool.AddUnresolvedSequence(atSeq, atSeq.Unique()); err != nil {
-			return nil, 0, err
+			return nil, err
 		}
 	}
 
@@ -736,50 +737,251 @@ func validationSetAssertionForEnforce(st *state.State, accountID, name string, s
 		return nil
 	}
 
+	opts := &RefreshAssertionsOptions{IsAutoRefresh: false}
 	if err := resolvePoolNoFallback(st, pool, checkBeforeCommit, userID, deviceCtx, opts); err != nil {
-		return nil, 0, err
+		return nil, err
 	}
 
-	latest, err := getLatest()
-	if err != nil {
-		return nil, 0, err
-	}
-	return vs, latest, err
+	return vs, err
 }
 
-// EnforceValidationSet tries to fetch the given validation set and enforce it.
-// If all validation sets constrains are satisfied, the current validation sets
-// tracking state is saved in validation sets history.
-func EnforceValidationSet(st *state.State, accountID, name string, sequence, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) error {
-	_, current, err := validationSetAssertionForEnforce(st, accountID, name, sequence, userID, snaps, ignoreValidation)
+// TryEnforcedValidationSets tries to fetch the given validation sets and
+// enforce them (together with currently tracked validation sets) against
+// installed snaps, but doesn't update tracking information in case of an error.
+// It may return snapasserts.ValidationSetsValidationError which can be used to
+// install/remove snaps as required to satisfy validation sets constraints.
+func TryEnforcedValidationSets(st *state.State, validationSets []string, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) error {
+	deviceCtx, err := snapstate.DevicePastSeeding(st, nil)
 	if err != nil {
 		return err
 	}
 
+	db := cachedDB(st)
+	pool := asserts.NewPool(db, maxGroups)
+
+	extraVsHeaders := make([]map[string]string, 0, len(validationSets))
+	newTracking := make([]*ValidationSetTracking, 0, len(validationSets))
+
+	for _, vsstr := range validationSets {
+		accountID, name, sequence, err := snapasserts.ParseValidationSet(vsstr)
+		if err != nil {
+			return err
+		}
+
+		// try to get existing from the db
+		headers := map[string]string{
+			"series":     release.Series,
+			"account-id": accountID,
+			"name":       name,
+		}
+		if sequence > 0 {
+			headers["sequence"] = fmt.Sprintf("%d", sequence)
+		}
+		atSeq := &asserts.AtSequence{
+			Type:        asserts.ValidationSetType,
+			SequenceKey: []string{release.Series, accountID, name},
+			Sequence:    sequence,
+			Revision:    asserts.RevisionNotKnown,
+			Pinned:      sequence > 0,
+		}
+
+		// prepare tracking data, note current is not known yet
+		tr := &ValidationSetTracking{
+			AccountID: headers["account-id"],
+			Name:      headers["name"],
+			Mode:      Enforce,
+			// may be 0 meaning no pinning
+			PinnedAt: sequence,
+		}
+
+		extraVsHeaders = append(extraVsHeaders, headers)
+		newTracking = append(newTracking, tr)
+
+		vs, err := getSpecificSequenceOrLatest(db, headers)
+		// found locally
+		if err == nil {
+			// update with pool
+			atSeq.Sequence = vs.Sequence()
+			atSeq.Revision = vs.Revision()
+			if err := pool.AddSequenceToUpdate(atSeq, atSeq.Unique()); err != nil {
+				return err
+			}
+		} else {
+			if !asserts.IsNotFound(err) {
+				return err
+			}
+			// try to resolve with pool
+			if err := pool.AddUnresolvedSequence(atSeq, atSeq.Unique()); err != nil {
+				return err
+			}
+		}
+	}
+
+	checkBeforeCommit := func(db *asserts.Database, bs asserts.Backstore) error {
+		tmpDb := db.WithStackedBackstore(bs)
+		// get the resolved validation set asserts, add to validation sets and check
+		var extraVs []*asserts.ValidationSet
+		for _, headers := range extraVsHeaders {
+			vs, err := getSpecificSequenceOrLatest(tmpDb, headers)
+			if err != nil {
+				return fmt.Errorf("internal error: cannot find validation set assertion: %v", err)
+			}
+			extraVs = append(extraVs, vs)
+		}
+
+		valsets, err := TrackedEnforcedValidationSets(st, extraVs...)
+		if err != nil {
+			return err
+		}
+		if err := valsets.Conflict(); err != nil {
+			return err
+		}
+		if err := valsets.CheckInstalledSnaps(snaps, ignoreValidation); err != nil {
+			// the returned error may be ValidationSetsValidationError which is normal and means we cannot enforce
+			// the new validation sets - the caller should resolve the error and retry.
+			return err
+		}
+
+		// all fine, will be committed (along with its prerequisites if any) on
+		// return by resolvePoolNoFallback
+		return nil
+	}
+
+	opts := &RefreshAssertionsOptions{}
+	if err := resolvePoolNoFallback(st, pool, checkBeforeCommit, userID, deviceCtx, opts); err != nil {
+		return err
+	}
+
+	// no error, all validation-sets can be enforced, update tracking for all vsets
+	for i, headers := range extraVsHeaders {
+		tr := newTracking[i]
+
+		if tr.PinnedAt == 0 {
+			// if unpinned, get latest assertion from the db to determine current
+			a, err := db.FindSequence(asserts.ValidationSetType, headers, -1, -1)
+			if err != nil {
+				// this is unexpected since all asserts should be resolved and committed at this point
+				return fmt.Errorf("internal error: cannot find validation set assertion: %v", err)
+			}
+			tr.Current = a.Sequence()
+		} else {
+			// no need to get latest since Current must be the same as pinned
+			tr.Current = tr.PinnedAt
+		}
+	}
+	for _, tr := range newTracking {
+		UpdateValidationSet(st, tr)
+	}
+
+	return addCurrentTrackingToValidationSetsHistory(st)
+}
+
+// ApplyEnforcedValidationSets enforces the supplied validation sets. It takes a map
+// of validation set keys to validation sets, pinned sequence numbers (if any),
+// installed snaps and ignored snaps. It fetches any pre-requisites necessary.
+func ApplyEnforcedValidationSets(st *state.State, valsets map[string]*asserts.ValidationSet, pinnedSeqs map[string]int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool, userID int) error {
+	deviceCtx, err := snapstate.DevicePastSeeding(st, nil)
+	if err != nil {
+		return err
+	}
+
+	db := cachedDB(st)
+	batch := asserts.NewBatch(handleUnsupported(db))
+
+	valsetsSlice := make([]*asserts.ValidationSet, 0, len(valsets))
+	valsetsTracking := make([]*ValidationSetTracking, 0, len(valsets))
+
+	for vsKey, vs := range valsets {
+		pinnedSeq := pinnedSeqs[vsKey]
+		if pinnedSeq != 0 && pinnedSeq != vs.Sequence() {
+			// shouldn't be possible save for programmer error since, if we have a pinned
+			// sequence here, it should've been used when fetching the assertion
+			return fmt.Errorf("internal error: trying to enforce validation set %q with sequence point %d different than pinned %d", vsKey, vs.Sequence(), pinnedSeq)
+		}
+
+		tr := &ValidationSetTracking{
+			AccountID: vs.AccountID(),
+			Name:      vs.Name(),
+			Mode:      Enforce,
+			Current:   vs.Sequence(),
+			// may be 0 meaning no pinning
+			PinnedAt: pinnedSeq,
+		}
+
+		valsetsTracking = append(valsetsTracking, tr)
+		valsetsSlice = append(valsetsSlice, vs)
+	}
+
+	err = doFetch(st, userID, deviceCtx, batch, func(f asserts.Fetcher) error {
+		for vsKey, vs := range valsets {
+			if err := f.Save(vs); err != nil {
+				return fmt.Errorf("cannot save assertion %q to batch: %v", vsKey, err)
+			}
+		}
+		return nil
+	})
+	if err != nil {
+		return err
+	}
+
+	valsetGroup, err := TrackedEnforcedValidationSets(st, valsetsSlice...)
+	if err != nil {
+		return err
+	}
+
+	if err := valsetGroup.Conflict(); err != nil {
+		return err
+	}
+
+	if err := valsetGroup.CheckInstalledSnaps(snaps, ignoreValidation); err != nil {
+		return err
+	}
+
+	if err := batch.CommitTo(db, nil); err != nil {
+		return err
+	}
+
+	for _, tr := range valsetsTracking {
+		UpdateValidationSet(st, tr)
+	}
+
+	return addCurrentTrackingToValidationSetsHistory(st)
+}
+
+// FetchAndApplyEnforcedValidationSet tries to fetch the given validation set and enforce it.
+// If all validation sets constrains are satisfied, the current validation sets
+// tracking state is saved in validation sets history.
+func FetchAndApplyEnforcedValidationSet(st *state.State, accountID, name string, sequence, userID int, snaps []*snapasserts.InstalledSnap, ignoreValidation map[string]bool) (*ValidationSetTracking, error) {
+	vs, err := validationSetAssertionForEnforce(st, accountID, name, sequence, userID, snaps, ignoreValidation)
+	if err != nil {
+		return nil, err
+	}
+
 	tr := ValidationSetTracking{
 		AccountID: accountID,
 		Name:      name,
 		Mode:      Enforce,
 		// note, sequence may be 0, meaning not pinned.
 		PinnedAt: sequence,
-		Current:  current,
+		Current:  vs.Sequence(),
 	}
 
 	UpdateValidationSet(st, &tr)
-	return addCurrentTrackingToValidationSetsHistory(st)
+	err = addCurrentTrackingToValidationSetsHistory(st)
+	return &tr, err
 }
 
 // MonitorValidationSet tries to fetch the given validation set and monitor it.
 // The current validation sets tracking state is saved in validation sets history.
-func MonitorValidationSet(st *state.State, accountID, name string, sequence int, userID int) error {
+func MonitorValidationSet(st *state.State, accountID, name string, sequence int, userID int) (*ValidationSetTracking, error) {
 	pinned := sequence > 0
 	opts := ResolveOptions{AllowLocalFallback: true}
 	as, local, err := validationSetAssertionForMonitor(st, accountID, name, sequence, pinned, userID, &opts)
 	if err != nil {
-		return fmt.Errorf("cannot get validation set assertion for %v: %v", ValidationSetKey(accountID, name), err)
+		return nil, fmt.Errorf("cannot get validation set assertion for %v: %v", ValidationSetKey(accountID, name), err)
 	}
 
-	tr := ValidationSetTracking{
+	tr := &ValidationSetTracking{
 		AccountID: accountID,
 		Name:      name,
 		Mode:      Monitor,
@@ -789,8 +991,8 @@ func MonitorValidationSet(st *state.State, accountID, name string, sequence int,
 		LocalOnly: local,
 	}
 
-	UpdateValidationSet(st, &tr)
-	return addCurrentTrackingToValidationSetsHistory(st)
+	UpdateValidationSet(st, tr)
+	return tr, addCurrentTrackingToValidationSetsHistory(st)
 }
 
 // TemporaryDB returns a temporary database stacked on top of the assertions
