@@ -21,6 +21,7 @@ package assertstate
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 
 	"github.com/snapcore/snapd/asserts"
@@ -66,6 +67,17 @@ func (vs *ValidationSetTracking) sameAs(tr *ValidationSetTracking) bool {
 		vs.Name == tr.Name && vs.PinnedAt == tr.PinnedAt
 }
 
+// Sequence returns the sequence number of the currently used validation set.
+func (vs *ValidationSetTracking) Sequence() int {
+	// Current was occasionally set to the latest sequence number even when Pinned != 0,
+	// this should no longer happen but return PinnedAt anyway to be safe
+	if vs.PinnedAt > 0 {
+		return vs.PinnedAt
+	}
+
+	return vs.Current
+}
+
 // ValidationSetKey formats the given account id and name into a validation set key.
 func ValidationSetKey(accountID, name string) string {
 	return fmt.Sprintf("%s/%s", accountID, name)
@@ -76,7 +88,7 @@ func ValidationSetKey(accountID, name string) string {
 func UpdateValidationSet(st *state.State, tr *ValidationSetTracking) {
 	var vsmap map[string]*json.RawMessage
 	err := st.Get("validation-sets", &vsmap)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		panic("internal error: cannot unmarshal validation set tracking state: " + err.Error())
 	}
 	if vsmap == nil {
@@ -97,7 +109,7 @@ func UpdateValidationSet(st *state.State, tr *ValidationSetTracking) {
 func ForgetValidationSet(st *state.State, accountID, name string) error {
 	var vsmap map[string]*json.RawMessage
 	err := st.Get("validation-sets", &vsmap)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		panic("internal error: cannot unmarshal validation set tracking state: " + err.Error())
 	}
 	if len(vsmap) == 0 {
@@ -139,17 +151,17 @@ func GetValidationSet(st *state.State, accountID, name string, tr *ValidationSet
 // ValidationSets retrieves all ValidationSetTracking data.
 func ValidationSets(st *state.State) (map[string]*ValidationSetTracking, error) {
 	var vsmap map[string]*ValidationSetTracking
-	if err := st.Get("validation-sets", &vsmap); err != nil && err != state.ErrNoState {
+	if err := st.Get("validation-sets", &vsmap); err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, err
 	}
 	return vsmap, nil
 }
 
-// EnforcedValidationSets returns ValidationSets object with all currently tracked
-// validation sets that are in enforcing mode. If extraVs is not nil then it is
-// added to the returned set and replaces a validation set with same account/name
-// in case one was tracked already.
-func EnforcedValidationSets(st *state.State, extraVs *asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
+// TrackedEnforcedValidationSets returns ValidationSets object with all currently tracked
+// validation sets that are in enforcing mode. If extraVss is not nil then they are
+// added to the returned set and replaces validation sets with same account/name
+// in case they were tracked already.
+func TrackedEnforcedValidationSets(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 	valsets, err := ValidationSets(st)
 	if err != nil {
 		return nil, err
@@ -158,8 +170,10 @@ func EnforcedValidationSets(st *state.State, extraVs *asserts.ValidationSet) (*s
 	db := DB(st)
 	sets := snapasserts.NewValidationSets()
 
-	if extraVs != nil {
+	skip := make(map[string]bool, len(extraVss))
+	for _, extraVs := range extraVss {
 		sets.Add(extraVs)
+		skip[fmt.Sprintf("%s:%s", extraVs.AccountID(), extraVs.Name())] = true
 	}
 
 	for _, vs := range valsets {
@@ -169,7 +183,7 @@ func EnforcedValidationSets(st *state.State, extraVs *asserts.ValidationSet) (*s
 
 		// if extraVs matches an already enforced validation set, then skip that one, extraVs has been added
 		// before the loop.
-		if extraVs != nil && extraVs.AccountID() == vs.AccountID && extraVs.Name() == vs.Name {
+		if skip[fmt.Sprintf("%s:%s", vs.AccountID, vs.Name)] {
 			continue
 		}
 
@@ -252,7 +266,7 @@ func addToValidationSetsHistory(st *state.State, validationSets map[string]*Vali
 // the validations sets tracking history.
 func validationSetsHistoryTop(st *state.State) (map[string]*ValidationSetTracking, error) {
 	var vshist []*json.RawMessage
-	if err := st.Get("validation-sets-history", &vshist); err != nil && err != state.ErrNoState {
+	if err := st.Get("validation-sets-history", &vshist); err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, err
 	}
 	if len(vshist) == 0 {
@@ -270,7 +284,7 @@ func validationSetsHistoryTop(st *state.State) (map[string]*ValidationSetTrackin
 // ValidationSetsHistory returns the complete history of validation sets tracking.
 func ValidationSetsHistory(st *state.State) ([]map[string]*ValidationSetTracking, error) {
 	var vshist []map[string]*ValidationSetTracking
-	if err := st.Get("validation-sets-history", &vshist); err != nil && err != state.ErrNoState {
+	if err := st.Get("validation-sets-history", &vshist); err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, err
 	}
 	return vshist, nil
