@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2017 Canonical Ltd
+ * Copyright (C) 2017-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -34,7 +34,7 @@ import (
 	"github.com/snapcore/snapd/overlord/assertstate"
 	"github.com/snapcore/snapd/overlord/assertstate/assertstatetest"
 	"github.com/snapcore/snapd/overlord/configstate/configcore"
-	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/testutil"
 )
 
@@ -80,11 +80,8 @@ PATH="/usr/bin"
 }
 
 func (s *proxySuite) TestConfigureProxyUnhappy(c *C) {
-	restore := release.MockOnClassic(false)
-	defer restore()
-
 	dirs.SetRootDir(c.MkDir())
-	err := configcore.Run(&mockConf{
+	err := configcore.Run(coreDev, &mockConf{
 		state: s.state,
 		conf: map[string]interface{}{
 			"proxy.http": "http://example.com",
@@ -94,14 +91,11 @@ func (s *proxySuite) TestConfigureProxyUnhappy(c *C) {
 }
 
 func (s *proxySuite) TestConfigureProxy(c *C) {
-	restore := release.MockOnClassic(false)
-	defer restore()
-
 	for _, proto := range []string{"http", "https", "ftp"} {
 		// populate with content
 		s.makeMockEtcEnvironment(c)
 
-		err := configcore.Run(&mockConf{
+		err := configcore.Run(coreDev, &mockConf{
 			state: s.state,
 			conf: map[string]interface{}{
 				fmt.Sprintf("proxy.%s", proto): fmt.Sprintf("%s://example.com", proto),
@@ -116,12 +110,9 @@ PATH="/usr/bin"
 }
 
 func (s *proxySuite) TestConfigureNoProxy(c *C) {
-	restore := release.MockOnClassic(false)
-	defer restore()
-
 	// populate with content
 	s.makeMockEtcEnvironment(c)
-	err := configcore.Run(&mockConf{
+	err := configcore.Run(coreDev, &mockConf{
 		state: s.state,
 		conf: map[string]interface{}{
 			"proxy.no-proxy": "example.com,bar.com",
@@ -135,8 +126,31 @@ no_proxy=example.com,bar.com`)
 }
 
 func (s *proxySuite) TestConfigureProxyStore(c *C) {
+	sessionResets := 0
+	defer configcore.MockDevicestateResetSession(func(s *state.State) error {
+		s.Unlock()
+		defer s.Lock()
+		sessionResets++
+		return nil
+	})()
+
+	// no change
+	err := configcore.Run(classicDev, &mockConf{
+		state: s.state,
+	})
+	c.Check(err, IsNil)
+
+	// no related change
+	err = configcore.Run(classicDev, &mockConf{
+		state: s.state,
+		changes: map[string]interface{}{
+			"refresh.rate-limit": "1MB",
+		},
+	})
+	c.Check(err, IsNil)
+
 	// set to ""
-	err := configcore.Run(&mockConf{
+	err = configcore.Run(classicDev, &mockConf{
 		state: s.state,
 		conf: map[string]interface{}{
 			"proxy.store": "",
@@ -147,14 +161,16 @@ func (s *proxySuite) TestConfigureProxyStore(c *C) {
 	// no assertion
 	conf := &mockConf{
 		state: s.state,
-		conf: map[string]interface{}{
+		changes: map[string]interface{}{
 			"proxy.store": "foo",
 		},
 	}
 
-	err = configcore.Run(conf)
+	err = configcore.Run(classicDev, conf)
 	c.Check(err, ErrorMatches, `cannot set proxy.store to "foo" without a matching store assertion`)
 
+	c.Check(sessionResets, Equals, 0)
+
 	operatorAcct := assertstest.NewAccount(s.storeSigning, "foo-operator", nil, "")
 	// have a store assertion
 	stoAs, err := s.storeSigning.Sign(asserts.StoreType, map[string]interface{}{
@@ -170,8 +186,26 @@ func (s *proxySuite) TestConfigureProxyStore(c *C) {
 		assertstatetest.AddMany(s.state, operatorAcct, stoAs)
 	}()
 
-	err = configcore.Run(conf)
+	err = configcore.Run(classicDev, conf)
 	c.Check(err, IsNil)
+
+	c.Check(sessionResets, Equals, 1)
+
+	// no value change
+	conf = &mockConf{
+		state: s.state,
+		conf: map[string]interface{}{
+			"proxy.store": "foo",
+		},
+		changes: map[string]interface{}{
+			"proxy.store": "foo",
+		},
+	}
+
+	err = configcore.Run(classicDev, conf)
+	c.Check(err, IsNil)
+
+	c.Check(sessionResets, Equals, 1)
 }
 
 func (s *proxySuite) TestConfigureProxyStoreNoURL(c *C) {
@@ -196,6 +230,6 @@ func (s *proxySuite) TestConfigureProxyStoreNoURL(c *C) {
 		assertstatetest.AddMany(s.state, operatorAcct, stoAs)
 	}()
 
-	err = configcore.Run(conf)
+	err = configcore.Run(coreDev, conf)
 	c.Check(err, ErrorMatches, `cannot set proxy.store to "foo" with a matching store assertion with url unset`)
 }
