@@ -1,8 +1,9 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
+//go:build !nomanagers
 // +build !nomanagers
 
 /*
- * Copyright (C) 2020 Canonical Ltd
+ * Copyright (C) 2020-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -26,6 +27,7 @@ import (
 
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/sysconfig"
 )
 
 func init() {
@@ -39,7 +41,9 @@ func init() {
 	addWithStateHandler(nil, setCloudInfoWhenSeeding, nil)
 
 	// proxy.{http,https,ftp}
-	addWithStateHandler(validateProxyStore, handleProxyConfiguration, coreOnly)
+	addWithStateHandler(nil, handleProxyConfiguration, coreOnly)
+	// proxy.store
+	addWithStateHandler(validateProxyStore, handleProxyStore, nil)
 
 	// resilience.vitality-hint
 	addWithStateHandler(validateVitalitySettings, handleVitalityConfiguration, nil)
@@ -56,6 +60,9 @@ func init() {
 	addWithStateHandler(validateRefreshSchedule, nil, validateOnly)
 	addWithStateHandler(validateRefreshRateLimit, nil, validateOnly)
 	addWithStateHandler(validateAutomaticSnapshotsExpiration, nil, validateOnly)
+
+	// netplan.*
+	addWithStateHandler(validateNetplanSettings, handleNetplanConfiguration, &flags{coreOnlyConfig: true})
 }
 
 type withStateHandler struct {
@@ -72,7 +79,7 @@ func (h *withStateHandler) validate(cfg config.ConfGetter) error {
 	return nil
 }
 
-func (h *withStateHandler) handle(cfg config.ConfGetter, opts *fsOnlyContext) error {
+func (h *withStateHandler) handle(dev sysconfig.Device, cfg config.ConfGetter, opts *fsOnlyContext) error {
 	conf := cfg.(config.Conf)
 	if h.handleFunc != nil {
 		return h.handleFunc(conf, opts)
@@ -104,11 +111,11 @@ func addWithStateHandler(validate func(config.Conf) error, handle func(config.Co
 	handlers = append(handlers, h)
 }
 
-func Run(cfg config.Conf) error {
-	return applyHandlers(cfg, handlers)
+func Run(dev sysconfig.Device, cfg config.Conf) error {
+	return applyHandlers(dev, cfg, handlers)
 }
 
-func applyHandlers(cfg config.Conf, handlers []configHandler) error {
+func applyHandlers(dev sysconfig.Device, cfg config.Conf, handlers []configHandler) error {
 	// check if the changes
 	for _, k := range cfg.Changes() {
 		switch {
@@ -116,6 +123,10 @@ func applyHandlers(cfg config.Conf, handlers []configHandler) error {
 			if !validCertOption(k) {
 				return fmt.Errorf("cannot set store ssl certificate under name %q: name must only contain word characters or a dash", k)
 			}
+		case isNetplanChange(k):
+			if release.OnClassic {
+				return fmt.Errorf("cannot set netplan configuration on classic")
+			}
 		case !supportedConfigurations[k]:
 			return fmt.Errorf("cannot set %q: unsupported system option", k)
 		}
@@ -128,17 +139,17 @@ func applyHandlers(cfg config.Conf, handlers []configHandler) error {
 	}
 
 	for _, h := range handlers {
-		if h.flags().coreOnlyConfig && release.OnClassic {
+		if h.flags().coreOnlyConfig && dev.Classic() {
 			continue
 		}
-		if err := h.handle(cfg, nil); err != nil {
+		if err := h.handle(dev, cfg, nil); err != nil {
 			return err
 		}
 	}
 	return nil
 }
 
-func Early(cfg config.Conf, values map[string]interface{}) error {
+func Early(dev sysconfig.Device, cfg config.Conf, values map[string]interface{}) error {
 	early, relevant := applyFilters(func(f flags) filterFunc {
 		return f.earlyConfigFilter
 	}, values)
@@ -147,5 +158,5 @@ func Early(cfg config.Conf, values map[string]interface{}) error {
 		return err
 	}
 
-	return applyHandlers(cfg, relevant)
+	return applyHandlers(dev, cfg, relevant)
 }
