@@ -24,12 +24,12 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
-	"time"
 
 	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/configstate/config"
+	"github.com/snapcore/snapd/sysconfig"
 	"github.com/snapcore/snapd/systemd"
 )
 
@@ -66,12 +66,13 @@ func switchDisableSSHService(sysd systemd.Systemd, serviceName string, disabled
 
 	sshCanary := filepath.Join(rootDir, "/etc/ssh/sshd_not_to_be_run")
 
+	units := []string{serviceName}
 	if disabled {
 		if err := ioutil.WriteFile(sshCanary, []byte("SSH has been disabled by snapd system configuration\n"), 0644); err != nil {
 			return err
 		}
 		if opts == nil {
-			return sysd.Stop(serviceName, 5*time.Minute)
+			return sysd.Stop(units)
 		}
 	} else {
 		err := os.Remove(sshCanary)
@@ -84,7 +85,7 @@ func switchDisableSSHService(sysd systemd.Systemd, serviceName string, disabled
 		sysd.Unmask("sshd.service")
 		sysd.Unmask("ssh.service")
 		if opts == nil {
-			return sysd.Start(serviceName)
+			return sysd.Start(units)
 		}
 	}
 	return nil
@@ -110,6 +111,7 @@ func switchDisableConsoleConfService(sysd systemd.Systemd, serviceName string, d
 		// XXX: instead of this hack we should look at the config
 		//      defaults and compare with the setting and exit if
 		//      they are the same but that requires some more changes.
+		// TODO: leverage sysconfig.Device instead
 		mode, _, _ := boot.ModeAndRecoverySystemFromKernelCommandLine()
 		if mode == boot.ModeInstall {
 			return nil
@@ -156,9 +158,10 @@ func switchDisableService(serviceName string, disabled bool, opts *fsOnlyContext
 		return switchDisableConsoleConfService(sysd, serviceName, disabled, opts)
 	}
 
+	units := []string{serviceName}
 	if opts == nil {
 		// ignore the service if not installed
-		status, err := sysd.Status(serviceName)
+		status, err := sysd.Status(units)
 		if err != nil {
 			return err
 		}
@@ -173,34 +176,39 @@ func switchDisableService(serviceName string, disabled bool, opts *fsOnlyContext
 
 	if disabled {
 		if opts == nil {
-			if err := sysd.Disable(serviceName); err != nil {
+			if err := sysd.DisableNoReload(units); err != nil {
 				return err
 			}
 		}
 		if err := sysd.Mask(serviceName); err != nil {
 			return err
 		}
+		// mask triggered a reload already
 		if opts == nil {
-			return sysd.Stop(serviceName, 5*time.Minute)
+			return sysd.Stop(units)
 		}
 	} else {
 		if err := sysd.Unmask(serviceName); err != nil {
 			return err
 		}
 		if opts == nil {
-			if err := sysd.Enable(serviceName); err != nil {
+			if err := sysd.EnableNoReload(units); err != nil {
+				return err
+			}
+			// enable does not trigger reloads, so issue one now
+			if err := sysd.DaemonReload(); err != nil {
 				return err
 			}
 		}
 		if opts == nil {
-			return sysd.Start(serviceName)
+			return sysd.Start(units)
 		}
 	}
 	return nil
 }
 
 // services that can be disabled
-func handleServiceDisableConfiguration(tr config.ConfGetter, opts *fsOnlyContext) error {
+func handleServiceDisableConfiguration(_ sysconfig.Device, tr config.ConfGetter, opts *fsOnlyContext) error {
 	for _, service := range services {
 		optionName := fmt.Sprintf("service.%s.disable", service.configName)
 		outputStr, err := coreCfg(tr, optionName)
