@@ -1,4 +1,5 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
+//go:build !nomanagers
 // +build !nomanagers
 
 /*
@@ -21,6 +22,7 @@
 package configcore
 
 import (
+	"errors"
 	"fmt"
 	"strings"
 
@@ -85,7 +87,7 @@ func handleVitalityConfiguration(tr config.Conf, opts *fsOnlyContext) error {
 		err := snapstate.Get(st, instanceName, &snapst)
 		// not installed, vitality-score will be applied when the snap
 		// gets installed
-		if err == state.ErrNoState {
+		if errors.Is(err, state.ErrNoState) {
 			continue
 		}
 		if err != nil {
@@ -110,6 +112,7 @@ func handleVitalityConfiguration(tr config.Conf, opts *fsOnlyContext) error {
 
 		// first get the device context to decide if we need to set
 		// RequireMountedSnapdSnap
+		// TODO: use sysconfig.Device instead
 		deviceCtx, err := snapstate.DeviceCtx(st, nil, nil)
 		if err != nil {
 			return err
