@@ -21,6 +21,7 @@ package configstate
 
 import (
 	"github.com/snapcore/snapd/overlord/configstate/config"
+	"github.com/snapcore/snapd/sysconfig"
 )
 
 var NewConfigureHandler = newConfigureHandler
@@ -33,7 +34,7 @@ func MockConfigcoreExportExperimentalFlags(mock func(tr config.ConfGetter) error
 	}
 }
 
-func MockConfigcoreEarly(mock func(cfg config.Conf, values map[string]interface{}) error) (restore func()) {
+func MockConfigcoreEarly(mock func(dev sysconfig.Device, cfg config.Conf, values map[string]interface{}) error) (restore func()) {
 	old := configcoreEarly
 	configcoreEarly = mock
 	return func() {
