@@ -99,13 +99,44 @@ func (s *deviceMgrBootconfigSuite) setupUC20Model(c *C) *asserts.Model {
 	return s.makeModelAssertionInState(c, "canonical", "pc-model-20", mockCore20ModelHeaders)
 }
 
+func (s *deviceMgrBootconfigSuite) setupClassicWithModesModel(c *C) *asserts.Model {
+	devicestatetest.SetDevice(s.state, &auth.DeviceState{
+		Brand:  "canonical",
+		Model:  "classic-with-modes",
+		Serial: "didididi",
+	})
+	return s.makeModelAssertionInState(c, "canonical", "classic-with-modes",
+		map[string]interface{}{
+			"architecture": "amd64",
+			"classic":      "true",
+			"distribution": "ubuntu",
+			"base":         "core22",
+			"snaps": []interface{}{
+				map[string]interface{}{
+					"name": "pc-linux",
+					"id":   "pclinuxdidididididididididididid",
+					"type": "kernel",
+				},
+				map[string]interface{}{
+					"name": "pc",
+					"id":   "pcididididididididididididididid",
+					"type": "gadget",
+				},
+			},
+		})
+}
+
 func (s *deviceMgrBootconfigSuite) testBootConfigUpdateRun(c *C, updateAttempted, applied bool, errMatch string) {
 	restore := release.MockOnClassic(false)
 	defer restore()
 
 	s.state.Lock()
 	tsk := s.state.NewTask("update-managed-boot-config", "update boot config")
-	chg := s.state.NewChange("dummy", "...")
+	tsk.Set("snap-setup", &snapstate.SnapSetup{
+		SideInfo: &s.gadgetSnapInfo.SideInfo,
+		Type:     snap.TypeGadget,
+	})
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(tsk)
 	chg.Set("system-restart-immediate", true)
 	s.state.Unlock()
@@ -141,6 +172,51 @@ func (s *deviceMgrBootconfigSuite) testBootConfigUpdateRun(c *C, updateAttempted
 	}
 }
 
+func (s *deviceMgrBootconfigSuite) testBootConfigUpdateRunClassic(c *C, updateAttempted, applied bool, errMatch string) {
+	restore := release.MockOnClassic(true)
+	defer restore()
+
+	s.state.Lock()
+	tsk := s.state.NewTask("update-managed-boot-config", "update boot config")
+	tsk.Set("snap-setup", &snapstate.SnapSetup{
+		SideInfo: &s.gadgetSnapInfo.SideInfo,
+		Type:     snap.TypeGadget,
+	})
+	chg := s.state.NewChange("sample", "...")
+	chg.AddTask(tsk)
+	chg.Set("system-restart-immediate", true)
+	s.state.Unlock()
+
+	s.settle(c)
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	if errMatch == "" {
+		c.Assert(chg.IsReady(), Equals, false)
+		c.Check(chg.Err(), IsNil)
+		c.Check(tsk.Status(), Equals, state.WaitStatus)
+	} else {
+		c.Assert(chg.IsReady(), Equals, true)
+		c.Check(chg.Err(), ErrorMatches, errMatch)
+		c.Check(tsk.Status(), Equals, state.ErrorStatus)
+	}
+	if updateAttempted {
+		c.Assert(s.managedbl.UpdateCalls, Equals, 1)
+		if errMatch == "" && applied {
+			// we log on success
+			log := tsk.Log()
+			c.Assert(log, HasLen, 2)
+			c.Check(log[0], Matches, ".* updated boot config assets")
+			c.Check(log[1], Matches, ".* Task set to wait until a manual system restart allows to continue")
+		}
+		// There must be no restart request
+		c.Check(s.restartRequests, HasLen, 0)
+	} else {
+		c.Assert(s.managedbl.UpdateCalls, Equals, 0)
+	}
+}
+
 func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunSuccess(c *C) {
 	s.state.Lock()
 	s.setupUC20Model(c)
@@ -160,6 +236,25 @@ func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunSuccess(c *C) {
 	})
 }
 
+func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunSuccessClassicWithModes(c *C) {
+	s.state.Lock()
+	s.setupClassicWithModesModel(c)
+	s.state.Unlock()
+
+	s.managedbl.Updated = true
+
+	updateAttempted := true
+	updateApplied := true
+	s.testBootConfigUpdateRunClassic(c, updateAttempted, updateApplied, "")
+
+	m, err := boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	c.Check([]string(m.CurrentKernelCommandLines), DeepEquals, []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1",
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 candidate",
+	})
+}
+
 func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateWithGadgetExtra(c *C) {
 	s.state.Lock()
 	s.setupUC20Model(c)
@@ -267,7 +362,7 @@ func (s *deviceMgrBootconfigSuite) TestBootConfigRemodelDoNothing(c *C) {
 	// be extra sure
 	c.Check(remodCtx.ForRemodeling(), Equals, true)
 	tsk := s.state.NewTask("update-managed-boot-config", "update boot config")
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(tsk)
 	remodCtx.Init(chg)
 	// replace the bootloader with something that always fails
