@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2020 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -29,6 +29,7 @@ import (
 	"strings"
 
 	. "gopkg.in/check.v1"
+	"gopkg.in/tomb.v2"
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/boot"
@@ -36,6 +37,7 @@ import (
 	"github.com/snapcore/snapd/bootloader/bootloadertest"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
+	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/devicestate"
@@ -207,7 +209,34 @@ func (s *deviceMgrGadgetSuite) setupUC20ModelWithGadget(c *C, gadget string) {
 	})
 }
 
-func (s *deviceMgrGadgetSuite) setupGadgetUpdate(c *C, modelGrade, gadgetYamlContent, gadgetYamlContentNext string) (chg *state.Change, tsk *state.Task) {
+func (s *deviceMgrGadgetSuite) setupClassicWithModesModel(c *C, gadget string) *asserts.Model {
+	devicestatetest.SetDevice(s.state, &auth.DeviceState{
+		Brand:  "canonical",
+		Model:  "classic-with-modes",
+		Serial: "didididi",
+	})
+	return s.makeModelAssertionInState(c, "canonical", "classic-with-modes",
+		map[string]interface{}{
+			"architecture": "amd64",
+			"classic":      "true",
+			"distribution": "ubuntu",
+			"base":         "core22",
+			"snaps": []interface{}{
+				map[string]interface{}{
+					"name": "pc-linux",
+					"id":   "pclinuxdidididididididididididid",
+					"type": "kernel",
+				},
+				map[string]interface{}{
+					"name": gadget,
+					"id":   "pcididididididididididididididid",
+					"type": "gadget",
+				},
+			},
+		})
+}
+
+func (s *deviceMgrGadgetSuite) setupGadgetUpdate(c *C, modelGrade, gadgetYamlContent, gadgetYamlContentNext string, isClassic bool) (chg *state.Change, tsk *state.Task) {
 	siCurrent := &snap.SideInfo{
 		RealName: "foo-gadget",
 		Revision: snap.R(33),
@@ -236,7 +265,9 @@ func (s *deviceMgrGadgetSuite) setupGadgetUpdate(c *C, modelGrade, gadgetYamlCon
 	s.state.Lock()
 	defer s.state.Unlock()
 
-	if modelGrade == "" {
+	if isClassic {
+		s.setupClassicWithModesModel(c, "foo-gadget")
+	} else if modelGrade == "" {
 		s.setupModelWithGadget(c, "foo-gadget")
 	} else {
 		s.setupUC20ModelWithGadget(c, "foo-gadget")
@@ -254,13 +285,13 @@ func (s *deviceMgrGadgetSuite) setupGadgetUpdate(c *C, modelGrade, gadgetYamlCon
 		SideInfo: si,
 		Type:     snap.TypeGadget,
 	})
-	chg = s.state.NewChange("dummy", "...")
+	chg = s.state.NewChange("sample", "...")
 	chg.AddTask(tsk)
 
 	return chg, tsk
 }
 
-func (s *deviceMgrGadgetSuite) testUpdateGadgetOnCoreSimple(c *C, grade string, encryption, immediate bool, gadgetYamlCont, gadgetYamlContNext string) {
+func (s *deviceMgrGadgetSuite) testUpdateGadgetSimple(c *C, grade string, encryption, immediate bool, gadgetYamlCont, gadgetYamlContNext string, isClassic bool) {
 	var updateCalled bool
 	var passedRollbackDir string
 
@@ -325,7 +356,7 @@ func (s *deviceMgrGadgetSuite) testUpdateGadgetOnCoreSimple(c *C, grade string,
 	})
 	defer restore()
 
-	chg, t := s.setupGadgetUpdate(c, grade, gadgetYamlCont, gadgetYamlContNext)
+	chg, t := s.setupGadgetUpdate(c, grade, gadgetYamlCont, gadgetYamlContNext, isClassic)
 
 	// procure modeenv and stamp that we sealed keys
 	if grade != "" {
@@ -360,40 +391,62 @@ func (s *deviceMgrGadgetSuite) testUpdateGadgetOnCoreSimple(c *C, grade string,
 
 	s.state.Lock()
 	defer s.state.Unlock()
-	c.Assert(chg.IsReady(), Equals, true)
 	c.Check(chg.Err(), IsNil)
-	c.Check(t.Status(), Equals, state.DoneStatus)
+	if isClassic {
+		c.Assert(chg.IsReady(), Equals, false)
+		c.Check(t.Status(), Equals, state.WaitStatus)
+	} else {
+		c.Assert(chg.IsReady(), Equals, true)
+		c.Check(t.Status(), Equals, state.DoneStatus)
+	}
 	c.Check(updateCalled, Equals, true)
 	rollbackDir := filepath.Join(dirs.SnapRollbackDir, "foo-gadget_34")
 	c.Check(rollbackDir, Equals, passedRollbackDir)
 	// should have been removed right after update
 	c.Check(osutil.IsDirectory(rollbackDir), Equals, false)
-	c.Check(s.restartRequests, DeepEquals, []restart.RestartType{expectedRst})
+	if isClassic {
+		c.Check(s.restartRequests, HasLen, 0)
+	} else {
+		c.Check(s.restartRequests, DeepEquals, []restart.RestartType{expectedRst})
+	}
 }
 
 func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreSimple(c *C) {
 	// unset grade
 	encryption := false
 	immediate := false
-	s.testUpdateGadgetOnCoreSimple(c, "", encryption, immediate, gadgetYaml, "")
+	isClassic := false
+	s.testUpdateGadgetSimple(c, "", encryption, immediate, gadgetYaml, "", isClassic)
+}
+
+func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnClassicWithModesSimple(c *C) {
+	r := release.MockOnClassic(true)
+	defer r()
+	encryption := false
+	immediate := false
+	isClassic := true
+	s.testUpdateGadgetSimple(c, "dangerous", encryption, immediate, gadgetYaml, "", isClassic)
 }
 
 func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnUC20CoreSimpleWithEncryption(c *C) {
 	encryption := true
 	immediate := false
-	s.testUpdateGadgetOnCoreSimple(c, "dangerous", encryption, immediate, uc20gadgetYaml, "")
+	isClassic := false
+	s.testUpdateGadgetSimple(c, "dangerous", encryption, immediate, uc20gadgetYaml, "", isClassic)
 }
 
 func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnUC20CoreSimpleNoEncryption(c *C) {
 	encryption := false
 	immediate := false
-	s.testUpdateGadgetOnCoreSimple(c, "dangerous", encryption, immediate, uc20gadgetYaml, "")
+	isClassic := false
+	s.testUpdateGadgetSimple(c, "dangerous", encryption, immediate, uc20gadgetYaml, "", isClassic)
 }
 
 func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnUC20CoreSimpleSystemRestartImmediate(c *C) {
 	encryption := false
 	immediate := true
-	s.testUpdateGadgetOnCoreSimple(c, "dangerous", encryption, immediate, uc20gadgetYaml, "")
+	isClassic := false
+	s.testUpdateGadgetSimple(c, "dangerous", encryption, immediate, uc20gadgetYaml, "", isClassic)
 }
 
 func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreNoUpdateNeeded(c *C) {
@@ -404,7 +457,8 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreNoUpdateNeeded(c *C) {
 	})
 	defer restore()
 
-	chg, t := s.setupGadgetUpdate(c, "", gadgetYaml, "")
+	isClassic := false
+	chg, t := s.setupGadgetUpdate(c, "", gadgetYaml, "", isClassic)
 
 	s.se.Ensure()
 	s.se.Wait()
@@ -430,7 +484,8 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreRollbackDirCreateFailed(c *
 	})
 	defer restore()
 
-	chg, t := s.setupGadgetUpdate(c, "", gadgetYaml, "")
+	isClassic := false
+	chg, t := s.setupGadgetUpdate(c, "", gadgetYaml, "", isClassic)
 
 	rollbackDir := filepath.Join(dirs.SnapRollbackDir, "foo-gadget_34")
 	err := os.MkdirAll(dirs.SnapRollbackDir, 0000)
@@ -456,7 +511,8 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreUpdateFailed(c *C) {
 		return errors.New("gadget exploded")
 	})
 	defer restore()
-	chg, t := s.setupGadgetUpdate(c, "", gadgetYaml, "")
+	isClassic := false
+	chg, t := s.setupGadgetUpdate(c, "", gadgetYaml, "", isClassic)
 
 	s.state.Lock()
 	s.state.Set("seeded", true)
@@ -502,7 +558,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreNotDuringFirstboot(c *C) {
 		SideInfo: si,
 		Type:     snap.TypeGadget,
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 
 	s.state.Unlock()
@@ -559,7 +615,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreBadGadgetYaml(c *C) {
 		SideInfo: si,
 		Type:     snap.TypeGadget,
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 
 	s.state.Unlock()
@@ -610,7 +666,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreParanoidChecks(c *C) {
 		SideInfo: si,
 		Type:     snap.TypeGadget,
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 
 	s.state.Unlock()
@@ -625,34 +681,6 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreParanoidChecks(c *C) {
 	c.Check(s.restartRequests, HasLen, 0)
 }
 
-func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnClassicErrorsOut(c *C) {
-	restore := release.MockOnClassic(true)
-	defer restore()
-
-	restore = devicestate.MockGadgetUpdate(func(model gadget.Model, current, update gadget.GadgetData, path string, policy gadget.UpdatePolicyFunc, _ gadget.ContentUpdateObserver) error {
-		return errors.New("unexpected call")
-	})
-	defer restore()
-
-	s.state.Lock()
-
-	s.state.Set("seeded", true)
-
-	t := s.state.NewTask("update-gadget-assets", "update gadget")
-	chg := s.state.NewChange("dummy", "...")
-	chg.AddTask(t)
-
-	s.state.Unlock()
-
-	s.settle(c)
-
-	s.state.Lock()
-	defer s.state.Unlock()
-	c.Assert(chg.IsReady(), Equals, true)
-	c.Check(chg.Err(), ErrorMatches, `(?s).*update gadget \(cannot run update gadget assets task on a classic system\).*`)
-	c.Check(t.Status(), Equals, state.ErrorStatus)
-}
-
 type mockUpdater struct{}
 
 func (m *mockUpdater) Backup() error { return nil }
@@ -705,11 +733,29 @@ volumes:
 		{"content.img", "updated content"},
 	})
 
+	r := gadget.MockVolumeStructureToLocationMap(func(_ gadget.GadgetData, _ gadget.Model, _ map[string]*gadget.LaidOutVolume) (map[string]map[int]gadget.StructureLocation, error) {
+		return map[string]map[int]gadget.StructureLocation{
+			"pc": {
+				0: {
+					Device: "/dev/foo",
+					Offset: quantity.OffsetMiB,
+				},
+			},
+		}, nil
+	})
+	defer r()
+
 	expectedRollbackDir := filepath.Join(dirs.SnapRollbackDir, "foo-gadget_34")
 	updaterForStructureCalls := 0
-	restore := gadget.MockUpdaterForStructure(func(ps *gadget.LaidOutStructure, rootDir, rollbackDir string, _ gadget.ContentUpdateObserver) (gadget.Updater, error) {
+	restore := gadget.MockUpdaterForStructure(func(loc gadget.StructureLocation, ps *gadget.LaidOutStructure, rootDir, rollbackDir string, _ gadget.ContentUpdateObserver) (gadget.Updater, error) {
 		updaterForStructureCalls++
 
+		c.Assert(loc, Equals, gadget.StructureLocation{
+			Device:         "/dev/foo",
+			Offset:         quantity.OffsetMiB,
+			RootMountPoint: "",
+		})
+
 		c.Assert(ps.Name, Equals, "foo")
 		c.Assert(rootDir, Equals, updateInfo.MountDir())
 		c.Assert(filepath.Join(rootDir, "content.img"), testutil.FileEquals, "updated content")
@@ -736,7 +782,7 @@ volumes:
 		SideInfo: si,
 		Type:     snap.TypeGadget,
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 
 	s.state.Unlock()
@@ -927,7 +973,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreHybridFirstboot(c *C) {
 		SideInfo: si,
 		Type:     snap.TypeGadget,
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(t)
 
 	s.state.Unlock()
@@ -947,7 +993,8 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreHybridFirstboot(c *C) {
 func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreHybridShouldWork(c *C) {
 	encryption := false
 	immediate := false
-	s.testUpdateGadgetOnCoreSimple(c, "", encryption, immediate, hybridGadgetYaml, "")
+	isClassic := false
+	s.testUpdateGadgetSimple(c, "", encryption, immediate, hybridGadgetYaml, "", isClassic)
 }
 
 func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreOldIsInvalidNowButShouldWork(c *C) {
@@ -958,7 +1005,8 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreOldIsInvalidNowButShouldWor
 	hybridGadgetYamlBroken := hybridGadgetYaml + `
         role: system-boot
 `
-	s.testUpdateGadgetOnCoreSimple(c, "", encryption, immediate, hybridGadgetYamlBroken, hybridGadgetYaml)
+	isClassic := false
+	s.testUpdateGadgetSimple(c, "", encryption, immediate, hybridGadgetYamlBroken, hybridGadgetYaml, isClassic)
 }
 
 func (s *deviceMgrGadgetSuite) makeMinimalKernelAssetsUpdateChange(c *C) (chg *state.Change, tsk *state.Task) {
@@ -1016,7 +1064,7 @@ volumes:
 		SideInfo: siNext,
 		Type:     snap.TypeKernel,
 	})
-	chg = s.state.NewChange("dummy", "...")
+	chg = s.state.NewChange("sample", "...")
 	chg.AddTask(tsk)
 
 	return chg, tsk
@@ -1108,8 +1156,8 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetOnCoreFromKernelRemodel(c *C) {
 	c.Check(rollbackDir, Equals, passedRollbackDir)
 }
 
-func (s *deviceMgrGadgetSuite) testGadgetCommandlineUpdateRun(c *C, fromFiles, toFiles [][]string, errMatch, logMatch string, updated bool) {
-	restore := release.MockOnClassic(false)
+func (s *deviceMgrGadgetSuite) testGadgetCommandlineUpdateRun(c *C, fromFiles, toFiles [][]string, errMatch, logMatch string, updated, isClassic bool) {
+	restore := release.MockOnClassic(isClassic)
 	defer restore()
 
 	s.state.Lock()
@@ -1135,7 +1183,7 @@ func (s *deviceMgrGadgetSuite) testGadgetCommandlineUpdateRun(c *C, fromFiles, t
 		SideInfo: &updateSi,
 		Type:     snap.TypeGadget,
 	})
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(tsk)
 	s.state.Unlock()
 
@@ -1144,19 +1192,33 @@ func (s *deviceMgrGadgetSuite) testGadgetCommandlineUpdateRun(c *C, fromFiles, t
 	s.state.Lock()
 	defer s.state.Unlock()
 
-	c.Assert(chg.IsReady(), Equals, true)
 	if errMatch == "" {
 		c.Check(chg.Err(), IsNil)
-		c.Check(tsk.Status(), Equals, state.DoneStatus)
+		if isClassic {
+			c.Assert(chg.IsReady(), Equals, false)
+			c.Check(tsk.Status(), Equals, state.WaitStatus)
+		} else {
+			c.Assert(chg.IsReady(), Equals, true)
+			c.Check(tsk.Status(), Equals, state.DoneStatus)
+		}
 		// we log on success
 		log := tsk.Log()
 		if logMatch != "" {
-			c.Assert(log, HasLen, 1)
+			if !isClassic {
+				c.Assert(log, HasLen, 1)
+			} else {
+				c.Assert(log, HasLen, 2)
+			}
 			c.Check(log[0], Matches, fmt.Sprintf(".* %v", logMatch))
+			if isClassic {
+				c.Check(log[1], Matches, ".* Task set to wait until a manual system restart allows to continue")
+			}
 		} else {
 			c.Check(log, HasLen, 0)
 		}
-		if updated {
+		if isClassic {
+			c.Check(s.restartRequests, HasLen, 0)
+		} else if updated {
 			// update was applied, thus a restart was requested
 			c.Check(s.restartRequests, DeepEquals, []restart.RestartType{restart.RestartSystem})
 		} else {
@@ -1164,6 +1226,7 @@ func (s *deviceMgrGadgetSuite) testGadgetCommandlineUpdateRun(c *C, fromFiles, t
 			c.Check(s.restartRequests, HasLen, 0)
 		}
 	} else {
+		c.Assert(chg.IsReady(), Equals, true)
 		c.Check(chg.Err(), ErrorMatches, errMatch)
 		c.Check(tsk.Status(), Equals, state.ErrorStatus)
 	}
@@ -1195,6 +1258,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineWithExistingArgs(c *C)
 	s.state.Unlock()
 
 	const update = true
+	const isClassic = false
 	s.testGadgetCommandlineUpdateRun(c,
 		[][]string{
 			{"meta/gadget.yaml", gadgetYaml},
@@ -1204,7 +1268,61 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineWithExistingArgs(c *C)
 			{"meta/gadget.yaml", gadgetYaml},
 			{"cmdline.extra", "args from updated gadget"},
 		},
-		"", "Updated kernel command line", update)
+		"", "Updated kernel command line", update, isClassic)
+
+	m, err = boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	c.Check([]string(m.CurrentKernelCommandLines), DeepEquals, []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 args from old gadget",
+		// gadget arguments are picked up for the candidate command line
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 args from updated gadget",
+	})
+	c.Check(s.managedbl.SetBootVarsCalls, Equals, 1)
+	vars, err := s.managedbl.GetBootVars("snapd_extra_cmdline_args")
+	c.Assert(err, IsNil)
+	// bootenv was cleared
+	c.Assert(vars, DeepEquals, map[string]string{
+		"snapd_extra_cmdline_args": "args from updated gadget",
+	})
+}
+
+func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineClassicWithModesWithExistingArgs(c *C) {
+	// arguments change
+	bootloader.Force(s.managedbl)
+	s.state.Lock()
+	s.setupClassicWithModesModel(c, "pc")
+	s.mockModeenvForMode(c, "run")
+	devicestate.SetBootOkRan(s.mgr, true)
+	s.state.Set("seeded", true)
+
+	// update the modeenv to have the gadget arguments included to mimic the
+	// state we would have in the system
+	m, err := boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	m.CurrentKernelCommandLines = []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 args from old gadget",
+	}
+	c.Assert(m.Write(), IsNil)
+	err = s.managedbl.SetBootVars(map[string]string{
+		"snapd_extra_cmdline_args": "args from old gadget",
+	})
+	c.Assert(err, IsNil)
+	s.managedbl.SetBootVarsCalls = 0
+
+	s.state.Unlock()
+
+	const update = true
+	const isClassic = true
+	s.testGadgetCommandlineUpdateRun(c,
+		[][]string{
+			{"meta/gadget.yaml", gadgetYaml},
+			{"cmdline.extra", "args from old gadget"},
+		},
+		[][]string{
+			{"meta/gadget.yaml", gadgetYaml},
+			{"cmdline.extra", "args from updated gadget"},
+		},
+		"", "Updated kernel command line", update, isClassic)
 
 	m, err = boot.ReadModeenv("")
 	c.Assert(err, IsNil)
@@ -1247,6 +1365,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineWithNewArgs(c *C) {
 	s.state.Unlock()
 
 	const update = true
+	const isClassic = false
 	s.testGadgetCommandlineUpdateRun(c,
 		[][]string{
 			{"meta/gadget.yaml", gadgetYaml},
@@ -1256,7 +1375,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineWithNewArgs(c *C) {
 			{"meta/gadget.yaml", gadgetYaml},
 			{"cmdline.extra", "args from new gadget"},
 		},
-		"", "Updated kernel command line", update)
+		"", "Updated kernel command line", update, isClassic)
 
 	m, err = boot.ReadModeenv("")
 	c.Assert(err, IsNil)
@@ -1299,6 +1418,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineDroppedArgs(c *C) {
 	s.state.Unlock()
 
 	const update = true
+	const isClassic = false
 	s.testGadgetCommandlineUpdateRun(c,
 		[][]string{
 			{"meta/gadget.yaml", gadgetYaml},
@@ -1309,7 +1429,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineDroppedArgs(c *C) {
 			{"meta/gadget.yaml", gadgetYaml},
 			// new one does not
 		},
-		"", "Updated kernel command line", update)
+		"", "Updated kernel command line", update, isClassic)
 
 	m, err = boot.ReadModeenv("")
 	c.Assert(err, IsNil)
@@ -1357,8 +1477,9 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineUnchanged(c *C) {
 	}
 	// old and new gadget have the same command line arguments, nothing changes
 	const update = false
+	const isClassic = false
 	s.testGadgetCommandlineUpdateRun(c, sameFiles, sameFiles,
-		"", "", update)
+		"", "", update, isClassic)
 
 	m, err = boot.ReadModeenv("")
 	c.Assert(err, IsNil)
@@ -1379,6 +1500,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineNonUC20(c *C) {
 
 	s.state.Unlock()
 	const update = false
+	const isClassic = false
 	s.testGadgetCommandlineUpdateRun(c,
 		[][]string{
 			{"meta/gadget.yaml", gadgetYaml},
@@ -1388,7 +1510,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineNonUC20(c *C) {
 			{"meta/gadget.yaml", gadgetYaml},
 			{"cmdline.extra", "args from new gadget"},
 		},
-		"", "", update)
+		"", "", update, isClassic)
 }
 
 func (s *deviceMgrGadgetSuite) TestGadgetCommandlineUpdateUndo(c *C) {
@@ -1445,7 +1567,7 @@ func (s *deviceMgrGadgetSuite) TestGadgetCommandlineUpdateUndo(c *C) {
 	})
 	terr := s.state.NewTask("error-trigger", "provoking total undo")
 	terr.WaitFor(tsk)
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(tsk)
 	chg.AddTask(terr)
 	chg.Set("system-restart-immediate", true)
@@ -1501,6 +1623,129 @@ func (s *deviceMgrGadgetSuite) TestGadgetCommandlineUpdateUndo(c *C) {
 	c.Check(s.managedbl.SetBootVarsCalls, Equals, 2)
 }
 
+func (s *deviceMgrGadgetSuite) TestGadgetCommandlineClassicWithModesUpdateUndo(c *C) {
+	restore := release.MockOnClassic(true)
+	defer restore()
+
+	bootloader.Force(s.managedbl)
+	s.state.Lock()
+	s.setupClassicWithModesModel(c, "pc")
+	s.mockModeenvForMode(c, "run")
+	devicestate.SetBootOkRan(s.mgr, true)
+	s.state.Set("seeded", true)
+
+	// mimic system state
+	m, err := boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	m.CurrentKernelCommandLines = []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 args from old gadget",
+	}
+	c.Assert(m.Write(), IsNil)
+
+	err = s.managedbl.SetBootVars(map[string]string{
+		"snapd_extra_cmdline_args": "args from old gadget",
+	})
+	c.Assert(err, IsNil)
+	s.managedbl.SetBootVarsCalls = 0
+
+	currentSi := &snap.SideInfo{
+		RealName: "pc",
+		Revision: snap.R(33),
+		SnapID:   "foo-id",
+	}
+	snapstate.Set(s.state, "pc", &snapstate.SnapState{
+		SnapType: "gadget",
+		Sequence: []*snap.SideInfo{currentSi},
+		Current:  currentSi.Revision,
+		Active:   true,
+	})
+	snaptest.MockSnapWithFiles(c, pcGadgetSnapYaml, currentSi, [][]string{
+		{"meta/gadget.yaml", gadgetYaml},
+		{"cmdline.extra", "args from old gadget"},
+	})
+	updateSi := *currentSi
+	updateSi.Revision = snap.R(34)
+	snaptest.MockSnapWithFiles(c, pcGadgetSnapYaml, &updateSi, [][]string{
+		{"meta/gadget.yaml", gadgetYaml},
+		{"cmdline.extra", "args from new gadget"},
+	})
+
+	erroringHandler := func(task *state.Task, _ *tomb.Tomb) error {
+		// We simulate the modeenv we would have after a reboot
+		m, err := boot.ReadModeenv("")
+		c.Assert(err, IsNil)
+		m.CurrentKernelCommandLines = []string{"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 args from new gadget"}
+		c.Assert(m.Write(), IsNil)
+		return errors.New("error out")
+	}
+	// FIXME the handler will be around for other tests, not sure if there
+	// is a way to remove it.
+	s.o.TaskRunner().AddHandler("error-save-mode-trigger", erroringHandler, nil)
+
+	tsk := s.state.NewTask("update-gadget-cmdline", "update gadget command line")
+	tsk.Set("snap-setup", &snapstate.SnapSetup{
+		SideInfo: &updateSi,
+		Type:     snap.TypeGadget,
+	})
+	terr := s.state.NewTask("error-save-mode-trigger", "provoking total undo")
+	terr.WaitFor(tsk)
+	chg := s.state.NewChange("sample", "...")
+	chg.AddTask(tsk)
+	chg.AddTask(terr)
+	chg.Set("system-restart-immediate", true)
+	s.state.Unlock()
+
+	restartCount := 0
+	s.restartObserve = func() {
+		// should not be called for classic with modes
+		restartCount++
+	}
+
+	s.settle(c)
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	// after manual reboot
+	c.Check(tsk.Status(), Equals, state.WaitStatus)
+	log := tsk.Log()
+	c.Assert(log, HasLen, 2)
+	c.Check(log[0], Matches, ".* Updated kernel command line")
+	c.Check(log[1], Matches, ".* Task set to wait until a manual system restart allows to continue")
+	c.Check(s.restartRequests, HasLen, 0)
+	c.Check(restartCount, Equals, 0)
+
+	// simulate restart
+	tsk.SetStatus(state.DoneStatus)
+
+	s.state.Unlock()
+	defer s.state.Lock()
+
+	s.settle(c)
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	// after manual reboot
+	c.Assert(chg.IsReady(), Equals, true)
+	c.Check(chg.Err(), ErrorMatches, "(?s)cannot perform the following tasks.*total undo.*")
+	c.Check(tsk.Status(), Equals, state.UndoneStatus)
+	log = tsk.Log()
+	c.Assert(log, HasLen, 4)
+	c.Check(log[2], Matches, ".* Reverted kernel command line change")
+	c.Check(log[3], Matches, ".* Skipped automatic system restart on classic system when undoing changes back to previous state")
+	// update was applied and then undone, but no automatic restarts happened
+	c.Check(s.restartRequests, HasLen, 0)
+	c.Check(restartCount, Equals, 0)
+	vars, err := s.managedbl.GetBootVars("snapd_extra_cmdline_args")
+	c.Assert(err, IsNil)
+	c.Assert(vars, DeepEquals, map[string]string{
+		"snapd_extra_cmdline_args": "args from old gadget",
+	})
+	// 2 calls, one to set the new arguments, and one to reset them back
+	c.Check(s.managedbl.SetBootVarsCalls, Equals, 2)
+}
+
 func (s *deviceMgrGadgetSuite) TestGadgetCommandlineUpdateNoChangeNoRebootsUndo(c *C) {
 	restore := release.MockOnClassic(false)
 	defer restore()
@@ -1554,7 +1799,7 @@ func (s *deviceMgrGadgetSuite) TestGadgetCommandlineUpdateNoChangeNoRebootsUndo(
 	})
 	terr := s.state.NewTask("error-trigger", "provoking total undo")
 	terr.WaitFor(tsk)
-	chg := s.state.NewChange("dummy", "...")
+	chg := s.state.NewChange("sample", "...")
 	chg.AddTask(tsk)
 	chg.AddTask(terr)
 	s.state.Unlock()
@@ -1603,6 +1848,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineWithFullArgs(c *C) {
 	s.state.Unlock()
 
 	const update = true
+	const isClassic = false
 	s.testGadgetCommandlineUpdateRun(c,
 		[][]string{
 			{"meta/gadget.yaml", gadgetYaml},
@@ -1612,7 +1858,7 @@ func (s *deviceMgrGadgetSuite) TestUpdateGadgetCommandlineWithFullArgs(c *C) {
 			{"meta/gadget.yaml", gadgetYaml},
 			{"cmdline.full", "full args"},
 		},
-		"", "Updated kernel command line", update)
+		"", "Updated kernel command line", update, isClassic)
 
 	m, err = boot.ReadModeenv("")
 	c.Assert(err, IsNil)
