@@ -19,39 +19,40 @@
 package devicestate
 
 import (
+	"errors"
 	"fmt"
 
 	"gopkg.in/tomb.v2"
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/boot"
+	"github.com/snapcore/snapd/overlord/restart"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
-	"github.com/snapcore/snapd/release"
 )
 
 func (m *DeviceManager) doUpdateManagedBootConfig(t *state.Task, _ *tomb.Tomb) error {
-	if release.OnClassic {
-		return fmt.Errorf("cannot run update boot config task on a classic system")
-	}
-
 	st := t.State()
 	st.Lock()
 	defer st.Unlock()
 
+	devCtx, err := DeviceCtx(st, t, nil)
+	if err != nil {
+		return err
+	}
+	if devCtx.IsClassicBoot() {
+		return fmt.Errorf("cannot run update boot config task on a classic system")
+	}
+
 	var seeded bool
-	err := st.Get("seeded", &seeded)
-	if err != nil && err != state.ErrNoState {
+	err = st.Get("seeded", &seeded)
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	if !seeded {
 		// do nothing during first boot & seeding
 		return nil
 	}
-	devCtx, err := DeviceCtx(st, t, nil)
-	if err != nil {
-		return err
-	}
 
 	if devCtx.Model().Grade() == asserts.ModelGradeUnset {
 		// pre UC20 system, do nothing
@@ -77,15 +78,17 @@ func (m *DeviceManager) doUpdateManagedBootConfig(t *state.Task, _ *tomb.Tomb) e
 	if err != nil {
 		return fmt.Errorf("cannot update boot config assets: %v", err)
 	}
+
+	// set this status already before returning to minimize wasteful redos
+	finalStatus := state.DoneStatus
 	if updated {
 		t.Logf("updated boot config assets")
 		// boot assets were updated, request a restart now so that the
 		// situation does not end up more complicated if more updates of
 		// boot assets were to be applied
-		snapstate.RestartSystem(t, nil)
+		return snapstate.FinishTaskWithRestart(t, finalStatus, restart.RestartSystem, nil)
+	} else {
+		t.SetStatus(finalStatus)
+		return nil
 	}
-
-	// minimize wasteful redos
-	t.SetStatus(state.DoneStatus)
-	return nil
 }
