@@ -1,6 +1,6 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 /*
- * Copyright (C) 2016-2017 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -19,6 +19,7 @@
 package devicestate
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -30,9 +31,9 @@ import (
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
 	"github.com/snapcore/snapd/logger"
+	"github.com/snapcore/snapd/overlord/restart"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
-	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 )
 
@@ -48,7 +49,7 @@ func makeRollbackDir(name string) (string, error) {
 
 func currentGadgetInfo(st *state.State, curDeviceCtx snapstate.DeviceContext) (*gadget.GadgetData, error) {
 	currentInfo, err := snapstate.GadgetInfo(st, curDeviceCtx)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, err
 	}
 	if currentInfo == nil {
@@ -81,10 +82,6 @@ var (
 )
 
 func (m *DeviceManager) doUpdateGadgetAssets(t *state.Task, _ *tomb.Tomb) error {
-	if release.OnClassic {
-		return fmt.Errorf("cannot run update gadget assets task on a classic system")
-	}
-
 	st := t.State()
 	st.Lock()
 	defer st.Unlock()
@@ -98,6 +95,9 @@ func (m *DeviceManager) doUpdateGadgetAssets(t *state.Task, _ *tomb.Tomb) error
 	if err != nil {
 		return err
 	}
+	if remodelCtx.IsClassicBoot() {
+		return fmt.Errorf("cannot run update gadget assets task on a classic system")
+	}
 	isRemodel := remodelCtx.ForRemodeling()
 	groundDeviceCtx := remodelCtx.GroundContext()
 
@@ -191,7 +191,7 @@ func (m *DeviceManager) doUpdateGadgetAssets(t *state.Task, _ *tomb.Tomb) error
 	// modify modeenv inside, which implicitly is guarded by the state lock;
 	// on top of that we do not expect the update to be moving large amounts
 	// of data
-	err = gadgetUpdate(*currentData, *updateData, snapRollbackDir, updatePolicy, updateObserver)
+	err = gadgetUpdate(model, *currentData, *updateData, snapRollbackDir, updatePolicy, updateObserver)
 	if err != nil {
 		if err == gadget.ErrNoUpdate {
 			// no update needed
@@ -201,17 +201,13 @@ func (m *DeviceManager) doUpdateGadgetAssets(t *state.Task, _ *tomb.Tomb) error
 		return err
 	}
 
-	t.SetStatus(state.DoneStatus)
-
 	if err := os.RemoveAll(snapRollbackDir); err != nil && !os.IsNotExist(err) {
 		logger.Noticef("failed to remove gadget update rollback directory %q: %v", snapRollbackDir, err)
 	}
 
 	// TODO: consider having the option to do this early via recovery in
 	// core20, have fallback code as well there
-	st.RequestRestart(state.RestartSystem)
-
-	return nil
+	return snapstate.FinishTaskWithRestart(t, state.DoneStatus, restart.RestartSystem, nil)
 }
 
 func (m *DeviceManager) updateGadgetCommandLine(t *state.Task, st *state.State, isUndo bool) (updated bool, err error) {
@@ -231,6 +227,9 @@ func (m *DeviceManager) updateGadgetCommandLine(t *state.Task, st *state.State,
 	if !isUndo {
 		// when updating, command line comes from the new gadget
 		gadgetData, err = pendingGadgetInfo(snapsup, devCtx)
+		if err != nil {
+			return false, err
+		}
 	} else {
 		// but when undoing, we use the current gadget which should have
 		// been restored
@@ -248,17 +247,21 @@ func (m *DeviceManager) updateGadgetCommandLine(t *state.Task, st *state.State,
 }
 
 func (m *DeviceManager) doUpdateGadgetCommandLine(t *state.Task, _ *tomb.Tomb) error {
-	if release.OnClassic {
-		return fmt.Errorf("internal error: cannot run update gadget kernel command line task on a classic system")
-	}
-
 	st := t.State()
 	st.Lock()
 	defer st.Unlock()
 
+	devCtx, err := DeviceCtx(st, t, nil)
+	if err != nil {
+		return err
+	}
+	if devCtx.IsClassicBoot() {
+		return fmt.Errorf("internal error: cannot run update gadget kernel command line task on a classic system")
+	}
+
 	var seeded bool
-	err := st.Get("seeded", &seeded)
-	if err != nil && err != state.ErrNoState {
+	err = st.Get("seeded", &seeded)
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	if !seeded {
@@ -277,29 +280,31 @@ func (m *DeviceManager) doUpdateGadgetCommandLine(t *state.Task, _ *tomb.Tomb) e
 	}
 	t.Logf("Updated kernel command line")
 
-	t.SetStatus(state.DoneStatus)
-
 	// TODO: consider optimization to avoid double reboot when the gadget
 	// snap carries an update to the gadget assets and a change in the
 	// kernel command line
 
 	// kernel command line was updated, request a reboot to make it effective
-	st.RequestRestart(state.RestartSystem)
-	return nil
+
+	return snapstate.FinishTaskWithRestart(t, state.DoneStatus, restart.RestartSystem, nil)
 }
 
 func (m *DeviceManager) undoUpdateGadgetCommandLine(t *state.Task, _ *tomb.Tomb) error {
-	if release.OnClassic {
-		return fmt.Errorf("internal error: cannot run update gadget kernel command line task on a classic system")
-	}
-
 	st := t.State()
 	st.Lock()
 	defer st.Unlock()
 
+	devCtx, err := DeviceCtx(st, t, nil)
+	if err != nil {
+		return err
+	}
+	if devCtx.IsClassicBoot() {
+		return fmt.Errorf("internal error: cannot run undo update gadget kernel command line task on a classic system")
+	}
+
 	var seeded bool
-	err := st.Get("seeded", &seeded)
-	if err != nil && err != state.ErrNoState {
+	err = st.Get("seeded", &seeded)
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	if !seeded {
@@ -318,9 +323,6 @@ func (m *DeviceManager) undoUpdateGadgetCommandLine(t *state.Task, _ *tomb.Tomb)
 	}
 	t.Logf("Reverted kernel command line change")
 
-	t.SetStatus(state.UndoneStatus)
-
 	// kernel command line was updated, request a reboot to make it effective
-	st.RequestRestart(state.RestartSystem)
-	return nil
+	return snapstate.FinishTaskWithRestart(t, state.UndoneStatus, restart.RestartSystem, nil)
 }
