@@ -223,11 +223,11 @@ version: 1
 `, "")
 
 	// pretend snap foo is held initially
-	_, err = snapstate.HoldRefresh(s.st, "snap1", 0, "foo")
+	_, err = snapstate.HoldRefresh(s.st, snapstate.HoldAutoRefresh, "snap1", 0, "foo")
 	c.Check(err, IsNil)
 	s.st.Unlock()
 
-	// sanity check
+	// validity check
 	var gating map[string]map[string]interface{}
 	s.st.Lock()
 	snapsHold := s.st.Get("snaps-hold", &gating)
