@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2020 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -22,6 +22,7 @@ package ifacestate
 import (
 	"bytes"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"os"
 	"sort"
@@ -30,19 +31,27 @@ import (
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
-	"github.com/snapcore/snapd/interfaces/backends"
 	"github.com/snapcore/snapd/interfaces/builtin"
 	"github.com/snapcore/snapd/interfaces/policy"
 	"github.com/snapcore/snapd/interfaces/utils"
 	"github.com/snapcore/snapd/jsonutil"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/assertstate"
+	"github.com/snapcore/snapd/overlord/ifacestate/schema"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/timings"
 )
 
+var (
+	snapdAppArmorServiceIsDisabled = snapdAppArmorServiceIsDisabledImpl
+	profilesNeedRegeneration       = profilesNeedRegenerationImpl
+
+	writeSystemKey = interfaces.WriteSystemKey
+)
+
 func (m *InterfaceManager) selectInterfaceMapper(snaps []*snap.Info) {
 	for _, snapInfo := range snaps {
 		if snapInfo.Type() == snap.TypeSnapd {
@@ -71,7 +80,7 @@ func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error
 	var snapdSnap snapstate.SnapState
 	var snapdSnapInfo *snap.Info
 	err := snapstate.Get(m.state, "snapd", &snapdSnap)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return fmt.Errorf("cannot access snapd snap state: %v", err)
 	}
 	if err == nil {
@@ -85,7 +94,7 @@ func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error
 	var coreSnap snapstate.SnapState
 	var coreSnapInfo *snap.Info
 	err = snapstate.Get(m.state, "core", &coreSnap)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return fmt.Errorf("cannot access core snap state: %v", err)
 	}
 	if err == nil {
@@ -100,7 +109,7 @@ func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error
 		CoreSnapInfo:  coreSnapInfo,
 		SnapdSnapInfo: snapdSnapInfo,
 	}
-	for _, backend := range backends.All {
+	for _, backend := range allSecurityBackends() {
 		if err := backend.Initialize(&opts); err != nil {
 			return err
 		}
@@ -140,8 +149,13 @@ func profilesNeedRegenerationImpl() bool {
 	return mismatch
 }
 
-var profilesNeedRegeneration = profilesNeedRegenerationImpl
-var writeSystemKey = interfaces.WriteSystemKey
+// snapdAppArmorServiceIsDisabledImpl returns true if the snapd.apparmor
+// service unit exists but is disabled
+func snapdAppArmorServiceIsDisabledImpl() bool {
+	sysd := systemd.New(systemd.SystemMode, nil)
+	isEnabled, err := sysd.IsEnabled("snapd.apparmor")
+	return err == nil && !isEnabled
+}
 
 // regenerateAllSecurityProfiles will regenerate all security profiles.
 func (m *InterfaceManager) regenerateAllSecurityProfiles(tm timings.Measurer) error {
@@ -176,7 +190,11 @@ func (m *InterfaceManager) regenerateAllSecurityProfiles(tm timings.Measurer) er
 		if err := snapstate.Get(m.state, snapName, &snapst); err != nil {
 			logger.Noticef("cannot get state of snap %q: %s", snapName, err)
 		}
-		return confinementOptions(snapst.Flags)
+		opts, err := buildConfinementOptions(m.state, snapst.InstanceName(), snapst.Flags)
+		if err != nil {
+			logger.Noticef("cannot get confinement options for snap %q: %s", snapName, err)
+		}
+		return opts
 	}
 
 	// For each backend:
@@ -246,14 +264,14 @@ var removeStaleConnections = func(st *state.State) error {
 		}
 		var snapst snapstate.SnapState
 		if err := snapstate.Get(st, connRef.PlugRef.Snap, &snapst); err != nil {
-			if err != state.ErrNoState {
+			if !errors.Is(err, state.ErrNoState) {
 				return err
 			}
 			staleConns = append(staleConns, id)
 			continue
 		}
 		if err := snapstate.Get(st, connRef.SlotRef.Snap, &snapst); err != nil {
-			if err != state.ErrNoState {
+			if !errors.Is(err, state.ErrNoState) {
 				return err
 			}
 			staleConns = append(staleConns, id)
@@ -273,7 +291,7 @@ var removeStaleConnections = func(st *state.State) error {
 func isBroken(st *state.State, snapName string) (bool, error) {
 	var snapst snapstate.SnapState
 	err := snapstate.Get(st, snapName, &snapst)
-	if err == state.ErrNoState {
+	if errors.Is(err, state.ErrNoState) {
 		return false, nil
 	}
 	if err != nil {
@@ -503,26 +521,6 @@ func addHotplugSlot(st *state.State, repo *interfaces.Repository, stateSlots map
 	return nil
 }
 
-type connState struct {
-	Auto      bool   `json:"auto,omitempty"`
-	ByGadget  bool   `json:"by-gadget,omitempty"`
-	Interface string `json:"interface,omitempty"`
-	// Undesired tracks connections that were manually disconnected after being auto-connected,
-	// so that they are not automatically reconnected again in the future.
-	Undesired        bool                   `json:"undesired,omitempty"`
-	StaticPlugAttrs  map[string]interface{} `json:"plug-static,omitempty"`
-	DynamicPlugAttrs map[string]interface{} `json:"plug-dynamic,omitempty"`
-	StaticSlotAttrs  map[string]interface{} `json:"slot-static,omitempty"`
-	DynamicSlotAttrs map[string]interface{} `json:"slot-dynamic,omitempty"`
-	// Hotplug-related attributes: HotplugGone indicates a connection that
-	// disappeared because the device was removed, but may potentially be
-	// restored in the future if we see the device again. HotplugKey is the
-	// key of the associated device; it's empty for connections of regular
-	// slots.
-	HotplugGone bool            `json:"hotplug-gone,omitempty"`
-	HotplugKey  snap.HotplugKey `json:"hotplug-key,omitempty"`
-}
-
 type gadgetConnect struct {
 	st   *state.State
 	task *state.Task
@@ -546,10 +544,10 @@ func newGadgetConnect(s *state.State, task *state.Task, repo *interfaces.Reposit
 // addGadgetConnections adds to newconns any applicable connections
 // from the gadget connections stanza.
 // conflictError is called to handle checkAutoconnectConflicts errors.
-func (gc *gadgetConnect) addGadgetConnections(newconns map[string]*interfaces.ConnRef, conns map[string]*connState, conflictError func(*state.Retry, error) error) error {
+func (gc *gadgetConnect) addGadgetConnections(newconns map[string]*interfaces.ConnRef, conns map[string]*schema.ConnState, conflictError func(*state.Retry, error) error) error {
 	var seeded bool
 	err := gc.st.Get("seeded", &seeded)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	// we apply gadget connections only during seeding or a remodeling
@@ -577,7 +575,7 @@ func (gc *gadgetConnect) addGadgetConnections(newconns map[string]*interfaces.Co
 
 	gconns, err := snapstate.GadgetConnections(gc.st, gc.deviceCtx)
 	if err != nil {
-		if err == state.ErrNoState {
+		if errors.Is(err, state.ErrNoState) {
 			// no gadget yet, nothing to do
 			return nil
 		}
@@ -633,7 +631,7 @@ func (gc *gadgetConnect) addGadgetConnections(newconns map[string]*interfaces.Co
 	return nil
 }
 
-func addNewConnection(st *state.State, task *state.Task, newconns map[string]*interfaces.ConnRef, conns map[string]*connState, plug *snap.PlugInfo, slot *snap.SlotInfo, conflictError func(*state.Retry, error) error) error {
+func addNewConnection(st *state.State, task *state.Task, newconns map[string]*interfaces.ConnRef, conns map[string]*schema.ConnState, plug *snap.PlugInfo, slot *snap.SlotInfo, conflictError func(*state.Retry, error) error) error {
 	connRef := interfaces.NewConnRef(plug, slot)
 	key := connRef.ID()
 	if _, ok := conns[key]; ok {
@@ -667,6 +665,10 @@ func addNewConnection(st *state.State, task *state.Task, newconns map[string]*in
 	return nil
 }
 
+// DebugAutoConnectCheck is a hook that can be set to debug auto-connection
+// candidates as they are checked.
+var DebugAutoConnectCheck func(*policy.ConnectCandidate, interfaces.SideArity, error)
+
 type autoConnectChecker struct {
 	st   *state.State
 	task *state.Task
@@ -749,6 +751,9 @@ func (c *autoConnectChecker) check(plug *interfaces.ConnectedPlug, slot *interfa
 	}
 
 	arity, err := ic.CheckAutoConnect()
+	if DebugAutoConnectCheck != nil {
+		DebugAutoConnectCheck(&ic, arity, err)
+	}
 	if err == nil {
 		return true, arity, nil
 	}
@@ -795,7 +800,7 @@ func filterUbuntuCoreSlots(candidates []*snap.SlotInfo, arities []interfaces.Sid
 // conns. cannotAutoConnectLog is called to build a log message in
 // case no applicable pair was found. conflictError is called
 // to handle checkAutoconnectConflicts errors.
-func (c *autoConnectChecker) addAutoConnections(newconns map[string]*interfaces.ConnRef, plugs []*snap.PlugInfo, filter func([]*snap.SlotInfo) []*snap.SlotInfo, conns map[string]*connState, cannotAutoConnectLog func(plug *snap.PlugInfo, candRefs []string) string, conflictError func(*state.Retry, error) error) error {
+func (c *autoConnectChecker) addAutoConnections(newconns map[string]*interfaces.ConnRef, plugs []*snap.PlugInfo, filter func([]*snap.SlotInfo) []*snap.SlotInfo, conns map[string]*schema.ConnState, cannotAutoConnectLog func(plug *snap.PlugInfo, candRefs []string) string, conflictError func(*state.Retry, error) error) error {
 	for _, plug := range plugs {
 		candSlots, arities := c.repo.AutoConnectCandidateSlots(plug.Snap.InstanceName(), plug.Name, c.check)
 
@@ -930,10 +935,10 @@ func getPlugAndSlotRefs(task *state.Task) (interfaces.PlugRef, interfaces.SlotRe
 // getConns returns information about connections from the state.
 //
 // Connections are transparently re-mapped according to remapIncomingConnRef
-func getConns(st *state.State) (conns map[string]*connState, err error) {
+func getConns(st *state.State) (conns map[string]*schema.ConnState, err error) {
 	var raw *json.RawMessage
 	err = st.Get("conns", &raw)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, fmt.Errorf("cannot obtain raw data about existing connections: %s", err)
 	}
 	if raw != nil {
@@ -943,9 +948,9 @@ func getConns(st *state.State) (conns map[string]*connState, err error) {
 		}
 	}
 	if conns == nil {
-		conns = make(map[string]*connState)
+		conns = make(map[string]*schema.ConnState)
 	}
-	remapped := make(map[string]*connState, len(conns))
+	remapped := make(map[string]*schema.ConnState, len(conns))
 	for id, cstate := range conns {
 		cref, err := interfaces.ParseConnRef(id)
 		if err != nil {
@@ -965,8 +970,8 @@ func getConns(st *state.State) (conns map[string]*connState, err error) {
 // setConns sets information about connections in the state.
 //
 // Connections are transparently re-mapped according to remapOutgoingConnRef
-func setConns(st *state.State, conns map[string]*connState) {
-	remapped := make(map[string]*connState, len(conns))
+func setConns(st *state.State, conns map[string]*schema.ConnState) {
+	remapped := make(map[string]*schema.ConnState, len(conns))
 	for id, cstate := range conns {
 		cref, err := interfaces.ParseConnRef(id)
 		if err != nil {
@@ -981,17 +986,46 @@ func setConns(st *state.State, conns map[string]*connState) {
 }
 
 // snapsWithSecurityProfiles returns all snaps that have active
-// security profiles: these are either snaps that are active, or about
-// to be active (pending link-snap) with a done setup-profiles
+// security profiles: these are either snaps that are active,
+// inactive snaps that are being operated on, whose profile state
+// is tracked with SnapState.PendingSecurity,
+// or snap about to be active (pending link-snap) with a done
+// setup-profiles
 func snapsWithSecurityProfiles(st *state.State) ([]*snap.Info, error) {
-	infos, err := snapstate.ActiveInfos(st)
+	all, err := snapstate.All(st)
 	if err != nil {
 		return nil, err
 	}
-	seen := make(map[string]bool, len(infos))
-	for _, info := range infos {
-		seen[info.InstanceName()] = true
+	infos := make([]*snap.Info, 0, len(all))
+	seen := make(map[string]bool, len(all))
+	for instanceName, snapst := range all {
+		if snapst.Active {
+			snapInfo, err := snapst.CurrentInfo()
+			if err != nil {
+				logger.Noticef("cannot retrieve info for snap %q: %s", instanceName, err)
+				continue
+			}
+			infos = append(infos, snapInfo)
+			seen[instanceName] = true
+		} else if snapst.PendingSecurity != nil {
+			// we tracked any pending security profiles for the snap
+			seen[instanceName] = true
+			si := snapst.PendingSecurity.SideInfo
+			if si == nil {
+				// profiles removed (already)
+				continue
+			}
+			snapInfo, err := snap.ReadInfo(instanceName, si)
+			if err != nil {
+				logger.Noticef("cannot retrieve info for snap %q: %s", instanceName, err)
+				continue
+			}
+			infos = append(infos, snapInfo)
+		}
 	}
+	// look at the changes for old snapds and also
+	// the situation that are being installed, so they do not
+	// have SnapState yet
 	for _, t := range st.Tasks() {
 		if t.Kind() != "link-snap" || t.Status().Ready() {
 			continue
@@ -1227,7 +1261,7 @@ func getHotplugAttrs(task *state.Task) (ifaceName string, hotplugKey snap.Hotplu
 
 func allocHotplugSeq(st *state.State) (int, error) {
 	var seq int
-	if err := st.Get("hotplug-seq", &seq); err != nil && err != state.ErrNoState {
+	if err := st.Get("hotplug-seq", &seq); err != nil && !errors.Is(err, state.ErrNoState) {
 		return 0, fmt.Errorf("internal error: cannot allocate hotplug sequence number: %s", err)
 	}
 	seq++
@@ -1279,7 +1313,7 @@ func getHotplugSlots(st *state.State) (map[string]*HotplugSlotInfo, error) {
 	var slots map[string]*HotplugSlotInfo
 	err := st.Get("hotplug-slots", &slots)
 	if err != nil {
-		if err != state.ErrNoState {
+		if !errors.Is(err, state.ErrNoState) {
 			return nil, err
 		}
 		slots = make(map[string]*HotplugSlotInfo)
@@ -1300,7 +1334,7 @@ func findHotplugSlot(stateSlots map[string]*HotplugSlotInfo, ifaceName string, h
 	return nil
 }
 
-func findConnsForHotplugKey(conns map[string]*connState, ifaceName string, hotplugKey snap.HotplugKey) []string {
+func findConnsForHotplugKey(conns map[string]*schema.ConnState, ifaceName string, hotplugKey snap.HotplugKey) []string {
 	var connsForDevice []string
 	for id, connSt := range conns {
 		if connSt.Interface != ifaceName || connSt.HotplugKey != hotplugKey {
