@@ -21,6 +21,7 @@ package ifacestate
 
 import (
 	"crypto/sha256"
+	"errors"
 	"fmt"
 	"strings"
 	"unicode"
@@ -133,7 +134,7 @@ func (m *InterfaceManager) hotplugDeviceAdded(devinfo *hotplug.HotplugDeviceInfo
 	}
 
 	gadget, err := snapstate.GadgetInfo(st, deviceCtx)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		logger.Noticef("internal error: cannot get gadget information: %v", err)
 	}
 
@@ -330,7 +331,7 @@ func (m *InterfaceManager) hotplugEnumerationDone() {
 
 func (m *InterfaceManager) hotplugEnabled() (bool, error) {
 	tr := config.NewTransaction(m.state)
-	return config.GetFeatureFlag(tr, features.Hotplug)
+	return features.Flag(tr, features.Hotplug)
 }
 
 // ensureUniqueName modifies proposedName so that it's unique according to isUnique predicate.
