@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2017 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -22,6 +22,7 @@
 package ifacestate
 
 import (
+	"errors"
 	"fmt"
 	"sync"
 	"time"
@@ -533,8 +534,11 @@ func delayedCrossMgrInit() {
 		})
 
 		// hook into conflict checks mechanisms
-		snapstate.AddAffectedSnapsByKind("connect", connectDisconnectAffectedSnaps)
-		snapstate.AddAffectedSnapsByKind("disconnect", connectDisconnectAffectedSnaps)
+		snapstate.RegisterAffectedSnapsByKind("connect", connectDisconnectAffectedSnaps)
+		snapstate.RegisterAffectedSnapsByKind("disconnect", connectDisconnectAffectedSnaps)
+
+		// hook into snap linking/unlinking and activation state changes
+		snapstate.AddLinkSnapParticipant(snapstate.LinkSnapParticipantFunc(OnSnapLinkageChanged))
 	})
 }
 
@@ -543,3 +547,31 @@ func MockConnectRetryTimeout(d time.Duration) (restore func()) {
 	connectRetryTimeout = d
 	return func() { connectRetryTimeout = old }
 }
+
+// OnSnapLinkageChanged is used to implement
+// snapstate.LinkSnapParticipant follow activation changes for snaps
+// so that we can track revisions with security profiles on disk for
+// temporarily inactive snaps.
+func OnSnapLinkageChanged(st *state.State, instanceName string) error {
+	var snapst snapstate.SnapState
+	if err := snapstate.Get(st, instanceName, &snapst); err != nil && !errors.Is(err, state.ErrNoState) {
+		return err
+	}
+	if !snapst.IsInstalled() {
+		// nothing to do
+		return nil
+	}
+
+	if snapst.Active {
+		// nothing to track
+		snapst.PendingSecurity = nil
+	} else {
+		// track the revision that was just unlinked that has
+		// still profiles
+		snapst.PendingSecurity = &snapstate.PendingSecurityState{
+			SideInfo: snapst.CurrentSideInfo(),
+		}
+	}
+	snapstate.Set(st, instanceName, &snapst)
+	return nil
+}
