@@ -20,11 +20,21 @@
 package servicestate_test
 
 import (
-	. "gopkg.in/check.v1"
+	"errors"
+	"io/ioutil"
+	"os"
+	"path/filepath"
+	"time"
 
+	"gopkg.in/check.v1"
+	. "gopkg.in/check.v1"
+	tomb "gopkg.in/tomb.v2"
+
+	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/servicestate"
+	"github.com/snapcore/snapd/overlord/servicestate/servicestatetest"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
@@ -82,7 +92,7 @@ func mockMixedQuotaGroup(st *state.State, name string, snaps []string) error {
 
 	var quotas map[string]*quota.Group
 	if err := st.Get("quotas", &quotas); err != nil {
-		if err != state.ErrNoState {
+		if !errors.Is(err, state.ErrNoState) {
 			return err
 		}
 		quotas = make(map[string]*quota.Group)
@@ -108,26 +118,15 @@ func (s *quotaHandlersSuite) TestDoQuotaControlCreate(c *C) {
 	snapstate.Set(s.state, "test-snap", s.testSnapState)
 	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
 
-	// make a fake task
-	t := st.NewTask("create-quota", "...")
-
-	qcs := []servicestate.QuotaControlAction{
-		{
-			Action:         "create",
-			QuotaName:      "foo-group",
-			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
-			AddSnaps:       []string{"test-snap"},
-		},
+	qcs := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo-group",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		AddSnaps:       []string{"test-snap"},
 	}
 
-	t.Set("quota-control-actions", &qcs)
-
-	st.Unlock()
-	err := s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
-
+	err := s.callDoQuotaControl(&qcs)
 	c.Assert(err, IsNil)
-	c.Assert(t.Status(), Equals, state.DoneStatus)
 
 	checkQuotaState(c, st, map[string]quotaGroupState{
 		"foo-group": {
@@ -137,6 +136,25 @@ func (s *quotaHandlersSuite) TestDoQuotaControlCreate(c *C) {
 	})
 }
 
+func (s *quotaHandlersSuite) getTasksOfKind(chg *state.Change, kind string) []*state.Task {
+	var tasks []*state.Task
+	for _, t := range chg.Tasks() {
+		if t.Kind() == kind {
+			tasks = append(tasks, t)
+		}
+	}
+	return tasks
+}
+
+func (s *quotaHandlersSuite) runRestartTasks(tasks []*state.Task) error {
+	for _, t := range tasks {
+		if err := s.o.ServiceManager().DoServiceControl(t, nil); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func (s *quotaHandlersSuite) TestDoQuotaControlCreateRestartOK(c *C) {
 	// test a situation where because of restart the task is reentered
 	r := s.mockSystemctlCalls(c, join(
@@ -155,7 +173,8 @@ func (s *quotaHandlersSuite) TestDoQuotaControlCreateRestartOK(c *C) {
 	snapstate.Set(s.state, "test-snap", s.testSnapState)
 	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
 
-	// make a fake task
+	// make a fake change with a single task
+	chg := st.NewChange("test", "")
 	t := st.NewTask("create-quota", "...")
 
 	qcs := []servicestate.QuotaControlAction{
@@ -168,6 +187,7 @@ func (s *quotaHandlersSuite) TestDoQuotaControlCreateRestartOK(c *C) {
 	}
 
 	t.Set("quota-control-actions", &qcs)
+	chg.AddTask(t)
 
 	expectedQuotaState := map[string]quotaGroupState{
 		"foo-group": {
@@ -175,25 +195,38 @@ func (s *quotaHandlersSuite) TestDoQuotaControlCreateRestartOK(c *C) {
 			Snaps:          []string{"test-snap"},
 		},
 	}
-
 	st.Unlock()
+
 	err := s.o.ServiceManager().DoQuotaControl(t, nil)
+
 	st.Lock()
 	c.Assert(err, IsNil)
-
 	c.Assert(t.Status(), Equals, state.DoneStatus)
 
+	t.SetStatus(state.DoingStatus)
+	restartTasks := s.getTasksOfKind(chg, "service-control")
+	st.Unlock()
+
+	err = s.runRestartTasks(restartTasks)
+	c.Assert(err, IsNil)
+
+	st.Lock()
+
 	checkQuotaState(c, st, expectedQuotaState)
 
-	t.SetStatus(state.DoingStatus)
-
 	st.Unlock()
 	err = s.o.ServiceManager().DoQuotaControl(t, nil)
+
 	st.Lock()
 	c.Assert(err, IsNil)
-
 	c.Assert(t.Status(), Equals, state.DoneStatus)
+	st.Unlock()
 
+	err = s.runRestartTasks(restartTasks)
+
+	st.Lock()
+	c.Assert(err, IsNil)
+	c.Assert(t.Status(), Equals, state.DoneStatus)
 	checkQuotaState(c, st, expectedQuotaState)
 }
 
@@ -213,7 +246,8 @@ func (s *quotaHandlersSuite) TestQuotaStateAlreadyUpdatedBehavior(c *C) {
 	snapstate.Set(s.state, "test-snap", s.testSnapState)
 	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
 
-	// make a fake task
+	// make a fake change with a task
+	chg := st.NewChange("test", "")
 	t := st.NewTask("create-quota", "...")
 
 	qcs := []servicestate.QuotaControlAction{
@@ -226,16 +260,26 @@ func (s *quotaHandlersSuite) TestQuotaStateAlreadyUpdatedBehavior(c *C) {
 	}
 
 	t.Set("quota-control-actions", &qcs)
-
+	chg.AddTask(t)
 	st.Unlock()
+
 	err := s.o.ServiceManager().DoQuotaControl(t, nil)
+
+	st.Lock()
+	c.Assert(err, IsNil)
+	c.Assert(t.Status(), Equals, state.DoneStatus)
+	c.Assert(len(chg.Tasks()), Equals, 2)
+
+	t.SetStatus(state.DoingStatus)
+	restartTasks := s.getTasksOfKind(chg, "service-control")
+	st.Unlock()
+
+	err = s.runRestartTasks(restartTasks)
+
 	st.Lock()
 	c.Assert(err, IsNil)
 
-	c.Assert(t.Status(), Equals, state.DoneStatus)
-	t.SetStatus(state.DoingStatus)
-
-	updated, appsToRestart, err := servicestate.QuotaStateAlreadyUpdated(t)
+	updated, appsToRestart, _, err := servicestate.QuotaStateAlreadyUpdated(t)
 	c.Assert(err, IsNil)
 	c.Check(updated, Equals, true)
 	c.Assert(appsToRestart, HasLen, 1)
@@ -249,20 +293,20 @@ func (s *quotaHandlersSuite) TestQuotaStateAlreadyUpdatedBehavior(c *C) {
 	r = servicestate.MockOsutilBootID("other-boot")
 	defer r()
 
-	updated, appsToRestart, err = servicestate.QuotaStateAlreadyUpdated(t)
+	updated, appsToRestart, _, err = servicestate.QuotaStateAlreadyUpdated(t)
 	c.Assert(err, IsNil)
 	c.Check(updated, Equals, true)
 	c.Check(appsToRestart, HasLen, 0)
 	r()
 
 	// restored
-	_, appsToRestart, err = servicestate.QuotaStateAlreadyUpdated(t)
+	_, appsToRestart, _, err = servicestate.QuotaStateAlreadyUpdated(t)
 	c.Assert(err, IsNil)
 	c.Check(appsToRestart, HasLen, 1)
 
 	// snap went missing
 	snapstate.Set(s.state, "test-snap", nil)
-	updated, appsToRestart, err = servicestate.QuotaStateAlreadyUpdated(t)
+	updated, appsToRestart, _, err = servicestate.QuotaStateAlreadyUpdated(t)
 	c.Assert(err, IsNil)
 	c.Check(updated, Equals, true)
 	c.Check(appsToRestart, HasLen, 0)
@@ -286,45 +330,25 @@ func (s *quotaHandlersSuite) TestDoQuotaControlUpdate(c *C) {
 	snapstate.Set(s.state, "test-snap", s.testSnapState)
 	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
 
-	// create a quota group
-	t := st.NewTask("create-quota", "...")
-
-	qcs := []servicestate.QuotaControlAction{
-		{
-			Action:         "create",
-			QuotaName:      "foo-group",
-			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
-			AddSnaps:       []string{"test-snap"},
-		},
+	qcs := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo-group",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		AddSnaps:       []string{"test-snap"},
 	}
 
-	t.Set("quota-control-actions", &qcs)
-
-	st.Unlock()
-	err := s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
+	err := s.callDoQuotaControl(&qcs)
 	c.Assert(err, IsNil)
 
-	// create a task for updating the quota group
-	t = st.NewTask("update-quota", "...")
-
 	// update the memory limit to be double
-	qcs = []servicestate.QuotaControlAction{
-		{
-			Action:         "update",
-			QuotaName:      "foo-group",
-			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB * 2).Build(),
-		},
+	qcs = servicestate.QuotaControlAction{
+		Action:         "update",
+		QuotaName:      "foo-group",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB * 2).Build(),
 	}
 
-	t.Set("quota-control-actions", &qcs)
-
-	st.Unlock()
-	err = s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
-
+	err = s.callDoQuotaControl(&qcs)
 	c.Assert(err, IsNil)
-	c.Assert(t.Status(), Equals, state.DoneStatus)
 
 	checkQuotaState(c, st, map[string]quotaGroupState{
 		"foo-group": {
@@ -353,39 +377,23 @@ func (s *quotaHandlersSuite) TestDoQuotaControlUpdateRestartOK(c *C) {
 	snapstate.Set(s.state, "test-snap", s.testSnapState)
 	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
 
-	// create a quota group
-	t := st.NewTask("create-quota", "...")
-
-	qcs := []servicestate.QuotaControlAction{
-		{
-			Action:         "create",
-			QuotaName:      "foo-group",
-			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
-			AddSnaps:       []string{"test-snap"},
-		},
+	qcs := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo-group",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		AddSnaps:       []string{"test-snap"},
 	}
 
-	t.Set("quota-control-actions", &qcs)
-
-	st.Unlock()
-	err := s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
+	err := s.callDoQuotaControl(&qcs)
 	c.Assert(err, IsNil)
 
-	// create a task for updating the quota group
-	t = st.NewTask("update-quota", "...")
-
 	// update the memory limit to be double
-	qcs = []servicestate.QuotaControlAction{
-		{
-			Action:         "update",
-			QuotaName:      "foo-group",
-			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB * 2).Build(),
-		},
+	qcs = servicestate.QuotaControlAction{
+		Action:         "update",
+		QuotaName:      "foo-group",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB * 2).Build(),
 	}
 
-	t.Set("quota-control-actions", &qcs)
-
 	expectedQuotaState := map[string]quotaGroupState{
 		"foo-group": {
 			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB * 2).Build(),
@@ -393,24 +401,14 @@ func (s *quotaHandlersSuite) TestDoQuotaControlUpdateRestartOK(c *C) {
 		},
 	}
 
-	st.Unlock()
-	err = s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
+	err = s.callDoQuotaControl(&qcs)
 	c.Assert(err, IsNil)
 
-	c.Assert(t.Status(), Equals, state.DoneStatus)
-
 	checkQuotaState(c, st, expectedQuotaState)
 
-	t.SetStatus(state.DoingStatus)
-
-	st.Unlock()
-	err = s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
+	err = s.callDoQuotaControl(&qcs)
 	c.Assert(err, IsNil)
 
-	c.Assert(t.Status(), Equals, state.DoneStatus)
-
 	checkQuotaState(c, st, expectedQuotaState)
 }
 
@@ -435,44 +433,24 @@ func (s *quotaHandlersSuite) TestDoQuotaControlRemove(c *C) {
 	snapstate.Set(s.state, "test-snap", s.testSnapState)
 	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
 
-	// create a quota group
-	t := st.NewTask("create-quota", "...")
-
-	qcs := []servicestate.QuotaControlAction{
-		{
-			Action:         "create",
-			QuotaName:      "foo-group",
-			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
-			AddSnaps:       []string{"test-snap"},
-		},
+	qcs := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo-group",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		AddSnaps:       []string{"test-snap"},
 	}
 
-	t.Set("quota-control-actions", &qcs)
-
-	st.Unlock()
-	err := s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
+	err := s.callDoQuotaControl(&qcs)
 	c.Assert(err, IsNil)
 
-	// create a task for removing the quota group
-	t = st.NewTask("remove-quota", "...")
-
 	// remove quota group
-	qcs = []servicestate.QuotaControlAction{
-		{
-			Action:    "remove",
-			QuotaName: "foo-group",
-		},
+	qcs = servicestate.QuotaControlAction{
+		Action:    "remove",
+		QuotaName: "foo-group",
 	}
 
-	t.Set("quota-control-actions", &qcs)
-
-	st.Unlock()
-	err = s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
-
+	err = s.callDoQuotaControl(&qcs)
 	c.Assert(err, IsNil)
-	c.Assert(t.Status(), Equals, state.DoneStatus)
 
 	checkQuotaState(c, st, nil)
 }
@@ -502,6 +480,7 @@ func (s *quotaHandlersSuite) TestDoQuotaControlRemoveRestartOK(c *C) {
 	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
 
 	// create a quota group
+	chg := st.NewChange("remove-quota", "...")
 	t := st.NewTask("create-quota", "...")
 
 	qcs := []servicestate.QuotaControlAction{
@@ -514,13 +493,22 @@ func (s *quotaHandlersSuite) TestDoQuotaControlRemoveRestartOK(c *C) {
 	}
 
 	t.Set("quota-control-actions", &qcs)
-
+	chg.AddTask(t)
 	st.Unlock()
+
 	err := s.o.ServiceManager().DoQuotaControl(t, nil)
+
 	st.Lock()
 	c.Assert(err, IsNil)
+	restartTasks := s.getTasksOfKind(chg, "service-control")
+	st.Unlock()
 
-	// create a task for removing the quota group
+	err = s.runRestartTasks(restartTasks)
+	c.Assert(err, IsNil)
+
+	st.Lock()
+	// create a change and a task for removing the quota group
+	chg = st.NewChange("remove-quota", "...")
 	t = st.NewTask("remove-quota", "...")
 
 	// remove quota group
@@ -532,23 +520,40 @@ func (s *quotaHandlersSuite) TestDoQuotaControlRemoveRestartOK(c *C) {
 	}
 
 	t.Set("quota-control-actions", &qcs)
-
+	chg.AddTask(t)
 	st.Unlock()
+
 	err = s.o.ServiceManager().DoQuotaControl(t, nil)
+
 	st.Lock()
 	c.Assert(err, IsNil)
-
 	c.Assert(t.Status(), Equals, state.DoneStatus)
 
+	t.SetStatus(state.DoingStatus)
+	restartTasks = s.getTasksOfKind(chg, "service-control")
+	st.Unlock()
+
+	err = s.runRestartTasks(restartTasks)
+	c.Assert(err, IsNil)
+
+	st.Lock()
+
 	checkQuotaState(c, st, nil)
 
-	t.SetStatus(state.DoingStatus)
-
 	st.Unlock()
+
 	err = s.o.ServiceManager().DoQuotaControl(t, nil)
+
 	st.Lock()
 	c.Assert(err, IsNil)
+	c.Assert(t.Status(), Equals, state.DoneStatus)
+	st.Unlock()
 
+	err = s.runRestartTasks(restartTasks)
+	c.Assert(err, IsNil)
+
+	st.Lock()
+	c.Assert(err, IsNil)
 	c.Assert(t.Status(), Equals, state.DoneStatus)
 
 	checkQuotaState(c, st, nil)
@@ -557,14 +562,21 @@ func (s *quotaHandlersSuite) TestDoQuotaControlRemoveRestartOK(c *C) {
 func (s *quotaHandlersSuite) callDoQuotaControl(action *servicestate.QuotaControlAction) error {
 	st := s.state
 	qcs := []*servicestate.QuotaControlAction{action}
+	chg := st.NewChange("quota-control", "...")
 	t := st.NewTask("quota-task", "...")
 	t.Set("quota-control-actions", &qcs)
+	chg.AddTask(t)
 
 	st.Unlock()
-	err := s.o.ServiceManager().DoQuotaControl(t, nil)
-	st.Lock()
+	defer st.Lock()
 
-	return err
+	if err := s.o.ServiceManager().DoQuotaControl(t, nil); err != nil {
+		return err
+	}
+	st.Lock()
+	restartTasks := s.getTasksOfKind(chg, "service-control")
+	st.Unlock()
+	return s.runRestartTasks(restartTasks)
 }
 
 func (s *quotaHandlersSuite) TestQuotaCreatePreseeding(c *C) {
@@ -641,7 +653,7 @@ func (s *quotaHandlersSuite) TestQuotaCreate(c *C) {
 
 	// trying to create a quota with too low of a memory limit fails
 	err = s.callDoQuotaControl(&qc2)
-	c.Assert(err, ErrorMatches, `memory limit 4096 is too small: size must be larger than 640KB`)
+	c.Assert(err, ErrorMatches, `memory limit 4096 is too small: size must be larger than 640 KiB`)
 
 	// but with an adequately sized memory limit, and a snap that exists, we can
 	// create it
@@ -747,7 +759,8 @@ func (s *quotaHandlersSuite) TestDoCreateSubGroupQuota(c *C) {
 	})
 
 	// foo-group exists as a slice too, but has no snap services in the slice
-	checkSliceState(c, systemd.EscapeUnitNamePath("foo-group"), quantity.SizeGiB)
+	checkSliceState(c, systemd.EscapeUnitNamePath("foo-group"),
+		quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build())
 }
 
 func (s *quotaHandlersSuite) TestQuotaRemove(c *C) {
@@ -900,7 +913,31 @@ func (s *quotaHandlersSuite) TestQuotaRemove(c *C) {
 	checkQuotaState(c, st, nil)
 
 	// foo is not mentioned in the service and doesn't exist
-	checkSvcAndSliceState(c, "test-snap.svc1", "foo", 0)
+	checkSvcAndSliceState(c, "test-snap.svc1", "foo", quota.NewResourcesBuilder().Build())
+}
+
+func (s *quotaHandlersSuite) TestQuotaRemoveWithLimitSetFails(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup the snap so it exists
+	snapstate.Set(s.state, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	err := servicestatetest.MockQuotaInState(st, "foo", "", []string{"test-snap"},
+		quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build())
+	c.Assert(err, IsNil)
+
+	// but we can remove the sub-group successfully first
+	qc := servicestate.QuotaControlAction{
+		Action:         "remove",
+		QuotaName:      "foo",
+		ResourceLimits: quota.NewResourcesBuilder().WithThreadLimit(16).Build(),
+	}
+
+	err = s.callDoQuotaControl(&qc)
+	c.Assert(err, ErrorMatches, `internal error, quota limit options cannot be used with remove action`)
 }
 
 func (s *quotaHandlersSuite) TestQuotaSnapModifyExistingMixable(c *C) {
@@ -1258,6 +1295,284 @@ func (s *quotaHandlersSuite) TestQuotaUpdateChangeMemLimit(c *C) {
 	c.Assert(err, ErrorMatches, "cannot update limits for group \"foo\": cannot decrease memory limit, remove and re-create it to decrease the limit")
 }
 
+func (s *quotaHandlersSuite) TestCreateJournalQuota(c *C) {
+	r := s.mockSystemctlCalls(c, join(
+		// CreateQuota for foo
+		systemctlCallsForCreateQuota("foo", "test-snap"),
+	))
+	defer r()
+
+	// Add fake handlers for the setup-profiles task which should be invoked
+	// when creating the journal quota.
+	var setupProfilesCalled int
+	fakeHandler := func(task *state.Task, _ *tomb.Tomb) error {
+		setupProfilesCalled++
+		task.State().Lock()
+		snapInfo, err := snapstate.TaskSnapSetup(task)
+		task.State().Unlock()
+		c.Assert(err, IsNil)
+		c.Check(snapInfo.InstanceName(), Equals, "test-snap")
+		c.Check(snapInfo.SideInfo.Revision, Equals, s.testSnapSideInfo.Revision)
+		return err
+	}
+	s.o.TaskRunner().AddHandler("setup-profiles", fakeHandler, fakeHandler)
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup the snap so it exists
+	snapstate.Set(s.state, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	qc := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo",
+		ResourceLimits: quota.NewResourcesBuilder().WithJournalSize(quantity.SizeMiB * 64).Build(),
+		AddSnaps:       []string{"test-snap"},
+	}
+	qcs := []*servicestate.QuotaControlAction{&qc}
+
+	chg := st.NewChange("quota-control-tasks", "...")
+	t := st.NewTask("quota-control", "...")
+	t.Set("quota-control-actions", &qcs)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err := s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Check(err, IsNil)
+	c.Check(setupProfilesCalled, Equals, 1)
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithJournalSize(quantity.SizeMiB * 64).Build(),
+			Snaps:          []string{"test-snap"},
+		},
+	})
+}
+
+func (s *quotaHandlersSuite) TestAddJournalQuota(c *C) {
+	r := s.mockSystemctlCalls(c, join(
+		// CreateQuota for foo
+		systemctlCallsForCreateQuota("foo", "test-snap"),
+
+		// UpdateQuota for foo
+		[]expectedSystemctl{{expArgs: []string{"daemon-reload"}}},
+		systemctlCallsForServiceRestart("test-snap"),
+	))
+	defer r()
+
+	// Add fake handlers for the setup-profiles task which should be invoked
+	// when creating the journal quota.
+	var setupProfilesCalled int
+	fakeHandler := func(task *state.Task, _ *tomb.Tomb) error {
+		setupProfilesCalled++
+		task.State().Lock()
+		snapInfo, err := snapstate.TaskSnapSetup(task)
+		task.State().Unlock()
+		c.Assert(err, IsNil)
+		c.Check(snapInfo.InstanceName(), Equals, "test-snap")
+		c.Check(snapInfo.SideInfo.Revision, Equals, s.testSnapSideInfo.Revision)
+		return err
+	}
+	s.o.TaskRunner().AddHandler("setup-profiles", fakeHandler, fakeHandler)
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup the snap so it exists
+	snapstate.Set(s.state, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	qc := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		AddSnaps:       []string{"test-snap"},
+	}
+	qcs := []*servicestate.QuotaControlAction{&qc}
+
+	chg := st.NewChange("quota-control-tasks", "...")
+	t := st.NewTask("quota-control", "...")
+	t.Set("quota-control-actions", &qcs)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err := s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Check(err, IsNil)
+	c.Check(setupProfilesCalled, Equals, 0)
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+			Snaps:          []string{"test-snap"},
+		},
+	})
+
+	qc = servicestate.QuotaControlAction{
+		Action:         "update",
+		QuotaName:      "foo",
+		ResourceLimits: quota.NewResourcesBuilder().WithJournalSize(quantity.SizeMiB * 64).Build(),
+	}
+	qcs = []*servicestate.QuotaControlAction{&qc}
+
+	chg = st.NewChange("quota-control-tasks", "...")
+	t = st.NewTask("quota-control", "...")
+	t.Set("quota-control-actions", &qcs)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err = s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Check(err, IsNil)
+	c.Check(setupProfilesCalled, Equals, 1)
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).WithJournalSize(quantity.SizeMiB * 64).Build(),
+			Snaps:          []string{"test-snap"},
+		},
+	})
+}
+
+func (s *quotaHandlersSuite) TestUpdateJournalQuota(c *C) {
+	r := s.mockSystemctlCalls(c, join(
+		[]expectedSystemctl{{expArgs: []string{"daemon-reload"}}},
+		systemctlCallsForSliceStart("foo"),
+		[]expectedSystemctl{
+			{expArgs: []string{"stop", "systemd-journald@snap-foo"}},
+			{
+				expArgs: []string{"show", "--property=ActiveState", "systemd-journald@snap-foo"},
+				output:  "ActiveState=inactive",
+			},
+			{expArgs: []string{"start", "systemd-journald@snap-foo"}},
+		},
+		systemctlCallsForServiceRestart("test-snap"),
+	))
+	defer r()
+
+	// Add fake handlers for the setup-profiles task which should be invoked
+	// when creating the journal quota.
+	var setupProfilesCalled int
+	fakeHandler := func(task *state.Task, _ *tomb.Tomb) error {
+		setupProfilesCalled++
+		task.State().Lock()
+		snapInfo, err := snapstate.TaskSnapSetup(task)
+		task.State().Unlock()
+		c.Assert(err, IsNil)
+		c.Check(snapInfo.InstanceName(), Equals, "test-snap")
+		c.Check(snapInfo.SideInfo.Revision, Equals, s.testSnapSideInfo.Revision)
+		return err
+	}
+	s.o.TaskRunner().AddHandler("setup-profiles", fakeHandler, fakeHandler)
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup the snap so it exists
+	snapstate.Set(s.state, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	// setup an existing quota group we can update it
+	err := servicestatetest.MockQuotaInState(st, "foo", "", []string{"test-snap"}, quota.NewResourcesBuilder().WithJournalSize(16*quantity.SizeMiB).Build())
+	c.Assert(err, check.IsNil)
+
+	// Create the journald config file in /etc/systemd/journald@snap-foo.conf
+	// this needs to be done to trigger the restart of the journald service for
+	// that specific group. This is not done in the test as we only setup the
+	// group as a mock, so manually do this here.
+	fooConfPath := filepath.Join(dirs.SnapSystemdDir, "journald@snap-foo.conf")
+	c.Assert(os.MkdirAll(filepath.Dir(fooConfPath), 0755), IsNil)
+	err = ioutil.WriteFile(fooConfPath, []byte(`[Journal]
+SystemMaxUse=16M
+`), 0644)
+	c.Assert(err, IsNil)
+
+	qc := servicestate.QuotaControlAction{
+		Action:         "update",
+		QuotaName:      "foo",
+		ResourceLimits: quota.NewResourcesBuilder().WithJournalRate(150, time.Millisecond*10).Build(),
+	}
+	qcs := []*servicestate.QuotaControlAction{&qc}
+
+	chg := st.NewChange("quota-control-tasks", "...")
+	t := st.NewTask("quota-control", "...")
+	t.Set("quota-control-actions", &qcs)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err = s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Check(err, IsNil)
+	c.Check(setupProfilesCalled, Equals, 0)
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithJournalSize(16*quantity.SizeMiB).WithJournalRate(150, time.Millisecond*10).Build(),
+			Snaps:          []string{"test-snap"},
+		},
+	})
+}
+
+func (s *quotaHandlersSuite) TestRemoveJournalQuota(c *C) {
+	r := s.mockSystemctlCalls(c, join(
+		// RemoveQuota for foo
+		[]expectedSystemctl{{expArgs: []string{"daemon-reload"}}},
+		systemctlCallsForSliceStop("foo"),
+		systemctlCallsForServiceRestart("test-snap"),
+	))
+	defer r()
+
+	// Add fake handlers for the setup-profiles task which should be invoked
+	// when creating the journal quota.
+	var setupProfilesCalled int
+	fakeHandler := func(task *state.Task, _ *tomb.Tomb) error {
+		setupProfilesCalled++
+		task.State().Lock()
+		snapInfo, err := snapstate.TaskSnapSetup(task)
+		task.State().Unlock()
+		c.Assert(err, IsNil)
+		c.Check(snapInfo.InstanceName(), Equals, "test-snap")
+		c.Check(snapInfo.SideInfo.Revision, Equals, s.testSnapSideInfo.Revision)
+		return err
+	}
+	s.o.TaskRunner().AddHandler("setup-profiles", fakeHandler, fakeHandler)
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup the snap so it exists
+	snapstate.Set(s.state, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	// setup an existing quota group we can remove
+	err := servicestatetest.MockQuotaInState(st, "foo", "", []string{"test-snap"}, quota.NewResourcesBuilder().WithJournalSize(16*quantity.SizeMiB).Build())
+	c.Assert(err, check.IsNil)
+
+	qc := servicestate.QuotaControlAction{
+		Action:    "remove",
+		QuotaName: "foo",
+	}
+	qcs := []*servicestate.QuotaControlAction{&qc}
+
+	chg := st.NewChange("quota-control-tasks", "...")
+	t := st.NewTask("quota-control", "...")
+	t.Set("quota-control-actions", &qcs)
+	chg.AddTask(t)
+
+	st.Unlock()
+	defer s.se.Stop()
+	err = s.o.Settle(5 * time.Second)
+	st.Lock()
+	c.Check(err, IsNil)
+	c.Check(setupProfilesCalled, Equals, 1)
+}
+
 func (s *quotaHandlersSuite) TestQuotaUpdateAddSnap(c *C) {
 	r := s.mockSystemctlCalls(c, join(
 		// CreateQuota for foo
@@ -1417,3 +1732,272 @@ func (s *quotaHandlersSuite) TestQuotaUpdateAddSnapAlreadyInOtherGroup(c *C) {
 		},
 	})
 }
+
+func (s *quotaHandlersSuite) TestDoQuotaAddSnap(c *C) {
+	r := s.mockSystemctlCalls(c, join(
+		// CreateQuota for foo
+		[]expectedSystemctl{{expArgs: []string{"daemon-reload"}}},
+		systemctlCallsForSliceStart("foo"),
+	))
+	defer r()
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup test-snap
+	snapstate.Set(s.state, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	// create a quota group
+	qc := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		AddSnaps:       nil,
+	}
+
+	err := s.callDoQuotaControl(&qc)
+	c.Assert(err, IsNil)
+
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		},
+	})
+
+	// The snap exists and the quota group exists, so we're able to test the
+	// DoQuotaAddSnap
+	task := s.state.NewTask("add-snap-to-quota", "test")
+
+	// now set the snap-setup parameter and try again
+	snapsup := &snapstate.SnapSetup{
+		SideInfo: &snap.SideInfo{
+			RealName: "test-snap",
+			Revision: snap.R(1),
+			SnapID:   "test-snap-id",
+		},
+	}
+	task.Set("snap-setup", snapsup)
+
+	st.Unlock()
+	err = s.mgr.DoQuotaAddSnap(task, nil)
+	st.Lock()
+	c.Assert(err.Error(), Equals, "internal error: cannot get quota-name: no state entry for key \"quota-name\"")
+
+	// and finally set the quota name as well, so it should succeed
+	task.Set("quota-name", "foo")
+	st.Unlock()
+	err = s.mgr.DoQuotaAddSnap(task, nil)
+	st.Lock()
+	c.Assert(err, IsNil)
+
+	// verify state
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+			Snaps:          []string{"test-snap"},
+		},
+	})
+}
+
+func (s *quotaHandlersSuite) TestDoQuotaAddSnapQuotaConflict(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup test-snap
+	snapstate.Set(st, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	// setup an existing quota group we can update it
+	err := servicestatetest.MockQuotaInState(st, "foo", "", nil, quota.NewResourcesBuilder().WithMemoryLimit(1*quantity.SizeGiB).Build())
+	c.Assert(err, check.IsNil)
+
+	// Create a change that has a quota-control task in it for quota group foo
+	chg := st.NewChange("quota-update", "update foo quota group")
+	tsk := st.NewTask("quota-control", "update limits")
+	tsk.Set("quota-control-actions", []servicestate.QuotaControlAction{
+		{
+			Action:    "update",
+			QuotaName: "foo",
+		},
+	})
+	chg.AddTask(tsk)
+	chg.SetStatus(state.DoingStatus)
+
+	// Now we create a task for QuotaAddSnap
+	_, err = servicestate.AddSnapToQuotaGroup(st, "test-snap", "foo")
+	c.Assert(err.Error(), Equals, "quota group \"foo\" has \"quota-update\" change in progress")
+}
+
+func (s *quotaHandlersSuite) TestDoQuotaAddSnapSnapConflict(c *C) {
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup test-snap
+	snapstate.Set(st, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	// setup an existing quota group we can update it
+	err := servicestatetest.MockQuotaInState(st, "foo2", "", nil, quota.NewResourcesBuilder().WithMemoryLimit(1*quantity.SizeGiB).Build())
+	c.Assert(err, check.IsNil)
+
+	// Create the initial change which will contain AddSnapToQuotaGroup
+	chg1 := st.NewChange("snap-install", "installing test-snap")
+	tsk1, err := servicestate.AddSnapToQuotaGroup(st, "test-snap", "foo")
+	c.Assert(err, IsNil)
+	snapsup := &snapstate.SnapSetup{
+		SideInfo: &snap.SideInfo{
+			RealName: "test-snap",
+			Revision: snap.R(1),
+			SnapID:   "test-snap-id",
+		},
+	}
+	tsk1.Set("snap-setup", snapsup)
+	chg1.AddTask(tsk1)
+	chg1.SetStatus(state.DoingStatus)
+
+	// Create a change that has a quota-control task in it for quota group foo
+	_, err = servicestate.UpdateQuota(st, "foo2", servicestate.QuotaGroupUpdate{
+		AddSnaps: []string{"test-snap"},
+	})
+	c.Assert(err.Error(), Equals, "snap \"test-snap\" has \"snap-install\" change in progress")
+}
+
+func (s *quotaHandlersSuite) TestDoAddSnapToJournalQuota(c *C) {
+	r := s.mockSystemctlCalls(c, join(
+		// CreateQuota for foo
+		[]expectedSystemctl{{expArgs: []string{"daemon-reload"}}},
+		systemctlCallsForSliceStart("foo"),
+	))
+	defer r()
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup test-snap
+	snapstate.Set(s.state, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	// create a quota group
+	qc := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo",
+		ResourceLimits: quota.NewResourcesBuilder().WithJournalSize(quantity.SizeGiB).Build(),
+		AddSnaps:       nil,
+	}
+
+	err := s.callDoQuotaControl(&qc)
+	c.Assert(err, IsNil)
+
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithJournalSize(quantity.SizeGiB).Build(),
+		},
+	})
+
+	// The snap exists and the quota group exists, so we're able to test the
+	// DoQuotaAddSnap
+	chg := s.state.NewChange("add-snap-to-quota", "test")
+	task := s.state.NewTask("add-snap-to-quota", "test")
+	snapsup := &snapstate.SnapSetup{
+		SideInfo: &snap.SideInfo{
+			RealName: "test-snap",
+			Revision: snap.R(1),
+			SnapID:   "test-snap-id",
+		},
+	}
+	task.Set("snap-setup", snapsup)
+	task.Set("quota-name", "foo")
+	chg.AddTask(task)
+	st.Unlock()
+	err = s.mgr.DoQuotaAddSnap(task, nil)
+	st.Lock()
+	c.Assert(err, IsNil)
+	c.Assert(len(chg.Tasks()), Equals, 2)
+	c.Check(chg.Tasks()[1].Kind(), Equals, "setup-profiles")
+
+	// verify state
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithJournalSize(quantity.SizeGiB).Build(),
+			Snaps:          []string{"test-snap"},
+		},
+	})
+}
+
+func (s *quotaHandlersSuite) TestUndoQuotaAddSnap(c *C) {
+	r := s.mockSystemctlCalls(c, join(
+		// CreateQuota for foo
+		[]expectedSystemctl{{expArgs: []string{"daemon-reload"}}},
+		systemctlCallsForSliceStart("foo"),
+		systemctlCallsForServiceRestart("test-snap"),
+
+		// System calls expected for the removal of the snap
+		[]expectedSystemctl{{expArgs: []string{"daemon-reload"}}},
+		systemctlCallsForServiceRestart("test-snap"),
+	))
+	defer r()
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// setup test-snap
+	snapstate.Set(s.state, "test-snap", s.testSnapState)
+	snaptest.MockSnapCurrent(c, testYaml, s.testSnapSideInfo)
+
+	// create a quota group
+	qc := servicestate.QuotaControlAction{
+		Action:         "create",
+		QuotaName:      "foo",
+		ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		AddSnaps:       []string{"test-snap"},
+	}
+
+	err := s.callDoQuotaControl(&qc)
+	c.Assert(err, IsNil)
+
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+			Snaps:          []string{"test-snap"},
+		},
+	})
+
+	// The snap exists and the quota group exists, so we're able to test the
+	// DoQuotaAddSnap
+	task := s.state.NewTask("undo-add-snap-to-quota", "test")
+
+	// Test that it correctly reports an error if parameters are missing
+	st.Unlock()
+	err = s.mgr.UndoQuotaAddSnap(task, nil)
+	st.Lock()
+	c.Assert(err.Error(), Equals, "no state entry for key \"snap-setup-task\"")
+
+	// Set correct parameters so it can run while we have the lock
+	// now set the snap-setup parameter and try again
+	snapsup := &snapstate.SnapSetup{
+		SideInfo: &snap.SideInfo{
+			RealName: "test-snap",
+			Revision: snap.R(1),
+			SnapID:   "test-snap-id",
+		},
+	}
+	task.Set("snap-setup", snapsup)
+
+	st.Unlock()
+	err = s.mgr.UndoQuotaAddSnap(task, nil)
+	st.Lock()
+	c.Assert(err, IsNil)
+
+	// verify state
+	checkQuotaState(c, st, map[string]quotaGroupState{
+		"foo": {
+			ResourceLimits: quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeGiB).Build(),
+		},
+	})
+}
