@@ -20,6 +20,7 @@
 package servicestate
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"os/exec"
@@ -30,6 +31,7 @@ import (
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/overlord/restart"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/progress"
@@ -56,6 +58,21 @@ func Manager(st *state.State, runner *state.TaskRunner) *ServiceManager {
 	}
 	// TODO: undo handler
 	runner.AddHandler("service-control", m.doServiceControl, nil)
+
+	// TODO: undo handler
+	runner.AddHandler("quota-control", m.doQuotaControl, nil)
+	RegisterAffectedQuotasByKind("quota-control", affectedQuotasForQuotaControl)
+	snapstate.RegisterAffectedSnapsByKind("quota-control", affectedSnapsForQuotaControl)
+
+	// We can't directly refer to the servicestate internals from snapstate,
+	// so this task encapsulate taking care of calling quotaUpdate
+	// with the correct setup. This task also supports proper handling of
+	// failure during install and correctly removes the snap again.
+	runner.AddHandler("quota-add-snap", m.doQuotaAddSnap, m.undoQuotaAddSnap)
+	RegisterAffectedQuotasByKind("quota-add-snap", affectedQuotasForQuotaAddSnap)
+	// quota-add-snap uses snap-setup and because of this retrieving the snap
+	// that is being added is implicitly already supported by snapstate/conflict.go
+
 	return m
 }
 
@@ -78,7 +95,7 @@ func (m *ServiceManager) ensureSnapServicesUpdated() (err error) {
 	// only run after we are seeded
 	var seeded bool
 	err = m.state.Get("seeded", &seeded)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	if !seeded {
@@ -90,7 +107,7 @@ func (m *ServiceManager) ensureSnapServicesUpdated() (err error) {
 
 	// ensure all snap services are updated
 	allStates, err := snapstate.All(m.state)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 
@@ -101,7 +118,7 @@ func (m *ServiceManager) ensureSnapServicesUpdated() (err error) {
 	}
 
 	allGrps, err := AllQuotas(m.state)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 
@@ -173,7 +190,7 @@ func (m *ServiceManager) ensureSnapServicesUpdated() (err error) {
 		// we need to immediately reboot in the hopes that this restores
 		// services to a functioning state
 
-		m.state.RequestRestart(state.RestartSystemNow)
+		restart.Request(m.state, restart.RestartSystemNow, nil)
 		return fmt.Errorf("error trying to restart killed services, immediately rebooting: %v", err)
 	}
 
@@ -192,7 +209,7 @@ func (m *ServiceManager) Ensure() error {
 
 func delayedCrossMgrInit() {
 	// hook into conflict checks mechanisms
-	snapstate.AddAffectedSnapsByAttr("service-action", serviceControlAffectedSnaps)
+	snapstate.RegisterAffectedSnapsByAttr("service-action", serviceControlAffectedSnaps)
 	snapstate.SnapServiceOptions = SnapServiceOptions
 	snapstate.EnsureSnapAbsentFromQuotaGroup = EnsureSnapAbsentFromQuota
 }
