@@ -153,7 +153,7 @@ func (s *snapshotSuite) TestLastSnapshotID(c *check.C) {
 	c.Assert(err, check.IsNil)
 	c.Check(setID, check.Equals, uint64(0))
 
-	// create snapshots dir and dummy snapshots
+	// create snapshots dir and test snapshots
 	os.MkdirAll(dirs.SnapshotsDir, os.ModePerm)
 	for _, name := range []string{
 		"9_some-snap-1.zip", "1234_not-a-snapshot", "12_other-snap.zip", "3_foo.zip",
@@ -557,21 +557,26 @@ func (s *snapshotSuite) TestList(c *check.C) {
 }
 
 func (s *snapshotSuite) TestAddDirToZipBails(c *check.C) {
-	snapshot := &client.Snapshot{SetID: 42, Snap: "a-snap"}
+	snapshot := &client.Snapshot{SetID: 42, Snap: "a-snap", Revision: snap.R(5)}
+
+	oldVal := os.Getenv("SNAPD_DEBUG")
+	c.Assert(os.Setenv("SNAPD_DEBUG", "1"), check.IsNil)
+	defer func() {
+		os.Setenv("SNAPD_DEBUG", oldVal)
+	}()
+
 	buf, restore := logger.MockLogger()
 	defer restore()
 	savingUserData := false
 	// note as the zip is nil this would panic if it didn't bail
-	c.Check(backend.AddDirToZip(nil, snapshot, nil, "", "an/entry", filepath.Join(s.root, "nonexistent"), savingUserData, nil), check.IsNil)
-	// no log for the non-existent case
-	c.Check(buf.String(), check.Equals, "")
-	buf.Reset()
-	c.Check(backend.AddDirToZip(nil, snapshot, nil, "", "an/entry", "/etc/passwd", savingUserData, nil), check.IsNil)
-	c.Check(buf.String(), check.Matches, "(?m).* is not a directory.")
+	c.Check(backend.AddSnapDirToZip(nil, snapshot, nil, "", "an/entry", filepath.Join(s.root, "nonexistent"), savingUserData, nil), check.IsNil)
+	c.Check(backend.AddSnapDirToZip(nil, snapshot, nil, "", "an/entry", "/etc/passwd", savingUserData, nil), check.IsNil)
+	c.Check(buf.String(), check.Matches, "(?m).* is does not exist.*")
 }
 
 func (s *snapshotSuite) TestAddDirToZipTarFails(c *check.C) {
-	d := filepath.Join(s.root, "foo")
+	rev := snap.R(5)
+	d := filepath.Join(s.root, rev.String())
 	c.Assert(os.MkdirAll(filepath.Join(d, "bar"), 0755), check.IsNil)
 	c.Assert(os.MkdirAll(filepath.Join(s.root, "common"), 0755), check.IsNil)
 
@@ -581,11 +586,12 @@ func (s *snapshotSuite) TestAddDirToZipTarFails(c *check.C) {
 	var buf bytes.Buffer
 	z := zip.NewWriter(&buf)
 	savingUserData := false
-	c.Assert(backend.AddDirToZip(ctx, nil, z, "", "an/entry", d, savingUserData, nil), check.ErrorMatches, ".* context canceled")
+	c.Assert(backend.AddSnapDirToZip(ctx, &client.Snapshot{Revision: rev}, z, "", "an/entry", s.root, savingUserData, nil), check.ErrorMatches, ".* context canceled")
 }
 
 func (s *snapshotSuite) TestAddDirToZip(c *check.C) {
-	d := filepath.Join(s.root, "foo")
+	rev := snap.R(5)
+	d := filepath.Join(s.root, rev.String())
 	c.Assert(os.MkdirAll(filepath.Join(d, "bar"), 0755), check.IsNil)
 	c.Assert(os.MkdirAll(filepath.Join(s.root, "common"), 0755), check.IsNil)
 	c.Assert(ioutil.WriteFile(filepath.Join(d, "bar", "baz"), []byte("hello\n"), 0644), check.IsNil)
@@ -594,9 +600,10 @@ func (s *snapshotSuite) TestAddDirToZip(c *check.C) {
 	z := zip.NewWriter(&buf)
 	snapshot := &client.Snapshot{
 		SHA3_384: map[string]string{},
+		Revision: rev,
 	}
 	savingUserData := false
-	c.Assert(backend.AddDirToZip(context.Background(), snapshot, z, "", "an/entry", d, savingUserData, nil), check.IsNil)
+	c.Assert(backend.AddSnapDirToZip(context.Background(), snapshot, z, "", "an/entry", s.root, savingUserData, nil), check.IsNil)
 	z.Close() // write out the central directory
 
 	c.Check(snapshot.SHA3_384, check.HasLen, 1)
@@ -617,6 +624,7 @@ func (s *snapshotSuite) TestAddDirToZipExclusions(c *check.C) {
 	z := zip.NewWriter(&buf)
 	snapshot := &client.Snapshot{
 		SHA3_384: map[string]string{},
+		Revision: snap.R("x1"),
 	}
 	defer z.Close()
 
@@ -673,7 +681,7 @@ func (s *snapshotSuite) TestAddDirToZipExclusions(c *check.C) {
 	} {
 		testLabel := check.Commentf("%s/%v", testData.excludes, testData.savingUserData)
 
-		err := backend.AddDirToZip(context.Background(), snapshot, z, "", "an/entry", d, testData.savingUserData, testData.excludes)
+		err := backend.AddSnapDirToZip(context.Background(), snapshot, z, "", "an/entry", s.root, testData.savingUserData, testData.excludes)
 		c.Check(err, check.ErrorMatches, "tar failed.*")
 		c.Check(tarArgs, check.DeepEquals, testData.expectedArgs, testLabel)
 	}
@@ -762,7 +770,7 @@ func (s *snapshotSuite) testHappyRoundtrip(c *check.C, marker string) {
 
 	for i := 0; i < 3; i++ {
 		comm := check.Commentf("%d", i)
-		// sanity check
+		// validity check
 		c.Check(diff().Run(), check.NotNil, comm)
 
 		// restore leaves things like they were (again and again)
@@ -840,7 +848,7 @@ func (s *snapshotSuite) TestRestoreRoundtripDifferentRevision(c *check.C) {
 		return cmd
 	}
 
-	// sanity check
+	// validity check
 	c.Check(diff().Run(), check.NotNil)
 
 	// restore leaves things like they were, but in the new dir
@@ -1336,7 +1344,7 @@ func createTestExportFile(filename string, flags *createTestExportFlags) error {
 
 		sha := map[string]string{}
 
-		// create dummy archive.tgz
+		// create test archive.tgz
 		archiveWriter, err := zipW.CreateHeader(&zip.FileHeader{Name: "archive.tgz"})
 		if err != nil {
 			return err
@@ -1424,13 +1432,13 @@ func makeMockSnapshotZipContent(c *check.C) []byte {
 	buf := bytes.NewBuffer(nil)
 	zipW := zip.NewWriter(buf)
 
-	// create dummy archive.tgz
+	// create test archive.tgz
 	archiveWriter, err := zipW.CreateHeader(&zip.FileHeader{Name: "archive.tgz"})
 	c.Assert(err, check.IsNil)
 	_, err = archiveWriter.Write([]byte("mock archive.tgz content"))
 	c.Assert(err, check.IsNil)
 
-	// create dummy meta.json
+	// create test meta.json
 	archiveWriter, err = zipW.CreateHeader(&zip.FileHeader{Name: "meta.json"})
 	c.Assert(err, check.IsNil)
 	_, err = archiveWriter.Write([]byte("{}"))
