@@ -73,12 +73,12 @@ func Manager(st *state.State, runner *state.TaskRunner) *SnapshotManager {
 	runner.AddHandler("forget-snapshot", doForget, nil)
 	runner.AddHandler("check-snapshot", doCheck, nil)
 	runner.AddHandler("restore-snapshot", doRestore, undoRestore)
-	runner.AddCleanup("restore-snapshot", cleanupRestore)
+	runner.AddHandler("cleanup-after-restore", doCleanupAfterRestore, nil)
 
 	manager := &SnapshotManager{
 		state: st,
 	}
-	snapstate.AddAffectedSnapsByAttr("snapshot-setup", manager.affectedSnaps)
+	snapstate.RegisterAffectedSnapsByAttr("snapshot-setup", manager.affectedSnaps)
 
 	return manager
 }
@@ -371,6 +371,23 @@ func undoRestore(task *state.Task, _ *tomb.Tomb) error {
 	return nil
 }
 
+func doCleanupAfterRestore(task *state.Task, tomb *tomb.Tomb) error {
+	st := task.State()
+	st.Lock()
+	restoreTasks := task.WaitTasks()
+	st.Unlock()
+	for _, t := range restoreTasks {
+		if err := cleanupRestore(t, tomb); err != nil {
+			logger.Noticef("Cleanup of restore task %s failed: %v", task.ID(), err)
+			// do not quit the loop: we must perform all cleanups anyway
+		}
+	}
+
+	// Also, do not return an error here: we don't want a failed cleanup to
+	// trigger an undo of the restore operation
+	return nil
+}
+
 func cleanupRestore(task *state.Task, _ *tomb.Tomb) error {
 	var restoreState backend.RestoreState
 
