@@ -27,6 +27,7 @@ import (
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/snapstate/backend"
 )
 
@@ -52,6 +53,7 @@ func (s *aliasesSuite) SetUpTest(c *C) {
 	err := os.MkdirAll(dirs.SnapBinariesDir, 0755)
 	c.Assert(err, IsNil)
 	c.Assert(os.MkdirAll(dirs.CompletersDir, 0755), IsNil)
+	c.Assert(os.MkdirAll(dirs.LegacyCompletersDir, 0755), IsNil)
 	c.Assert(os.MkdirAll(filepath.Dir(dirs.CompleteShPath(s.base)), 0755), IsNil)
 	c.Assert(ioutil.WriteFile(dirs.CompleteShPath(s.base), nil, 0644), IsNil)
 }
@@ -92,7 +94,7 @@ func (s *aliasesSuite) TestMissingAliases(c *C) {
 	c.Check(missingCs, DeepEquals, []*backend.Alias{{Name: "foo", Target: "x.foo"}})
 }
 
-func matchingAliases(aliases []*backend.Alias) (matchingAs, matchingCs []*backend.Alias, err error) {
+func matchingAliasesCommon(aliases []*backend.Alias, completersDir string) (matchingAs, matchingCs []*backend.Alias, err error) {
 	for _, cand := range aliases {
 		target, err := os.Readlink(filepath.Join(dirs.SnapBinariesDir, cand.Name))
 		if err == nil {
@@ -103,7 +105,7 @@ func matchingAliases(aliases []*backend.Alias) (matchingAs, matchingCs []*backen
 			return nil, nil, err
 		}
 
-		target, err = os.Readlink(filepath.Join(dirs.CompletersDir, cand.Name))
+		target, err = os.Readlink(filepath.Join(completersDir, cand.Name))
 		if err == nil {
 			if target == cand.Target {
 				matchingCs = append(matchingCs, cand)
@@ -115,6 +117,14 @@ func matchingAliases(aliases []*backend.Alias) (matchingAs, matchingCs []*backen
 	return matchingAs, matchingCs, nil
 }
 
+func matchingAliases(aliases []*backend.Alias) (matchingAs, matchingCs []*backend.Alias, err error) {
+	return matchingAliasesCommon(aliases, dirs.CompletersDir)
+}
+
+func matchingAliasesLegacy(aliases []*backend.Alias) (matchingAs, matchingCs []*backend.Alias, err error) {
+	return matchingAliasesCommon(aliases, dirs.LegacyCompletersDir)
+}
+
 func (s *aliasesSuite) TestMatchingAliases(c *C) {
 	err := os.Symlink("x.foo", filepath.Join(dirs.SnapBinariesDir, "foo"))
 	c.Assert(err, IsNil)
@@ -147,6 +157,12 @@ func mkCompleters(c *C, base string, apps ...string) {
 	}
 }
 
+func mkCompletersLegacy(c *C, base string, apps ...string) {
+	for _, app := range apps {
+		c.Assert(os.Symlink(dirs.CompleteShPath(base), filepath.Join(dirs.LegacyCompletersDir, app)), IsNil)
+	}
+}
+
 func (s *aliasesSuite) TestUpdateAliasesAddWithCompleter(c *C) {
 	mkCompleters(c, s.base, "x.bar", "x.foo")
 	aliases := []*backend.Alias{{Name: "foo", Target: "x.foo"}, {Name: "bar", Target: "x.bar"}}
@@ -160,6 +176,38 @@ func (s *aliasesSuite) TestUpdateAliasesAddWithCompleter(c *C) {
 	c.Check(matchingCs, DeepEquals, aliases)
 }
 
+func (s *aliasesSuite) TestUpdateAliasesAddWithLegacyCompleter(c *C) {
+	mkCompletersLegacy(c, s.base, "x.bar", "x.foo")
+	aliases := []*backend.Alias{{Name: "foo", Target: "x.foo"}, {Name: "bar", Target: "x.bar"}}
+
+	err := s.be.UpdateAliases(aliases, nil)
+	c.Assert(err, IsNil)
+
+	matchingAs, matchingCs, err := matchingAliasesLegacy(aliases)
+	c.Assert(err, IsNil)
+	c.Check(matchingAs, DeepEquals, aliases)
+	c.Check(matchingCs, DeepEquals, aliases)
+}
+
+func (s *aliasesSuite) TestUpdateAliasesAddWithLegacyCompleterMigrate(c *C) {
+	mkCompleters(c, s.base, "x.bar", "x.foo")
+	aliases := []*backend.Alias{{Name: "foo", Target: "x.foo"}, {Name: "bar", Target: "x.bar"}}
+
+	c.Assert(os.Symlink("foo", filepath.Join(dirs.LegacyCompletersDir, "x.foo")), IsNil)
+	c.Assert(os.Symlink("bar", filepath.Join(dirs.LegacyCompletersDir, "x.bar")), IsNil)
+
+	err := s.be.UpdateAliases(aliases, nil)
+	c.Assert(err, IsNil)
+
+	matchingAs, matchingCs, err := matchingAliases(aliases)
+	c.Assert(err, IsNil)
+	c.Check(matchingAs, DeepEquals, aliases)
+	c.Check(matchingCs, DeepEquals, aliases)
+
+	c.Check(osutil.FileExists(filepath.Join(dirs.LegacyCompletersDir, "x.foo")), Equals, false)
+	c.Check(osutil.FileExists(filepath.Join(dirs.LegacyCompletersDir, "x.bar")), Equals, false)
+}
+
 func (s *aliasesSuite) TestUpdateAliasesAddIdempot(c *C) {
 	aliases := []*backend.Alias{{Name: "foo", Target: "x.foo"}, {Name: "bar", Target: "x.bar"}}
 
@@ -242,6 +290,21 @@ func (s *aliasesSuite) TestUpdateAliasesWithCompleterRemove(c *C) {
 	c.Check(matchingCs, HasLen, 0)
 }
 
+func (s *aliasesSuite) TestUpdateAliasesWithLegacyCompleterRemove(c *C) {
+	aliases := []*backend.Alias{{Name: "foo", Target: "x.foo"}, {Name: "bar", Target: "x.bar"}}
+	err := s.be.UpdateAliases(aliases, nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(os.Symlink("foo", filepath.Join(dirs.LegacyCompletersDir, "x.foo")), IsNil)
+	c.Assert(os.Symlink("bar", filepath.Join(dirs.LegacyCompletersDir, "x.bar")), IsNil)
+
+	err = s.be.UpdateAliases(nil, aliases)
+	c.Assert(err, IsNil)
+
+	c.Check(osutil.FileExists(filepath.Join(dirs.LegacyCompletersDir, "x.foo")), Equals, false)
+	c.Check(osutil.FileExists(filepath.Join(dirs.LegacyCompletersDir, "x.bar")), Equals, false)
+}
+
 func (s *aliasesSuite) TestUpdateAliasesRemoveIdempot(c *C) {
 	aliases := []*backend.Alias{{Name: "foo", Target: "x.foo"}, {Name: "bar", Target: "x.bar"}}
 
