@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2016 Canonical Ltd
+ * Copyright (C) 2014-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -95,7 +95,7 @@ func (s *copydataSuite) testCopyData(c *C, snapDir string, opts *dirs.SnapDirOpt
 
 	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
 	// just creates data dirs in this case
-	err = s.be.CopySnapData(v1, nil, progress.Null, opts)
+	err = s.be.CopySnapData(v1, nil, opts, progress.Null)
 	c.Assert(err, IsNil)
 
 	canaryDataFile := filepath.Join(v1.DataDir(), "canary.txt")
@@ -110,7 +110,7 @@ func (s *copydataSuite) testCopyData(c *C, snapDir string, opts *dirs.SnapDirOpt
 	c.Assert(err, IsNil)
 
 	v2 := snaptest.MockSnap(c, helloYaml2, &snap.SideInfo{Revision: snap.R(20)})
-	err = s.be.CopySnapData(v2, v1, progress.Null, opts)
+	err = s.be.CopySnapData(v2, v1, opts, progress.Null)
 	c.Assert(err, IsNil)
 
 	newCanaryDataFile := filepath.Join(dirs.SnapDataDir, "hello/20", "canary.txt")
@@ -133,11 +133,11 @@ func (s *copydataSuite) TestCopyDataBails(c *C) {
 	defer func() { dirs.SnapDataHomeGlob = oldSnapDataHomeGlob }()
 
 	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
-	c.Assert(s.be.CopySnapData(v1, nil, progress.Null, nil), IsNil)
+	c.Assert(s.be.CopySnapData(v1, nil, nil, progress.Null), IsNil)
 	c.Assert(os.Chmod(v1.DataDir(), 0), IsNil)
 
 	v2 := snaptest.MockSnap(c, helloYaml2, &snap.SideInfo{Revision: snap.R(20)})
-	err := s.be.CopySnapData(v2, v1, progress.Null, nil)
+	err := s.be.CopySnapData(v2, v1, nil, progress.Null)
 	c.Check(err, ErrorMatches, "cannot copy .*")
 }
 
@@ -150,7 +150,7 @@ func (s *copydataSuite) TestCopyDataNoUserHomes(c *C) {
 	dirs.SnapDataHomeGlob = filepath.Join(s.tempdir, "no-such-home", "*", "snap")
 
 	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
-	err := s.be.CopySnapData(v1, nil, progress.Null, nil)
+	err := s.be.CopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	canaryDataFile := filepath.Join(v1.DataDir(), "canary.txt")
@@ -161,7 +161,7 @@ func (s *copydataSuite) TestCopyDataNoUserHomes(c *C) {
 	c.Assert(err, IsNil)
 
 	v2 := snaptest.MockSnap(c, helloYaml2, &snap.SideInfo{Revision: snap.R(20)})
-	err = s.be.CopySnapData(v2, v1, progress.Null, nil)
+	err = s.be.CopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	_, err = os.Stat(filepath.Join(v2.DataDir(), "canary.txt"))
@@ -169,7 +169,7 @@ func (s *copydataSuite) TestCopyDataNoUserHomes(c *C) {
 	_, err = os.Stat(filepath.Join(v2.CommonDataDir(), "canary.common"))
 	c.Assert(err, IsNil)
 
-	// sanity atm
+	// validity atm
 	c.Check(v1.DataDir(), Not(Equals), v2.DataDir())
 	c.Check(v1.CommonDataDir(), Equals, v2.CommonDataDir())
 }
@@ -234,7 +234,7 @@ func (s *copydataSuite) testCopyDataUndo(c *C, snapDir string, opts *dirs.SnapDi
 	v2 := snaptest.MockSnap(c, helloYaml2, &snap.SideInfo{Revision: snap.R(20)})
 
 	// copy data
-	err := s.be.CopySnapData(v2, v1, progress.Null, opts)
+	err := s.be.CopySnapData(v2, v1, opts, progress.Null)
 	c.Assert(err, IsNil)
 	v2data := filepath.Join(dirs.SnapDataDir, "hello/20")
 	l, err := filepath.Glob(filepath.Join(v2data, "*"))
@@ -245,7 +245,7 @@ func (s *copydataSuite) testCopyDataUndo(c *C, snapDir string, opts *dirs.SnapDi
 	c.Assert(err, IsNil)
 	c.Assert(l, HasLen, 1)
 
-	err = s.be.UndoCopySnapData(v2, v1, progress.Null, opts)
+	err = s.be.UndoCopySnapData(v2, v1, opts, progress.Null)
 	c.Assert(err, IsNil)
 
 	// now removed
@@ -268,14 +268,14 @@ func (s *copydataSuite) TestCopyDataDoUndoNoUserHomes(c *C) {
 	v2 := snaptest.MockSnap(c, helloYaml2, &snap.SideInfo{Revision: snap.R(20)})
 
 	// copy data
-	err := s.be.CopySnapData(v2, v1, progress.Null, nil)
+	err := s.be.CopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 	v2data := filepath.Join(dirs.SnapDataDir, "hello/20")
 	l, err := filepath.Glob(filepath.Join(v2data, "*"))
 	c.Assert(err, IsNil)
 	c.Assert(l, HasLen, 1)
 
-	err = s.be.UndoCopySnapData(v2, v1, progress.Null, nil)
+	err = s.be.UndoCopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	// now removed
@@ -287,14 +287,14 @@ func (s *copydataSuite) TestCopyDataDoUndoFirstInstall(c *C) {
 	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
 
 	// first install
-	err := s.be.CopySnapData(v1, nil, progress.Null, nil)
+	err := s.be.CopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 	_, err = os.Stat(v1.DataDir())
 	c.Assert(err, IsNil)
 	_, err = os.Stat(v1.CommonDataDir())
 	c.Assert(err, IsNil)
 
-	err = s.be.UndoCopySnapData(v1, nil, progress.Null, nil)
+	err = s.be.UndoCopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 	_, err = os.Stat(v1.DataDir())
 	c.Check(os.IsNotExist(err), Equals, true)
@@ -320,7 +320,7 @@ func (s *copydataSuite) testCopyDataDoABA(c *C, opts *dirs.SnapDirOptions) {
 	c.Check(s.populatedData("20"), Equals, "20\n")
 
 	// and now we pretend to refresh back to v1 (r10)
-	c.Check(s.be.CopySnapData(v1, v2, progress.Null, opts), IsNil)
+	c.Check(s.be.CopySnapData(v1, v2, opts, progress.Null), IsNil)
 
 	// so 10 now has 20's data
 	c.Check(s.populatedData("10"), Equals, "20\n")
@@ -345,14 +345,14 @@ func (s *copydataSuite) TestCopyDataDoUndoABA(c *C) {
 	c.Check(s.populatedData("20"), Equals, "20\n")
 
 	// and now we pretend to refresh back to v1 (r10)
-	c.Check(s.be.CopySnapData(v1, v2, progress.Null, nil), IsNil)
+	c.Check(s.be.CopySnapData(v1, v2, nil, progress.Null), IsNil)
 
 	// so v1 (r10) now has v2 (r20)'s data and we have trash
 	c.Check(s.populatedData("10"), Equals, "20\n")
 	c.Check(s.populatedData("10.old"), Equals, "10\n")
 
 	// but oh no! we have to undo it!
-	c.Check(s.be.UndoCopySnapData(v1, v2, progress.Null, nil), IsNil)
+	c.Check(s.be.UndoCopySnapData(v1, v2, nil, progress.Null), IsNil)
 
 	// so now v1 (r10) has v1 (r10)'s data and v2 (r20) has v2 (r20)'s data and we have no trash
 	c.Check(s.populatedData("10"), Equals, "10\n")
@@ -372,10 +372,10 @@ func (s *copydataSuite) TestCopyDataDoIdempotent(c *C) {
 	v2 := snaptest.MockSnap(c, helloYaml2, &snap.SideInfo{Revision: snap.R(20)})
 
 	// copy data
-	err := s.be.CopySnapData(v2, v1, progress.Null, nil)
+	err := s.be.CopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
-	err = s.be.CopySnapData(v2, v1, progress.Null, nil)
+	err = s.be.CopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	v2data := filepath.Join(dirs.SnapDataDir, "hello/20")
@@ -399,15 +399,15 @@ func (s *copydataSuite) TestCopyDataUndoIdempotent(c *C) {
 	v2 := snaptest.MockSnap(c, helloYaml2, &snap.SideInfo{Revision: snap.R(20)})
 
 	// copy data
-	err := s.be.CopySnapData(v2, v1, progress.Null, nil)
+	err := s.be.CopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	v2data := filepath.Join(dirs.SnapDataDir, "hello/20")
 
-	err = s.be.UndoCopySnapData(v2, v1, progress.Null, nil)
+	err = s.be.UndoCopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
-	err = s.be.UndoCopySnapData(v2, v1, progress.Null, nil)
+	err = s.be.UndoCopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	// now removed
@@ -422,10 +422,10 @@ func (s *copydataSuite) TestCopyDataDoFirstInstallIdempotent(c *C) {
 	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
 
 	// first install
-	err := s.be.CopySnapData(v1, nil, progress.Null, nil)
+	err := s.be.CopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 
-	err = s.be.CopySnapData(v1, nil, progress.Null, nil)
+	err = s.be.CopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	_, err = os.Stat(v1.DataDir())
@@ -433,7 +433,7 @@ func (s *copydataSuite) TestCopyDataDoFirstInstallIdempotent(c *C) {
 	_, err = os.Stat(v1.CommonDataDir())
 	c.Assert(err, IsNil)
 
-	err = s.be.UndoCopySnapData(v1, nil, progress.Null, nil)
+	err = s.be.UndoCopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 	_, err = os.Stat(v1.DataDir())
 	c.Check(os.IsNotExist(err), Equals, true)
@@ -445,17 +445,17 @@ func (s *copydataSuite) TestCopyDataUndoFirstInstallIdempotent(c *C) {
 	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
 
 	// first install
-	err := s.be.CopySnapData(v1, nil, progress.Null, nil)
+	err := s.be.CopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 	_, err = os.Stat(v1.DataDir())
 	c.Assert(err, IsNil)
 	_, err = os.Stat(v1.CommonDataDir())
 	c.Assert(err, IsNil)
 
-	err = s.be.UndoCopySnapData(v1, nil, progress.Null, nil)
+	err = s.be.UndoCopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 
-	err = s.be.UndoCopySnapData(v1, nil, progress.Null, nil)
+	err = s.be.UndoCopySnapData(v1, nil, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	_, err = os.Stat(v1.DataDir())
@@ -478,7 +478,7 @@ func (s *copydataSuite) TestCopyDataCopyFailure(c *C) {
 	}
 
 	// copy data will fail
-	err := s.be.CopySnapData(v2, v1, progress.Null, nil)
+	err := s.be.CopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, ErrorMatches, fmt.Sprintf(`cannot copy %s to %s: .*: "cp: boom" \(3\)`, q(v1.DataDir()), q(v2.DataDir())))
 }
 
@@ -492,7 +492,7 @@ func (s *copydataSuite) TestCopyDataPartialFailure(c *C) {
 	// pretend we install a new version
 	v2 := snaptest.MockSnap(c, helloYaml2, &snap.SideInfo{Revision: snap.R(20)})
 
-	// sanity check: the 20 dirs don't exist yet (but 10 do)
+	// precondition check: the 20 dirs don't exist yet (but 10 do)
 	for _, dir := range []string{dirs.SnapDataDir, homedir1, homedir2} {
 		c.Assert(osutil.FileExists(filepath.Join(dir, "hello", "20")), Equals, false, Commentf(dir))
 		c.Assert(osutil.FileExists(filepath.Join(dir, "hello", "10")), Equals, true, Commentf(dir))
@@ -501,7 +501,7 @@ func (s *copydataSuite) TestCopyDataPartialFailure(c *C) {
 	c.Assert(os.Chmod(filepath.Join(homedir2, "hello", "10", "canary.home"), 0), IsNil)
 
 	// try to copy data
-	err := s.be.CopySnapData(v2, v1, progress.Null, nil)
+	err := s.be.CopySnapData(v2, v1, nil, progress.Null)
 	c.Assert(err, NotNil)
 
 	// the copy data failed, so check it cleaned up after itself (but not too much!)
@@ -532,7 +532,7 @@ func (s *copydataSuite) TestCopyDataSameRevision(c *C) {
 	}
 
 	// copy data works
-	err := s.be.CopySnapData(v1, v1, progress.Null, nil)
+	err := s.be.CopySnapData(v1, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	// the data is still there :-)
@@ -568,7 +568,7 @@ func (s *copydataSuite) TestUndoCopyDataSameRevision(c *C) {
 	}
 
 	// undo copy data works
-	err := s.be.UndoCopySnapData(v1, v1, progress.Null, nil)
+	err := s.be.UndoCopySnapData(v1, v1, nil, progress.Null)
 	c.Assert(err, IsNil)
 
 	// the data is still there :-)
@@ -582,6 +582,108 @@ func (s *copydataSuite) TestUndoCopyDataSameRevision(c *C) {
 	}
 }
 
+const (
+	mountRunMntUbuntuSaveFmt = `26 27 8:3 / %s/run/mnt/ubuntu-save rw,relatime shared:7 - ext4 /dev/fakedevice0p1 rw,data=ordered`
+	mountSnapSaveFmt         = `26 27 8:3 / %s/var/lib/snapd/save rw,relatime shared:7 - ext4 /dev/fakedevice0p1 rw,data=ordered`
+)
+
+func (s *copydataSuite) TestSetupCommonSaveDataClassic(c *C) {
+	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
+
+	// first install
+	err := s.be.SetupSnapSaveData(v1, mockClassicDev, progress.Null)
+	c.Assert(err, IsNil)
+	_, err = os.Stat(v1.CommonDataSaveDir())
+	c.Assert(err.Error(), Equals, fmt.Sprintf("stat %s/var/lib/snapd/save/snap/hello: no such file or directory", dirs.GlobalRootDir))
+}
+
+func (s *copydataSuite) TestSetupCommonSaveDataCoreNoMount(c *C) {
+	restore := osutil.MockMountInfo("")
+	defer restore()
+	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
+
+	// first install
+	err := s.be.SetupSnapSaveData(v1, mockDev, progress.Null)
+	c.Assert(err, IsNil)
+	_, err = os.Stat(v1.CommonDataSaveDir())
+	c.Assert(err.Error(), Equals, fmt.Sprintf("stat %s/var/lib/snapd/save/snap/hello: no such file or directory", dirs.GlobalRootDir))
+}
+
+func (s *copydataSuite) TestSetupCommonSaveDataFirstInstall(c *C) {
+	restore := osutil.MockMountInfo(fmt.Sprintf(mountRunMntUbuntuSaveFmt, dirs.GlobalRootDir) + "\n" +
+		fmt.Sprintf(mountSnapSaveFmt, dirs.GlobalRootDir))
+	defer restore()
+
+	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
+
+	// first install
+	err := s.be.SetupSnapSaveData(v1, mockDev, progress.Null)
+	c.Assert(err, IsNil)
+	_, err = os.Stat(v1.CommonDataSaveDir())
+	c.Assert(err, IsNil)
+
+	// create a test file to make sure this also gets removed
+	c.Assert(ioutil.WriteFile(filepath.Join(v1.CommonDataSaveDir(), "canary.txt"), nil, 0644), IsNil)
+
+	// removes correctly when no previous info is present
+	err = s.be.UndoSetupSnapSaveData(v1, nil, mockDev, progress.Null)
+	c.Assert(err, IsNil)
+	_, err = os.Stat(v1.CommonDataSaveDir())
+	c.Check(os.IsNotExist(err), Equals, true)
+
+	// verify that the root (snap) folder has not been touched
+	exists, isDir, err := osutil.DirExists(dirs.SnapDataSaveDir)
+	c.Check(err, IsNil)
+	c.Check(exists && isDir, Equals, true)
+}
+
+func (s *copydataSuite) TestSetupCommonSaveDataSameRevision(c *C) {
+	restore := osutil.MockMountInfo(fmt.Sprintf(mountRunMntUbuntuSaveFmt, dirs.GlobalRootDir) + "\n" +
+		fmt.Sprintf(mountSnapSaveFmt, dirs.GlobalRootDir))
+	defer restore()
+
+	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
+
+	c.Assert(os.MkdirAll(v1.CommonDataSaveDir(), 0755), IsNil)
+	c.Assert(ioutil.WriteFile(filepath.Join(v1.CommonDataSaveDir(), "canary.txt"), nil, 0644), IsNil)
+	c.Assert(osutil.FileExists(filepath.Join(v1.CommonDataSaveDir(), "canary.txt")), Equals, true)
+
+	// setup snap save data works
+	err := s.be.SetupSnapSaveData(v1, mockDev, progress.Null)
+	c.Assert(err, IsNil)
+
+	// assert data still is there
+	c.Assert(osutil.FileExists(filepath.Join(v1.CommonDataSaveDir(), "canary.txt")), Equals, true)
+}
+
+func (s *copydataSuite) TestUndoSetupCommonSaveDataClassic(c *C) {
+	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
+
+	c.Assert(os.MkdirAll(v1.CommonDataSaveDir(), 0755), IsNil)
+	c.Assert(ioutil.WriteFile(filepath.Join(v1.CommonDataSaveDir(), "canary.txt"), nil, 0644), IsNil)
+	c.Assert(osutil.FileExists(filepath.Join(v1.CommonDataSaveDir(), "canary.txt")), Equals, true)
+
+	// make sure that undo doesn't do anything on a classic system
+	err := s.be.UndoSetupSnapSaveData(v1, v1, mockClassicDev, progress.Null)
+	c.Assert(err, IsNil)
+
+	c.Assert(osutil.FileExists(filepath.Join(v1.CommonDataSaveDir(), "canary.txt")), Equals, true)
+}
+
+func (s *copydataSuite) TestUndoSetupCommonSaveDataSameRevision(c *C) {
+	v1 := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
+
+	c.Assert(os.MkdirAll(v1.CommonDataSaveDir(), 0755), IsNil)
+	c.Assert(ioutil.WriteFile(filepath.Join(v1.CommonDataSaveDir(), "canary.txt"), nil, 0644), IsNil)
+	c.Assert(osutil.FileExists(filepath.Join(v1.CommonDataSaveDir(), "canary.txt")), Equals, true)
+
+	// make sure that undo doesn't do anything with a previous version present
+	err := s.be.UndoSetupSnapSaveData(v1, v1, mockDev, progress.Null)
+	c.Assert(err, IsNil)
+
+	c.Assert(osutil.FileExists(filepath.Join(v1.CommonDataSaveDir(), "canary.txt")), Equals, true)
+}
+
 func (s *copydataSuite) TestHideSnapData(c *C) {
 	info := snaptest.MockSnap(c, helloYaml1, &snap.SideInfo{Revision: snap.R(10)})
 
@@ -973,7 +1075,7 @@ func (s *copydataSuite) TestInitSnapUserHome(c *C) {
 	dirPath := filepath.Join(revDir, "dir")
 	c.Assert(os.Mkdir(dirPath, 0775), IsNil)
 
-	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev)
+	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev, opts)
 	c.Assert(err, IsNil)
 	exposedHome := filepath.Join(homeDir, dirs.ExposedSnapHomeDir, snapName)
 	c.Check(undoInfo.Created, DeepEquals, []string{exposedHome})
@@ -1025,7 +1127,7 @@ func (s *copydataSuite) TestInitExposedHomeIgnoreXDGDirs(c *C) {
 	localPath := filepath.Join(revDir, ".local", "share")
 	c.Assert(os.MkdirAll(localPath, 0700), IsNil)
 
-	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev)
+	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev, opts)
 	c.Assert(err, IsNil)
 	exposedHome := snap.UserExposedHomeDir(homeDir, snapName)
 	c.Check(undoInfo.Created, DeepEquals, []string{exposedHome})
@@ -1074,7 +1176,7 @@ func (s *copydataSuite) TestInitSnapFailOnFirstErr(c *C) {
 	rev, err := snap.ParseRevision("2")
 	c.Assert(err, IsNil)
 
-	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev)
+	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev, nil)
 	c.Assert(err, ErrorMatches, ".*: boom")
 	c.Check(undoInfo, IsNil)
 
@@ -1123,7 +1225,7 @@ func (s *copydataSuite) TestInitSnapUndoOnErr(c *C) {
 	rev, err := snap.ParseRevision("2")
 	c.Assert(err, IsNil)
 
-	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev)
+	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev, nil)
 	c.Assert(err, ErrorMatches, ".*: boom")
 	c.Check(undoInfo, IsNil)
 
@@ -1152,7 +1254,7 @@ func (s *copydataSuite) TestInitSnapNothingToCopy(c *C) {
 	rev, err := snap.ParseRevision("2")
 	c.Assert(err, IsNil)
 
-	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev)
+	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev, nil)
 	c.Assert(err, IsNil)
 	c.Check(undoInfo.Created, DeepEquals, []string{snap.UserExposedHomeDir(usr.HomeDir, snapName)})
 
@@ -1187,7 +1289,7 @@ func (s *copydataSuite) TestInitAlreadyExistsFile(c *C) {
 	rev, err := snap.ParseRevision("2")
 	c.Assert(err, IsNil)
 
-	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev)
+	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev, nil)
 	c.Assert(err, ErrorMatches, fmt.Sprintf("cannot initialize new user HOME %q: already exists but is not a directory", newHome))
 	c.Check(undoInfo, IsNil)
 
@@ -1217,7 +1319,7 @@ func (s *copydataSuite) TestInitAlreadyExistsDir(c *C) {
 	rev, err := snap.ParseRevision("2")
 	c.Assert(err, IsNil)
 
-	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev)
+	undoInfo, err := s.be.InitExposedSnapHome(snapName, rev, nil)
 	c.Assert(err, IsNil)
 	c.Check(undoInfo.Created, HasLen, 0)
 
