@@ -20,6 +20,7 @@
 package snapstate
 
 import (
+	"errors"
 	"fmt"
 	"reflect"
 
@@ -51,6 +52,11 @@ func (e *ChangeConflictError) Error() string {
 	return fmt.Sprintf("snap %q has changes in progress", e.Snap)
 }
 
+func (e *ChangeConflictError) Is(err error) bool {
+	_, ok := err.(*ChangeConflictError)
+	return ok
+}
+
 // An AffectedSnapsFunc returns a list of affected snap names for the given supported task.
 type AffectedSnapsFunc func(*state.Task) ([]string, error)
 
@@ -59,13 +65,13 @@ var (
 	affectedSnapsByKind = make(map[string]AffectedSnapsFunc)
 )
 
-// AddAffectedSnapsByAttr registers an AffectedSnapsFunc for returning the affected snaps for tasks sporting the given identifying attribute, to use in conflicts detection.
-func AddAffectedSnapsByAttr(attr string, f AffectedSnapsFunc) {
+// RegisterAffectedSnapsByAttr registers an AffectedSnapsFunc for returning the affected snaps for tasks sporting the given identifying attribute, to use in conflicts detection.
+func RegisterAffectedSnapsByAttr(attr string, f AffectedSnapsFunc) {
 	affectedSnapsByAttr[attr] = f
 }
 
-// AddAffectedSnapsByKind registers an AffectedSnapsFunc for returning the affected snaps for tasks of the given kind, to use in conflicts detection. Whenever possible using AddAffectedSnapsByAttr should be preferred.
-func AddAffectedSnapsByKind(kind string, f AffectedSnapsFunc) {
+// RegisterAffectedSnapsByKind registers an AffectedSnapsFunc for returning the affected snaps for tasks of the given kind, to use in conflicts detection. Whenever possible using RegisterAffectedSnapsByAttr should be preferred.
+func RegisterAffectedSnapsByKind(kind string, f AffectedSnapsFunc) {
 	affectedSnapsByKind[kind] = f
 }
 
@@ -225,7 +231,7 @@ func checkChangeConflictIgnoringOneChange(st *state.State, instanceName string,
 		// install, while getting the snap info; for refresh, when
 		// getting what needs refreshing).
 		var cursnapst SnapState
-		if err := Get(st, instanceName, &cursnapst); err != nil && err != state.ErrNoState {
+		if err := Get(st, instanceName, &cursnapst); err != nil && !errors.Is(err, state.ErrNoState) {
 			return err
 		}
 
