@@ -48,7 +48,7 @@ func logstr(task *state.Task) string {
 }
 
 func changeWithLanesAndSnapSetups(st *state.State, snapNames ...string) *state.Change {
-	chg := st.NewChange("dummy", "...")
+	chg := st.NewChange("sample", "...")
 	for _, snapName := range snapNames {
 		lane := st.NewLane()
 		tsk := st.NewTask(fmt.Sprintf("a-task-for-snap-%s-in-lane-%d", snapName, lane), "test")
@@ -80,7 +80,7 @@ func (s *reRefreshSuite) TestDoCheckReRefreshFailsWithoutReRefreshSetup(c *C) {
 }
 
 func (s *reRefreshSuite) TestDoCheckReRefreshFailsIfUpdateFails(c *C) {
-	defer snapstate.MockReRefreshUpdateMany(func(context.Context, *state.State, []string, int, snapstate.UpdateFilter, *snapstate.Flags, string) ([]string, []*state.TaskSet, error) {
+	defer snapstate.MockReRefreshUpdateMany(func(context.Context, *state.State, []string, []*snapstate.RevisionOptions, int, snapstate.UpdateFilter, *snapstate.Flags, string) ([]string, []*state.TaskSet, error) {
 		return nil, nil, errors.New("bzzt")
 	})()
 
@@ -103,7 +103,7 @@ func (s *reRefreshSuite) TestDoCheckReRefreshFailsIfUpdateFails(c *C) {
 
 func (s *reRefreshSuite) TestDoCheckReRefreshNoReRefreshes(c *C) {
 	updaterCalled := false
-	defer snapstate.MockReRefreshUpdateMany(func(context.Context, *state.State, []string, int, snapstate.UpdateFilter, *snapstate.Flags, string) ([]string, []*state.TaskSet, error) {
+	defer snapstate.MockReRefreshUpdateMany(func(context.Context, *state.State, []string, []*snapstate.RevisionOptions, int, snapstate.UpdateFilter, *snapstate.Flags, string) ([]string, []*state.TaskSet, error) {
 		updaterCalled = true
 		return nil, nil, nil
 	})()
@@ -128,7 +128,7 @@ func (s *reRefreshSuite) TestDoCheckReRefreshNoReRefreshes(c *C) {
 
 func (s *reRefreshSuite) TestDoCheckReRefreshPassesReRefreshSetupData(c *C) {
 	var chgID string
-	defer snapstate.MockReRefreshUpdateMany(func(ctx context.Context, st *state.State, snaps []string, userID int, filter snapstate.UpdateFilter, flags *snapstate.Flags, changeID string) ([]string, []*state.TaskSet, error) {
+	defer snapstate.MockReRefreshUpdateMany(func(_ context.Context, _ *state.State, snaps []string, _ []*snapstate.RevisionOptions, userID int, _ snapstate.UpdateFilter, flags *snapstate.Flags, changeID string) ([]string, []*state.TaskSet, error) {
 		c.Check(changeID, Equals, chgID)
 		expected := []string{"won", "too", "tree"}
 		sort.Strings(expected)
@@ -165,7 +165,7 @@ func (s *reRefreshSuite) TestDoCheckReRefreshPassesReRefreshSetupData(c *C) {
 }
 
 func (s *reRefreshSuite) TestDoCheckReRefreshAddsNewTasks(c *C) {
-	defer snapstate.MockReRefreshUpdateMany(func(ctx context.Context, st *state.State, snaps []string, userID int, filter snapstate.UpdateFilter, flags *snapstate.Flags, changeID string) ([]string, []*state.TaskSet, error) {
+	defer snapstate.MockReRefreshUpdateMany(func(_ context.Context, st *state.State, snaps []string, _ []*snapstate.RevisionOptions, _ int, _ snapstate.UpdateFilter, _ *snapstate.Flags, _ string) ([]string, []*state.TaskSet, error) {
 		expected := []string{"won", "too", "tree"}
 		sort.Strings(expected)
 		sort.Strings(snaps)
@@ -216,13 +216,13 @@ func refreshedSnaps(task *state.Task) string {
 // for a snap with t1snap, the second one with status t2status.
 func addLane(st *state.State, chg *state.Change, t1snap string, t2status state.Status) {
 	lane := st.NewLane()
-	t1 := st.NewTask("dummy1", "...")
+	t1 := st.NewTask("test1", "...")
 	t1.JoinLane(lane)
 	t1.Set("snap-setup", snapstate.SnapSetup{SideInfo: &snap.SideInfo{RealName: t1snap}})
 	t1.SetStatus(state.DoneStatus)
 	chg.AddTask(t1)
 
-	t2 := st.NewTask("dummy2", "...")
+	t2 := st.NewTask("test2", "...")
 	t2.JoinLane(lane)
 	t2.WaitFor(t1)
 	t2.SetStatus(t2status)
@@ -299,11 +299,11 @@ func (s *reRefreshSuite) TestLaneSnapsTwoSetups(c *C) {
 	defer s.state.Unlock()
 
 	ts := state.NewTaskSet()
-	t1 := s.state.NewTask("dummy1", "...")
+	t1 := s.state.NewTask("test1", "...")
 	t1.Set("snap-setup", snapstate.SnapSetup{SideInfo: &snap.SideInfo{RealName: "one"}})
 	t1.SetStatus(state.DoneStatus)
 	ts.AddTask(t1)
-	t2 := s.state.NewTask("dummy2", "...")
+	t2 := s.state.NewTask("test2", "...")
 	t2.Set("snap-setup", snapstate.SnapSetup{SideInfo: &snap.SideInfo{RealName: "two"}})
 	t2.WaitFor(t1)
 	ts.AddTask(t2)
@@ -324,11 +324,11 @@ func (s *reRefreshSuite) TestLaneSnapsBadSetup(c *C) {
 	defer s.state.Unlock()
 
 	ts := state.NewTaskSet()
-	t1 := s.state.NewTask("dummy1", "...")
+	t1 := s.state.NewTask("test1", "...")
 	t1.Set("snap-setup", "what is this")
 	t1.SetStatus(state.DoneStatus)
 	ts.AddTask(t1)
-	t2 := s.state.NewTask("dummy2", "...")
+	t2 := s.state.NewTask("test2", "...")
 	t2.Set("snap-setup", snapstate.SnapSetup{SideInfo: &snap.SideInfo{RealName: "two"}})
 	t2.WaitFor(t1)
 	ts.AddTask(t2)
@@ -373,8 +373,10 @@ func (s *reRefreshSuite) TestFilterReturnsFalseIfEpochEqualZero(c *C) {
 	c.Check(snapstate.ReRefreshFilter(&snap.Info{Epoch: snap.Epoch{}}, snapst), Equals, false)
 }
 
+// validation-sets related tests
+
 func (s *refreshSuite) TestMaybeRestoreValidationSetsAndRevertSnaps(c *C) {
-	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVs *asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
+	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 		return nil, nil
 	})
 	defer restore()
@@ -385,14 +387,14 @@ func (s *refreshSuite) TestMaybeRestoreValidationSetsAndRevertSnaps(c *C) {
 
 	refreshedSnaps := []string{"foo", "bar"}
 	// nothing to do with no enforced validation sets
-	ts, err := snapstate.MaybeRestoreValidationSetsAndRevertSnaps(st, refreshedSnaps)
+	ts, err := snapstate.MaybeRestoreValidationSetsAndRevertSnaps(st, refreshedSnaps, "")
 	c.Assert(err, IsNil)
 	c.Check(ts, IsNil)
 }
 
 func (s *validationSetsSuite) TestMaybeRestoreValidationSetsAndRevertSnapsOneRevert(c *C) {
 	var enforcedValidationSetsCalled int
-	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVs *asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
+	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 		enforcedValidationSetsCalled++
 
 		vs := snapasserts.NewValidationSets()
@@ -484,9 +486,18 @@ func (s *validationSetsSuite) TestMaybeRestoreValidationSetsAndRevertSnapsOneRev
 	})
 	snaptest.MockSnap(c, `name: some-snap3`, si3)
 
+	chg := s.state.NewChange("install change", "...")
+	t1 := s.state.NewTask("link-snap", "...")
+	t1.Set("snap-setup", &snapstate.SnapSetup{SideInfo: &snap.SideInfo{RealName: "some-snap1", SnapID: "aaqKhntON3vR7kwEbVPsILm7bUViPDzx", Revision: snap.R(1)}})
+	t1.SetStatus(state.DoneStatus)
+	t2 := s.state.NewTask("check-rerefresh", "...")
+	chg.AddTask(t1)
+	chg.AddTask(t2)
+
 	// some-snap2 failed to refresh
 	refreshedSnaps := []string{"some-snap1", "some-snap3"}
-	ts, err := snapstate.MaybeRestoreValidationSetsAndRevertSnaps(st, refreshedSnaps)
+	// pass change id to make sure revert doesn't conflict
+	ts, err := snapstate.MaybeRestoreValidationSetsAndRevertSnaps(st, refreshedSnaps, chg.ID())
 	c.Assert(err, IsNil)
 
 	// we expect revert of snap1
@@ -518,9 +529,60 @@ func (s *validationSetsSuite) TestMaybeRestoreValidationSetsAndRevertSnapsOneRev
 	c.Check(enforcedValidationSetsCalled, Equals, 2)
 }
 
+func (s *validationSetsSuite) TestMaybeRestoreValidationSetsAndRevertNoSnapsRefreshed(c *C) {
+	var enforcedValidationSetsCalled int
+	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
+		enforcedValidationSetsCalled++
+
+		vs := snapasserts.NewValidationSets()
+		snap1 := map[string]interface{}{
+			"id":       "aaqKhntON3vR7kwEbVPsILm7bUViPDzx",
+			"name":     "some-snap1",
+			"presence": "required",
+			"revision": "3",
+		}
+
+		c.Assert(enforcedValidationSetsCalled, Equals, 1, Commentf("unexpected call to EnforcedValidatioSets"))
+		vsa1 := s.mockValidationSetAssert(c, "bar", "2", snap1)
+		vs.Add(vsa1.(*asserts.ValidationSet))
+		return vs, nil
+	})
+	defer restore()
+
+	var restoreValidationSetsTrackingCalled int
+	restoreRestoreValidationSetsTracking := snapstate.MockRestoreValidationSetsTracking(func(*state.State) error {
+		restoreValidationSetsTrackingCalled++
+		return nil
+	})
+	defer restoreRestoreValidationSetsTracking()
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// snaps in the system
+	si1 := &snap.SideInfo{RealName: "some-snap1", SnapID: "aaqKhntON3vR7kwEbVPsILm7bUViPDzx", Revision: snap.R(1)}
+	snapstate.Set(s.state, "some-snap1", &snapstate.SnapState{
+		Active:   true,
+		Sequence: []*snap.SideInfo{si1},
+		Current:  snap.R(1),
+		SnapType: "app",
+	})
+	snaptest.MockSnap(c, `name: some-snap1`, si1)
+
+	// no snaps get refreshed
+	ts, err := snapstate.MaybeRestoreValidationSetsAndRevertSnaps(st, nil, "")
+	c.Assert(err, IsNil)
+
+	// we expect no snap reverts
+	c.Assert(ts, HasLen, 0)
+	c.Check(restoreValidationSetsTrackingCalled, Equals, 1)
+	c.Check(enforcedValidationSetsCalled, Equals, 1)
+}
+
 func (s *validationSetsSuite) TestMaybeRestoreValidationSetsAndRevertJustValidationSetsRestore(c *C) {
 	var enforcedValidationSetsCalled int
-	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVs *asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
+	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 		enforcedValidationSetsCalled++
 
 		vs := snapasserts.NewValidationSets()
@@ -589,7 +651,7 @@ func (s *validationSetsSuite) TestMaybeRestoreValidationSetsAndRevertJustValidat
 	snaptest.MockSnap(c, `name: some-snap2`, si3)
 
 	refreshedSnaps := []string{"some-snap2"}
-	ts, err := snapstate.MaybeRestoreValidationSetsAndRevertSnaps(st, refreshedSnaps)
+	ts, err := snapstate.MaybeRestoreValidationSetsAndRevertSnaps(st, refreshedSnaps, "")
 	c.Assert(err, IsNil)
 
 	// we expect no snap reverts
@@ -597,3 +659,72 @@ func (s *validationSetsSuite) TestMaybeRestoreValidationSetsAndRevertJustValidat
 	c.Check(restoreValidationSetsTrackingCalled, Equals, 1)
 	c.Check(enforcedValidationSetsCalled, Equals, 2)
 }
+
+func (s *validationSetsSuite) TestMaybeRestoreValidationSetsAndRevertStillValid(c *C) {
+	var enforcedValidationSetsCalled int
+	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
+		enforcedValidationSetsCalled++
+
+		vs := snapasserts.NewValidationSets()
+		snap2 := map[string]interface{}{
+			"id":       "abcKhntON3vR7kwEbVPsILm7bUViPDzx",
+			"name":     "some-snap2",
+			"presence": "required",
+		}
+
+		c.Assert(enforcedValidationSetsCalled, Equals, 1, Commentf("unexpected call to EnforcedValidatioSets"))
+
+		// refreshed validation sets
+		// snap1 revision 3 is now required (but snap wasn't refreshed)
+		snap1 := map[string]interface{}{
+			"id":       "aaqKhntON3vR7kwEbVPsILm7bUViPDzx",
+			"name":     "some-snap1",
+			"presence": "required",
+		}
+		vsa1 := s.mockValidationSetAssert(c, "bar", "3", snap1, snap2)
+		vs.Add(vsa1.(*asserts.ValidationSet))
+		return vs, nil
+	})
+	defer restore()
+
+	var restoreValidationSetsTrackingCalled int
+	restoreRestoreValidationSetsTracking := snapstate.MockRestoreValidationSetsTracking(func(*state.State) error {
+		restoreValidationSetsTrackingCalled++
+		return nil
+	})
+	defer restoreRestoreValidationSetsTracking()
+
+	st := s.state
+	st.Lock()
+	defer st.Unlock()
+
+	// snaps in the system after partial refresh
+	si1 := &snap.SideInfo{RealName: "some-snap1", SnapID: "aaqKhntON3vR7kwEbVPsILm7bUViPDzx", Revision: snap.R(3)}
+	snapstate.Set(s.state, "some-snap1", &snapstate.SnapState{
+		Active:   true,
+		Sequence: []*snap.SideInfo{si1},
+		Current:  snap.R(3),
+		SnapType: "app",
+	})
+	snaptest.MockSnap(c, `name: some-snap1`, si1)
+
+	si3 := &snap.SideInfo{RealName: "some-snap2", SnapID: "abcKhntON3vR7kwEbVPsILm7bUViPDzx", Revision: snap.R(1)}
+	snapstate.Set(s.state, "some-snap2", &snapstate.SnapState{
+		Active:   true,
+		Sequence: []*snap.SideInfo{si3},
+		Current:  snap.R(1),
+		SnapType: "app",
+	})
+	snaptest.MockSnap(c, `name: some-snap2`, si3)
+
+	// pretend that some-snap1 was refreshed (and some-snap2 failed), some-snap1 is now at revision 3; validation set
+	// with sequence 3 is still valid though so no snap reverts.
+	refreshedSnaps := []string{"some-snap1"}
+	ts, err := snapstate.MaybeRestoreValidationSetsAndRevertSnaps(st, refreshedSnaps, "")
+	c.Assert(err, IsNil)
+
+	// we expect no snap reverts
+	c.Assert(ts, HasLen, 0)
+	c.Check(restoreValidationSetsTrackingCalled, Equals, 0)
+	c.Check(enforcedValidationSetsCalled, Equals, 1)
+}
