@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2017 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -98,6 +98,8 @@ type SnapSetup struct {
 
 	SnapPath string `json:"snap-path,omitempty"`
 
+	ExpectedProvenance string `json:"provenance,omitempty"`
+
 	DownloadInfo *snap.DownloadInfo `json:"download-info,omitempty"`
 	SideInfo     *snap.SideInfo     `json:"side-info,omitempty"`
 	auxStoreInfo
@@ -232,6 +234,19 @@ type SnapState struct {
 	// MigratedToExposedHome is set if ~/Snap was created and initialized. If set, ~/Snap
 	// should be used as the snap's HOME.
 	MigratedToExposedHome bool `json:"migrated-exposed-home,omitempty"`
+
+	// PendingSecurity tracks information about snaps that have
+	// their security profiles set up but are not active.
+	// It is managed by ifacestate.
+	PendingSecurity *PendingSecurityState `json:"pending-security,omitempty"`
+}
+
+// PendingSecurityState holds information about snaps that have
+// their security profiles set up but are not active.
+type PendingSecurityState struct {
+	// SideInfo of the revision for which security profiles are or
+	// should be set up if any.
+	SideInfo *snap.SideInfo `json:"side-info,omitempty"`
 }
 
 func (snapst *SnapState) SetTrackingChannel(s string) error {
@@ -528,11 +543,14 @@ func Manager(st *state.State, runner *state.TaskRunner) (*SnapManager, error) {
 
 	// misc
 	runner.AddHandler("switch-snap", m.doSwitchSnap, nil)
+	runner.AddHandler("migrate-snap-home", m.doMigrateSnapHome, m.undoMigrateSnapHome)
+	// no undo for now since it's last task in valset auto-resolution change
+	runner.AddHandler("enforce-validation-sets", m.doEnforceValidationSets, nil)
 
 	// control serialisation
 	runner.AddBlocked(m.blockedTask)
 
-	AddAffectedSnapsByKind("conditional-auto-refresh", conditionalAutoRefreshAffectedSnaps)
+	RegisterAffectedSnapsByKind("conditional-auto-refresh", conditionalAutoRefreshAffectedSnaps)
 
 	return m, nil
 }
@@ -562,7 +580,7 @@ func genRefreshRequestSalt(st *state.State) error {
 	st.Lock()
 	defer st.Unlock()
 
-	if err := st.Get("refresh-privacy-key", &refreshPrivacyKey); err != nil && err != state.ErrNoState {
+	if err := st.Get("refresh-privacy-key", &refreshPrivacyKey); err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	if refreshPrivacyKey != "" {
@@ -599,8 +617,7 @@ func (m *SnapManager) NextRefresh() time.Time {
 }
 
 // EffectiveRefreshHold returns the time until to which refreshes are
-// held if refresh.hold configuration is set and accounting for the
-// max postponement since the last refresh.
+// held if refresh.hold configuration is set.
 // The caller should be holding the state lock.
 func (m *SnapManager) EffectiveRefreshHold() (time.Time, error) {
 	return m.autoRefresh.EffectiveRefreshHold()
@@ -653,8 +670,8 @@ func (m *SnapManager) ensureVulnerableSnapRemoved(name string) error {
 	// in them from being available to abuse for fixed vulnerabilies that are
 	// not exploitable in the current versions of snapd/core snaps.
 	var alreadyRemoved bool
-	key := fmt.Sprintf("%s-snap-cve-2021-44731-vuln-removed", name)
-	if err := m.state.Get(key, &alreadyRemoved); err != nil && err != state.ErrNoState {
+	key := fmt.Sprintf("%s-snap-cve-2022-3328-vuln-removed", name)
+	if err := m.state.Get(key, &alreadyRemoved); err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	if alreadyRemoved {
@@ -662,10 +679,10 @@ func (m *SnapManager) ensureVulnerableSnapRemoved(name string) error {
 	}
 	var snapSt SnapState
 	err := Get(m.state, name, &snapSt)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
-	if err == state.ErrNoState {
+	if errors.Is(err, state.ErrNoState) {
 		// not installed, nothing to do
 		return nil
 	}
@@ -680,8 +697,8 @@ func (m *SnapManager) ensureVulnerableSnapRemoved(name string) error {
 		if err != nil {
 			return err
 		}
-		// res is < 0 if "ver" is lower than "2.54.3"
-		res, err := strutil.VersionCompare(ver, "2.54.3")
+		// res is < 0 if "ver" is lower than "2.57.6"
+		res, err := strutil.VersionCompare(ver, "2.57.6")
 		if err != nil {
 			return err
 		}
@@ -756,7 +773,7 @@ func (m *SnapManager) ensureVulnerableSnapConfineVersionsRemovedOnClassic() erro
 
 	// we have to remove vulnerable versions of both the core and snapd snaps
 	// only when we now have fixed versions installed / active
-	// the fixed version is 2.54.3, so if the version of the current core/snapd
+	// the fixed version is 2.57.6, so if the version of the current core/snapd
 	// snap is that or higher, then we proceed (if we didn't already do this)
 
 	if err := m.ensureVulnerableSnapRemoved("snapd"); err != nil {
@@ -782,7 +799,7 @@ func (m *SnapManager) ensureForceDevmodeDropsDevmodeFromState() error {
 
 	// int because we might want to come back and do a second pass at cleanup
 	var fixed int
-	if err := m.state.Get("fix-forced-devmode", &fixed); err != nil && err != state.ErrNoState {
+	if err := m.state.Get("fix-forced-devmode", &fixed); err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 
@@ -792,7 +809,7 @@ func (m *SnapManager) ensureForceDevmodeDropsDevmodeFromState() error {
 
 	for _, name := range []string{"core", "ubuntu-core"} {
 		var snapst SnapState
-		if err := Get(m.state, name, &snapst); err == state.ErrNoState {
+		if err := Get(m.state, name, &snapst); errors.Is(err, state.ErrNoState) {
 			// nothing to see here
 			continue
 		} else if err != nil {
@@ -836,7 +853,7 @@ func (m *SnapManager) ensureSnapdSnapTransition() error {
 	// check if snapd snap is installed
 	var snapst SnapState
 	err := Get(m.state, "snapd", &snapst)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	// nothing to do
@@ -869,7 +886,7 @@ func (m *SnapManager) ensureSnapdSnapTransition() error {
 	// Note that state.ErrNoState should never happen in practise. However
 	// if it *does* happen we still want to fix those systems by installing
 	// the snapd snap.
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	coreChannel := snapst.TrackingChannel
@@ -886,7 +903,7 @@ func (m *SnapManager) ensureSnapdSnapTransition() error {
 	// ensure we limit the retries in case something goes wrong
 	var lastSnapdTransitionAttempt time.Time
 	err = m.state.Get("snapd-transition-last-retry-time", &lastSnapdTransitionAttempt)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	now := time.Now()
@@ -897,7 +914,7 @@ func (m *SnapManager) ensureSnapdSnapTransition() error {
 
 	var retryCount int
 	err = m.state.Get("snapd-transition-retry", &retryCount)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	m.state.Set("snapd-transition-retry", retryCount+1)
@@ -922,10 +939,10 @@ func (m *SnapManager) ensureUbuntuCoreTransition() error {
 
 	var snapst SnapState
 	err := Get(m.state, "ubuntu-core", &snapst)
-	if err == state.ErrNoState {
+	if errors.Is(err, state.ErrNoState) {
 		return nil
 	}
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 
@@ -939,7 +956,7 @@ func (m *SnapManager) ensureUbuntuCoreTransition() error {
 	// ensure we limit the retries in case something goes wrong
 	var lastUbuntuCoreTransitionAttempt time.Time
 	err = m.state.Get("ubuntu-core-transition-last-retry-time", &lastUbuntuCoreTransitionAttempt)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	now := time.Now()
@@ -957,7 +974,7 @@ func (m *SnapManager) ensureUbuntuCoreTransition() error {
 
 	var retryCount int
 	err = m.state.Get("ubuntu-core-transition-retry", &retryCount)
-	if err != nil && err != state.ErrNoState {
+	if err != nil && !errors.Is(err, state.ErrNoState) {
 		return err
 	}
 	m.state.Set("ubuntu-core-transition-retry", retryCount+1)
@@ -981,7 +998,7 @@ func (m *SnapManager) atSeed() error {
 	defer m.state.Unlock()
 	var seeded bool
 	err := m.state.Get("seeded", &seeded)
-	if err != state.ErrNoState {
+	if !errors.Is(err, state.ErrNoState) {
 		// already seeded or other error
 		return err
 	}
