@@ -21,9 +21,9 @@ package snapstate
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"sort"
-	"strings"
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/asserts/snapasserts"
@@ -41,7 +41,11 @@ var currentSnaps = currentSnapsImpl
 // EnforcedValidationSets allows to hook getting of validation sets in enforce
 // mode into installation/refresh/removal of snaps. It gets hooked from
 // assertstate.
-var EnforcedValidationSets func(st *state.State, extraVs *asserts.ValidationSet) (*snapasserts.ValidationSets, error)
+var EnforcedValidationSets func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error)
+
+// EnforceValidationSets allows to hook enforcing validation sets without
+// fetching them. It's hooked from assertstate.
+var EnforceValidationSets func(*state.State, map[string]*asserts.ValidationSet, map[string]int, []*snapasserts.InstalledSnap, map[string]bool, int) error
 
 func userIDForSnap(st *state.State, snapst *SnapState, fallbackUserID int) (int, error) {
 	userID := snapst.UserID
@@ -84,7 +88,7 @@ func refreshOptions(st *state.State, origOpts *store.RefreshOptions) (*store.Ref
 		opts = *origOpts
 	}
 
-	if err := st.Get("refresh-privacy-key", &opts.PrivacyKey); err != nil && err != state.ErrNoState {
+	if err := st.Get("refresh-privacy-key", &opts.PrivacyKey); err != nil && !errors.Is(err, state.ErrNoState) {
 		return nil, fmt.Errorf("cannot obtain store request salt: %v", err)
 	}
 	if opts.PrivacyKey == "" {
@@ -210,10 +214,9 @@ var installSize = func(st *state.State, snaps []minimalInstallInfo, userID int)
 	return total, nil
 }
 
-func setActionValidationSetsAndRequiredRevision(action *store.SnapAction, valsets []string, requiredRevision snap.Revision) {
+func setActionValidationSetsAndRequiredRevision(action *store.SnapAction, valsets []snapasserts.ValidationSetKey, requiredRevision snap.Revision) {
 	for _, vs := range valsets {
-		keyParts := strings.Split(vs, "/")
-		action.ValidationSets = append(action.ValidationSets, keyParts)
+		action.ValidationSets = append(action.ValidationSets, vs)
 	}
 	if !requiredRevision.Unset() {
 		action.Revision = requiredRevision
@@ -249,36 +252,41 @@ func installInfo(ctx context.Context, st *state.State, name string, revOpts *Rev
 	}
 
 	var requiredRevision snap.Revision
-	var requiredValSets []string
+	var requiredValSets []snapasserts.ValidationSetKey
 
 	if !flags.IgnoreValidation {
-		enforcedSets, err := EnforcedValidationSets(st, nil)
-		if err != nil {
-			return store.SnapActionResult{}, err
-		}
-
-		if enforcedSets != nil {
-			// check for invalid presence first to have a list of sets where it's invalid
-			invalidForValSets, err := enforcedSets.CheckPresenceInvalid(naming.Snap(name))
-			if err != nil {
-				if _, ok := err.(*snapasserts.PresenceConstraintError); !ok {
-					return store.SnapActionResult{}, err
-				} // else presence is optional or required, carry on
-			}
-			if len(invalidForValSets) > 0 {
-				return store.SnapActionResult{}, fmt.Errorf("cannot install snap %q due to enforcing rules of validation set %s", name, strings.Join(invalidForValSets, ","))
-			}
-			requiredValSets, requiredRevision, err = enforcedSets.CheckPresenceRequired(naming.Snap(name))
+		if len(revOpts.ValidationSets) > 0 {
+			requiredRevision = revOpts.Revision
+			requiredValSets = revOpts.ValidationSets
+		} else {
+			enforcedSets, err := EnforcedValidationSets(st)
 			if err != nil {
 				return store.SnapActionResult{}, err
 			}
-		}
-	}
 
-	// check if desired revision matches the revision required by validation sets
-	if !requiredRevision.Unset() && !revOpts.Revision.Unset() && revOpts.Revision.N != requiredRevision.N {
-		return store.SnapActionResult{}, fmt.Errorf("cannot install snap %q at requested revision %s without --ignore-validation, revision %s required by validation sets: %s",
-			name, revOpts.Revision, requiredRevision, strings.Join(requiredValSets, ","))
+			if enforcedSets != nil {
+				// check for invalid presence first to have a list of sets where it's invalid
+				invalidForValSets, err := enforcedSets.CheckPresenceInvalid(naming.Snap(name))
+				if err != nil {
+					if _, ok := err.(*snapasserts.PresenceConstraintError); !ok {
+						return store.SnapActionResult{}, err
+					} // else presence is optional or required, carry on
+				}
+				if len(invalidForValSets) > 0 {
+					return store.SnapActionResult{}, fmt.Errorf("cannot install snap %q due to enforcing rules of validation set %s", name, snapasserts.ValidationSetKeySlice(invalidForValSets).CommaSeparated())
+				}
+				requiredValSets, requiredRevision, err = enforcedSets.CheckPresenceRequired(naming.Snap(name))
+				if err != nil {
+					return store.SnapActionResult{}, err
+				}
+			}
+
+			// check if desired revision matches the revision required by validation sets
+			if !requiredRevision.Unset() && !revOpts.Revision.Unset() && revOpts.Revision.N != requiredRevision.N {
+				return store.SnapActionResult{}, fmt.Errorf("cannot install snap %q at requested revision %s without --ignore-validation, revision %s required by validation sets: %s",
+					name, revOpts.Revision, requiredRevision, snapasserts.ValidationSetKeySlice(requiredValSets).CommaSeparated())
+			}
+		}
 	}
 
 	if len(requiredValSets) > 0 {
@@ -337,19 +345,30 @@ func updateInfo(st *state.State, snapst *SnapState, opts *RevisionOptions, userI
 		Flags:   storeFlags,
 	}
 
+	if len(opts.ValidationSets) > 0 {
+		// update to a specific revision is handled by updateToRevisionInfo.
+		// updating without a revision while enforcing validation sets is not a
+		// viable scenario (although we could handle it if desired), we only install/refresh
+		// what's missing and explicitly required by requested validation sets.
+		return nil, fmt.Errorf("internal error: list of validation sets is not expected for update without revision")
+	}
+
+	var requiredRevision snap.Revision
+	var requiredValsets []snapasserts.ValidationSetKey
+
 	if !flags.IgnoreValidation {
-		enforcedSets, err := EnforcedValidationSets(st, nil)
+		enforcedSets, err := EnforcedValidationSets(st)
 		if err != nil {
 			return nil, err
 		}
 		if enforcedSets != nil {
-			requiredValsets, requiredRevision, err := enforcedSets.CheckPresenceRequired(naming.Snap(curInfo.InstanceName()))
+			requiredValsets, requiredRevision, err = enforcedSets.CheckPresenceRequired(naming.Snap(curInfo.InstanceName()))
 			if err != nil {
 				return nil, err
 			}
 			if !requiredRevision.Unset() && snapst.Current == requiredRevision {
 				logger.Debugf("snap %q is already at the revision %s required by validation sets: %s, skipping",
-					curInfo.InstanceName(), snapst.Current, strings.Join(requiredValsets, ","))
+					curInfo.InstanceName(), snapst.Current, snapasserts.ValidationSetKeySlice(requiredValsets).CommaSeparated())
 				return nil, store.ErrNoUpdateAvailable
 			}
 			if len(requiredValsets) > 0 {
@@ -441,7 +460,7 @@ func singleActionResult(name, action string, results []store.SnapActionResult, e
 	return store.SnapActionResult{}, e
 }
 
-func updateToRevisionInfo(st *state.State, snapst *SnapState, revision snap.Revision, userID int, flags Flags, deviceCtx DeviceContext) (*snap.Info, error) {
+func updateToRevisionInfo(st *state.State, snapst *SnapState, revOpts *RevisionOptions, userID int, flags Flags, deviceCtx DeviceContext) (*snap.Info, error) {
 	curSnaps, err := currentSnaps(st)
 	if err != nil {
 		return nil, err
@@ -462,44 +481,53 @@ func updateToRevisionInfo(st *state.State, snapst *SnapState, revision snap.Revi
 		SnapID:       curInfo.SnapID,
 		InstanceName: curInfo.InstanceName(),
 		// the desired revision
-		Revision: revision,
+		Revision: revOpts.Revision,
 	}
 
+	var requiredRevision snap.Revision
+	var requiredValsets []snapasserts.ValidationSetKey
+
 	var storeFlags store.SnapActionFlags
 	if !flags.IgnoreValidation {
-		enforcedSets, err := EnforcedValidationSets(st, nil)
-		if err != nil {
-			return nil, err
-		}
-		if enforcedSets != nil {
-			requiredValsets, requiredRevision, err := enforcedSets.CheckPresenceRequired(naming.Snap(curInfo.InstanceName()))
+		if len(revOpts.ValidationSets) > 0 {
+			requiredRevision = revOpts.Revision
+			requiredValsets = revOpts.ValidationSets
+		} else {
+			enforcedSets, err := EnforcedValidationSets(st)
 			if err != nil {
 				return nil, err
 			}
-			if !requiredRevision.Unset() {
-				if revision != requiredRevision {
-					return nil, fmt.Errorf("cannot update snap %q to revision %s without --ignore-validation, revision %s is required by validation sets: %s",
-						curInfo.InstanceName(), revision, requiredRevision, strings.Join(requiredValsets, ","))
+			if enforcedSets != nil {
+				requiredValsets, requiredRevision, err = enforcedSets.CheckPresenceRequired(naming.Snap(curInfo.InstanceName()))
+				if err != nil {
+					return nil, err
 				}
-				// note, not checking if required revision matches snapst.Current because
-				// this is already indirectly prevented by infoForUpdate().
+				if !requiredRevision.Unset() {
+					if revOpts.Revision != requiredRevision {
+						return nil, fmt.Errorf("cannot update snap %q to revision %s without --ignore-validation, revision %s is required by validation sets: %s",
+							curInfo.InstanceName(), revOpts.Revision, requiredRevision, snapasserts.ValidationSetKeySlice(requiredValsets).CommaSeparated())
+					}
+					// note, not checking if required revision matches snapst.Current because
+					// this is already indirectly prevented by infoForUpdate().
 
-				// specific revision is required, reset cohort in current snaps
-				for _, sn := range curSnaps {
-					if sn.InstanceName == curInfo.InstanceName() {
-						sn.CohortKey = ""
-						break
+					// specific revision is required, reset cohort in current snaps
+					for _, sn := range curSnaps {
+						if sn.InstanceName == curInfo.InstanceName() {
+							sn.CohortKey = ""
+							break
+						}
 					}
 				}
 			}
-			if len(requiredValsets) > 0 {
-				setActionValidationSetsAndRequiredRevision(action, requiredValsets, requiredRevision)
-			}
 		}
 	} else {
 		storeFlags = store.SnapActionIgnoreValidation
 	}
 
+	if len(requiredValsets) > 0 {
+		setActionValidationSetsAndRequiredRevision(action, requiredValsets, requiredRevision)
+	}
+
 	action.Flags = storeFlags
 
 	theStore := Store(st, deviceCtx)
@@ -570,7 +598,7 @@ func collectCurrentSnaps(snapStates map[string]*SnapState, consider func(*store.
 	return curSnaps, nil
 }
 
-func refreshCandidates(ctx context.Context, st *state.State, names []string, user *auth.UserState, opts *store.RefreshOptions) ([]*snap.Info, map[string]*SnapState, map[string]bool, error) {
+func refreshCandidates(ctx context.Context, st *state.State, names []string, revOpts []*RevisionOptions, user *auth.UserState, opts *store.RefreshOptions) ([]*snap.Info, map[string]*SnapState, map[string]bool, error) {
 	snapStates, err := All(st)
 	if err != nil {
 		return nil, nil, nil, err
@@ -603,9 +631,21 @@ func refreshCandidates(ctx context.Context, st *state.State, names []string, use
 	ignoreValidationByInstanceName := make(map[string]bool)
 	nCands := 0
 
-	enforcedSets, err := EnforcedValidationSets(st, nil)
-	if err != nil {
-		return nil, nil, nil, err
+	var enforcedSets *snapasserts.ValidationSets
+	var revOptsByName map[string]*RevisionOptions
+
+	// if refreshing to specific revision to enforce a new validation set, we've
+	// already checked against other enforced sets
+	if revOpts == nil {
+		enforcedSets, err = EnforcedValidationSets(st)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+	} else {
+		revOptsByName = make(map[string]*RevisionOptions, len(revOpts))
+		for i, opts := range revOpts {
+			revOptsByName[names[i]] = opts
+		}
 	}
 
 	addCand := func(installed *store.CurrentSnap, snapst *SnapState) error {
@@ -631,8 +671,14 @@ func refreshCandidates(ctx context.Context, st *state.State, names []string, use
 		}
 
 		if !snapst.IgnoreValidation {
-			if enforcedSets != nil {
-				requiredValsets, requiredRevision, err := enforcedSets.CheckPresenceRequired(naming.Snap(installed.InstanceName))
+			var requiredValsets []snapasserts.ValidationSetKey
+			var requiredRevision snap.Revision
+
+			if revOpts != nil {
+				opts := revOptsByName[installed.InstanceName]
+				requiredValsets, requiredRevision = opts.ValidationSets, opts.Revision
+			} else if enforcedSets != nil {
+				requiredValsets, requiredRevision, err = enforcedSets.CheckPresenceRequired(naming.Snap(installed.InstanceName))
 				// note, this errors out the entire refresh
 				if err != nil {
 					return err
@@ -642,9 +688,10 @@ func refreshCandidates(ctx context.Context, st *state.State, names []string, use
 				if !requiredRevision.Unset() && installed.Revision == requiredRevision {
 					return nil
 				}
-				if len(requiredValsets) > 0 {
-					setActionValidationSetsAndRequiredRevision(action, requiredValsets, requiredRevision)
-				}
+			}
+
+			if len(requiredValsets) > 0 {
+				setActionValidationSetsAndRequiredRevision(action, requiredValsets, requiredRevision)
 			}
 		}
 
@@ -730,7 +777,7 @@ func refreshCandidates(ctx context.Context, st *state.State, names []string, use
 	return updates, stateByInstanceName, ignoreValidationByInstanceName, nil
 }
 
-func installCandidates(st *state.State, names []string, channel string, user *auth.UserState) ([]store.SnapActionResult, error) {
+func installCandidates(st *state.State, names []string, revOpts []*RevisionOptions, channel string, user *auth.UserState) ([]store.SnapActionResult, error) {
 	curSnaps, err := currentSnaps(st)
 	if err != nil {
 		return nil, err
@@ -741,9 +788,14 @@ func installCandidates(st *state.State, names []string, channel string, user *au
 		return nil, err
 	}
 
-	enforcedSets, err := EnforcedValidationSets(st, nil)
-	if err != nil {
-		return nil, err
+	// if installing a specific revision, we may be trying to enforce a validation
+	// set so don't check against current ones.
+	var enforcedSets *snapasserts.ValidationSets
+	if revOpts == nil {
+		enforcedSets, err = EnforcedValidationSets(st)
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	actions := make([]*store.SnapAction, len(names))
@@ -755,7 +807,13 @@ func installCandidates(st *state.State, names []string, channel string, user *au
 			Channel: channel,
 		}
 
-		if enforcedSets != nil {
+		var requiredValSets []snapasserts.ValidationSetKey
+		var requiredRevision snap.Revision
+
+		if revOpts != nil {
+			requiredValSets = revOpts[i].ValidationSets
+			requiredRevision = revOpts[i].Revision
+		} else if enforcedSets != nil {
 			// check for invalid presence first to have a list of sets where it's invalid
 			invalidForValSets, err := enforcedSets.CheckPresenceInvalid(naming.Snap(name))
 			if err != nil {
@@ -763,17 +821,20 @@ func installCandidates(st *state.State, names []string, channel string, user *au
 					return nil, err
 				} // else presence is optional or required, carry on
 			}
+
 			if len(invalidForValSets) > 0 {
-				return nil, fmt.Errorf("cannot install snap %q due to enforcing rules of validation set %s", name, strings.Join(invalidForValSets, ","))
+				return nil, fmt.Errorf("cannot install snap %q due to enforcing rules of validation set %s", name, snapasserts.ValidationSetKeySlice(invalidForValSets).CommaSeparated())
 			}
-			requiredValSets, requiredRevision, err := enforcedSets.CheckPresenceRequired(naming.Snap(name))
+			requiredValSets, requiredRevision, err = enforcedSets.CheckPresenceRequired(naming.Snap(name))
 			if err != nil {
 				return nil, err
 			}
-			if len(requiredValSets) > 0 {
-				setActionValidationSetsAndRequiredRevision(action, requiredValSets, requiredRevision)
-			}
 		}
+
+		if len(requiredValSets) > 0 {
+			setActionValidationSetsAndRequiredRevision(action, requiredValSets, requiredRevision)
+		}
+
 		actions[i] = action
 	}
 
