@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2020 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -39,8 +39,6 @@ import (
 type Backend interface {
 	Checkpoint(data []byte) error
 	EnsureBefore(d time.Duration)
-	// TODO: take flags to ask for reboot vs restart?
-	RequestRestart(t RestartType)
 }
 
 type customData map[string]*json.RawMessage
@@ -48,7 +46,7 @@ type customData map[string]*json.RawMessage
 func (data customData) get(key string, value interface{}) error {
 	entryJSON := data[key]
 	if entryJSON == nil {
-		return ErrNoState
+		return &NoStateError{Key: key}
 	}
 	err := json.Unmarshal(*entryJSON, value)
 	if err != nil {
@@ -74,21 +72,6 @@ func (data customData) set(key string, value interface{}) {
 	data[key] = &entryJSON
 }
 
-type RestartType int
-
-const (
-	RestartUnset RestartType = iota
-	RestartDaemon
-	RestartSystem
-	// RestartSystemNow is like RestartSystem but action is immediate
-	RestartSystemNow
-	// RestartSocket will restart the daemon so that it goes into
-	// socket activation mode.
-	RestartSocket
-	// Stop just stops the daemon (used with image pre-seeding)
-	StopDaemon
-)
-
 // State represents an evolving system state that persists across restarts.
 //
 // The State is concurrency-safe, and all reads and writes to it must be
@@ -115,21 +98,20 @@ type State struct {
 
 	cache map[interface{}]interface{}
 
-	restarting RestartType
-	restartLck sync.Mutex
-	bootID     string
+	pendingChangeByAttr map[string]func(*Change) bool
 }
 
 // New returns a new empty state.
 func New(backend Backend) *State {
 	return &State{
-		backend:  backend,
-		data:     make(customData),
-		changes:  make(map[string]*Change),
-		tasks:    make(map[string]*Task),
-		warnings: make(map[string]*Warning),
-		modified: true,
-		cache:    make(map[interface{}]interface{}),
+		backend:             backend,
+		data:                make(customData),
+		changes:             make(map[string]*Change),
+		tasks:               make(map[string]*Task),
+		warnings:            make(map[string]*Warning),
+		modified:            true,
+		cache:               make(map[interface{}]interface{}),
+		pendingChangeByAttr: make(map[string]func(*Change) bool),
 	}
 }
 
@@ -259,72 +241,31 @@ func (s *State) EnsureBefore(d time.Duration) {
 	}
 }
 
-// RequestRestart asks for a restart of the managing process.
-// The state needs to be locked to request a RestartSystem.
-func (s *State) RequestRestart(t RestartType) {
-	if s.backend != nil {
-		if t == RestartSystem || t == RestartSystemNow {
-			if s.bootID == "" {
-				panic("internal error: cannot request a system restart if current boot ID was not provided via VerifyReboot")
-			}
-			s.Set("system-restart-from-boot-id", s.bootID)
-		}
-		s.restartLck.Lock()
-		s.restarting = t
-		s.restartLck.Unlock()
-		s.backend.RequestRestart(t)
-	}
-}
-
-// Restarting returns whether a restart was requested with RequestRestart and of which type.
-func (s *State) Restarting() (bool, RestartType) {
-	s.restartLck.Lock()
-	defer s.restartLck.Unlock()
-	return s.restarting != RestartUnset, s.restarting
-}
-
-var ErrExpectedReboot = errors.New("expected reboot did not happen")
-
-// VerifyReboot checks if the state remembers that a system restart was
-// requested and whether it succeeded based on the provided current
-// boot id.  It returns ErrExpectedReboot if the expected reboot did
-// not happen yet.  It must be called early in the usage of state and
-// before an RequestRestart with RestartSystem is attempted.
-// It must be called with the state lock held.
-func (s *State) VerifyReboot(curBootID string) error {
-	var fromBootID string
-	err := s.Get("system-restart-from-boot-id", &fromBootID)
-	if err != nil && err != ErrNoState {
-		return err
-	}
-	s.bootID = curBootID
-	if fromBootID == "" {
-		return nil
-	}
-	if fromBootID == curBootID {
-		return ErrExpectedReboot
-	}
-	// we rebooted alright
-	s.ClearReboot()
-	return nil
-}
-
-// ClearReboot clears state information about tracking requested reboots.
-func (s *State) ClearReboot() {
-	s.Set("system-restart-from-boot-id", nil)
-}
-
-func MockRestarting(s *State, restarting RestartType) RestartType {
-	s.restartLck.Lock()
-	defer s.restartLck.Unlock()
-	old := s.restarting
-	s.restarting = restarting
-	return old
-}
-
 // ErrNoState represents the case of no state entry for a given key.
 var ErrNoState = errors.New("no state entry for key")
 
+// NoStateError represents the case where no state could be found for a given key.
+type NoStateError struct {
+	// Key is the key for which no state could be found.
+	Key string
+}
+
+func (e *NoStateError) Error() string {
+	var keyMsg string
+	if e.Key != "" {
+		keyMsg = fmt.Sprintf(" %q", e.Key)
+	}
+
+	return fmt.Sprintf("no state entry for key%s", keyMsg)
+}
+
+// Is returns true if the error is of type *NoStateError or equal to ErrNoState.
+// NoStateError's key isn't compared between errors.
+func (e *NoStateError) Is(err error) bool {
+	_, ok := err.(*NoStateError)
+	return ok || errors.Is(err, ErrNoState)
+}
+
 // Get unmarshals the stored value associated with the provided key
 // into the value parameter.
 // It returns ErrNoState if there is no entry for key.
@@ -333,6 +274,12 @@ func (s *State) Get(key string, value interface{}) error {
 	return s.data.get(key, value)
 }
 
+// Has returns whether the provided key has an associated value.
+func (s *State) Has(key string) bool {
+	s.reading()
+	return s.data.has(key)
+}
+
 // Set associates value with key for future consulting by managers.
 // The provided value must properly marshal and unmarshal with encoding/json.
 func (s *State) Set(key string, value interface{}) {
@@ -441,10 +388,18 @@ func (s *State) tasksIn(tids []string) []*Task {
 	return res
 }
 
+// RegisterPendingChangeByAttr registers predicates that will be invoked by
+// Prune on changes with the specified attribute set to check whether even if
+// they meet the time criteria they must not be aborted yet.
+func (s *State) RegisterPendingChangeByAttr(attr string, f func(*Change) bool) {
+	s.pendingChangeByAttr[attr] = f
+}
+
 // Prune does several cleanup tasks to the in-memory state:
 //
 //  * it removes changes that became ready for more than pruneWait and aborts
-//    tasks spawned for more than abortWait.
+//    tasks spawned for more than abortWait unless prevented by predicates
+//    registered with RegisterPendingChangeByAttr.
 //
 //  * it removes tasks unlinked to changes after pruneWait. When there are more
 //    changes than the limit set via "maxReadyChanges" those changes in ready
@@ -478,6 +433,7 @@ func (s *State) Prune(startOfOperation time.Time, pruneWait, abortWait time.Dura
 		}
 	}
 
+NextChange:
 	for _, chg := range changes {
 		readyTime := chg.ReadyTime()
 		spawnTime := chg.SpawnTime()
@@ -489,7 +445,12 @@ func (s *State) Prune(startOfOperation time.Time, pruneWait, abortWait time.Dura
 				chg.Abort()
 				delete(s.changes, chg.ID())
 			} else if spawnTime.Before(abortLimit) {
-				chg.Abort()
+				for attr, pending := range s.pendingChangeByAttr {
+					if chg.Has(attr) && pending(chg) {
+						continue NextChange
+					}
+				}
+				chg.AbortUnreadyLanes()
 			}
 			continue
 		}
@@ -515,8 +476,7 @@ func (s *State) Prune(startOfOperation time.Time, pruneWait, abortWait time.Dura
 
 // GetMaybeTimings implements timings.GetSaver
 func (s *State) GetMaybeTimings(timings interface{}) error {
-	err := s.Get("timings", timings)
-	if err != nil && err != ErrNoState {
+	if err := s.Get("timings", timings); err != nil && !errors.Is(err, ErrNoState) {
 		return err
 	}
 	return nil
@@ -540,5 +500,6 @@ func ReadState(backend Backend, r io.Reader) (*State, error) {
 	s.backend = backend
 	s.modified = false
 	s.cache = make(map[interface{}]interface{})
+	s.pendingChangeByAttr = make(map[string]func(*Change) bool)
 	return s, err
 }
