@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -46,6 +46,18 @@ func (r *Retry) Error() string {
 	return "task should be retried"
 }
 
+// Wait is returned from a handler to signal that the task cannot
+// proceed at the moment maybe because some manual action from the
+// user required at this point or because of errors. The task
+// will be set to WaitStatus.
+type Wait struct {
+	Reason string
+}
+
+func (r *Wait) Error() string {
+	return "task set to wait, manual action required"
+}
+
 type blockedFunc func(t *Task, running []*Task) bool
 
 // TaskRunner controls the running of goroutines to execute known task kinds.
@@ -197,7 +209,7 @@ func (r *TaskRunner) run(t *Task) {
 	r.tombs[t.ID()] = tomb
 	tomb.Go(func() error {
 		// Capture the error result with tomb.Kill so we can
-		// use tomb.Err uniformily to consider both it or a
+		// use tomb.Err uniformly to consider both it or a
 		// overriding previous Kill reason.
 		t0 := time.Now()
 		tomb.Kill(handler(t, tomb))
@@ -222,7 +234,7 @@ func (r *TaskRunner) run(t *Task) {
 		switch err.(type) {
 		case nil:
 			// we are ok
-		case *Retry:
+		case *Retry, *Wait:
 			// preserve
 		default:
 			if r.stopped {
@@ -235,13 +247,19 @@ func (r *TaskRunner) run(t *Task) {
 		switch x := err.(type) {
 		case *Retry:
 			// Handler asked to be called again later.
-			// TODO Allow postponing retries past the next Ensure.
 			if t.Status() == AbortStatus {
 				// Would work without it but might take two ensures.
 				r.tryUndo(t)
 			} else if x.After != 0 {
 				t.At(timeNow().Add(x.After))
 			}
+		case *Wait:
+			if t.Status() == AbortStatus {
+				// Would work without it but might take two ensures.
+				r.tryUndo(t)
+			} else {
+				t.SetStatus(WaitStatus)
+			}
 		case nil:
 			var next []*Task
 			switch t.Status() {
@@ -401,6 +419,10 @@ ConsiderTasks:
 			}
 			continue
 		}
+		if status == WaitStatus {
+			// nothing more to run
+			continue
+		}
 
 		if mustWait(t) {
 			// Dependencies still unhandled.
