@@ -21,23 +21,17 @@ package release
 
 import (
 	"bufio"
-	"bytes"
-	"io/ioutil"
+	"fmt"
 	"os"
 	"strings"
 	"unicode"
 
-	"github.com/snapcore/snapd/sandbox/apparmor"
-	"github.com/snapcore/snapd/sandbox/cgroup"
 	"github.com/snapcore/snapd/strutil"
 )
 
 // Series holds the Ubuntu Core series for snapd to use.
 var Series = "16"
 
-// For testing only
-var mockedForceDevMode *bool
-
 // OS contains information about the system extracted from /etc/os-release.
 type OS struct {
 	ID        string   `json:"id"`
@@ -45,20 +39,6 @@ type OS struct {
 	VersionID string   `json:"version-id,omitempty"`
 }
 
-// ForceDevMode returns true if the distribution doesn't implement required
-// security features for confinement and devmode is forced.
-func (o *OS) ForceDevMode() bool {
-	if mockedForceDevMode != nil {
-		return *mockedForceDevMode
-	}
-
-	apparmorFull := apparmor.ProbedLevel() == apparmor.Full
-	// TODO: update once security backends affected by cgroupv2 are fully
-	// supported
-	cgroupv2 := cgroup.IsUnified()
-	return !apparmorFull || cgroupv2
-}
-
 // DistroLike checks if the distribution ID or ID_LIKE matches one of the given names.
 func DistroLike(distros ...string) bool {
 	for _, distro := range distros {
@@ -124,23 +104,77 @@ func readOSRelease() OS {
 	return osRelease
 }
 
-var ioutilReadFile = ioutil.ReadFile
-
-func isWSL() bool {
-	version, err := ioutilReadFile("/proc/version")
-	if err == nil && bytes.Contains(version, []byte("Microsoft")) {
-		return true
-	}
-
-	return false
+// Note that osutil.FileExists cannot be used here as an osutil import will create a cyclic import
+var fileExists = func(path string) bool {
+	_, err := os.Stat(path)
+	return err == nil
 }
 
-// PreseedMode states whether snapd is running in a presseding mode
-// to perform a partial first boot and touch only filesystem state
-// inside a chroot.
-// TODO: handle via devicestate, similar to DeviceContext.
-var PreseedMode = func() bool {
-	return os.Getenv("SNAPD_PRESEED") != ""
+var procMountsPath = "/proc/mounts"
+
+// filesystemRootType returns the filesystem type mounted at "/".
+func filesystemRootType() (string, error) {
+	// We scan /proc/mounts, which contains space-separated values:
+	// [irrelevant] [mount point] [fstype] [irrelevant...]
+	// Here are some examples on some platforms:
+	// WSL1       :  rootfs / wslfs rw,noatime 0 0
+	// WSL2       :  /dev/sdc / ext4 rw,relatime,discard,errors=remount-ro,data=ordered 0 0
+	// lxc on WSL2:  /dev/loop0 / btrfs rw,relatime,idmapped,space_cache,user_subvol_rm_allowed,subvolid=259,subvol=/containers/testlxd 0 0
+	// We search for mount point = "/", and return the fstype.
+	//
+	// This should be done by osutil.LoadMountInfo but that would cause a dependency cycle
+	file, err := os.Open(procMountsPath)
+	if err != nil {
+		return "", fmt.Errorf("cannot find root filesystem type: %v", err)
+	}
+	defer file.Close()
+
+	scanner := bufio.NewScanner(file)
+	for scanner.Scan() {
+		data := strings.Fields(scanner.Text())
+		if len(data) < 3 || data[1] != "/" {
+			continue
+		}
+		return data[2], nil
+	}
+
+	if err = scanner.Err(); err != nil {
+		return "", fmt.Errorf("cannot find root filesystem type: %v", err)
+	}
+
+	return "", fmt.Errorf("cannot find root filesystem type: not in list")
+}
+
+// We detect WSL via the existence of /proc/sys/fs/binfmt_misc/WSLInterop
+// Under some undocumented circumstances this file may be missing. We have /run/WSL as a backup.
+//
+// We detect WSL1 via the root filesystem type:
+// - wslfs or lxfs mean WSL1
+// - Anything else means WSL2
+// After knowing we're in WSL, if any error occurs we assume WSL2 as it is the more flexible version
+func getWSLVersion() int {
+	if !fileExists("/proc/sys/fs/binfmt_misc/WSLInterop") && !fileExists("/run/WSL") {
+		return 0
+	}
+	fstype, err := filesystemRootType()
+	if err != nil {
+		// TODO log error here once logger can be imported without circular imports
+		return 2
+	}
+
+	if fstype == "wslfs" || fstype == "lxfs" {
+		return 1
+	}
+	return 2
+}
+
+// SystemctlSupportsUserUnits returns true if the systemctl utility
+// supports user units.
+func SystemctlSupportsUserUnits() bool {
+	// Ubuntu 14.04's systemctl does not support the arguments
+	// needed to enable user session units. Further more, it does
+	// not ship with a systemd user instance.
+	return !(ReleaseInfo.ID == "ubuntu" && ReleaseInfo.VersionID == "14.04")
 }
 
 // OnClassic states whether the process is running inside a
@@ -151,6 +185,10 @@ var OnClassic bool
 // Subsystem for Linux
 var OnWSL bool
 
+// If the previous is true, WSLVersion states whether the process is running inside WSL1 or WSL2
+// Otherwise it is set to 0
+var WSLVersion int
+
 // ReleaseInfo contains data loaded from /etc/os-release on startup.
 var ReleaseInfo OS
 
@@ -159,14 +197,8 @@ func init() {
 
 	OnClassic = (ReleaseInfo.ID != "ubuntu-core")
 
-	OnWSL = isWSL()
-}
-
-// MockPreseedMode fakes preseed mode.
-func MockPreseedMode(preseedMode func() bool) (restore func()) {
-	old := PreseedMode
-	PreseedMode = preseedMode
-	return func() { PreseedMode = old }
+	WSLVersion = getWSLVersion()
+	OnWSL = WSLVersion != 0
 }
 
 // MockOnClassic forces the process to appear inside a classic
@@ -184,13 +216,3 @@ func MockReleaseInfo(osRelease *OS) (restore func()) {
 	ReleaseInfo = *osRelease
 	return func() { ReleaseInfo = old }
 }
-
-// MockForcedDevmode fake the system to believe its in a distro
-// that is in ForcedDevmode
-func MockForcedDevmode(isDevmode bool) (restore func()) {
-	old := mockedForceDevMode
-	mockedForceDevMode = &isDevmode
-	return func() {
-		mockedForceDevMode = old
-	}
-}
