@@ -20,7 +20,9 @@
 package apparmor_test
 
 import (
+	"errors"
 	"fmt"
+	"io"
 	"io/ioutil"
 	"os"
 	"path/filepath"
@@ -29,6 +31,7 @@ import (
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/sandbox/apparmor"
 	"github.com/snapcore/snapd/testutil"
 )
@@ -37,10 +40,22 @@ func TestApparmor(t *testing.T) {
 	TestingT(t)
 }
 
-type apparmorSuite struct{}
+type apparmorSuite struct {
+	testutil.BaseTest
+}
 
 var _ = Suite(&apparmorSuite{})
 
+func (s *apparmorSuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+	s.AddCleanup(func() {
+		configFile := filepath.Join(dirs.GlobalRootDir, "/etc/apparmor.d/tunables/home.d/snapd")
+		if err := os.Remove(configFile); err != nil {
+			c.Assert(os.IsNotExist(err), Equals, true)
+		}
+	})
+}
+
 func (*apparmorSuite) TestAppArmorLevelTypeStringer(c *C) {
 	c.Check(apparmor.Unknown.String(), Equals, "unknown")
 	c.Check(apparmor.Unsupported.String(), Equals, "none")
@@ -129,13 +144,13 @@ func (*apparmorSuite) TestMockAppArmorFeatures(c *C) {
 	restore()
 
 	// Complete parser features but apparmor is unusable because of missing required kernel features.
-	// The dummy feature is there to pretend that apparmor in the kernel is not entirely disabled.
-	restore = apparmor.MockFeatures([]string{"dummy-feature"}, nil, apparmor.RequiredParserFeatures, nil)
+	// The test feature is there to pretend that apparmor in the kernel is not entirely disabled.
+	restore = apparmor.MockFeatures([]string{"test-feature"}, nil, apparmor.RequiredParserFeatures, nil)
 	c.Check(apparmor.ProbedLevel(), Equals, apparmor.Unusable)
 	c.Check(apparmor.Summary(), Equals, "apparmor is enabled but required kernel features are missing: file")
 	features, err = apparmor.KernelFeatures()
 	c.Assert(err, IsNil)
-	c.Check(features, DeepEquals, []string{"dummy-feature"})
+	c.Check(features, DeepEquals, []string{"test-feature"})
 	features, err = apparmor.ParserFeatures()
 	c.Assert(err, IsNil)
 	c.Check(features, DeepEquals, apparmor.RequiredParserFeatures)
@@ -199,28 +214,32 @@ func (s *apparmorSuite) TestProbeAppArmorParserFeatures(c *C) {
 		expFeatures []string
 	}{
 		{
-			exitCodes: []int{1, 1, 1, 1, 1},
+			exitCodes: []int{1, 1, 1, 1, 1, 1},
 		},
 		{
-			exitCodes:   []int{1, 0, 1, 1, 1},
+			exitCodes:   []int{1, 0, 1, 1, 1, 1},
 			expFeatures: []string{"qipcrtr-socket"},
 		},
 		{
-			exitCodes:   []int{0, 1, 1, 1, 1},
+			exitCodes:   []int{0, 1, 1, 1, 1, 1},
 			expFeatures: []string{"unsafe"},
 		},
 		{
-			exitCodes:   []int{1, 1, 1, 0, 1},
+			exitCodes:   []int{1, 1, 1, 0, 1, 1},
 			expFeatures: []string{"cap-audit-read"},
 		},
 		{
-			exitCodes:   []int{0, 0, 1, 1, 1},
+			exitCodes:   []int{0, 0, 1, 1, 1, 1},
 			expFeatures: []string{"qipcrtr-socket", "unsafe"},
 		},
 		{
-			exitCodes:   []int{0, 0, 0, 0, 0},
+			exitCodes:   []int{0, 0, 0, 0, 0, 1},
 			expFeatures: []string{"cap-audit-read", "cap-bpf", "mqueue", "qipcrtr-socket", "unsafe"},
 		},
+		{
+			exitCodes:   []int{0, 0, 0, 0, 0, 0},
+			expFeatures: []string{"cap-audit-read", "cap-bpf", "mqueue", "qipcrtr-socket", "unsafe", "xdp"},
+		},
 	}
 
 	for _, t := range testcases {
@@ -274,6 +293,9 @@ profile snap-test {
 profile snap-test {
  mqueue,
 }
+profile snap-test {
+ network xdp,
+}
 `)
 	}
 
@@ -306,7 +328,7 @@ func (s *apparmorSuite) TestInterfaceSystemKey(c *C) {
 	c.Check(features, DeepEquals, []string{"network", "policy"})
 	features, err = apparmor.ParserFeatures()
 	c.Assert(err, IsNil)
-	c.Check(features, DeepEquals, []string{"cap-audit-read", "cap-bpf", "mqueue", "qipcrtr-socket", "unsafe"})
+	c.Check(features, DeepEquals, []string{"cap-audit-read", "cap-bpf", "mqueue", "qipcrtr-socket", "unsafe", "xdp"})
 }
 
 func (s *apparmorSuite) TestAppArmorParserMtime(c *C) {
@@ -346,7 +368,7 @@ func (s *apparmorSuite) TestFeaturesProbedOnce(c *C) {
 	c.Check(features, DeepEquals, []string{"network", "policy"})
 	features, err = apparmor.ParserFeatures()
 	c.Assert(err, IsNil)
-	c.Check(features, DeepEquals, []string{"cap-audit-read", "cap-bpf", "mqueue", "qipcrtr-socket", "unsafe"})
+	c.Check(features, DeepEquals, []string{"cap-audit-read", "cap-bpf", "mqueue", "qipcrtr-socket", "unsafe", "xdp"})
 
 	// this makes probing fails but is not done again
 	err = os.RemoveAll(d)
@@ -378,3 +400,45 @@ func (s *apparmorSuite) TestValidateFreeFromAAREhappy(c *C) {
 		c.Check(apparmor.ValidateNoAppArmorRegexp(s), IsNil, Commentf("%q raised an error but shouldn't", s))
 	}
 }
+
+func (s *apparmorSuite) TestUpdateHomedirsTunableMkdirFail(c *C) {
+	restore := apparmor.MockMkdirAll(func(string, os.FileMode) error {
+		return errors.New("mkdir failure")
+	})
+	defer restore()
+
+	err := apparmor.UpdateHomedirsTunable([]string{"does", "not", "matter"})
+	c.Check(err, ErrorMatches, `cannot create AppArmor tunable directory: mkdir failure`)
+}
+
+func (s *apparmorSuite) TestUpdateHomedirsTunableWriteFail(c *C) {
+	restore := apparmor.MockMkdirAll(func(string, os.FileMode) error {
+		return nil
+	})
+	defer restore()
+
+	restore = apparmor.MockAtomicWrite(func(string, io.Reader, os.FileMode, osutil.AtomicWriteFlags) error {
+		return errors.New("write failure")
+	})
+	defer restore()
+
+	err := apparmor.UpdateHomedirsTunable([]string{"does", "not", "matter"})
+	c.Check(err, ErrorMatches, `write failure`)
+}
+
+func (s *apparmorSuite) TestUpdateHomedirsTunableHappy(c *C) {
+	err := apparmor.UpdateHomedirsTunable([]string{"/home/a", "/dir2"})
+	c.Assert(err, IsNil)
+	configFile := filepath.Join(dirs.GlobalRootDir, "/etc/apparmor.d/tunables/home.d/snapd")
+	fileContents, err := ioutil.ReadFile(configFile)
+	c.Assert(err, IsNil)
+	c.Check(string(fileContents), Equals,
+		`# Generated by snapd -- DO NOT EDIT!`+"\n"+`@{HOMEDIRS}+="/home/a" "/dir2"`)
+}
+
+func (s *apparmorSuite) TestUpdateHomedirsTunableHappyNoDirs(c *C) {
+	err := apparmor.UpdateHomedirsTunable([]string{})
+	c.Check(err, IsNil)
+	configFile := filepath.Join(dirs.GlobalRootDir, "/etc/apparmor.d/tunables/home.d/snapd")
+	c.Check(osutil.FileExists(configFile), Equals, false)
+}
