@@ -24,6 +24,7 @@ import (
 	"io/ioutil"
 	"os"
 	"strings"
+	"time"
 
 	"github.com/godbus/dbus"
 	. "gopkg.in/check.v1"
@@ -50,24 +51,27 @@ var _ = Suite(&trackingSuite{})
 
 func (s *trackingSuite) SetUpTest(c *C) {
 	dirs.SetRootDir(c.MkDir())
+	cgroup.MockVersion(cgroup.V2, nil)
 }
 
 func (s *trackingSuite) TearDownTest(c *C) {
 	dirs.SetRootDir("")
 }
 
-// CreateTransientScopeForTracking is a no-op when refresh app awareness is off
+// CreateTransientScopeForTracking always attempts to track, even when refresh app awareness flag is off.
 func (s *trackingSuite) TestCreateTransientScopeForTrackingFeatureDisabled(c *C) {
 	noDBus := func() (*dbus.Conn, error) {
-		c.Error("test sequence violated")
-		return nil, fmt.Errorf("dbus should not have been used")
+		return nil, fmt.Errorf("dbus not available")
 	}
 	restore := dbusutil.MockConnections(noDBus, noDBus)
 	defer restore()
 
+	// The feature is disabled but we still track applications. The feature
+	// flag is now only observed in side snapd snap manager, while considering
+	// snap refreshes.
 	c.Assert(features.RefreshAppAwareness.IsEnabled(), Equals, false)
 	err := cgroup.CreateTransientScopeForTracking("snap.pkg.app", nil)
-	c.Check(err, IsNil)
+	c.Assert(err, ErrorMatches, "cannot track application process")
 }
 
 // CreateTransientScopeForTracking does stuff when refresh app awareness is on
@@ -85,14 +89,28 @@ func (s *trackingSuite) TestCreateTransientScopeForTrackingFeatureEnabled(c *C)
 	uuid := "cc98cd01-6a25-46bd-b71b-82069b71b770"
 	restore = cgroup.MockRandomUUID(uuid)
 	defer restore()
+	signalChan := make(chan struct{})
 	// Replace interactions with DBus so that only session bus is available and responds with our logic.
-	conn, err := dbustest.Connection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
+	conn, inject, err := dbustest.InjectableConnection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
 		switch n {
 		case 0:
+			return []*dbus.Message{checkSystemdSignalSubscribe(c, msg)}, nil
+		case 1:
+			defer func() {
+				close(signalChan)
+			}()
 			return []*dbus.Message{checkAndRespondToStartTransientUnit(c, msg, "snap.pkg.app."+uuid+".scope", 312123)}, nil
+		case 2:
+			return []*dbus.Message{checkSystemSignalUnsubscribe(c, msg)}, nil
 		}
 		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
 	})
+	go func() {
+		select {
+		case <-signalChan:
+			inject(mockJobRemovedSignal("snap.pkg.app."+uuid+".scope", "done"))
+		}
+	}()
 	c.Assert(err, IsNil)
 	restore = dbusutil.MockOnlySessionBusAvailable(conn)
 	defer restore()
@@ -392,6 +410,112 @@ func (s *trackingSuite) TestCreateTransientScopeForRootOnSystemBus(c *C) {
 	c.Assert(err, IsNil)
 }
 
+type testTransientScopeConfirm struct {
+	uuid        string
+	confirmUnit string
+	confirmErr  error
+	expectedErr string
+}
+
+func (s *trackingSuite) testCreateTransientScopeConfirm(c *C, tc testTransientScopeConfirm) {
+	enableFeatures(c, features.RefreshAppAwareness)
+
+	logBuf, restore := logger.MockLogger()
+	defer restore()
+	os.Setenv("SNAPD_DEBUG", "true")
+	defer os.Unsetenv("SNAPD_DEBUG")
+	restore = cgroup.MockOsGetuid(12345)
+	defer restore()
+	restore = cgroup.MockOsGetpid(312123)
+	defer restore()
+	restore = cgroup.MockRandomUUID(tc.uuid)
+	defer restore()
+	sessionBus, err := dbustest.Connection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
+		// we are not expecting any dbus traffic
+		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
+	})
+
+	c.Assert(err, IsNil)
+	restore = dbusutil.MockOnlySessionBusAvailable(sessionBus)
+	defer restore()
+	restore = cgroup.MockDoCreateTransientScope(func(conn *dbus.Conn, unitName string, pid int) error {
+		c.Assert(conn, Equals, sessionBus)
+		c.Assert(unitName, Equals, "snap.pkg.app."+tc.uuid+".scope")
+		return nil
+	})
+	defer restore()
+
+	restore = cgroup.MockCgroupProcessPathInTrackingCgroup(func(pid int) (string, error) {
+		return tc.confirmUnit, tc.confirmErr
+	})
+	defer restore()
+
+	// creating transient scope fails when systemd reports that the job failed
+	err = cgroup.CreateTransientScopeForTracking("snap.pkg.app", nil)
+	if tc.expectedErr == "" {
+		c.Assert(err, IsNil)
+	} else {
+		c.Assert(err, ErrorMatches, tc.expectedErr)
+		if tc.confirmErr == nil {
+			c.Check(logBuf.String(), testutil.Contains, "systemd could not associate process 312123 with transient scope")
+		}
+	}
+}
+
+func (s *trackingSuite) TestCreateTransientScopeConfirmHappy(c *C) {
+	uuid := "cc98cd01-6a25-46bd-b71b-82069b71b770"
+	s.testCreateTransientScopeConfirm(c, testTransientScopeConfirm{
+		uuid:        uuid,
+		confirmUnit: "foo/bar/baz/snap.pkg.app." + uuid + ".scope",
+	})
+}
+
+func (s *trackingSuite) TestCreateTransientScopeConfirmOtherUnit(c *C) {
+	uuid := "cc98cd01-6a25-46bd-b71b-82069b71b770"
+	s.testCreateTransientScopeConfirm(c, testTransientScopeConfirm{
+		uuid:        uuid,
+		confirmUnit: "foo/bar/baz/other-unit.scope",
+		expectedErr: cgroup.ErrCannotTrackProcess.Error(),
+	})
+}
+
+func (s *trackingSuite) TestCreateTransientScopeConfirmCheckError(c *C) {
+	uuid := "cc98cd01-6a25-46bd-b71b-82069b71b770"
+	s.testCreateTransientScopeConfirm(c, testTransientScopeConfirm{
+		uuid:        uuid,
+		confirmErr:  fmt.Errorf("mock failure"),
+		expectedErr: "mock failure",
+	})
+}
+
+const systemdSignalMatch = `type='signal',interface='org.freedesktop.systemd1.Manager',member='JobRemoved'`
+
+func checkSystemdSignalSubscribe(c *C, msg *dbus.Message) *dbus.Message {
+	var rule string
+	dbus.Store(msg.Body, &rule)
+	c.Check(rule, Equals, systemdSignalMatch)
+	return &dbus.Message{
+		Type: dbus.TypeMethodReply,
+		Headers: map[dbus.HeaderField]dbus.Variant{
+			dbus.FieldReplySerial: dbus.MakeVariant(msg.Serial()),
+			dbus.FieldSender:      dbus.MakeVariant(":1"), // This does not matter.
+		},
+	}
+}
+
+func checkSystemSignalUnsubscribe(c *C, msg *dbus.Message) *dbus.Message {
+	var rule string
+	dbus.Store(msg.Body, &rule)
+	c.Check(rule, Equals, systemdSignalMatch)
+	return &dbus.Message{
+		Type: dbus.TypeMethodReply,
+		Headers: map[dbus.HeaderField]dbus.Variant{
+			dbus.FieldReplySerial: dbus.MakeVariant(msg.Serial()),
+			dbus.FieldSender:      dbus.MakeVariant(":1"), // This does not matter.
+		},
+	}
+}
+
 func checkAndRespondToStartTransientUnit(c *C, msg *dbus.Message, scopeName string, pid int) *dbus.Message {
 	// XXX: Those types might live in a package somewhere
 	type Property struct {
@@ -451,15 +575,216 @@ func checkAndFailToStartTransientUnit(c *C, msg *dbus.Message, errMsg string) *d
 	}
 }
 
-func (s *trackingSuite) TestDoCreateTransientScopeHappy(c *C) {
+func mockJobRemovedSignal(unit, result string) *dbus.Message {
+	return &dbus.Message{
+		Type: dbus.TypeSignal,
+		Headers: map[dbus.HeaderField]dbus.Variant{
+			dbus.FieldSender:    dbus.MakeVariant(":1"), // This does not matter.
+			dbus.FieldPath:      dbus.MakeVariant(dbus.ObjectPath("/org/freedesktop/systemd1")),
+			dbus.FieldInterface: dbus.MakeVariant("org.freedesktop.systemd1.Manager"),
+			dbus.FieldMember:    dbus.MakeVariant("JobRemoved"),
+			dbus.FieldSignature: dbus.MakeVariant(
+				dbus.SignatureOf(uint32(0), dbus.ObjectPath(""), "", "")),
+		},
+		Body: []interface{}{
+			uint32(1),
+			dbus.ObjectPath("/org/freedesktop/systemd1/job/1462"),
+			unit,
+			result,
+		},
+	}
+}
+
+func (s *trackingSuite) TestCreateTransientScopeHappyWithRetriedCheckCgroupV1(c *C) {
+	enableFeatures(c, features.RefreshAppAwareness)
+
+	restore := cgroup.MockVersion(cgroup.V1, nil)
+	defer restore()
+	logBuf, restore := logger.MockLogger()
+	defer restore()
+	os.Setenv("SNAPD_DEBUG", "true")
+	defer os.Unsetenv("SNAPD_DEBUG")
+	restore = cgroup.MockOsGetuid(12345)
+	defer restore()
+	restore = cgroup.MockOsGetpid(312123)
+	defer restore()
+	uuid := "cc98cd01-6a25-46bd-b71b-82069b71b770"
+	restore = cgroup.MockRandomUUID(uuid)
+	defer restore()
+	sessionBus, err := dbustest.Connection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
+		c.Logf("%v got msg: %v", n, msg)
+		switch n {
+		case 0:
+			return []*dbus.Message{checkAndRespondToStartTransientUnit(c, msg, "snap.pkg.app."+uuid+".scope", 312123)}, nil
+			// CreateTransientScopeForTracking is called twice in the test
+		case 1:
+			return []*dbus.Message{checkAndRespondToStartTransientUnit(c, msg, "snap.pkg.app."+uuid+".scope", 312123)}, nil
+		}
+		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
+	})
+
+	c.Assert(err, IsNil)
+	restore = dbusutil.MockOnlySessionBusAvailable(sessionBus)
+	defer restore()
+
+	pathInTrackingCgroupCallsToSuccess := 5
+	pathInTrackingCgroupCalls := 0
+	restore = cgroup.MockCgroupProcessPathInTrackingCgroup(func(pid int) (string, error) {
+		pathInTrackingCgroupCalls++
+		if pathInTrackingCgroupCalls < pathInTrackingCgroupCallsToSuccess {
+			return "vte-spawn-1234-1234-1234.scope", nil
+		}
+		return "snap.pkg.app." + uuid + ".scope", nil
+	})
+	defer restore()
+
+	// creating transient scope succeeds even if check is retried
+	err = cgroup.CreateTransientScopeForTracking("snap.pkg.app", nil)
+	c.Assert(err, IsNil)
+	c.Check(pathInTrackingCgroupCalls, Equals, 5)
+	c.Check(logBuf.String(), Not(testutil.Contains), "systemd could not associate process 312123 with transient scope")
+
+	// try again, but exhaust the limit if attempts which should be set to 100
+	pathInTrackingCgroupCalls = 0
+	pathInTrackingCgroupCallsToSuccess = 99999
+	logBuf.Reset()
+	err = cgroup.CreateTransientScopeForTracking("snap.pkg.app", nil)
+	c.Assert(err, ErrorMatches, "cannot track application process")
+	c.Check(pathInTrackingCgroupCalls, Equals, 100)
+	c.Check(logBuf.String(), testutil.Contains, "systemd could not associate process 312123 with transient scope")
+}
+
+func (s *trackingSuite) TestCreateTransientScopeUnhappyJobFailed(c *C) {
+	enableFeatures(c, features.RefreshAppAwareness)
+
+	restore := cgroup.MockOsGetuid(12345)
+	defer restore()
+	restore = cgroup.MockOsGetpid(312123)
+	defer restore()
+	uuid := "cc98cd01-6a25-46bd-b71b-82069b71b770"
+	restore = cgroup.MockRandomUUID(uuid)
+	defer restore()
+	signalChan := make(chan struct{})
+	sessionBus, inject, err := dbustest.InjectableConnection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
+		c.Logf("dbus message %v", n)
+		switch n {
+		case 0:
+			return []*dbus.Message{checkSystemdSignalSubscribe(c, msg)}, nil
+		case 1:
+			defer func() {
+				close(signalChan)
+			}()
+			return []*dbus.Message{checkAndRespondToStartTransientUnit(c, msg, "snap.pkg.app."+uuid+".scope", 312123)}, nil
+		case 2:
+			return []*dbus.Message{checkSystemSignalUnsubscribe(c, msg)}, nil
+		}
+		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
+	})
+	go func() {
+		select {
+		case <-signalChan:
+			inject(mockJobRemovedSignal("snap.pkg.app."+uuid+".scope", "failed"))
+		}
+	}()
+
+	c.Assert(err, IsNil)
+	restore = dbusutil.MockOnlySessionBusAvailable(sessionBus)
+	defer restore()
+
+	restore = cgroup.MockCgroupProcessPathInTrackingCgroup(func(pid int) (string, error) {
+		c.Fatal("unexpected call")
+		return "", fmt.Errorf("unexpected call")
+	})
+	defer restore()
+
+	// creating transient scope fails when systemd reports that the job failed
+	err = cgroup.CreateTransientScopeForTracking("snap.pkg.app", nil)
+	c.Assert(err, ErrorMatches, "transient scope could not be started, job /org/freedesktop/systemd1/job/1462 finished with result failed")
+}
+
+func (s *trackingSuite) TestCreateTransientScopeUnhappyJobTimeout(c *C) {
+	enableFeatures(c, features.RefreshAppAwareness)
+
+	restore := cgroup.MockOsGetuid(12345)
+	defer restore()
+	restore = cgroup.MockOsGetpid(312123)
+	defer restore()
+	uuid := "cc98cd01-6a25-46bd-b71b-82069b71b770"
+	restore = cgroup.MockRandomUUID(uuid)
+	defer restore()
+	restore = cgroup.MockCreateScopeJobTimeout(100 * time.Millisecond)
+	defer restore()
+	// mock a connection, but without support for injecting signal messages
+	sessionBus, err := dbustest.Connection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
+		c.Logf("dbus message %v", n)
+		switch n {
+		case 0:
+			return []*dbus.Message{checkSystemdSignalSubscribe(c, msg)}, nil
+		case 1:
+			return []*dbus.Message{checkAndRespondToStartTransientUnit(c, msg, "snap.pkg.app."+uuid+".scope", 312123)}, nil
+		case 2:
+			return []*dbus.Message{checkSystemSignalUnsubscribe(c, msg)}, nil
+		}
+		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
+	})
+
+	c.Assert(err, IsNil)
+	restore = dbusutil.MockOnlySessionBusAvailable(sessionBus)
+	defer restore()
+
+	restore = cgroup.MockCgroupProcessPathInTrackingCgroup(func(pid int) (string, error) {
+		c.Fatal("unexpected call")
+		return "", fmt.Errorf("unexpected call")
+	})
+	defer restore()
+
+	// creating transient scope fails when systemd reports that the job failed
+	err = cgroup.CreateTransientScopeForTracking("snap.pkg.app", nil)
+	c.Assert(err, ErrorMatches, "transient scope not created in 100ms")
+}
+
+func (s *trackingSuite) TestDoCreateTransientScopeHappyCgroupV2(c *C) {
+	signalChan := make(chan struct{})
+	conn, inject, err := dbustest.InjectableConnection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
+		c.Logf("message: %v", msg)
+		switch n {
+		case 0:
+			return []*dbus.Message{checkSystemdSignalSubscribe(c, msg)}, nil
+		case 1:
+			defer func() {
+				close(signalChan)
+			}()
+			return []*dbus.Message{checkAndRespondToStartTransientUnit(c, msg, "foo.scope", 312123)}, nil
+		case 2:
+			return []*dbus.Message{checkSystemSignalUnsubscribe(c, msg)}, nil
+		}
+		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
+	})
+	go func() {
+		select {
+		case <-signalChan:
+			inject(mockJobRemovedSignal("foo.scope", "done"))
+		}
+	}()
+
+	c.Assert(err, IsNil)
+	defer conn.Close()
+	err = cgroup.DoCreateTransientScope(conn, "foo.scope", 312123)
+	c.Assert(err, IsNil)
+}
+
+func (s *trackingSuite) TestDoCreateTransientScopeHappyCgroupV1(c *C) {
+	restore := cgroup.MockVersion(cgroup.V1, nil)
+	defer restore()
+
 	conn, err := dbustest.Connection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
+		c.Logf("message: %v", msg)
 		switch n {
 		case 0:
 			return []*dbus.Message{checkAndRespondToStartTransientUnit(c, msg, "foo.scope", 312123)}, nil
 		}
 		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
 	})
-
 	c.Assert(err, IsNil)
 	defer conn.Close()
 	err = cgroup.DoCreateTransientScope(conn, "foo.scope", 312123)
@@ -479,7 +804,11 @@ func (s *trackingSuite) TestDoCreateTransientScopeForwardedErrors(c *C) {
 		conn, err := dbustest.Connection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
 			switch n {
 			case 0:
+				return []*dbus.Message{checkSystemdSignalSubscribe(c, msg)}, nil
+			case 1:
 				return []*dbus.Message{checkAndFailToStartTransientUnit(c, msg, t.dbusError)}, nil
+			case 2:
+				return []*dbus.Message{checkSystemSignalUnsubscribe(c, msg)}, nil
 			}
 			return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
 		})
@@ -498,7 +827,11 @@ func (s *trackingSuite) TestDoCreateTransientScopeClashingScopeName(c *C) {
 	conn, err := dbustest.Connection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
 		switch n {
 		case 0:
+			return []*dbus.Message{checkSystemdSignalSubscribe(c, msg)}, nil
+		case 1:
 			return []*dbus.Message{checkAndFailToStartTransientUnit(c, msg, errMsg)}, nil
+		case 2:
+			return []*dbus.Message{checkSystemSignalUnsubscribe(c, msg)}, nil
 		}
 		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
 	})
@@ -514,7 +847,11 @@ func (s *trackingSuite) TestDoCreateTransientScopeOtherDBusErrors(c *C) {
 	conn, err := dbustest.Connection(func(msg *dbus.Message, n int) ([]*dbus.Message, error) {
 		switch n {
 		case 0:
+			return []*dbus.Message{checkSystemdSignalSubscribe(c, msg)}, nil
+		case 1:
 			return []*dbus.Message{checkAndFailToStartTransientUnit(c, msg, errMsg)}, nil
+		case 2:
+			return []*dbus.Message{checkSystemSignalUnsubscribe(c, msg)}, nil
 		}
 		return nil, fmt.Errorf("unexpected message #%d: %s", n, msg)
 	})
