@@ -1,8 +1,9 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
+//go:build nosecboot
 // +build nosecboot
 
 /*
- * Copyright (C) 2020 Canonical Ltd
+ * Copyright (C) 2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,6 +21,22 @@
 
 package secboot
 
-func (k RecoveryKey) String() string {
-	return "not-implemented"
+import (
+	"github.com/snapcore/snapd/secboot/keys"
+)
+
+func EnsureRecoveryKey(string, []RecoveryKeyDevice) (keys.RecoveryKey, error) {
+	return keys.RecoveryKey{}, errBuildWithoutSecboot
+}
+
+func RemoveRecoveryKeys(map[RecoveryKeyDevice]string) error {
+	return errBuildWithoutSecboot
+}
+
+func StageEncryptionKeyChange(node string, key keys.EncryptionKey) error {
+	return errBuildWithoutSecboot
+}
+
+func TransitionEncryptionKeyChange(mountpoint string, key keys.EncryptionKey) error {
+	return errBuildWithoutSecboot
 }
