@@ -107,6 +107,12 @@ version: 1.0
 	"arm-kernel=20": `name: arm-kernel
 type: kernel
 version: 1.0
+`,
+	"test-devmode=20": `name: test-devmode
+type: app
+base: core20
+version: 1.0
+confinement: devmode
 `,
 }
 
