@@ -24,10 +24,13 @@ package quota
 import (
 	"bytes"
 	"fmt"
+	"path/filepath"
 	"runtime"
 	"sort"
+	"time"
 
 	// TODO: move this to snap/quantity? or similar
+	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/snap/naming"
@@ -52,9 +55,32 @@ type GroupQuotaCPU struct {
 	// in Percentage and Count is ignored.
 	Percentage int `json:"percentage,omitempty"`
 
-	// AllowedCPUs is a list of CPU core indices that are allowed to be used by the group. Each value
+	// CPUSet is a list of CPU core indices that are allowed to be used by the group. Each value
 	// in the list refers to the CPU core number. If the list is empty, all CPU cores are allowed.
-	AllowedCPUs []int `json:"allowed-cpus,omitempty"`
+	CPUSet []int `json:"allowed-cpus,omitempty"`
+}
+
+// GroupQuotaJournal contains the supported limits for journald. Any limit set here
+// applies only to the quota group itself. Journal limits will not be inherited by the
+// sub-groups as this behaviour is not supported by systemd.
+type GroupQuotaJournal struct {
+	// Size is the maximum allowed size of the journal for the group.
+	// If the size is set below current usage, systemd will automatically treat
+	// the current usage of the journald namespace as the minimum limit and
+	// render whatever set here ineffective. The maximum allowed size for
+	// journald namespaces is 4GB. A value of 0 here means no limit is present.
+	Size quantity.Size `json:"size,omitempty"`
+
+	// RateEnabled tells us whether or not the values provided in RateCount and
+	// RatePeriod should be written.
+	RateEnabled bool `json:"rate-enabled,omitempty"`
+	// RateCount is the number of messages allowed each RatePeriod. A zero value
+	// in this field will disable the rate-limit.
+	RateCount int `json:"rate-count,omitempty"`
+	// RatePeriod is the time-period for when the rate resets. Each RatePeriod,
+	// RateCount number of messages is allowed. A zero value in this field will
+	// disable the rate-limit.
+	RatePeriod time.Duration `json:"rate-period,omitempty"`
 }
 
 // Group is a quota group of snaps, services or sub-groups that are all subject
@@ -88,10 +114,15 @@ type Group struct {
 	// and which cores (requires cgroupsv2) are allowed to be used.
 	CPULimit *GroupQuotaCPU `json:"cpu-limit,omitempty"`
 
-	// TaskLimit is the limit of threads/processes that can be active at once in
+	// ThreadLimit is the limit of threads/processes that can be active at once in
 	// the group. Once the limit is reached, further forks() or clones() will be blocked
 	// for processes in the group.
-	TaskLimit int `json:"task-limit,omitempty"`
+	ThreadLimit int `json:"task-limit,omitempty"`
+
+	// JournalLimit is the limits that apply to the journal for this quota group. When
+	// this limit is present, then the quota group will be assigned a log namespace for
+	// journald.
+	JournalLimit *GroupQuotaJournal `json:"journal-limit,omitempty"`
 
 	// ParentGroup is the the parent group that this group is a child of. If it
 	// is empty, then this is a "root" quota group.
@@ -135,12 +166,25 @@ func (grp *Group) GetQuotaResources() Resources {
 		if grp.CPULimit.Percentage != 0 {
 			resourcesBuilder.WithCPUPercentage(grp.CPULimit.Percentage)
 		}
-		if len(grp.CPULimit.AllowedCPUs) != 0 {
-			resourcesBuilder.WithAllowedCPUs(grp.CPULimit.AllowedCPUs)
+		if len(grp.CPULimit.CPUSet) != 0 {
+			resourcesBuilder.WithCPUSet(grp.CPULimit.CPUSet)
 		}
 	}
-	if grp.TaskLimit != 0 {
-		resourcesBuilder.WithThreadLimit(grp.TaskLimit)
+	if grp.ThreadLimit != 0 {
+		resourcesBuilder.WithThreadLimit(grp.ThreadLimit)
+	}
+	if grp.JournalLimit != nil {
+		resourcesBuilder.WithJournalNamespace()
+		if grp.JournalLimit.Size != 0 {
+			resourcesBuilder.WithJournalSize(grp.JournalLimit.Size)
+		}
+		// We cannot just check for RateCount and RatePeriod and call WithJournalRate()
+		// only if both are non-zero, because not calling WithJournalRate() causes the
+		// system's default rate count and rate period to be used; what we really want
+		// here is to be able to completely disable the rate-limit for a journal quota.
+		if grp.JournalLimit.RateEnabled {
+			resourcesBuilder.WithJournalRate(grp.JournalLimit.RateCount, grp.JournalLimit.RatePeriod)
+		}
 	}
 	return resourcesBuilder.Build()
 }
@@ -224,8 +268,38 @@ func (grp *Group) SliceFileName() string {
 	return buf.String()
 }
 
+// JournalNamespaceName returns the snap formatted name of the log namespace
+func (grp *Group) JournalNamespaceName() string {
+	return fmt.Sprintf("snap-%s", grp.Name)
+}
+
+// JournalConfFileName returns the name of the journal configuration file that should
+// be used for this quota group. As an example, a group named "foo" will return a name
+// of journald@snap-foo.conf
+func (grp *Group) JournalConfFileName() string {
+	return fmt.Sprintf("journald@%s.conf", grp.JournalNamespaceName())
+}
+
+// JournalServiceName returns the systemd service name for the quota group.
+func (grp *Group) JournalServiceName() string {
+	return fmt.Sprintf("systemd-journald@%s.service", grp.JournalNamespaceName())
+}
+
+// JournalServiceFile returns the directory specific to this quota group for
+// its journal service unit drop-in.
+func (grp *Group) JournalServiceDropInDir() string {
+	return filepath.Join(dirs.SnapServicesDir, grp.JournalServiceName()+".d")
+}
+
+// JournalServiceDropInFile returns the full path to the journal service unit drop-in
+// file for the quota group.
+func (grp *Group) JournalServiceDropInFile() string {
+	return filepath.Join(grp.JournalServiceDropInDir(), "00-snap.conf")
+}
+
 // groupQuotaAllocations contains information about current quotas of a group
-// and is used by getQuotaAllocations to contain this information.
+// and is used by getQuotaAllocations to contain this information. This only accounts
+// for quotas that support inheritance, which currently does not include journal quotas.
 // There are two types of values for each quota - the quota limit set by this group,
 // and the quota reserved by children of this group. Examples:
 // Group that has a non-memory quota, but has a child group that has a memory quota of 512mb:
@@ -250,8 +324,8 @@ type groupQuotaAllocations struct {
 	ThreadsLimit              int
 	ThreadsReservedByChildren int
 
-	AllowedCPUsLimit              []int
-	AllowedCPUsReservedByChildren []int
+	CPUSetLimit              []int
+	CPUSetReservedByChildren []int
 }
 
 func max(a, b int) int {
@@ -271,10 +345,10 @@ func maxq(a, b quantity.Size) quantity.Size {
 // GetLocalCPUSetQuota returns the current CPU set quota for the group. This
 // does not return any inheritted CPU set quota.
 func (grp *Group) GetLocalCPUSetQuota() []int {
-	if grp.CPULimit == nil || len(grp.CPULimit.AllowedCPUs) == 0 {
+	if grp.CPULimit == nil || len(grp.CPULimit.CPUSet) == 0 {
 		return []int{}
 	}
-	return grp.CPULimit.AllowedCPUs
+	return grp.CPULimit.CPUSet
 }
 
 // GetCPUSetQuota returns the currently active CPU set quota for this group, which
@@ -287,8 +361,8 @@ func (grp *Group) GetCPUSetQuota() []int {
 
 	parent := grp.parentGroup
 	for parent != nil {
-		if parent.CPULimit != nil && len(parent.CPULimit.AllowedCPUs) != 0 {
-			return parent.CPULimit.AllowedCPUs
+		if parent.CPULimit != nil && len(parent.CPULimit.CPUSet) != 0 {
+			return parent.CPULimit.CPUSet
 		}
 		parent = parent.parentGroup
 	}
@@ -331,10 +405,10 @@ func (grp *Group) getCurrentCPUAllocation() int {
 // tree and store them in the allQuotas paramater
 func (grp *Group) getQuotaAllocations(allQuotas map[string]*groupQuotaAllocations) *groupQuotaAllocations {
 	limits := &groupQuotaAllocations{
-		MemoryLimit:      grp.MemoryLimit,
-		CPULimit:         grp.getCurrentCPUAllocation(),
-		ThreadsLimit:     grp.TaskLimit,
-		AllowedCPUsLimit: grp.GetLocalCPUSetQuota(),
+		MemoryLimit:  grp.MemoryLimit,
+		CPULimit:     grp.getCurrentCPUAllocation(),
+		ThreadsLimit: grp.ThreadLimit,
+		CPUSetLimit:  grp.GetLocalCPUSetQuota(),
 	}
 
 	// sliceUniqueAndSort sorts an array of ints in ascending order and removes duplicates
@@ -366,16 +440,16 @@ func (grp *Group) getQuotaAllocations(allQuotas map[string]*groupQuotaAllocation
 
 		// We need to merge the allowed CPUs lists, but we need to make sure that the list is unique, since cpu cores
 		// can be reused between sub-groups.
-		if len(subGroupLimits.AllowedCPUsLimit) > 0 {
-			limits.AllowedCPUsReservedByChildren = append(limits.AllowedCPUsReservedByChildren, subGroupLimits.AllowedCPUsLimit...)
-		} else if len(subGroupLimits.AllowedCPUsReservedByChildren) > 0 {
-			limits.AllowedCPUsReservedByChildren = append(limits.AllowedCPUsReservedByChildren, subGroupLimits.AllowedCPUsReservedByChildren...)
+		if len(subGroupLimits.CPUSetLimit) > 0 {
+			limits.CPUSetReservedByChildren = append(limits.CPUSetReservedByChildren, subGroupLimits.CPUSetLimit...)
+		} else if len(subGroupLimits.CPUSetReservedByChildren) > 0 {
+			limits.CPUSetReservedByChildren = append(limits.CPUSetReservedByChildren, subGroupLimits.CPUSetReservedByChildren...)
 		}
 	}
 
 	// Sort the allowed CPUs list, and remove duplicates.
-	if len(limits.AllowedCPUsReservedByChildren) > 0 {
-		limits.AllowedCPUsReservedByChildren = sliceUniqueAndSort(limits.AllowedCPUsReservedByChildren)
+	if len(limits.CPUSetReservedByChildren) > 0 {
+		limits.CPUSetReservedByChildren = sliceUniqueAndSort(limits.CPUSetReservedByChildren)
 	}
 
 	// Store the retrieved limits for the group
@@ -484,11 +558,11 @@ func (grp *Group) validateCPUResourceFit(allQuotas map[string]*groupQuotaAllocat
 						cpuRequested, parent.Name, cpuAvailable)
 				}
 				break
-			} else if len(limits.AllowedCPUsLimit) > 0 {
-				maxCPUAvailableInSet := len(limits.AllowedCPUsLimit) * 100
+			} else if len(limits.CPUSetLimit) > 0 {
+				maxCPUAvailableInSet := len(limits.CPUSetLimit) * 100
 				if cpuRequested > maxCPUAvailableInSet {
 					return fmt.Errorf("sub-group cpu limit of %d%% is too large to fit inside group %q with allowed CPU set %v",
-						cpuRequested, parent.Name, limits.AllowedCPUsLimit)
+						cpuRequested, parent.Name, limits.CPUSetLimit)
 				}
 				break
 			}
@@ -526,9 +600,9 @@ func (grp *Group) validateCPUsAllowedResourceFit(allQuotas map[string]*groupQuot
 	// recursive descent as we already have counted up the usage of our children.
 	currentLimits := allQuotas[grp.Name]
 	if currentLimits != nil {
-		if !isSuperset(cpusAllowed, currentLimits.AllowedCPUsReservedByChildren) {
+		if !isSuperset(cpusAllowed, currentLimits.CPUSetReservedByChildren) {
 			return fmt.Errorf("group cpu-set %v is not a superset of current subgroup usage of %v",
-				cpusAllowed, currentLimits.AllowedCPUsReservedByChildren)
+				cpusAllowed, currentLimits.CPUSetReservedByChildren)
 		}
 
 		// If we are doing further restrictions (i.e the new cpu set is a subset of the current)
@@ -544,10 +618,10 @@ func (grp *Group) validateCPUsAllowedResourceFit(allQuotas map[string]*groupQuot
 	parent := grp.parentGroup
 	for parent != nil {
 		limits := allQuotas[parent.Name]
-		if limits != nil && len(limits.AllowedCPUsLimit) != 0 {
-			if !isSuperset(limits.AllowedCPUsLimit, cpusAllowed) {
+		if limits != nil && len(limits.CPUSetLimit) != 0 {
+			if !isSuperset(limits.CPUSetLimit, cpusAllowed) {
 				return fmt.Errorf("sub-group cpu-set %v is not a subset of group %q cpu-set %v",
-					cpusAllowed, parent.Name, limits.AllowedCPUsLimit)
+					cpusAllowed, parent.Name, limits.CPUSetLimit)
 			}
 			break
 		}
@@ -565,7 +639,7 @@ func (grp *Group) validateThreadResourceFit(allQuotas map[string]*groupQuotaAllo
 	// make sure current usage does not exceed the new limit, we can avoid any
 	// recursive descent as we already have counted up the usage of our children.
 	currentLimits := allQuotas[grp.Name]
-	threadsReserved := grp.TaskLimit
+	threadsReserved := grp.ThreadLimit
 	if currentLimits != nil {
 		if currentLimits.ThreadsReservedByChildren > threadLimit {
 			return fmt.Errorf("group thread limit of %d is too small to fit current subgroup usage of %d",
@@ -574,7 +648,7 @@ func (grp *Group) validateThreadResourceFit(allQuotas map[string]*groupQuotaAllo
 
 		// if we are reducing the limit, then we don't need to check upper parents,
 		// as we can assume it will fit by this point
-		if threadLimit < grp.TaskLimit {
+		if threadLimit < grp.ThreadLimit {
 			return nil
 		}
 
@@ -667,10 +741,23 @@ func (grp *Group) UpdateQuotaLimits(resourceLimits Resources) error {
 		if grp.CPULimit == nil {
 			grp.CPULimit = &GroupQuotaCPU{}
 		}
-		grp.CPULimit.AllowedCPUs = resourceLimits.CPUSet.CPUs
+		grp.CPULimit.CPUSet = resourceLimits.CPUSet.CPUs
 	}
 	if resourceLimits.Threads != nil {
-		grp.TaskLimit = resourceLimits.Threads.Limit
+		grp.ThreadLimit = resourceLimits.Threads.Limit
+	}
+	if resourceLimits.Journal != nil {
+		if grp.JournalLimit == nil {
+			grp.JournalLimit = &GroupQuotaJournal{}
+		}
+		if resourceLimits.Journal.Size != nil {
+			grp.JournalLimit.Size = resourceLimits.Journal.Size.Limit
+		}
+		if resourceLimits.Journal.Rate != nil {
+			grp.JournalLimit.RateEnabled = true
+			grp.JournalLimit.RateCount = resourceLimits.Journal.Rate.Count
+			grp.JournalLimit.RatePeriod = resourceLimits.Journal.Rate.Period
+		}
 	}
 	return nil
 }
