@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2020 Canonical Ltd
+ * Copyright (C) 2014-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -30,6 +30,7 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"os/exec"
 	"path"
 	"strconv"
 	"strings"
@@ -48,6 +49,8 @@ import (
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/channel"
+	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/snapdenv"
 	"github.com/snapcore/snapd/strutil"
 )
@@ -65,6 +68,8 @@ const (
 	UbuntuCoreWireProtocol = "1"
 )
 
+var requestTimeout = 10 * time.Second
+
 // the LimitTime should be slightly more than 3 times of our http.Client
 // Timeout value
 var defaultRetryStrategy = retry.LimitCount(6, retry.LimitTime(38*time.Second,
@@ -88,6 +93,10 @@ type Config struct {
 	StoreBaseURL      *url.URL
 	AssertionsBaseURL *url.URL
 
+	// Authorizer used to authorize requests, can be nil and a default
+	// will be used.
+	Authorizer Authorizer
+
 	// StoreID is the store id used if we can't get one through the DeviceAndAuthContext.
 	StoreID string
 
@@ -141,11 +150,12 @@ type Store struct {
 	infoFields   []string
 	findFields   []string
 	deltaFormat  string
+
+	auth Authorizer
 	// reused http client
 	client *http.Client
 
-	dauthCtx  DeviceAndAuthContext
-	sessionMu sync.Mutex
+	dauthCtx DeviceAndAuthContext
 
 	mu                sync.Mutex
 	suggestedCurrency string
@@ -156,6 +166,12 @@ type Store struct {
 	proxyConnectHeader http.Header
 
 	userAgent string
+
+	xdeltaCheckLock sync.Mutex
+	// whether we should use deltas or not
+	shouldUseDeltas *bool
+	// which xdelta3 we picked when we checked the deltas
+	xdelta3CmdFunc func(args ...string) *exec.Cmd
 }
 
 var ErrTooManyRequests = errors.New("too many requests")
@@ -385,9 +401,19 @@ func New(cfg *Config, dauthCtx DeviceAndAuthContext) *Store {
 		userAgent:          userAgent,
 	}
 	store.client = store.newHTTPClient(&httputil.ClientOptions{
-		Timeout:    10 * time.Second,
+		Timeout:    requestTimeout,
 		MayLogBody: true,
 	})
+	auth := cfg.Authorizer
+	if auth == nil {
+		if dauthCtx != nil {
+			auth = &deviceAuthorizer{endpointURL: store.endpointURL}
+		} else {
+			auth = UserAuthorizer{}
+		}
+	}
+	store.auth = auth
+
 	store.SetCacheDownloads(cfg.CacheDownloads)
 
 	return store
@@ -483,177 +509,13 @@ func (s *Store) LoginUser(username, password, otp string) (string, string, error
 	return macaroon, discharge, nil
 }
 
-// authAvailable returns true if there is a user and/or device session setup
-func (s *Store) authAvailable(user *auth.UserState) (bool, error) {
-	if user.HasStoreAuth() {
-		return true, nil
-	} else {
-		var device *auth.DeviceState
-		var err error
-		if s.dauthCtx != nil {
-			device, err = s.dauthCtx.Device()
-			if err != nil {
-				return false, err
-			}
-		}
-		return device != nil && device.SessionMacaroon != "", nil
-	}
-}
-
-// authenticateUser will add the store expected Macaroon Authorization header for user
-func authenticateUser(r *http.Request, user *auth.UserState) {
-	var buf bytes.Buffer
-	fmt.Fprintf(&buf, `Macaroon root="%s"`, user.StoreMacaroon)
-
-	// deserialize root macaroon (we need its signature to do the discharge binding)
-	root, err := auth.MacaroonDeserialize(user.StoreMacaroon)
-	if err != nil {
-		logger.Debugf("cannot deserialize root macaroon: %v", err)
-		return
-	}
-
-	for _, d := range user.StoreDischarges {
-		// prepare discharge for request
-		discharge, err := auth.MacaroonDeserialize(d)
-		if err != nil {
-			logger.Debugf("cannot deserialize discharge macaroon: %v", err)
-			return
-		}
-		discharge.Bind(root.Signature())
-
-		serializedDischarge, err := auth.MacaroonSerialize(discharge)
-		if err != nil {
-			logger.Debugf("cannot re-serialize discharge macaroon: %v", err)
-			return
-		}
-		fmt.Fprintf(&buf, `, discharge="%s"`, serializedDischarge)
-	}
-	r.Header.Set("Authorization", buf.String())
-}
-
-// refreshDischarges will request refreshed discharge macaroons for the user
-func refreshDischarges(httpClient *http.Client, user *auth.UserState) ([]string, error) {
-	newDischarges := make([]string, len(user.StoreDischarges))
-	for i, d := range user.StoreDischarges {
-		discharge, err := auth.MacaroonDeserialize(d)
-		if err != nil {
-			return nil, err
-		}
-		if discharge.Location() != UbuntuoneLocation {
-			newDischarges[i] = d
-			continue
-		}
-
-		refreshedDischarge, err := refreshDischargeMacaroon(httpClient, d)
-		if err != nil {
-			return nil, err
-		}
-		newDischarges[i] = refreshedDischarge
-	}
-	return newDischarges, nil
-}
-
-// refreshUser will refresh user discharge macaroon and update state
-func (s *Store) refreshUser(user *auth.UserState) error {
-	if s.dauthCtx == nil {
-		return fmt.Errorf("user credentials need to be refreshed but update in place only supported in snapd")
-	}
-	newDischarges, err := refreshDischarges(s.client, user)
-	if err != nil {
-		return err
-	}
-
-	curUser, err := s.dauthCtx.UpdateUserAuth(user, newDischarges)
-	if err != nil {
-		return err
-	}
-	// update in place
-	*user = *curUser
-
-	return nil
-}
-
-// refreshDeviceSession will set or refresh the device session in the state
-func (s *Store) refreshDeviceSession(device *auth.DeviceState) error {
-	if s.dauthCtx == nil {
-		return fmt.Errorf("internal error: no device and auth context")
-	}
-
-	s.sessionMu.Lock()
-	defer s.sessionMu.Unlock()
-	// check that no other goroutine has already got a new session etc...
-	device1, err := s.dauthCtx.Device()
-	if err != nil {
-		return err
-	}
-	// We can replace device with "device1" here because Device
-	// and UpdateDeviceAuth (and the underlying SetDevice)
-	// require/use the global state lock, so the reading/setting
-	// values have a total order, and device1 cannot come before
-	// device in that order. See also:
-	// https://github.com/snapcore/snapd/pull/6716#discussion_r277025834
-	if *device1 != *device {
-		// nothing to do
-		*device = *device1
-		return nil
-	}
-
-	nonce, err := requestStoreDeviceNonce(s.client, s.endpointURL(deviceNonceEndpPath, nil).String())
-	if err != nil {
-		return err
-	}
-
-	devSessReqParams, err := s.dauthCtx.DeviceSessionRequestParams(nonce)
-	if err != nil {
-		return err
-	}
-
-	session, err := requestDeviceSession(s.client, s.endpointURL(deviceSessionEndpPath, nil).String(), devSessReqParams, device.SessionMacaroon)
-	if err != nil {
-		return err
-	}
-
-	curDevice, err := s.dauthCtx.UpdateDeviceAuth(device, session)
-	if err != nil {
-		return err
-	}
-	// update in place
-	*device = *curDevice
-	return nil
-}
-
 // EnsureDeviceSession makes sure the store has a device session available.
 // Expects the store to have an AuthContext.
-func (s *Store) EnsureDeviceSession() (*auth.DeviceState, error) {
-	if s.dauthCtx == nil {
-		return nil, fmt.Errorf("internal error: no authContext")
-	}
-
-	device, err := s.dauthCtx.Device()
-	if err != nil {
-		return nil, err
-	}
-
-	if device.SessionMacaroon != "" {
-		return device, nil
-	}
-	if device.Serial == "" {
-		return nil, ErrNoSerial
-	}
-	// we don't have a session yet but have a serial, try
-	// to get a session
-	err = s.refreshDeviceSession(device)
-	if err != nil {
-		return nil, err
-	}
-	return device, err
-}
-
-// authenticateDevice will add the store expected Macaroon X-Device-Authorization header for device
-func authenticateDevice(r *http.Request, device *auth.DeviceState, apiLevel apiLevel) {
-	if device != nil && device.SessionMacaroon != "" {
-		r.Header.Set(hdrSnapDeviceAuthorization[apiLevel], fmt.Sprintf(`Macaroon root="%s"`, device.SessionMacaroon))
+func (s *Store) EnsureDeviceSession() error {
+	if a, ok := s.auth.(*deviceAuthorizer); ok {
+		return a.EnsureDeviceSession(s.dauthCtx, s.client)
 	}
+	return nil
 }
 
 func (s *Store) setStoreID(r *http.Request, apiLevel apiLevel) (customStore bool) {
@@ -691,6 +553,7 @@ var (
 type deviceAuthNeed int
 
 const (
+	//nolint:deadcode
 	deviceAuthPreferred deviceAuthNeed = iota
 	deviceAuthCustomStoreOnly
 )
@@ -841,28 +704,30 @@ func (s *Store) doRequest(ctx context.Context, client *http.Client, reqOptions *
 			return nil, err
 		}
 
-		wwwAuth := resp.Header.Get("WWW-Authenticate")
 		if resp.StatusCode == 401 && authRefreshes < 4 {
 			// 4 tries: 2 tries for each in case both user
 			// and device need refreshing
-			var refreshNeed authRefreshNeed
-			if user != nil && strings.Contains(wwwAuth, "needs_refresh=1") {
+			wwwAuth := resp.Header.Get("WWW-Authenticate")
+			var refreshNeed AuthRefreshNeed
+			if strings.Contains(wwwAuth, "needs_refresh=1") {
 				// refresh user
-				refreshNeed.user = true
+				refreshNeed.User = true
 			}
 			if strings.Contains(wwwAuth, "refresh_device_session=1") {
 				// refresh device session
-				refreshNeed.device = true
+				refreshNeed.Device = true
 			}
 			if refreshNeed.needed() {
-				err := s.refreshAuth(user, refreshNeed)
-				if err != nil {
-					return nil, err
+				if a, ok := s.auth.(RefreshingAuthorizer); ok {
+					err := a.RefreshAuth(refreshNeed, s.dauthCtx, user, s.client)
+					if err != nil {
+						return nil, err
+					}
+					// close previous response and retry
+					resp.Body.Close()
+					authRefreshes++
+					continue
 				}
-				// close previous response and retry
-				resp.Body.Close()
-				authRefreshes++
-				continue
 			}
 		}
 
@@ -870,39 +735,29 @@ func (s *Store) doRequest(ctx context.Context, client *http.Client, reqOptions *
 	}
 }
 
-type authRefreshNeed struct {
-	device bool
-	user   bool
-}
-
-func (rn *authRefreshNeed) needed() bool {
-	return rn.device || rn.user
-}
-
-func (s *Store) refreshAuth(user *auth.UserState, need authRefreshNeed) error {
-	if need.user {
-		// refresh user
-		err := s.refreshUser(user)
-		if err != nil {
-			return err
-		}
+func (s *Store) buildLocationString() (string, error) {
+	if s.dauthCtx == nil {
+		return "", nil
 	}
-	if need.device {
-		// refresh device session
-		if s.dauthCtx == nil {
-			return fmt.Errorf("internal error: no device and auth context")
-		}
-		device, err := s.dauthCtx.Device()
-		if err != nil {
-			return err
-		}
 
-		err = s.refreshDeviceSession(device)
-		if err != nil {
-			return err
-		}
+	cloudInfo, err := s.dauthCtx.CloudInfo()
+	if err != nil {
+		return "", err
 	}
-	return nil
+
+	if cloudInfo == nil {
+		return "", nil
+	}
+
+	cdnParams := []string{fmt.Sprintf("cloud-name=%q", cloudInfo.Name)}
+	if cloudInfo.Region != "" {
+		cdnParams = append(cdnParams, fmt.Sprintf("region=%q", cloudInfo.Region))
+	}
+	if cloudInfo.AvailabilityZone != "" {
+		cdnParams = append(cdnParams, fmt.Sprintf("availability-zone=%q", cloudInfo.AvailabilityZone))
+	}
+
+	return strings.Join(cdnParams, " "), nil
 }
 
 // build a new http.Request with headers for the store
@@ -918,23 +773,20 @@ func (s *Store) newRequest(ctx context.Context, reqOptions *requestOptions, user
 	}
 
 	customStore := s.setStoreID(req, reqOptions.APILevel)
-
-	if s.dauthCtx != nil && (customStore || reqOptions.DeviceAuthNeed != deviceAuthCustomStoreOnly) {
-		device, err := s.EnsureDeviceSession()
+	authOpts := AuthorizeOptions{apiLevel: reqOptions.APILevel}
+	authOpts.deviceAuth = customStore || reqOptions.DeviceAuthNeed != deviceAuthCustomStoreOnly
+	if authOpts.deviceAuth {
+		err := s.EnsureDeviceSession()
 		if err != nil && err != ErrNoSerial {
 			return nil, err
 		}
 		if err == ErrNoSerial {
 			// missing serial assertion, log and continue without device authentication
 			logger.Debugf("cannot set device session: %v", err)
-		} else {
-			authenticateDevice(req, device, reqOptions.APILevel)
 		}
 	}
-
-	// only set user authentication if user logged in to the store
-	if user.HasStoreAuth() {
-		authenticateUser(req, user)
+	if err := s.auth.Authorize(req, s.dauthCtx, user, &authOpts); err != nil {
+		logger.Debugf("cannot authorize store request: %v", err)
 	}
 
 	req.Header.Set("User-Agent", s.userAgent)
@@ -943,6 +795,13 @@ func (s *Store) newRequest(ctx context.Context, reqOptions *requestOptions, user
 	req.Header.Set(hdrSnapDeviceSeries[reqOptions.APILevel], s.series)
 	req.Header.Set(hdrSnapClassic[reqOptions.APILevel], strconv.FormatBool(release.OnClassic))
 	req.Header.Set("Snap-Device-Capabilities", "default-tracks")
+	locationHeader, err := s.buildLocationString()
+	if err != nil {
+		return nil, err
+	}
+	if locationHeader != "" {
+		req.Header.Set("Snap-Device-Location", locationHeader)
+	}
 	if cua := ClientUserAgent(ctx); cua != "" {
 		req.Header.Set("Snap-Client-User-Agent", cua)
 	}
@@ -1017,7 +876,7 @@ func (s *Store) decorateOrders(snaps []*snap.Info, user *auth.UserState) error {
 		}
 	}
 
-	if user == nil {
+	if !s.auth.CanAuthorizeForUser(user) {
 		return nil
 	}
 
@@ -1076,35 +935,14 @@ type SnapSpec struct {
 
 // SnapInfo returns the snap.Info for the store-hosted snap matching the given spec, or an error.
 func (s *Store) SnapInfo(ctx context.Context, snapSpec SnapSpec, user *auth.UserState) (*snap.Info, error) {
-	query := url.Values{}
-	query.Set("fields", strings.Join(s.infoFields, ","))
-	query.Set("architecture", s.architecture)
+	fields := strings.Join(s.infoFields, ",")
 
-	u := s.endpointURL(path.Join(snapInfoEndpPath, snapSpec.Name), query)
-	reqOptions := &requestOptions{
-		Method:   "GET",
-		URL:      u,
-		APILevel: apiV2Endps,
-	}
-
-	var remote storeInfo
-	resp, err := s.retryRequestDecodeJSON(ctx, reqOptions, user, &remote, nil)
+	si, resp, err := s.snapInfo(ctx, snapSpec.Name, fields, user)
 	if err != nil {
 		return nil, err
 	}
 
-	// check statusCode
-	switch resp.StatusCode {
-	case 200:
-		// OK
-	case 404:
-		return nil, ErrSnapNotFound
-	default:
-		msg := fmt.Sprintf("get details for snap %q", snapSpec.Name)
-		return nil, respToError(resp, msg)
-	}
-
-	info, err := infoFromStoreInfo(&remote)
+	info, err := infoFromStoreInfo(si)
 	if err != nil {
 		return nil, err
 	}
@@ -1119,6 +957,51 @@ func (s *Store) SnapInfo(ctx context.Context, snapSpec SnapSpec, user *auth.User
 	return info, nil
 }
 
+func (s *Store) snapInfo(ctx context.Context, snapName string, fields string, user *auth.UserState) (*storeInfo, *http.Response, error) {
+	query := url.Values{}
+	query.Set("fields", fields)
+	query.Set("architecture", s.architecture)
+
+	u := s.endpointURL(path.Join(snapInfoEndpPath, snapName), query)
+	reqOptions := &requestOptions{
+		Method:   "GET",
+		URL:      u,
+		APILevel: apiV2Endps,
+	}
+
+	var remote storeInfo
+	resp, err := s.retryRequestDecodeJSON(ctx, reqOptions, user, &remote, nil)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	// check statusCode
+	switch resp.StatusCode {
+	case 200:
+		// OK
+	case 404:
+		return nil, nil, ErrSnapNotFound
+	default:
+		msg := fmt.Sprintf("get details for snap %q", snapName)
+		return nil, nil, respToError(resp, msg)
+	}
+
+	return &remote, resp, err
+}
+
+// SnapInfo checks whether the store-hosted snap matching the given spec exists and returns a reference with it name and snap-id and default channel, or an error.
+func (s *Store) SnapExists(ctx context.Context, snapSpec SnapSpec, user *auth.UserState) (naming.SnapRef, *channel.Channel, error) {
+	// request the minimal amount information
+	fields := "channel-map"
+
+	si, _, err := s.snapInfo(ctx, snapSpec.Name, fields, user)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	return minimalFromStoreInfo(si)
+}
+
 // A Search is what you do in order to Find something
 type Search struct {
 	// Query is a term to search by or a prefix (if Prefix is true)
@@ -1136,7 +1019,7 @@ type Search struct {
 // Find finds  (installable) snaps from the store, matching the
 // given Search.
 func (s *Store) Find(ctx context.Context, search *Search, user *auth.UserState) ([]*snap.Info, error) {
-	if search.Private && user == nil {
+	if search.Private && !s.auth.CanAuthorizeForUser(user) {
 		return nil, ErrUnauthenticated
 	}
 
@@ -1347,7 +1230,7 @@ func (s *Store) Sections(ctx context.Context, user *auth.UserState) ([]string, e
 	}
 
 	if resp.StatusCode != 200 {
-		return nil, respToError(resp, "sections")
+		return nil, respToError(resp, "retrieve sections")
 	}
 
 	if ct := resp.Header.Get("Content-Type"); ct != halJsonContentType {
@@ -1466,7 +1349,7 @@ func (s *Store) Buy(options *client.BuyOptions, user *auth.UserState) (*client.B
 	if options.Currency == "" {
 		return buyOptionError("currency missing")
 	}
-	if user == nil {
+	if !s.auth.CanAuthorizeForUser(user) {
 		return nil, ErrUnauthenticated
 	}
 
@@ -1541,7 +1424,7 @@ type storeCustomer struct {
 
 // ReadyToBuy returns nil if the user's account has accepted T&Cs and has a payment method registered, and an error otherwise
 func (s *Store) ReadyToBuy(user *auth.UserState) error {
-	if user == nil {
+	if !s.auth.CanAuthorizeForUser(user) {
 		return ErrUnauthenticated
 	}
 
