@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2020 Canonical Ltd
+ * Copyright (C) 2014-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -29,6 +29,7 @@ import (
 	"time"
 
 	"github.com/snapcore/snapd/asserts"
+	"github.com/snapcore/snapd/asserts/snapasserts"
 	"github.com/snapcore/snapd/jsonutil"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/auth"
@@ -56,8 +57,8 @@ type CurrentSnap struct {
 	Block            []snap.Revision
 	Epoch            snap.Epoch
 	CohortKey        string
-	// ValidationSets is an optional array of validation sets primary keys.
-	ValidationSets [][]string
+	// ValidationSets is an optional array of validation set primary keys.
+	ValidationSets []snapasserts.ValidationSetKey
 }
 
 type AssertionQuery interface {
@@ -77,7 +78,7 @@ type currentSnapV2JSON struct {
 	RefreshedDate    *time.Time `json:"refreshed-date,omitempty"`
 	IgnoreValidation bool       `json:"ignore-validation,omitempty"`
 	CohortKey        string     `json:"cohort-key,omitempty"`
-	// ValidationSets is an optional array of validation sets primary keys.
+	// ValidationSets is an optional array of validation set primary keys.
 	ValidationSets [][]string `json:"validation-sets,omitempty"`
 }
 
@@ -97,9 +98,9 @@ type SnapAction struct {
 	CohortKey    string
 	Flags        SnapActionFlags
 	Epoch        snap.Epoch
-	// ValidationSets is an optional array of validation sets primary keys
+	// ValidationSets is an optional array of validation set primary keys
 	// (relevant for install and refresh actions).
-	ValidationSets [][]string
+	ValidationSets []snapasserts.ValidationSetKey
 }
 
 func isValidAction(action string) bool {
@@ -237,27 +238,29 @@ func (s *Store) SnapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 
 		if saErr, ok := err.(*SnapActionError); ok && authRefreshes < 2 && len(saErr.Other) > 0 {
 			// do we need to try to refresh auths?, 2 tries
-			var refreshNeed authRefreshNeed
+			var refreshNeed AuthRefreshNeed
 			for _, otherErr := range saErr.Other {
 				switch otherErr {
 				case errUserAuthorizationNeedsRefresh:
-					refreshNeed.user = true
+					refreshNeed.User = true
 				case errDeviceAuthorizationNeedsRefresh:
-					refreshNeed.device = true
+					refreshNeed.Device = true
 				}
 			}
 			if refreshNeed.needed() {
-				err := s.refreshAuth(user, refreshNeed)
-				if err != nil {
-					// best effort
-					logger.Noticef("cannot refresh soft-expired authorisation: %v", err)
+				if a, ok := s.auth.(RefreshingAuthorizer); ok {
+					err := a.RefreshAuth(refreshNeed, s.dauthCtx, user, s.client)
+					if err != nil {
+						// best effort
+						logger.Noticef("cannot refresh soft-expired authorisation: %v", err)
+					}
+					authRefreshes++
+					// TODO: we could avoid retrying here
+					// if refreshAuth gave no error we got
+					// as many non-error results from the
+					// store as actions anyway
+					continue
 				}
-				authRefreshes++
-				// TODO: we could avoid retrying here
-				// if refreshAuth gave no error we got
-				// as many non-error results from the
-				// store as actions anyway
-				continue
 			}
 		}
 
@@ -327,6 +330,12 @@ func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 		if !curSnap.RefreshedDate.IsZero() {
 			refreshedDate = &curSnap.RefreshedDate
 		}
+
+		valsetKeys := make([][]string, 0, len(curSnap.ValidationSets))
+		for _, vsKey := range curSnap.ValidationSets {
+			valsetKeys = append(valsetKeys, vsKey.Components())
+		}
+
 		curSnapJSONs[i] = &currentSnapV2JSON{
 			SnapID:           curSnap.SnapID,
 			InstanceKey:      instanceKey,
@@ -336,7 +345,7 @@ func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 			RefreshedDate:    refreshedDate,
 			Epoch:            curSnap.Epoch,
 			CohortKey:        curSnap.CohortKey,
-			ValidationSets:   curSnap.ValidationSets,
+			ValidationSets:   valsetKeys,
 		}
 	}
 
@@ -368,6 +377,11 @@ func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 			ignoreValidation = &f
 		}
 
+		valsetKeyComponents := make([][]string, 0, len(a.ValidationSets))
+		for _, vsKey := range a.ValidationSets {
+			valsetKeyComponents = append(valsetKeyComponents, vsKey.Components())
+		}
+
 		var instanceKey string
 		aJSON := &snapActionJSON{
 			Action:           a.Action,
@@ -375,7 +389,7 @@ func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 			Channel:          a.Channel,
 			Revision:         a.Revision.N,
 			CohortKey:        a.CohortKey,
-			ValidationSets:   a.ValidationSets,
+			ValidationSets:   valsetKeyComponents,
 			IgnoreValidation: ignoreValidation,
 		}
 		if !a.Revision.Unset() {
@@ -433,7 +447,7 @@ func (s *Store) snapAction(ctx context.Context, currentSnaps []*CurrentSnap, act
 			for j, at := range ats {
 				aj := &assertAtJSON{
 					Type:       at.Type.Name,
-					PrimaryKey: at.PrimaryKey,
+					PrimaryKey: asserts.ReducePrimaryKey(at.Type, at.PrimaryKey),
 				}
 				rev := at.Revision
 				if rev != asserts.RevisionNotKnown {
@@ -677,7 +691,7 @@ func reportFetchAssertionsError(res *snapActionResult, assertq AssertionQuery) e
 	errl := res.ErrorList
 	carryingRef := func(ent *errorListEntry) bool {
 		aType := asserts.Type(ent.Type)
-		return aType != nil && len(ent.PrimaryKey) == len(aType.PrimaryKey)
+		return aType != nil && aType.AcceptablePrimaryKey(ent.PrimaryKey)
 	}
 	carryingSeqKey := func(ent *errorListEntry) bool {
 		aType := asserts.Type(ent.Type)
