@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2020 Canonical Ltd
+ * Copyright (C) 2014-2022 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -32,7 +32,6 @@ import (
 	"os/exec"
 	"path/filepath"
 	"strconv"
-	"strings"
 	"sync"
 	"time"
 
@@ -50,6 +49,8 @@ import (
 	"github.com/snapcore/snapd/snapdtool"
 )
 
+var commandFromSystemSnap = snapdtool.CommandFromSystemSnap
+
 var downloadRetryStrategy = retry.LimitCount(7, retry.LimitTime(90*time.Second,
 	retry.Exponential{
 		Initial: 500 * time.Millisecond,
@@ -80,14 +81,79 @@ func init() {
 }
 
 // Deltas enabled by default on classic, but allow opting in or out on both classic and core.
-func useDeltas() bool {
-	// only xdelta3 is supported for now, so check the binary exists here
-	// TODO: have a per-format checker instead
-	if _, err := getXdelta3Cmd(); err != nil {
+func (s *Store) useDeltas() (use bool) {
+	s.xdeltaCheckLock.Lock()
+	defer s.xdeltaCheckLock.Unlock()
+
+	// check the cached value if available
+	if s.shouldUseDeltas != nil {
+		return *s.shouldUseDeltas
+	}
+
+	defer func() {
+		// cache whatever value we return for next time
+		s.shouldUseDeltas = &use
+	}()
+
+	// check if deltas were disabled by the environment
+	if !osutil.GetenvBool("SNAPD_USE_DELTAS_EXPERIMENTAL", true) {
+		// then the env var is explicitly false, we can't use deltas
+		logger.Debugf("delta usage disabled by environment variable")
 		return false
 	}
 
-	return osutil.GetenvBool("SNAPD_USE_DELTAS_EXPERIMENTAL", true)
+	// TODO: have a per-format checker instead, we currently only support
+	// xdelta3 as a format for deltas
+
+	// check if the xdelta3 config command works from the system snap
+	cmd, err := commandFromSystemSnap("/usr/bin/xdelta3", "config")
+	if err == nil {
+		// we have a xdelta3 from the system snap, make sure it works
+		if runErr := cmd.Run(); runErr == nil {
+			// success using the system snap provided one, setup the callback to
+			// use the cmd we got from CommandFromSystemSnap, but with a small
+			// tweak - this cmd to run xdelta3 from the system snap will likely
+			// have other arguments and a different main exe usually, so
+			// use it exactly as we got it from CommandFromSystemSnap,
+			// but drop the last arg which we know is "config"
+			exe := cmd.Path
+			args := cmd.Args[:len(cmd.Args)-1]
+			env := cmd.Env
+			dir := cmd.Dir
+			s.xdelta3CmdFunc = func(xDelta3args ...string) *exec.Cmd {
+				return &exec.Cmd{
+					Path: exe,
+					Args: append(args, xDelta3args...),
+					Env:  env,
+					Dir:  dir,
+				}
+			}
+			return true
+		} else {
+			logger.Noticef("unable to use system snap provided xdelta3, running config command failed: %v", runErr)
+		}
+	}
+
+	// we didn't have one from a system snap or it didn't work, fallback to
+	// trying xdelta3 from the system
+	loc, err := exec.LookPath("xdelta3")
+	if err != nil {
+		// no xdelta3 in the env, so no deltas
+		logger.Noticef("no host system xdelta3 available to use deltas")
+		return false
+	}
+
+	if err := exec.Command(loc, "config").Run(); err != nil {
+		// xdelta3 in the env failed to run, so no deltas
+		logger.Noticef("unable to use host system xdelta3, running config command failed: %v", err)
+		return false
+	}
+
+	// the xdelta3 in the env worked, so use that one
+	s.xdelta3CmdFunc = func(args ...string) *exec.Cmd {
+		return exec.Command(loc, args...)
+	}
+	return true
 }
 
 func (s *Store) cdnHeader() (string, error) {
@@ -95,10 +161,6 @@ func (s *Store) cdnHeader() (string, error) {
 		return "none", nil
 	}
 
-	if s.dauthCtx == nil {
-		return "", nil
-	}
-
 	// set Snap-CDN from cloud instance information
 	// if available
 
@@ -106,25 +168,7 @@ func (s *Store) cdnHeader() (string, error) {
 	// where we first to send this header and if the
 	// operation fails that way to even get the connection
 	// then we retry without sending this?
-
-	cloudInfo, err := s.dauthCtx.CloudInfo()
-	if err != nil {
-		return "", err
-	}
-
-	if cloudInfo != nil {
-		cdnParams := []string{fmt.Sprintf("cloud-name=%q", cloudInfo.Name)}
-		if cloudInfo.Region != "" {
-			cdnParams = append(cdnParams, fmt.Sprintf("region=%q", cloudInfo.Region))
-		}
-		if cloudInfo.AvailabilityZone != "" {
-			cdnParams = append(cdnParams, fmt.Sprintf("availability-zone=%q", cloudInfo.AvailabilityZone))
-		}
-
-		return strings.Join(cdnParams, " "), nil
-	}
-
-	return "", nil
+	return s.buildLocationString()
 }
 
 type HashError struct {
@@ -157,7 +201,7 @@ func (s *Store) Download(ctx context.Context, name string, targetPath string, do
 		return nil
 	}
 
-	if useDeltas() {
+	if s.useDeltas() {
 		logger.Debugf("Available deltas returned by store: %v", downloadInfo.Deltas)
 
 		if len(downloadInfo.Deltas) == 1 {
@@ -197,16 +241,7 @@ func (s *Store) Download(ctx context.Context, name string, targetPath string, do
 		logger.Debugf("Starting download of %q.", partialPath)
 	}
 
-	authAvail, err := s.authAvailable(user)
-	if err != nil {
-		return err
-	}
-
-	url := downloadInfo.AnonDownloadURL
-	if url == "" || authAvail {
-		url = downloadInfo.DownloadURL
-	}
-
+	url := downloadInfo.DownloadURL
 	if downloadInfo.Size == 0 || resume < downloadInfo.Size {
 		err = download(ctx, name, downloadInfo.Sha3_384, url, user, s, w, resume, pbar, dlOpts)
 		if err != nil {
@@ -479,7 +514,7 @@ func downloadImpl(ctx context.Context, name, sha3_384, downloadURL string, user
 		if resp.ContentLength == 0 {
 			logger.Noticef("Unexpected Content-Length: 0 for %s", downloadURL)
 		} else {
-			logger.Noticef("Download size for %s: %d", downloadURL, resp.ContentLength)
+			logger.Debugf("Download size for %s: %d", downloadURL, resp.ContentLength)
 		}
 		pbar.Start(name, dlSize)
 		mw := io.MultiWriter(w, h, pbar, tc)
@@ -559,17 +594,7 @@ func (s *Store) DownloadStream(ctx context.Context, name string, downloadInfo *s
 		return file, 206, nil
 	}
 
-	authAvail, err := s.authAvailable(user)
-	if err != nil {
-		return nil, 0, err
-	}
-
-	downloadURL := downloadInfo.AnonDownloadURL
-	if downloadURL == "" || authAvail {
-		downloadURL = downloadInfo.DownloadURL
-	}
-
-	storeURL, err := url.Parse(downloadURL)
+	storeURL, err := url.Parse(downloadInfo.DownloadURL)
 	if err != nil {
 		return nil, 0, err
 	}
@@ -610,28 +635,17 @@ func (s *Store) downloadDelta(deltaName string, downloadInfo *snap.DownloadInfo,
 		return fmt.Errorf("store returned unsupported delta format %q (only xdelta3 currently)", deltaInfo.Format)
 	}
 
-	authAvail, err := s.authAvailable(user)
-	if err != nil {
-		return err
-	}
-
-	url := deltaInfo.AnonDownloadURL
-	if url == "" || authAvail {
-		url = deltaInfo.DownloadURL
-	}
+	url := deltaInfo.DownloadURL
 
 	return download(context.TODO(), deltaName, deltaInfo.Sha3_384, url, user, s, w, 0, pbar, dlOpts)
 }
 
-func getXdelta3Cmd(args ...string) (*exec.Cmd, error) {
-	if osutil.ExecutableExists("xdelta3") {
-		return exec.Command("xdelta3", args...), nil
-	}
-	return snapdtool.CommandFromSystemSnap("/usr/bin/xdelta3", args...)
+// applyDelta generates a target snap from a previously downloaded snap and a downloaded delta.
+var applyDelta = func(s *Store, name string, deltaPath string, deltaInfo *snap.DeltaInfo, targetPath string, targetSha3_384 string) error {
+	return s.applyDeltaImpl(name, deltaPath, deltaInfo, targetPath, targetSha3_384)
 }
 
-// applyDelta generates a target snap from a previously downloaded snap and a downloaded delta.
-var applyDelta = func(name string, deltaPath string, deltaInfo *snap.DeltaInfo, targetPath string, targetSha3_384 string) error {
+func (s *Store) applyDeltaImpl(name string, deltaPath string, deltaInfo *snap.DeltaInfo, targetPath string, targetSha3_384 string) error {
 	snapBase := fmt.Sprintf("%s_%d.snap", name, deltaInfo.FromRevision)
 	snapPath := filepath.Join(dirs.SnapBlobDir, snapBase)
 
@@ -646,16 +660,20 @@ var applyDelta = func(name string, deltaPath string, deltaInfo *snap.DeltaInfo,
 	partialTargetPath := targetPath + ".partial"
 
 	xdelta3Args := []string{"-d", "-s", snapPath, deltaPath, partialTargetPath}
-	cmd, err := getXdelta3Cmd(xdelta3Args...)
-	if err != nil {
-		return err
+
+	// validity check that deltas are available and that the path for the xdelta3
+	// command is set
+	if ok := s.useDeltas(); !ok {
+		return fmt.Errorf("internal error: applyDelta used when deltas are not available")
 	}
 
-	if err := cmd.Run(); err != nil {
+	// run the xdelta3 command, cleaning up if we fail and logging about it
+	if runErr := s.xdelta3CmdFunc(xdelta3Args...).Run(); runErr != nil {
+		logger.Noticef("encountered error applying delta: %v", runErr)
 		if err := os.Remove(partialTargetPath); err != nil {
-			logger.Noticef("failed to remove partial delta target %q: %s", partialTargetPath, err)
+			logger.Noticef("error cleaning up partial delta target %q: %s", partialTargetPath, err)
 		}
-		return err
+		return runErr
 	}
 
 	if err := os.Chmod(partialTargetPath, 0600); err != nil {
@@ -705,7 +723,7 @@ func (s *Store) downloadAndApplyDelta(name, targetPath string, downloadInfo *sna
 	}
 
 	logger.Debugf("Successfully downloaded delta for %q at %s", name, deltaPath)
-	if err := applyDelta(name, deltaPath, deltaInfo, targetPath, downloadInfo.Sha3_384); err != nil {
+	if err := applyDelta(s, name, deltaPath, deltaInfo, targetPath, downloadInfo.Sha3_384); err != nil {
 		return err
 	}
 
