@@ -56,17 +56,30 @@ func Test(t *testing.T) { TestingT(t) }
 
 // systemd's testsuite
 type SystemdTestSuite struct {
+	// 'systemctl' command except for stop. These all
+	// run synchronously so they can share attributes.
 	i      int
 	argses [][]string
 	errors []error
 	outs   [][]byte
+	// 'systemctl stop' command requires some special
+	// mocking because it runs in its own thread. We
+	// have to track its own set of attributes otherwise
+	// the unit test arguments checks will intermittendly
+	// fail due to the random completion ordering. We
+	// do not care about output for 'systemctl stop'.
+	stopDelays []time.Duration
+	stopArgses [][]string
+	stopErrors []error
+	stopIter   int
 
-	j        int
-	jns      []string
-	jsvcs    [][]string
-	jouts    [][]byte
-	jerrs    []error
-	jfollows []bool
+	j           int
+	jns         []string
+	jsvcs       [][]string
+	jouts       [][]byte
+	jerrs       []error
+	jfollows    []bool
+	jnamespaces []bool
 
 	rep *testreporter
 
@@ -83,16 +96,22 @@ func (s *SystemdTestSuite) SetUpTest(c *C) {
 	c.Assert(err, IsNil)
 	err = os.MkdirAll(dirs.SnapRuntimeServicesDir, 0755)
 	c.Assert(err, IsNil)
+	c.Assert(os.MkdirAll(filepath.Join(dirs.SnapServicesDir, "snapd.mounts.target.wants"), 0755), IsNil)
+	// This is only to handle older version of snapd
 	c.Assert(os.MkdirAll(filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants"), 0755), IsNil)
 
 	// force UTC timezone, for reproducible timestamps
 	os.Setenv("TZ", "")
 
-	s.restoreSystemctl = MockSystemctl(s.myRun)
+	s.restoreSystemctl = MockSystemctlWithDelay(s.myRun)
 	s.i = 0
 	s.argses = nil
 	s.errors = nil
 	s.outs = nil
+	s.stopDelays = nil
+	s.stopArgses = nil
+	s.stopErrors = nil
+	s.stopIter = 0
 
 	s.restoreJournalctl = MockJournalctl(s.myJctl)
 	s.j = 0
@@ -101,6 +120,7 @@ func (s *SystemdTestSuite) SetUpTest(c *C) {
 	s.jouts = nil
 	s.jerrs = nil
 	s.jfollows = nil
+	s.jnamespaces = nil
 
 	s.rep = new(testreporter)
 
@@ -113,25 +133,45 @@ func (s *SystemdTestSuite) TearDownTest(c *C) {
 	s.restoreSELinux()
 }
 
-func (s *SystemdTestSuite) myRun(args ...string) (out []byte, err error) {
-	s.argses = append(s.argses, args)
-	if s.i < len(s.outs) {
-		out = s.outs[s.i]
+func (s *SystemdTestSuite) myRun(args ...string) (out []byte, delay time.Duration, err error) {
+	// Default completion time for 'systemctl' operations
+	delayReq := time.Duration(0)
+
+	// We deal with 'systemctl stop' separately because this is a
+	// blocking call running inside a go routine (separate thread)
+	if args[0] == "stop" {
+		// 'systemctl stop' cases
+		s.stopArgses = append(s.stopArgses, args)
+		if s.stopIter < len(s.stopErrors) {
+			err = s.stopErrors[s.stopIter]
+		}
+		if s.stopIter < len(s.stopDelays) {
+			delayReq = s.stopDelays[s.stopIter]
+		}
+		s.stopIter++
+	} else {
+		// Other 'systemctl' cases
+		s.argses = append(s.argses, args)
+		if s.i < len(s.outs) {
+			out = s.outs[s.i]
+		}
+		if s.i < len(s.errors) {
+			err = s.errors[s.i]
+		}
+		s.i++
 	}
-	if s.i < len(s.errors) {
-		err = s.errors[s.i]
-	}
-	s.i++
-	return out, err
+
+	return out, delayReq, err
 }
 
-func (s *SystemdTestSuite) myJctl(svcs []string, n int, follow bool) (io.ReadCloser, error) {
+func (s *SystemdTestSuite) myJctl(svcs []string, n int, follow, namespaces bool) (io.ReadCloser, error) {
 	var err error
 	var out []byte
 
 	s.jns = append(s.jns, strconv.Itoa(n))
 	s.jsvcs = append(s.jsvcs, svcs)
 	s.jfollows = append(s.jfollows, follow)
+	s.jnamespaces = append(s.jnamespaces, namespaces)
 
 	if s.j < len(s.jouts) {
 		out = s.jouts[s.j]
@@ -228,58 +268,177 @@ func (s *SystemdTestSuite) TestStartNoBlock(c *C) {
 	c.Check(s.argses, DeepEquals, [][]string{{"start", "--no-block", "foo", "bar"}})
 }
 
-func (s *SystemdTestSuite) TestStop(c *C) {
-	restore := MockStopDelays(time.Millisecond, 25*time.Second)
+// Stop failed because systemctl stop returned an error
+func (s *SystemdTestSuite) TestStopSystemCtlStopError(c *C) {
+	// Poll rate = 2ms, Silence timeout = 4ms
+	restore := MockStopDelays(2*time.Millisecond, 4*time.Millisecond)
 	defer restore()
+	s.stopErrors = []error{errors.New("mock error")}
+	err := New(SystemMode, s.rep).Stop([]string{"foo"})
+	c.Assert(err, ErrorMatches, "mock error")
+	c.Assert(s.stopArgses, HasLen, 1)
+	c.Check(s.stopArgses[0], DeepEquals, []string{"stop", "foo"})
+	// No notifications
+	c.Assert(len(s.rep.msgs), Equals, 0)
+}
+
+// Stop failed because systemctl show returned an error
+func (s *SystemdTestSuite) TestSystemCtlShowError(c *C) {
+	// Poll rate = 2ms, Silence timeout = 4ms
+	restore := MockStopDelays(2*time.Millisecond, 4*time.Millisecond)
+	defer restore()
+
+	// 'systemctl stop'
+	s.stopDelays = []time.Duration{
+		// The delay we set here applies to the systemctl stop
+		// command. The purpose is to simulate how long systemd
+		// tries to stop the supplied services before returning.
+		50 * time.Millisecond,
+	}
+
+	// 'systemctl show'
+	s.errors = []error{errors.New("mock error")}
+
+	err := New(SystemMode, s.rep).Stop([]string{"foo"})
+	c.Assert(err, ErrorMatches, "mock error")
+	c.Assert(s.stopArgses, HasLen, 1)
+	c.Check(s.stopArgses[0], DeepEquals, []string{"stop", "foo"})
+	c.Assert(s.argses, HasLen, 1)
+	c.Check(s.argses[0], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+	// No notifications
+	c.Assert(len(s.rep.msgs), Equals, 0)
+}
+
+// The stop completed before the notification silence period expired so
+// no output (notification) is expected
+func (s *SystemdTestSuite) TestStopBeforeNotify(c *C) {
+	// Poll rate = 2ms, Silence timeout = 4ms
+	restore := MockStopDelays(2*time.Millisecond, 4*time.Millisecond)
+	defer restore()
+
+	// 'systemctl stop'
+	s.stopDelays = []time.Duration{
+		// The delay we set here applies to the systemctl stop
+		// command. The purpose is to simulate how long systemd
+		// tries to stop the supplied services before returning.
+		50 * time.Millisecond,
+	}
+	s.stopErrors = []error{nil, errors.New("mock error")}
+
+	// 'systemctl show'
 	s.outs = [][]byte{
-		nil, // for the "stop" itself
+		[]byte("ActiveState=inactive\n"),
+	}
+	s.errors = []error{nil, errors.New("mock error")}
+
+	err := New(SystemMode, s.rep).Stop([]string{"foo"})
+	c.Assert(err, IsNil)
+	// 'systemctl stop'
+	c.Assert(s.stopArgses, HasLen, 1)
+	c.Check(s.stopArgses[0], DeepEquals, []string{"stop", "foo"})
+	// 'systemctl show'
+	c.Assert(s.argses, HasLen, 1)
+	c.Check(s.argses[0], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+
+	// No notifications
+	c.Assert(len(s.rep.msgs), Equals, 0)
+}
+
+// The stop completed after the notification silence period expired
+func (s *SystemdTestSuite) TestStopAfterNotify(c *C) {
+	// Poll rate = 2ms, Silence timeout = 4ms
+	restore := MockStopDelays(2*time.Millisecond, 4*time.Millisecond)
+	defer restore()
+
+	// 'systemctl stop'
+	s.stopDelays = []time.Duration{
+		// The delay we set here applies to the systemctl stop
+		// command. The purpose is to simulate how long systemd
+		// tries to stop the supplied services before returning.
+		50 * time.Millisecond,
+	}
+	s.stopErrors = []error{nil, errors.New("mock error")}
+
+	// 'systemctl show'
+	s.outs = [][]byte{
+		[]byte("ActiveState=whatever\n"),
+		[]byte("ActiveState=whatever\n"),
 		[]byte("ActiveState=whatever\n"),
 		[]byte("ActiveState=active\n"),
 		[]byte("ActiveState=inactive\n"),
 	}
-	s.errors = []error{nil, nil, nil, nil, &Timeout{}}
-	err := New(SystemMode, s.rep).Stop([]string{"foo"}, 1*time.Second)
+	// The timeout error output should never by reached because the 's.outs'
+	// entry before this slot indicates the unit has stopped
+	s.errors = []error{nil, nil, nil, nil, nil, errors.New("mock error")}
+
+	err := New(SystemMode, s.rep).Stop([]string{"foo"})
 	c.Assert(err, IsNil)
-	c.Assert(s.argses, HasLen, 4)
-	c.Check(s.argses[0], DeepEquals, []string{"stop", "foo"})
-	c.Check(s.argses[1], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
-	c.Check(s.argses[1], DeepEquals, s.argses[2])
-	c.Check(s.argses[1], DeepEquals, s.argses[3])
+	// 'systemctl stop'
+	c.Assert(s.stopArgses, HasLen, 1)
+	c.Check(s.stopArgses[0], DeepEquals, []string{"stop", "foo"})
+	// 'systemctl show'
+	c.Assert(s.argses, HasLen, 5)
+	c.Check(s.argses[0], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+	c.Check(s.argses[0], DeepEquals, s.argses[1])
+	c.Check(s.argses[0], DeepEquals, s.argses[2])
+	c.Check(s.argses[0], DeepEquals, s.argses[3])
+	c.Check(s.argses[0], DeepEquals, s.argses[4])
+
+	// Only the initial notification will be shown because no
+	// list update are possible with only a single service unit
+	c.Assert(len(s.rep.msgs), Equals, 1)
+	c.Check(s.rep.msgs[0], Equals, `Waiting for "foo" to stop.`)
 }
 
 func (s *SystemdTestSuite) TestStopMany(c *C) {
-	restore := MockStopDelays(time.Millisecond, 25*time.Second)
+	// Poll rate = 2ms, Silence timeout = 4ms
+	restore := MockStopDelays(2*time.Millisecond, 4*time.Millisecond)
 	defer restore()
+
+	// 'systemctl stop'
+	s.stopDelays = []time.Duration{
+		// The delay we set here applies to the systemctl stop
+		// command. The purpose is to simulate how long systemd
+		// tries to stop the supplied services before returning.
+		50 * time.Millisecond,
+	}
+	s.stopErrors = []error{nil, errors.New("mock error")}
+
+	// 'systemctl show'
 	s.outs = [][]byte{
-		nil,                              // for the "stop" itself
+		[]byte("ActiveState=whatever\n"), // foo
+		[]byte("ActiveState=whatever\n"), // bar
+		[]byte("ActiveState=whatever\n"), // foo
+		[]byte("ActiveState=whatever\n"), // bar
+		[]byte("ActiveState=active\n"),   // foo
+		[]byte("ActiveState=whatever\n"), // bar
 		[]byte("ActiveState=inactive\n"), // foo
+		[]byte("ActiveState=active\n"),   // bar
 		[]byte("ActiveState=inactive\n"), // bar
 	}
-	s.errors = []error{nil, nil, nil, nil, &Timeout{}}
-	err := New(SystemMode, s.rep).Stop([]string{"foo", "bar"}, 1*time.Second)
+	// The timeout error output should never by reached because the 's.outs'
+	// entry before this slot indicates the unit has stopped
+	s.errors = []error{nil, nil, nil, nil, nil, nil, nil, nil, nil, errors.New("mock error")}
+	err := New(SystemMode, s.rep).Stop([]string{"foo", "bar"})
 	c.Assert(err, IsNil)
-	c.Assert(s.argses, HasLen, 3)
-	c.Check(s.argses[0], DeepEquals, []string{"stop", "foo", "bar"})
-	c.Check(s.argses[1], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
-	c.Check(s.argses[2], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
-}
+	// 'systemctl stop'
+	c.Assert(s.stopArgses, HasLen, 1)
+	c.Check(s.stopArgses[0], DeepEquals, []string{"stop", "foo", "bar"})
+	// 'systemctl show'
+	c.Assert(s.argses, HasLen, 9)
+	c.Check(s.argses[0], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+	c.Check(s.argses[1], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
+	c.Check(s.argses[2], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+	c.Check(s.argses[3], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
+	c.Check(s.argses[4], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+	c.Check(s.argses[5], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
+	c.Check(s.argses[6], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+	c.Check(s.argses[7], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
+	c.Check(s.argses[8], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
 
-func (s *SystemdTestSuite) TestStopTimesOut(c *C) {
-	restore := MockStopDelays(time.Millisecond, 4*time.Millisecond)
-	defer restore()
-	s.outs = [][]byte{
-		nil, // for the "stop" itself
-		[]byte("ActiveState=active\n"),
-		[]byte("ActiveState=active\n"),
-		[]byte("ActiveState=active\n"),
-	}
-	s.errors = []error{nil, nil, nil, nil, nil}
-	err := New(SystemMode, s.rep).Stop([]string{"foo", "bar"}, 3*time.Millisecond)
-	c.Assert(err, ErrorMatches, `"foo", "bar" failed to stop: timeout`)
-	c.Assert(len(s.argses) >= 3, Equals, true)
-	c.Check(s.argses[0], DeepEquals, []string{"stop", "foo", "bar"})
-	c.Check(s.argses[1], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
-	c.Check(s.argses[2], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
+	c.Assert(len(s.rep.msgs), Equals, 2)
+	c.Check(s.rep.msgs[0], Equals, `Waiting for "foo", "bar" to stop.`)
+	c.Check(s.rep.msgs[1], Equals, `Waiting for "bar" to stop.`)
 }
 
 func (s *SystemdTestSuite) TestStatus(c *C) {
@@ -603,24 +762,6 @@ NeedDaemonReload=no
 	c.Check(out, IsNil)
 }
 
-func (s *SystemdTestSuite) TestStopTimeout(c *C) {
-	restore := MockStopDelays(time.Millisecond, 25*time.Second)
-	defer restore()
-	err := New(SystemMode, s.rep).Stop([]string{"foo"}, 10*time.Millisecond)
-	c.Assert(err, FitsTypeOf, &Timeout{})
-	c.Assert(len(s.rep.msgs) > 0, Equals, true)
-	c.Check(s.rep.msgs[0], Equals, `Waiting for "foo" to stop.`)
-}
-
-func (s *SystemdTestSuite) TestStopManyTimeout(c *C) {
-	restore := MockStopDelays(time.Millisecond, 25*time.Second)
-	defer restore()
-	err := New(SystemMode, s.rep).Stop([]string{"foo", "bar"}, 10*time.Millisecond)
-	c.Assert(err, FitsTypeOf, &Timeout{})
-	c.Assert(len(s.rep.msgs) > 0, Equals, true)
-	c.Check(s.rep.msgs[0], Equals, `Waiting for "foo", "bar" to stop.`)
-}
-
 func (s *SystemdTestSuite) TestDisable(c *C) {
 	sysd := New(SystemMode, s.rep)
 	err := sysd.DisableNoReload([]string{"foo"})
@@ -724,39 +865,57 @@ func (s *SystemdTestSuite) TestUnmaskUnderRoot(c *C) {
 }
 
 func (s *SystemdTestSuite) TestRestart(c *C) {
-	restore := MockStopDelays(time.Millisecond, 25*time.Second)
+	restore := MockStopDelays(2*time.Millisecond, 4*time.Millisecond)
 	defer restore()
+
+	// 'systemctl stop'
+	s.stopDelays = []time.Duration{
+		// The delay we set here applies to the systemctl stop
+		// command. The purpose is to simulate how long systemd
+		// tries to stop the supplied services before returning.
+		50 * time.Millisecond,
+	}
+
 	s.outs = [][]byte{
-		nil, // for the "stop" itself
 		[]byte("ActiveState=inactive\n"),
 		nil, // for the "start"
 	}
-	s.errors = []error{nil, nil, nil, nil, &Timeout{}}
-	err := New(SystemMode, s.rep).Restart([]string{"foo"}, 100*time.Millisecond)
+	s.errors = []error{nil, nil, errors.New("mock error")}
+	err := New(SystemMode, s.rep).Restart([]string{"foo"})
 	c.Assert(err, IsNil)
-	c.Check(s.argses, HasLen, 3)
-	c.Check(s.argses[0], DeepEquals, []string{"stop", "foo"})
-	c.Check(s.argses[1], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
-	c.Check(s.argses[2], DeepEquals, []string{"start", "foo"})
+	c.Check(s.stopArgses, HasLen, 1)
+	c.Check(s.stopArgses[0], DeepEquals, []string{"stop", "foo"})
+	c.Check(s.argses, HasLen, 2)
+	c.Check(s.argses[0], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+	c.Check(s.argses[1], DeepEquals, []string{"start", "foo"})
 }
 
 func (s *SystemdTestSuite) TestRestartMany(c *C) {
-	restore := MockStopDelays(time.Millisecond, 25*time.Second)
+	restore := MockStopDelays(2*time.Millisecond, 4*time.Millisecond)
 	defer restore()
+
+	// 'systemctl stop'
+	s.stopDelays = []time.Duration{
+		// The delay we set here applies to the systemctl stop
+		// command. The purpose is to simulate how long systemd
+		// tries to stop the supplied services before returning.
+		50 * time.Millisecond,
+	}
+
 	s.outs = [][]byte{
-		nil,                              // for the "stop" itself
 		[]byte("ActiveState=inactive\n"), // foo
 		[]byte("ActiveState=inactive\n"), // bar
 		nil,                              // for the "start"
 	}
-	s.errors = []error{nil, nil, nil, nil, &Timeout{}}
-	err := New(SystemMode, s.rep).Restart([]string{"foo", "bar"}, 100*time.Millisecond)
+	s.errors = []error{nil, nil, nil, errors.New("mock error")}
+	err := New(SystemMode, s.rep).Restart([]string{"foo", "bar"})
 	c.Assert(err, IsNil)
-	c.Check(s.argses, HasLen, 4)
-	c.Check(s.argses[0], DeepEquals, []string{"stop", "foo", "bar"})
-	c.Check(s.argses[1], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
-	c.Check(s.argses[2], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
-	c.Check(s.argses[3], DeepEquals, []string{"start", "foo", "bar"})
+	c.Check(s.stopArgses, HasLen, 1)
+	c.Check(s.stopArgses[0], DeepEquals, []string{"stop", "foo", "bar"})
+	c.Check(s.argses, HasLen, 3)
+	c.Check(s.argses[0], DeepEquals, []string{"show", "--property=ActiveState", "foo"})
+	c.Check(s.argses[1], DeepEquals, []string{"show", "--property=ActiveState", "bar"})
+	c.Check(s.argses[2], DeepEquals, []string{"start", "foo", "bar"})
 }
 
 func (s *SystemdTestSuite) TestKill(c *C) {
@@ -764,20 +923,16 @@ func (s *SystemdTestSuite) TestKill(c *C) {
 	c.Check(s.argses, DeepEquals, [][]string{{"kill", "foo", "-s", "HUP", "--kill-who=all"}})
 }
 
-func (s *SystemdTestSuite) TestIsTimeout(c *C) {
-	c.Check(IsTimeout(os.ErrInvalid), Equals, false)
-	c.Check(IsTimeout(&Timeout{}), Equals, true)
-}
-
 func (s *SystemdTestSuite) TestLogErrJctl(c *C) {
-	s.jerrs = []error{&Timeout{}}
+	s.jerrs = []error{errors.New("mock journalctl error")}
 
-	reader, err := New(SystemMode, s.rep).LogReader([]string{"foo"}, 24, false)
+	reader, err := New(SystemMode, s.rep).LogReader([]string{"foo"}, 24, false, false)
 	c.Check(err, NotNil)
 	c.Check(reader, IsNil)
 	c.Check(s.jns, DeepEquals, []string{"24"})
 	c.Check(s.jsvcs, DeepEquals, [][]string{{"foo"}})
 	c.Check(s.jfollows, DeepEquals, []bool{false})
+	c.Check(s.jnamespaces, DeepEquals, []bool{false})
 	c.Check(s.j, Equals, 1)
 }
 
@@ -787,7 +942,7 @@ func (s *SystemdTestSuite) TestLogs(c *C) {
 `
 	s.jouts = [][]byte{[]byte(expected)}
 
-	reader, err := New(SystemMode, s.rep).LogReader([]string{"foo"}, 24, false)
+	reader, err := New(SystemMode, s.rep).LogReader([]string{"foo"}, 24, false, false)
 	c.Check(err, IsNil)
 	logs, err := ioutil.ReadAll(reader)
 	c.Assert(err, IsNil)
@@ -795,6 +950,7 @@ func (s *SystemdTestSuite) TestLogs(c *C) {
 	c.Check(s.jns, DeepEquals, []string{"24"})
 	c.Check(s.jsvcs, DeepEquals, [][]string{{"foo"}})
 	c.Check(s.jfollows, DeepEquals, []bool{false})
+	c.Check(s.jnamespaces, DeepEquals, []bool{false})
 	c.Check(s.j, Equals, 1)
 }
 
@@ -1010,8 +1166,9 @@ func (s *SystemdTestSuite) TestAddMountUnit(c *C) {
 	c.Assert(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(`
 [Unit]
 Description=Mount unit for foo, revision 42
-Before=snapd.service
-After=zfs-mount.service
+After=snapd.mounts-pre.target
+Before=snapd.mounts.target
+Before=local-fs.target
 
 [Mount]
 What=%s
@@ -1021,6 +1178,7 @@ Options=nodev,ro,x-gdu.hide,x-gvfs-hide
 LazyUnmount=yes
 
 [Install]
+WantedBy=snapd.mounts.target
 WantedBy=multi-user.target
 `[1:], mockSnapPath))
 
@@ -1044,8 +1202,9 @@ func (s *SystemdTestSuite) TestAddMountUnitForDirs(c *C) {
 	c.Assert(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(`
 [Unit]
 Description=Mount unit for foodir, revision x1
-Before=snapd.service
-After=zfs-mount.service
+After=snapd.mounts-pre.target
+Before=snapd.mounts.target
+Before=local-fs.target
 
 [Mount]
 What=%s
@@ -1055,6 +1214,7 @@ Options=nodev,ro,x-gdu.hide,x-gvfs-hide,bind
 LazyUnmount=yes
 
 [Install]
+WantedBy=snapd.mounts.target
 WantedBy=multi-user.target
 `[1:], snapDir))
 
@@ -1090,8 +1250,9 @@ func (s *SystemdTestSuite) TestAddMountUnitTransient(c *C) {
 	c.Assert(filepath.Join(dirs.SnapRuntimeServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(`
 [Unit]
 Description=Mount unit for foo via bar
-Before=snapd.service
-After=zfs-mount.service
+After=snapd.mounts-pre.target
+Before=snapd.mounts.target
+Before=local-fs.target
 
 [Mount]
 What=%s
@@ -1101,6 +1262,7 @@ Options=remount,ro
 LazyUnmount=yes
 
 [Install]
+WantedBy=snapd.mounts.target
 WantedBy=multi-user.target
 X-SnapdOrigin=bar
 `[1:], mockSnapPath))
@@ -1133,8 +1295,9 @@ func (s *SystemdTestSuite) TestWriteSELinuxMountUnit(c *C) {
 	c.Assert(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(`
 [Unit]
 Description=Mount unit for foo, revision 42
-Before=snapd.service
-After=zfs-mount.service
+After=snapd.mounts-pre.target
+Before=snapd.mounts.target
+Before=local-fs.target
 
 [Mount]
 What=%s
@@ -1144,6 +1307,7 @@ Options=nodev,context=system_u:object_r:snappy_snap_t:s0,ro,x-gdu.hide,x-gvfs-hi
 LazyUnmount=yes
 
 [Install]
+WantedBy=snapd.mounts.target
 WantedBy=multi-user.target
 `[1:], mockSnapPath))
 }
@@ -1177,8 +1341,9 @@ exit 0
 	c.Check(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(`
 [Unit]
 Description=Mount unit for foo, revision x1
-Before=snapd.service
-After=zfs-mount.service
+After=snapd.mounts-pre.target
+Before=snapd.mounts.target
+Before=local-fs.target
 
 [Mount]
 What=%s
@@ -1188,6 +1353,7 @@ Options=nodev,ro,x-gdu.hide,x-gvfs-hide,allow_other
 LazyUnmount=yes
 
 [Install]
+WantedBy=snapd.mounts.target
 WantedBy=multi-user.target
 `[1:], mockSnapPath))
 }
@@ -1217,8 +1383,9 @@ exit 0
 	c.Assert(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(`
 [Unit]
 Description=Mount unit for foo, revision x1
-Before=snapd.service
-After=zfs-mount.service
+After=snapd.mounts-pre.target
+Before=snapd.mounts.target
+Before=local-fs.target
 
 [Mount]
 What=%s
@@ -1228,6 +1395,7 @@ Options=nodev,ro,x-gdu.hide,x-gvfs-hide
 LazyUnmount=yes
 
 [Install]
+WantedBy=snapd.mounts.target
 WantedBy=multi-user.target
 `[1:], mockSnapPath))
 }
@@ -1236,20 +1404,23 @@ func (s *SystemdTestSuite) TestJctl(c *C) {
 	var args []string
 	var err error
 	MockOsutilStreamCommand(func(name string, myargs ...string) (io.ReadCloser, error) {
-		c.Check(cap(myargs) <= len(myargs)+2, Equals, true, Commentf("cap:%d, len:%d", cap(myargs), len(myargs)))
+		c.Check(cap(myargs) <= len(myargs)+3, Equals, true, Commentf("cap:%d, len:%d", cap(myargs), len(myargs)))
 		args = myargs
 		return nil, nil
 	})
 
-	_, err = Jctl([]string{"foo", "bar"}, 10, false)
+	_, err = Jctl([]string{"foo", "bar"}, 10, false, false)
 	c.Assert(err, IsNil)
 	c.Check(args, DeepEquals, []string{"-o", "json", "--no-pager", "-n", "10", "-u", "foo", "-u", "bar"})
-	_, err = Jctl([]string{"foo", "bar", "baz"}, 99, true)
+	_, err = Jctl([]string{"foo", "bar", "baz"}, 99, true, false)
 	c.Assert(err, IsNil)
 	c.Check(args, DeepEquals, []string{"-o", "json", "--no-pager", "-n", "99", "-f", "-u", "foo", "-u", "bar", "-u", "baz"})
-	_, err = Jctl([]string{"foo", "bar"}, -1, false)
+	_, err = Jctl([]string{"foo", "bar"}, -1, false, false)
 	c.Assert(err, IsNil)
 	c.Check(args, DeepEquals, []string{"-o", "json", "--no-pager", "--no-tail", "-u", "foo", "-u", "bar"})
+	_, err = Jctl([]string{"foo", "bar"}, -1, false, true)
+	c.Assert(err, IsNil)
+	c.Check(args, DeepEquals, []string{"-o", "json", "--no-pager", "--no-tail", "--namespace=*", "-u", "foo", "-u", "bar"})
 }
 
 func (s *SystemdTestSuite) TestIsActiveUnderRoot(c *C) {
@@ -1427,8 +1598,8 @@ func (s *SystemdTestSuite) TestGlobalUserMode(c *C) {
 	c.Check(sysd.DaemonReexec, Panics, "cannot call daemon-reexec with GlobalUserMode")
 	c.Check(func() { sysd.Start([]string{"foo"}) }, Panics, "cannot call start with GlobalUserMode")
 	c.Check(func() { sysd.StartNoBlock([]string{"foo"}) }, Panics, "cannot call start with GlobalUserMode")
-	c.Check(func() { sysd.Stop([]string{"foo"}, 0) }, Panics, "cannot call stop with GlobalUserMode")
-	c.Check(func() { sysd.Restart([]string{"foo"}, 0) }, Panics, "cannot call restart with GlobalUserMode")
+	c.Check(func() { sysd.Stop([]string{"foo"}) }, Panics, "cannot call stop with GlobalUserMode")
+	c.Check(func() { sysd.Restart([]string{"foo"}) }, Panics, "cannot call restart with GlobalUserMode")
 	c.Check(func() { sysd.Kill("foo", "HUP", "") }, Panics, "cannot call kill with GlobalUserMode")
 	c.Check(func() { sysd.IsActive("foo") }, Panics, "cannot call is-active with GlobalUserMode")
 }
@@ -1478,8 +1649,9 @@ func (s *SystemdTestSuite) TestEmulationModeBackend(c *C) {
 const unitTemplate = `
 [Unit]
 Description=Mount unit for foo, revision 42
-Before=snapd.service
-After=zfs-mount.service
+After=snapd.mounts-pre.target
+Before=snapd.mounts.target
+Before=local-fs.target
 
 [Mount]
 What=%s
@@ -1489,6 +1661,7 @@ Options=%s
 LazyUnmount=yes
 
 [Install]
+WantedBy=snapd.mounts.target
 WantedBy=multi-user.target
 `
 
@@ -1513,6 +1686,8 @@ func (s *SystemdTestSuite) TestPreseedModeAddMountUnit(c *C) {
 	// mount was called
 	c.Check(mockMountCmd.Calls()[0], DeepEquals, []string{"mount", "-t", "squashfs", mockSnapPath, "/snap/snapname/123", "-o", "nodev,ro,x-gdu.hide,x-gvfs-hide"})
 	// unit was enabled with a symlink
+	c.Check(osutil.IsSymlink(filepath.Join(dirs.SnapServicesDir, "snapd.mounts.target.wants", mountUnitName)), Equals, true)
+	// This is only to handle older version of snapd
 	c.Check(osutil.IsSymlink(filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants", mountUnitName)), Equals, true)
 	c.Check(filepath.Join(dirs.SnapServicesDir, mountUnitName), testutil.FileEquals, fmt.Sprintf(unitTemplate[1:], mockSnapPath, "squashfs", "nodev,ro,x-gdu.hide,x-gvfs-hide"))
 }
@@ -1568,14 +1743,48 @@ func (s *SystemdTestSuite) TestPreseedModeRemoveMountUnit(c *C) {
 	sysd := NewEmulationMode(dirs.GlobalRootDir)
 
 	mountUnit := makeMockMountUnit(c, mountDir)
-	symlinkPath := filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants", filepath.Base(mountUnit))
+	symlinkPath := filepath.Join(dirs.SnapServicesDir, "snapd.mounts.target.wants", filepath.Base(mountUnit))
 	c.Assert(os.Symlink(mountUnit, symlinkPath), IsNil)
+	// This is only to handle older version of snapd
+	oldSymlinkPath := filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants", filepath.Base(mountUnit))
+	c.Assert(os.Symlink(mountUnit, oldSymlinkPath), IsNil)
+
+	// Create a symlink in a different target, like if it was manually created.
+	manualTargetWants := filepath.Join(dirs.SnapServicesDir, "some-other.target.wants")
+	os.MkdirAll(manualTargetWants, os.ModePerm)
+	c.Assert(osutil.IsDirectory(manualTargetWants), Equals, true)
+	manualSymlinkPath := filepath.Join(manualTargetWants, filepath.Base(mountUnit))
+	c.Assert(os.Symlink(mountUnit, manualSymlinkPath), IsNil)
+
+	// Create yet another symlink in a different target, like if it was manually created.
+	// This time we use `.requires` instead of `.wants`. And we add another unit in that target
+	// to test the case we should not remove the `.requires`/`.wants` directory.
+	nonEmptyManualTargetWants := filepath.Join(dirs.SnapServicesDir, "some-non-empty-other.target.requires")
+	os.MkdirAll(nonEmptyManualTargetWants, os.ModePerm)
+	c.Assert(osutil.IsDirectory(nonEmptyManualTargetWants), Equals, true)
+	manualSymlinkPathInNonEmpty := filepath.Join(nonEmptyManualTargetWants, filepath.Base(mountUnit))
+	c.Assert(os.Symlink(mountUnit, manualSymlinkPathInNonEmpty), IsNil)
+	otherMountDir := dirs.GlobalRootDir + "/snap/bar/51"
+	otherMountUnit := makeMockMountUnit(c, otherMountDir)
+	otherSymlinkPath := filepath.Join(nonEmptyManualTargetWants, filepath.Base(otherMountUnit))
+	c.Assert(os.Symlink(otherMountUnit, otherSymlinkPath), IsNil)
+
 	c.Assert(sysd.RemoveMountUnitFile(mountDir), IsNil)
 
 	// the file is gone
 	c.Check(osutil.FileExists(mountUnit), Equals, false)
 	// unit symlink is gone
 	c.Check(osutil.IsSymlink(symlinkPath), Equals, false)
+	// unit symlink manually instaled is gone
+	c.Check(osutil.IsSymlink(manualSymlinkPath), Equals, false)
+	c.Assert(osutil.IsDirectory(manualTargetWants), Equals, false)
+	// Make sure we do not remove non empty .requires directory
+	c.Check(osutil.IsSymlink(manualSymlinkPathInNonEmpty), Equals, false)
+	c.Assert(osutil.IsDirectory(nonEmptyManualTargetWants), Equals, true)
+	c.Check(osutil.IsSymlink(otherSymlinkPath), Equals, true)
+	// unit symlink manually instaled is gone
+	c.Check(osutil.IsSymlink(manualSymlinkPath), Equals, false)
+	c.Assert(osutil.IsDirectory(manualTargetWants), Equals, false)
 	// and systemd was not called
 	c.Check(s.argses, HasLen, 0)
 	// umount was called
@@ -1596,8 +1805,11 @@ func (s *SystemdTestSuite) TestPreseedModeRemoveMountUnitUnmounted(c *C) {
 
 	sysd := NewEmulationMode(dirs.GlobalRootDir)
 	mountUnit := makeMockMountUnit(c, mountDir)
-	symlinkPath := filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants", filepath.Base(mountUnit))
+	symlinkPath := filepath.Join(dirs.SnapServicesDir, "snapd.mounts.target.wants", filepath.Base(mountUnit))
 	c.Assert(os.Symlink(mountUnit, symlinkPath), IsNil)
+	// This is only to handle older version of snapd
+	oldSymlinkPath := filepath.Join(dirs.SnapServicesDir, "multi-user.target.wants", filepath.Base(mountUnit))
+	c.Assert(os.Symlink(mountUnit, oldSymlinkPath), IsNil)
 
 	c.Assert(sysd.RemoveMountUnitFile(mountDir), IsNil)
 
@@ -1709,7 +1921,7 @@ Type=doesntmatter
 Options=do,not,matter,either
 
 [Install]
-WantedBy=multi-user.target
+WantedBy=doesntmatter.target
 X-SnapdOrigin=%s
 `, snapName, where, origin)
 		return ioutil.WriteFile(path, []byte(contents), 0644)
@@ -1953,6 +2165,57 @@ func (s *SystemdTestSuite) TestInactiveEnterTimestampMalformedMore(c *C) {
 	c.Check(stamp.IsZero(), Equals, true)
 }
 
+func (s *SystemdTestSuite) TestSystemdRunError(c *C) {
+	sr := testutil.MockCommand(c, "systemd-run", `echo "fail"; exit 11`)
+	defer sr.Restore()
+
+	sysd := New(SystemMode, s.rep)
+	output, err := sysd.Run([]string{"bad-cmd", "arg1"}, nil)
+	c.Check(output, IsNil)
+	c.Assert(err, ErrorMatches, `cannot run \["bad-cmd" "arg1"\]: fail`)
+}
+
+func (s *SystemdTestSuite) TestSystemdRunHappy(c *C) {
+	sr := testutil.MockCommand(c, "systemd-run", `echo "happy output" && >&2 echo "to stderr"`)
+	defer sr.Restore()
+
+	sysd := New(SystemMode, s.rep)
+	output, err := sysd.Run([]string{"happy-cmd", "arg1"}, nil)
+	c.Check(string(output), Equals, "happy output\nto stderr\n")
+	c.Check(err, IsNil)
+	c.Check(sr.Calls(), DeepEquals, [][]string{
+		{"systemd-run", "--wait", "--pipe", "--collect", "--service-type=exec", "--quiet", "--", "happy-cmd", "arg1"},
+	})
+}
+
+func (s *SystemdTestSuite) TestSystemdRunHappyWithStdin(c *C) {
+	sr := testutil.MockCommand(c, "systemd-run", `echo "some output" && cat - `)
+	defer sr.Restore()
+
+	sysd := New(SystemMode, s.rep)
+	opts := &RunOptions{Stdin: bytes.NewBufferString("stdin input\n")}
+	output, err := sysd.Run([]string{"cmd-with-stdin", "arg1"}, opts)
+	c.Check(string(output), Equals, "some output\nstdin input\n")
+	c.Check(err, IsNil)
+	c.Check(sr.Calls(), DeepEquals, [][]string{
+		{"systemd-run", "--wait", "--pipe", "--collect", "--service-type=exec", "--quiet", "--", "cmd-with-stdin", "arg1"},
+	})
+}
+
+func (s *SystemdTestSuite) TestSystemdRunKeyringMode(c *C) {
+	sr := testutil.MockCommand(c, "systemd-run", `echo "happy output"`)
+	defer sr.Restore()
+
+	sysd := New(SystemMode, s.rep)
+	opts := &RunOptions{KeyringMode: KeyringModePrivate}
+	output, err := sysd.Run([]string{"happy-cmd", "arg1"}, opts)
+	c.Check(string(output), Equals, "happy output\n")
+	c.Check(err, IsNil)
+	c.Check(sr.Calls(), DeepEquals, [][]string{
+		{"systemd-run", "--wait", "--pipe", "--collect", "--service-type=exec", "--quiet", "--property=KeyringMode=private", "--", "happy-cmd", "arg1"},
+	})
+}
+
 type systemdErrorSuite struct{}
 
 var _ = Suite(&systemdErrorSuite{})
