@@ -21,25 +21,15 @@ package agent
 
 import (
 	"syscall"
-	"time"
 )
 
 var (
-	SessionInfoCmd    = sessionInfoCmd
-	ServiceControlCmd = serviceControlCmd
+	SessionInfoCmd                = sessionInfoCmd
+	ServiceControlCmd             = serviceControlCmd
+	PendingRefreshNotificationCmd = pendingRefreshNotificationCmd
+	FinishRefreshNotificationCmd  = finishRefreshNotificationCmd
 )
 
-func MockStopTimeouts(stop, kill time.Duration) (restore func()) {
-	oldStopTimeout := stopTimeout
-	stopTimeout = stop
-	oldKillWait := killWait
-	killWait = kill
-	return func() {
-		stopTimeout = oldStopTimeout
-		killWait = oldKillWait
-	}
-}
-
 func MockUcred(ucred *syscall.Ucred, err error) (restore func()) {
 	old := sysGetsockoptUcred
 	sysGetsockoptUcred = func(fd, level, opt int) (*syscall.Ucred, error) {
@@ -49,3 +39,12 @@ func MockUcred(ucred *syscall.Ucred, err error) (restore func()) {
 		sysGetsockoptUcred = old
 	}
 }
+
+// MockNoBus temporarily unsets the D-Bus connection of a SessionAgent
+func MockNoBus(agent *SessionAgent) (restore func()) {
+	bus := agent.bus
+	agent.bus = nil
+	return func() {
+		agent.bus = bus
+	}
+}
