@@ -20,17 +20,21 @@
 package agent_test
 
 import (
+	"bytes"
 	"encoding/json"
 	"fmt"
+	"io/ioutil"
 	"net"
 	"net/http"
 	"os"
+	"path/filepath"
 	"syscall"
 	"testing"
 	"time"
 
 	. "gopkg.in/check.v1"
 
+	"github.com/snapcore/snapd/desktop/notification/notificationtest"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil/sys"
@@ -41,6 +45,7 @@ import (
 func Test(t *testing.T) { TestingT(t) }
 
 type sessionAgentSuite struct {
+	testutil.DBusTest
 	socketPath string
 	client     *http.Client
 }
@@ -48,6 +53,7 @@ type sessionAgentSuite struct {
 var _ = Suite(&sessionAgentSuite{})
 
 func (s *sessionAgentSuite) SetUpTest(c *C) {
+	s.DBusTest.SetUpTest(c)
 	dirs.SetRootDir(c.MkDir())
 	xdgRuntimeDir := fmt.Sprintf("%s/%d", dirs.XdgRuntimeDirBase, os.Getuid())
 	c.Assert(os.MkdirAll(xdgRuntimeDir, 0700), IsNil)
@@ -65,6 +71,7 @@ func (s *sessionAgentSuite) SetUpTest(c *C) {
 func (s *sessionAgentSuite) TearDownTest(c *C) {
 	dirs.SetRootDir("")
 	logger.SetLogger(logger.NullLogger)
+	s.DBusTest.TearDownTest(c)
 }
 
 func (s *sessionAgentSuite) TestStartStop(c *C) {
@@ -74,6 +81,12 @@ func (s *sessionAgentSuite) TestStartStop(c *C) {
 	agent.Start()
 	defer func() { c.Check(agent.Stop(), IsNil) }()
 
+	// The agent has connected to the session bus
+	var hasOwner bool
+	c.Check(s.DBusTest.SessionBus.BusObject().Call("org.freedesktop.DBus.NameHasOwner", 0, "io.snapcraft.SessionAgent").Store(&hasOwner), IsNil)
+	c.Check(hasOwner, Equals, true)
+
+	// The agent is listening for REST API requests
 	response, err := s.client.Get("http://localhost/v1/session-info")
 	c.Assert(err, IsNil)
 	defer response.Body.Close()
@@ -142,6 +155,93 @@ func (s *sessionAgentSuite) TestExitOnIdle(c *C) {
 	}
 }
 
+func (s *sessionAgentSuite) TestFdoNotification(c *C) {
+	desktopFile := "[Desktop Entry]\nIcon=/path/appicon.png"
+	c.Assert(os.MkdirAll(filepath.Join(dirs.SnapDesktopFilesDir), 0755), IsNil)
+	c.Assert(ioutil.WriteFile(filepath.Join(dirs.SnapDesktopFilesDir, "app.desktop"), []byte(desktopFile), 0644), IsNil)
+
+	backend, err := notificationtest.NewFdoServer()
+	c.Assert(err, IsNil)
+	defer backend.Stop()
+
+	agent, err := agent.New()
+	c.Assert(err, IsNil)
+	agent.IdleTimeout = 150 * time.Millisecond
+	agent.Start()
+	defer agent.Stop()
+
+	// simulate snap refresh message
+	makeRequest := func() {
+		data := bytes.NewBufferString(`{"instance-name":"some-snap", "busy-app-name":"App", "busy-app-desktop-entry":"app"}`)
+		response, err := s.client.Post("http://localhost/v1/notifications/pending-refresh", "application/json", data)
+		c.Assert(err, IsNil)
+		defer response.Body.Close()
+		c.Check(response.StatusCode, Equals, 200)
+	}
+	makeRequest()
+
+	// wait for a while, we want the message to trigger FDO notification
+	time.Sleep(50 * time.Millisecond)
+
+	// our fake FDO backend should receive the notification
+	fdoNotification := backend.Get(1)
+	c.Assert(fdoNotification, NotNil)
+	c.Check(fdoNotification.AppName, Equals, "App")
+	c.Check(fdoNotification.Icon, Equals, "/path/appicon.png")
+
+	// trigger notification close signal over dbus
+	c.Assert(backend.Close(1, 0), IsNil)
+
+	select {
+	case <-agent.Dying():
+	case <-time.After(2 * time.Second):
+		c.Fatal("agent did not exit after idle timeout expired")
+	}
+}
+
+func (s *sessionAgentSuite) TestGtkNotification(c *C) {
+	desktopFile := "[Desktop Entry]\nIcon=/path/appicon.png"
+	c.Assert(os.MkdirAll(filepath.Join(dirs.SnapDesktopFilesDir), 0755), IsNil)
+	c.Assert(ioutil.WriteFile(filepath.Join(dirs.SnapDesktopFilesDir, "app.desktop"), []byte(desktopFile), 0644), IsNil)
+
+	backend, err := notificationtest.NewGtkServer()
+	c.Assert(err, IsNil)
+	defer backend.Stop()
+
+	agent, err := agent.New()
+	c.Assert(err, IsNil)
+	agent.IdleTimeout = 150 * time.Millisecond
+	agent.Start()
+	defer agent.Stop()
+
+	// simulate snap refresh message
+	makeRequest := func() {
+		data := bytes.NewBufferString(`{"instance-name":"some-snap", "busy-app-name":"App", "busy-app-desktop-entry":"app"}`)
+		response, err := s.client.Post("http://localhost/v1/notifications/pending-refresh", "application/json", data)
+		c.Assert(err, IsNil)
+		defer response.Body.Close()
+		c.Check(response.StatusCode, Equals, 200)
+	}
+	makeRequest()
+
+	// wait for a while, we want the message to trigger FDO notification
+	time.Sleep(50 * time.Millisecond)
+
+	// our fake FDO backend should receive the notification
+	gtkNotification := backend.Get("some-snap")
+	c.Assert(gtkNotification, NotNil)
+	c.Check(gtkNotification.DesktopID, Equals, "io.snapcraft.SessionAgent")
+
+	// trigger notification close signal over dbus
+	c.Assert(backend.Close("some-snap"), IsNil)
+
+	select {
+	case <-agent.Dying():
+	case <-time.After(2 * time.Second):
+		c.Fatal("agent did not exit after idle timeout expired")
+	}
+}
+
 func (s *sessionAgentSuite) TestConnectFromOtherUser(c *C) {
 	logbuf, restore := logger.MockLogger()
 	defer restore()
