@@ -122,13 +122,13 @@ type Settings struct {
 	conn *dbus.Conn
 }
 
-// Name returns the name of the interface this object implements
-func (s *Settings) Name() string {
+// Interface returns the name of the interface this object implements
+func (s *Settings) Interface() string {
 	return "io.snapcraft.Settings"
 }
 
-// BasePath returns the base path of the object
-func (s *Settings) BasePath() dbus.ObjectPath {
+// ObjectPath returns the path that the object is exported as
+func (s *Settings) ObjectPath() dbus.ObjectPath {
 	return "/io/snapcraft/Settings"
 }
 
@@ -234,6 +234,10 @@ func checkOutput(cmd *exec.Cmd, command string, setspec *settingSpec) (string, *
 //
 // Example usage: dbus-send --session --dest=io.snapcraft.Settings --type=method_call --print-reply /io/snapcraft/Settings io.snapcraft.Settings.Check string:'default-web-browser' string:'firefox.desktop'
 func (s *Settings) Check(setting string, check string, sender dbus.Sender) (string, *dbus.Error) {
+	if err := checkOnClassic(); err != nil {
+		return "", err
+	}
+
 	settingMain := &settingSpec{setting: setting}
 	desktopFile, err := desktopFileFromValueForSetting(s, "check", settingMain, check, sender)
 	if err != nil {
@@ -254,6 +258,10 @@ func (s *Settings) Check(setting string, check string, sender dbus.Sender) (stri
 //
 // Example usage: dbus-send --session --dest=io.snapcraft.Settings --type=method_call --print-reply /io/snapcraft/Settings io.snapcraft.Settings.CheckSub string:'default-url-scheme-handler' string:'irc' string:'ircclient.desktop'
 func (s *Settings) CheckSub(setting string, subproperty string, check string, sender dbus.Sender) (string, *dbus.Error) {
+	if err := checkOnClassic(); err != nil {
+		return "", err
+	}
+
 	settingSub := &settingSpec{setting: setting, subproperty: subproperty}
 	desktopFile, err := desktopFileFromValueForSetting(s, "check", settingSub, check, sender)
 	if err != nil {
@@ -274,6 +282,10 @@ func (s *Settings) CheckSub(setting string, subproperty string, check string, se
 //
 // Example usage: dbus-send --session --dest=io.snapcraft.Settings --type=method_call --print-reply /io/snapcraft/Settings io.snapcraft.Settings.Get string:'default-web-browser'
 func (s *Settings) Get(setting string, sender dbus.Sender) (string, *dbus.Error) {
+	if err := checkOnClassic(); err != nil {
+		return "", err
+	}
+
 	settingMain := &settingSpec{setting: setting}
 	if err := settingMain.validate(); err != nil {
 		return "", err
@@ -293,6 +305,10 @@ func (s *Settings) Get(setting string, sender dbus.Sender) (string, *dbus.Error)
 //
 // Example usage: dbus-send --session --dest=io.snapcraft.Settings --type=method_call --print-reply /io/snapcraft/Settings io.snapcraft.Settings.GetSub string:'default-url-scheme-handler' string:'irc'
 func (s *Settings) GetSub(setting string, subproperty string, sender dbus.Sender) (string, *dbus.Error) {
+	if err := checkOnClassic(); err != nil {
+		return "", err
+	}
+
 	settingSub := &settingSpec{setting: setting, subproperty: subproperty}
 	if err := settingSub.validate(); err != nil {
 		return "", err
@@ -312,6 +328,10 @@ func (s *Settings) GetSub(setting string, subproperty string, sender dbus.Sender
 //
 // Example usage: dbus-send --session --dest=io.snapcraft.Settings --type=method_call --print-reply /io/snapcraft/Settings io.snapcraft.Settings.Set string:'default-web-browser' string:'chromium-browser.desktop'
 func (s *Settings) Set(setting string, new string, sender dbus.Sender) *dbus.Error {
+	if err := checkOnClassic(); err != nil {
+		return err
+	}
+
 	settingMain := &settingSpec{setting: setting}
 	desktopFile, err := desktopFileFromValueForSetting(s, "set", settingMain, new, sender)
 	if err != nil {
@@ -335,6 +355,10 @@ func (s *Settings) Set(setting string, new string, sender dbus.Sender) *dbus.Err
 //
 // Example usage: dbus-send --session --dest=io.snapcraft.Settings --type=method_call --print-reply /io/snapcraft/Settings io.snapcraft.Settings.SetSub string:'default-url-scheme-handler' string:'irc' string:'ircclient.desktop'
 func (s *Settings) SetSub(setting string, subproperty string, new string, sender dbus.Sender) *dbus.Error {
+	if err := checkOnClassic(); err != nil {
+		return err
+	}
+
 	settingSub := &settingSpec{setting: setting, subproperty: subproperty}
 	desktopFile, err := desktopFileFromValueForSetting(s, "set", settingSub, new, sender)
 	if err != nil {
