@@ -50,7 +50,7 @@ import (
 	"github.com/snapcore/snapd/usersession/client"
 )
 
-type interacter interface {
+type Interacter interface {
 	Notify(status string)
 }
 
@@ -100,8 +100,8 @@ CPUAccounting=true
 		fmt.Fprintf(buf, "CPUQuota=%d%%\n", min(cpuQuotaSnap, cpuQuotaMax))
 	}
 
-	if grp.CPULimit != nil && len(grp.CPULimit.AllowedCPUs) != 0 {
-		allowedCpusValue := strutil.IntsToCommaSeparated(grp.CPULimit.AllowedCPUs)
+	if grp.CPULimit != nil && len(grp.CPULimit.CPUSet) != 0 {
+		allowedCpusValue := strutil.IntsToCommaSeparated(grp.CPULimit.CPUSet)
 		fmt.Fprintf(buf, "AllowedCPUs=%s\n", allowedCpusValue)
 	}
 
@@ -132,8 +132,8 @@ TasksAccounting=true
 `
 	buf := bytes.NewBufferString(header)
 
-	if grp.TaskLimit != 0 {
-		fmt.Fprintf(buf, "TasksMax=%d\n", grp.TaskLimit)
+	if grp.ThreadLimit != 0 {
+		fmt.Fprintf(buf, "TasksMax=%d\n", grp.ThreadLimit)
 	}
 	return buf.String()
 }
@@ -159,7 +159,48 @@ X-Snappy=yes
 	return buf.Bytes()
 }
 
-func stopUserServices(cli *client.Client, inter interacter, services ...string) error {
+func formatJournalSizeConf(grp *quota.Group) string {
+	if grp.JournalLimit.Size == 0 {
+		return ""
+	}
+	return fmt.Sprintf(`SystemMaxUse=%[1]d
+RuntimeMaxUse=%[1]d
+`, grp.JournalLimit.Size)
+}
+
+func formatJournalRateConf(grp *quota.Group) string {
+	if !grp.JournalLimit.RateEnabled {
+		return ""
+	}
+	return fmt.Sprintf(`RateLimitIntervalSec=%dus
+RateLimitBurst=%d
+`, grp.JournalLimit.RatePeriod.Microseconds(), grp.JournalLimit.RateCount)
+}
+
+func generateJournaldConfFile(grp *quota.Group) []byte {
+	if grp.JournalLimit == nil {
+		return nil
+	}
+
+	sizeOptions := formatJournalSizeConf(grp)
+	rateOptions := formatJournalRateConf(grp)
+	// Set Storage=auto for all journal namespaces we create. This is
+	// the setting for the default namespace, and 'persistent' is the default
+	// setting for all namespaces. However we want namespaces to honor the
+	// journal.persistent setting, and this only works if Storage is set
+	// to 'auto'.
+	// See https://www.freedesktop.org/software/systemd/man/journald.conf.html#Storage=
+	template := `# Journald configuration for snap quota group %[1]s
+[Journal]
+Storage=auto
+`
+	buf := bytes.Buffer{}
+	fmt.Fprintf(&buf, template, grp.Name)
+	fmt.Fprint(&buf, sizeOptions, rateOptions)
+	return buf.Bytes()
+}
+
+func stopUserServices(cli *client.Client, inter Interacter, services ...string) error {
 	ctx, cancel := context.WithTimeout(context.Background(), time.Duration(timeout.DefaultTimeout))
 	defer cancel()
 	failures, err := cli.ServicesStop(ctx, services)
@@ -169,7 +210,7 @@ func stopUserServices(cli *client.Client, inter interacter, services ...string)
 	return err
 }
 
-func startUserServices(cli *client.Client, inter interacter, services ...string) error {
+func startUserServices(cli *client.Client, inter Interacter, services ...string) error {
 	ctx, cancel := context.WithTimeout(context.Background(), time.Duration(timeout.DefaultTimeout))
 	defer cancel()
 	startFailures, stopFailures, err := cli.ServicesStart(ctx, services)
@@ -182,44 +223,33 @@ func startUserServices(cli *client.Client, inter interacter, services ...string)
 	return err
 }
 
-func stopService(sysd systemd.Systemd, app *snap.AppInfo, inter interacter) error {
-	serviceName := app.ServiceName()
-	tout := serviceStopTimeout(app)
+func stopService(sysd systemd.Systemd, app *snap.AppInfo, inter Interacter) error {
+	var serviceList []string
 
-	var extraServices []string
+	// Add application sockets
 	for _, socket := range app.Sockets {
-		extraServices = append(extraServices, filepath.Base(socket.File()))
+		serviceList = append(serviceList, filepath.Base(socket.File()))
 	}
+	// Add application timers
 	if app.Timer != nil {
-		extraServices = append(extraServices, filepath.Base(app.Timer.File()))
+		serviceList = append(serviceList, filepath.Base(app.Timer.File()))
 	}
+	// Add application service
+	serviceList = append(serviceList, app.ServiceName())
 
 	switch app.DaemonScope {
 	case snap.SystemDaemon:
-		var stopErrors error
-		if len(extraServices) > 0 {
-			stopErrors = sysd.Stop(extraServices, tout)
-		}
-
-		if err := sysd.Stop([]string{serviceName}, tout); err != nil {
-			if !systemd.IsTimeout(err) {
-				return err
-			}
-			inter.Notify(fmt.Sprintf("%s refused to stop, killing.", serviceName))
-			// ignore errors for kill; nothing we'd do differently at this point
-			sysd.Kill(serviceName, "TERM", "")
-			time.Sleep(killWait)
-			sysd.Kill(serviceName, "KILL", "")
-		}
-
-		if stopErrors != nil {
-			return stopErrors
+		if err := sysd.Stop(serviceList); err != nil {
+			return err
 		}
 
 	case snap.UserDaemon:
-		extraServices = append(extraServices, serviceName)
 		cli := client.New()
-		return stopUserServices(cli, inter, extraServices...)
+		if err := stopUserServices(cli, inter, serviceList...); err != nil {
+			return err
+		}
+	default:
+		panic("unknown app.DaemonScope")
 	}
 
 	return nil
@@ -233,7 +263,7 @@ type StartServicesFlags struct {
 // StartServices starts service units for the applications from the snap which
 // are services. Service units will be started in the order provided by the
 // caller.
-func StartServices(apps []*snap.AppInfo, disabledSvcs []string, flags *StartServicesFlags, inter interacter, tm timings.Measurer) (err error) {
+func StartServices(apps []*snap.AppInfo, disabledSvcs []string, flags *StartServicesFlags, inter Interacter, tm timings.Measurer) (err error) {
 	if flags == nil {
 		flags = &StartServicesFlags{}
 	}
@@ -471,6 +501,323 @@ type EnsureSnapServicesOptions struct {
 	RequireMountedSnapdSnap bool
 }
 
+// ensureSnapServicesContext is the context for EnsureSnapServices.
+// EnsureSnapServices supports transactional update/write of systemd service
+// files and slice files. A part of this is to support rollback of files and
+// also keep track of whether a restart of systemd daemon is required.
+type ensureSnapServicesContext struct {
+	// snaps, observeChange, opts and inter are the arguments
+	// taken by EnsureSnapServices. They are here to allow sub-functions
+	// to easier access these, and keep parameter lists shorter.
+	snaps         map[*snap.Info]*SnapServiceOptions
+	observeChange ObserveChangeCallback
+	opts          *EnsureSnapServicesOptions
+	inter         Interacter
+
+	// note: is not used when preseeding is set in opts.Preseeding
+	sysd                     systemd.Systemd
+	systemDaemonReloadNeeded bool
+	userDaemonReloadNeeded   bool
+	// modifiedUnits is the set of units that were modified and the previous
+	// state of the unit before modification that we can roll back to if there
+	// are any issues.
+	// note that the rollback is best effort, if we are rebooted in the middle,
+	// there is no guarantee about the state of files, some may have been
+	// updated and some may have been rolled back, higher level tasks/changes
+	// should have do/undo handlers to properly handle the case where this
+	// function is interrupted midway
+	modifiedUnits map[string]*osutil.MemoryFileState
+}
+
+// restore is a helper function which should be called in case any errors happen
+// during the write/update of systemd files
+func (es *ensureSnapServicesContext) restore() {
+	for file, state := range es.modifiedUnits {
+		if state == nil {
+			// we don't have anything to rollback to, so just remove the
+			// file
+			if err := os.Remove(file); err != nil {
+				es.inter.Notify(fmt.Sprintf("while trying to remove %s due to previous failure: %v", file, err))
+			}
+		} else {
+			// rollback the file to the previous state
+			if err := osutil.EnsureFileState(file, state); err != nil {
+				es.inter.Notify(fmt.Sprintf("while trying to rollback %s due to previous failure: %v", file, err))
+			}
+		}
+	}
+
+	if !es.opts.Preseeding {
+		if es.systemDaemonReloadNeeded {
+			if err := es.sysd.DaemonReload(); err != nil {
+				es.inter.Notify(fmt.Sprintf("while trying to perform systemd daemon-reload due to previous failure: %v", err))
+			}
+		}
+		if es.userDaemonReloadNeeded {
+			if err := userDaemonReload(); err != nil {
+				es.inter.Notify(fmt.Sprintf("while trying to perform user systemd daemon-reload due to previous failure: %v", err))
+			}
+		}
+	}
+}
+
+// reloadModified uses the modifiedSystemServices/userDaemonReloadNeeded to determine whether a reload
+// is required from systemd to take the new systemd files into effect. This is a NOP
+// if opts.Preseeding is set
+func (es *ensureSnapServicesContext) reloadModified() error {
+	if es.opts.Preseeding {
+		return nil
+	}
+
+	if es.systemDaemonReloadNeeded {
+		if err := es.sysd.DaemonReload(); err != nil {
+			return err
+		}
+	}
+	if es.userDaemonReloadNeeded {
+		if err := userDaemonReload(); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+// ensureSnapSystemdUnits takes care of writing .service files for all services
+// registered in snap.Info apps.
+func (es *ensureSnapServicesContext) ensureSnapSystemdUnits(snapInfo *snap.Info, opts *AddSnapServicesOptions) error {
+	handleFileModification := func(app *snap.AppInfo, unitType string, name, path string, content []byte) error {
+		old, modifiedFile, err := tryFileUpdate(path, content)
+		if err != nil {
+			return err
+		}
+
+		if modifiedFile {
+			if es.observeChange != nil {
+				var oldContent []byte
+				if old != nil {
+					oldContent = old.Content
+				}
+				es.observeChange(app, nil, unitType, name, string(oldContent), string(content))
+			}
+			es.modifiedUnits[path] = old
+
+			// also mark that we need to reload either the system or
+			// user instance of systemd
+			switch app.DaemonScope {
+			case snap.SystemDaemon:
+				es.systemDaemonReloadNeeded = true
+			case snap.UserDaemon:
+				es.userDaemonReloadNeeded = true
+			}
+		}
+
+		return nil
+	}
+
+	// note that the Preseeding option is not used here at all
+	for _, app := range snapInfo.Apps {
+		if !app.IsService() {
+			continue
+		}
+
+		// create services first; this doesn't trigger systemd
+
+		// Generate new service file state
+		path := app.ServiceFile()
+		content, err := generateSnapServiceFile(app, opts)
+		if err != nil {
+			return err
+		}
+
+		if err := handleFileModification(app, "service", app.Name, path, content); err != nil {
+			return err
+		}
+
+		// Generate systemd .socket files if needed
+		socketFiles, err := generateSnapSocketFiles(app)
+		if err != nil {
+			return err
+		}
+		for name, content := range socketFiles {
+			path := app.Sockets[name].File()
+			if err := handleFileModification(app, "socket", name, path, content); err != nil {
+				return err
+			}
+		}
+
+		if app.Timer != nil {
+			content, err := generateSnapTimerFile(app)
+			if err != nil {
+				return err
+			}
+			path := app.Timer.File()
+			if err := handleFileModification(app, "timer", "", path, content); err != nil {
+				return err
+			}
+		}
+	}
+	return nil
+}
+
+// ensureSnapsSystemdServices takes care of writing .service files for all apps in the provided snaps
+// list, and also returns a quota group set that represents all quota groups for the set of snaps
+// provided if they are a part of any.
+func (es *ensureSnapServicesContext) ensureSnapsSystemdServices() (*quota.QuotaGroupSet, error) {
+	neededQuotaGrps := &quota.QuotaGroupSet{}
+
+	for s, snapSvcOpts := range es.snaps {
+		if s.Type() == snap.TypeSnapd {
+			return nil, fmt.Errorf("internal error: adding explicit services for snapd snap is unexpected")
+		}
+
+		// always use RequireMountedSnapdSnap options from the global options
+		genServiceOpts := &AddSnapServicesOptions{
+			RequireMountedSnapdSnap: es.opts.RequireMountedSnapdSnap,
+		}
+		if snapSvcOpts != nil {
+			// and if there are per-snap options specified, use that for
+			// VitalityRank
+			genServiceOpts.VitalityRank = snapSvcOpts.VitalityRank
+			genServiceOpts.QuotaGroup = snapSvcOpts.QuotaGroup
+
+			if snapSvcOpts.QuotaGroup != nil {
+				if err := neededQuotaGrps.AddAllNecessaryGroups(snapSvcOpts.QuotaGroup); err != nil {
+					// this error can basically only be a circular reference
+					// in the quota group tree
+					return nil, err
+				}
+			}
+		}
+
+		if err := es.ensureSnapSystemdUnits(s, genServiceOpts); err != nil {
+			return nil, err
+		}
+	}
+	return neededQuotaGrps, nil
+}
+
+func (es *ensureSnapServicesContext) ensureSnapSlices(quotaGroups *quota.QuotaGroupSet) error {
+	handleSliceModification := func(grp *quota.Group, path string, content []byte) error {
+		old, modifiedFile, err := tryFileUpdate(path, content)
+		if err != nil {
+			return err
+		}
+
+		if modifiedFile {
+			if es.observeChange != nil {
+				var oldContent []byte
+				if old != nil {
+					oldContent = old.Content
+				}
+				es.observeChange(nil, grp, "slice", grp.Name, string(oldContent), string(content))
+			}
+
+			es.modifiedUnits[path] = old
+
+			// also mark that we need to reload the system instance of systemd
+			// TODO: also handle reloading the user instance of systemd when
+			// needed
+			es.systemDaemonReloadNeeded = true
+		}
+
+		return nil
+	}
+
+	// now make sure that all of the slice units exist
+	for _, grp := range quotaGroups.AllQuotaGroups() {
+		content := generateGroupSliceFile(grp)
+
+		sliceFileName := grp.SliceFileName()
+		path := filepath.Join(dirs.SnapServicesDir, sliceFileName)
+		if err := handleSliceModification(grp, path, content); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+func (es *ensureSnapServicesContext) ensureSnapJournaldUnits(quotaGroups *quota.QuotaGroupSet) error {
+	handleJournalModification := func(grp *quota.Group, path string, content []byte) error {
+		old, fileModified, err := tryFileUpdate(path, content)
+		if err != nil {
+			return err
+		}
+
+		if !fileModified {
+			return nil
+		}
+
+		// suppress any event and restart if we actually did not do anything
+		// as it seems modifiedFile is set even when the file does not exist
+		// and when the new content is nil.
+		if (old == nil || len(old.Content) == 0) && len(content) == 0 {
+			return nil
+		}
+
+		if es.observeChange != nil {
+			var oldContent []byte
+			if old != nil {
+				oldContent = old.Content
+			}
+			es.observeChange(nil, grp, "journald", grp.Name, string(oldContent), string(content))
+		}
+
+		es.modifiedUnits[path] = old
+		return nil
+	}
+
+	for _, grp := range quotaGroups.AllQuotaGroups() {
+		contents := generateJournaldConfFile(grp)
+		fileName := grp.JournalConfFileName()
+
+		path := filepath.Join(dirs.SnapSystemdDir, fileName)
+		if err := handleJournalModification(grp, path, contents); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+// ensureJournalQuotaServiceUnits takes care of writing service drop-in files for all journal namespaces.
+func (es *ensureSnapServicesContext) ensureJournalQuotaServiceUnits(quotaGroups *quota.QuotaGroupSet) error {
+	handleFileModification := func(grp *quota.Group, path string, content []byte) error {
+		old, fileModified, err := tryFileUpdate(path, content)
+		if err != nil {
+			return err
+		}
+
+		if fileModified {
+			if es.observeChange != nil {
+				var oldContent []byte
+				if old != nil {
+					oldContent = old.Content
+				}
+				es.observeChange(nil, grp, "service", grp.Name, string(oldContent), string(content))
+			}
+			es.modifiedUnits[path] = old
+		}
+		return nil
+	}
+
+	for _, grp := range quotaGroups.AllQuotaGroups() {
+		if grp.JournalLimit == nil {
+			continue
+		}
+
+		if err := os.MkdirAll(grp.JournalServiceDropInDir(), 0755); err != nil {
+			return err
+		}
+
+		dropInPath := grp.JournalServiceDropInFile()
+		content := genJournalServiceFile(grp)
+		if err := handleFileModification(grp, dropInPath, content); err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
 // EnsureSnapServices will ensure that the specified snap services' file states
 // are up to date with the specified options and infos. It will add new services
 // if those units don't already exist, but it does not delete existing service
@@ -488,210 +835,45 @@ type EnsureSnapServicesOptions struct {
 // produce immediate side-effects, as the changes are in effect only
 // if the function did not return an error.
 // This function is idempotent.
-func EnsureSnapServices(snaps map[*snap.Info]*SnapServiceOptions, opts *EnsureSnapServicesOptions, observeChange ObserveChangeCallback, inter interacter) (err error) {
-	// note, sysd is not used when preseeding
-	sysd := systemd.New(systemd.SystemMode, inter)
-
+func EnsureSnapServices(snaps map[*snap.Info]*SnapServiceOptions, opts *EnsureSnapServicesOptions, observeChange ObserveChangeCallback, inter Interacter) (err error) {
 	if opts == nil {
 		opts = &EnsureSnapServicesOptions{}
 	}
 
-	// we only consider the global EnsureSnapServicesOptions to decide if we
-	// are preseeding or not to reduce confusion about which set of options
-	// determines whether we are preseeding or not during the ensure operation
-	preseeding := opts.Preseeding
-
-	// modifiedUnitsPreviousState is the set of units that were modified and the previous
-	// state of the unit before modification that we can roll back to if there
-	// are any issues.
-	// note that the rollback is best effort, if we are rebooted in the middle,
-	// there is no guarantee about the state of files, some may have been
-	// updated and some may have been rolled back, higher level tasks/changes
-	// should have do/undo handlers to properly handle the case where this
-	// function is interrupted midway
-	modifiedUnitsPreviousState := make(map[string]*osutil.MemoryFileState)
-	var modifiedSystem, modifiedUser bool
+	context := &ensureSnapServicesContext{
+		snaps:         snaps,
+		observeChange: observeChange,
+		opts:          opts,
+		inter:         inter,
+		sysd:          systemd.New(systemd.SystemMode, inter),
+		modifiedUnits: make(map[string]*osutil.MemoryFileState),
+	}
 
 	defer func() {
 		if err == nil {
 			return
 		}
-		for file, state := range modifiedUnitsPreviousState {
-			if state == nil {
-				// we don't have anything to rollback to, so just remove the
-				// file
-				if e := os.Remove(file); e != nil {
-					inter.Notify(fmt.Sprintf("while trying to remove %s due to previous failure: %v", file, e))
-				}
-			} else {
-				// rollback the file to the previous state
-				if e := osutil.EnsureFileState(file, state); e != nil {
-					inter.Notify(fmt.Sprintf("while trying to rollback %s due to previous failure: %v", file, e))
-				}
-			}
-		}
-		if modifiedSystem && !preseeding {
-			if e := sysd.DaemonReload(); e != nil {
-				inter.Notify(fmt.Sprintf("while trying to perform systemd daemon-reload due to previous failure: %v", e))
-			}
-		}
-		if modifiedUser && !preseeding {
-			if e := userDaemonReload(); e != nil {
-				inter.Notify(fmt.Sprintf("while trying to perform user systemd daemon-reload due to previous failure: %v", e))
-			}
-		}
+		context.restore()
 	}()
 
-	handleFileModification := func(app *snap.AppInfo, unitType string, name, path string, content []byte) error {
-		old, modifiedFile, err := tryFileUpdate(path, content)
-		if err != nil {
-			return err
-		}
-
-		if modifiedFile {
-			if observeChange != nil {
-				var oldContent []byte
-				if old != nil {
-					oldContent = old.Content
-				}
-				observeChange(app, nil, unitType, name, string(oldContent), string(content))
-			}
-			modifiedUnitsPreviousState[path] = old
-
-			// also mark that we need to reload either the system or
-			// user instance of systemd
-			switch app.DaemonScope {
-			case snap.SystemDaemon:
-				modifiedSystem = true
-			case snap.UserDaemon:
-				modifiedUser = true
-			}
-		}
-
-		return nil
+	quotaGroups, err := context.ensureSnapsSystemdServices()
+	if err != nil {
+		return err
 	}
 
-	neededQuotaGrps := &quota.QuotaGroupSet{}
-
-	for s, snapSvcOpts := range snaps {
-		if s.Type() == snap.TypeSnapd {
-			return fmt.Errorf("internal error: adding explicit services for snapd snap is unexpected")
-		}
-
-		// always use RequireMountedSnapdSnap options from the global options
-		genServiceOpts := &AddSnapServicesOptions{
-			RequireMountedSnapdSnap: opts.RequireMountedSnapdSnap,
-		}
-		if snapSvcOpts != nil {
-			// and if there are per-snap options specified, use that for
-			// VitalityRank
-			genServiceOpts.VitalityRank = snapSvcOpts.VitalityRank
-			genServiceOpts.QuotaGroup = snapSvcOpts.QuotaGroup
-
-			if snapSvcOpts.QuotaGroup != nil {
-				if err := neededQuotaGrps.AddAllNecessaryGroups(snapSvcOpts.QuotaGroup); err != nil {
-					// this error can basically only be a circular reference
-					// in the quota group tree
-					return err
-				}
-			}
-		}
-		// note that the Preseeding option is not used here at all
-
-		for _, app := range s.Apps {
-			if !app.IsService() {
-				continue
-			}
-
-			// create services first; this doesn't trigger systemd
-
-			// Generate new service file state
-			path := app.ServiceFile()
-			content, err := generateSnapServiceFile(app, genServiceOpts)
-			if err != nil {
-				return err
-			}
-
-			if err := handleFileModification(app, "service", app.Name, path, content); err != nil {
-				return err
-			}
-
-			// Generate systemd .socket files if needed
-			socketFiles, err := generateSnapSocketFiles(app)
-			if err != nil {
-				return err
-			}
-			for name, content := range socketFiles {
-				path := app.Sockets[name].File()
-				if err := handleFileModification(app, "socket", name, path, content); err != nil {
-					return err
-				}
-			}
-
-			if app.Timer != nil {
-				content, err := generateSnapTimerFile(app)
-				if err != nil {
-					return err
-				}
-				path := app.Timer.File()
-				if err := handleFileModification(app, "timer", "", path, content); err != nil {
-					return err
-				}
-			}
-		}
+	if err := context.ensureSnapSlices(quotaGroups); err != nil {
+		return err
 	}
 
-	handleSliceModification := func(grp *quota.Group, path string, content []byte) error {
-		old, modifiedFile, err := tryFileUpdate(path, content)
-		if err != nil {
-			return err
-		}
-
-		if modifiedFile {
-			if observeChange != nil {
-				var oldContent []byte
-				if old != nil {
-					oldContent = old.Content
-				}
-				observeChange(nil, grp, "slice", grp.Name, string(oldContent), string(content))
-			}
-
-			modifiedUnitsPreviousState[path] = old
-
-			// also mark that we need to reload the system instance of systemd
-			// TODO: also handle reloading the user instance of systemd when
-			// needed
-			modifiedSystem = true
-		}
-
-		return nil
+	if err := context.ensureSnapJournaldUnits(quotaGroups); err != nil {
+		return err
 	}
 
-	// now make sure that all of the slice units exist
-	for _, grp := range neededQuotaGrps.AllQuotaGroups() {
-		content := generateGroupSliceFile(grp)
-
-		sliceFileName := grp.SliceFileName()
-		path := filepath.Join(dirs.SnapServicesDir, sliceFileName)
-		if err := handleSliceModification(grp, path, content); err != nil {
-			return err
-		}
+	if err := context.ensureJournalQuotaServiceUnits(quotaGroups); err != nil {
+		return err
 	}
 
-	if !preseeding {
-		if modifiedSystem {
-			if err = sysd.DaemonReload(); err != nil {
-				return err
-			}
-		}
-		if modifiedUser {
-			if err = userDaemonReload(); err != nil {
-				return err
-			}
-		}
-	}
-
-	return nil
+	return context.reloadModified()
 }
 
 // AddSnapServicesOptions is a struct for controlling the generated service
@@ -717,7 +899,7 @@ type AddSnapServicesOptions struct {
 
 // AddSnapServices adds service units for the applications from the snap which
 // are services. The services do not get enabled or started.
-func AddSnapServices(s *snap.Info, opts *AddSnapServicesOptions, inter interacter) error {
+func AddSnapServices(s *snap.Info, opts *AddSnapServicesOptions, inter Interacter) error {
 	m := map[*snap.Info]*SnapServiceOptions{
 		s: {},
 	}
@@ -744,7 +926,7 @@ type StopServicesFlags struct {
 
 // StopServices stops and optionally disables service units for the applications
 // from the snap which are services.
-func StopServices(apps []*snap.AppInfo, flags *StopServicesFlags, reason snap.ServiceStopReason, inter interacter, tm timings.Measurer) error {
+func StopServices(apps []*snap.AppInfo, flags *StopServicesFlags, reason snap.ServiceStopReason, inter Interacter, tm timings.Measurer) error {
 	sysd := systemd.New(systemd.SystemMode, inter)
 	if flags == nil {
 		flags = &StopServicesFlags{}
@@ -784,17 +966,6 @@ func StopServices(apps []*snap.AppInfo, flags *StopServicesFlags, reason snap.Se
 		if err != nil {
 			return err
 		}
-
-		// ensure the service is really stopped on remove regardless
-		// of stop-mode
-		if reason == snap.StopReasonRemove && !app.StopMode.KillAll() && app.DaemonScope == snap.SystemDaemon {
-			// FIXME: make this smarter and avoid the killWait
-			//        delay if not needed (i.e. if all processes
-			//        have died)
-			sysd.Kill(app.ServiceName(), "TERM", "all")
-			time.Sleep(killWait)
-			sysd.Kill(app.ServiceName(), "KILL", "")
-		}
 	}
 	if len(disableServices) > 0 {
 		if err := sysd.DisableNoReload(disableServices); err != nil {
@@ -809,7 +980,7 @@ func StopServices(apps []*snap.AppInfo, flags *StopServicesFlags, reason snap.Se
 
 // ServicesEnableState returns a map of service names from the given snap,
 // together with their enable/disable status.
-func ServicesEnableState(s *snap.Info, inter interacter) (map[string]bool, error) {
+func ServicesEnableState(s *snap.Info, inter Interacter) (map[string]bool, error) {
 	sysd := systemd.New(systemd.SystemMode, inter)
 
 	// loop over all services in the snap, querying systemd for the current
@@ -838,7 +1009,7 @@ func ServicesEnableState(s *snap.Info, inter interacter) (map[string]bool, error
 // with sub-groups, one must remove all the sub-groups first.
 // This function is idempotent, if the slice file doesn't exist no error is
 // returned.
-func RemoveQuotaGroup(grp *quota.Group, inter interacter) error {
+func RemoveQuotaGroup(grp *quota.Group, inter Interacter) error {
 	// TODO: it only works on leaf sub-groups currently
 	if len(grp.SubGroups) != 0 {
 		return fmt.Errorf("internal error: cannot remove quota group with sub-groups")
@@ -864,7 +1035,7 @@ func RemoveQuotaGroup(grp *quota.Group, inter interacter) error {
 // RemoveSnapServices disables and removes service units for the applications
 // from the snap which are services. The optional flag indicates whether
 // services are removed as part of undoing of first install of a given snap.
-func RemoveSnapServices(s *snap.Info, inter interacter) error {
+func RemoveSnapServices(s *snap.Info, inter Interacter) error {
 	if s.Type() == snap.TypeSnapd {
 		return fmt.Errorf("internal error: removing explicit services for snapd snap is unexpected")
 	}
@@ -1079,6 +1250,9 @@ OOMScoreAdjust={{.OOMAdjustScore}}
 {{- if .SliceUnit}}
 Slice={{.SliceUnit}}
 {{- end}}
+{{- if .LogNamespace}}
+LogNamespace={{.LogNamespace}}
+{{- end}}
 {{- if not (or .App.Sockets .App.Timer .App.ActivatesOn) }}
 
 [Install]
@@ -1151,6 +1325,7 @@ WantedBy={{.ServicesTarget}}
 		After                    []string
 		InterfaceServiceSnippets string
 		SliceUnit                string
+		LogNamespace             string
 
 		Home    string
 		EnvVars string
@@ -1195,6 +1370,9 @@ WantedBy={{.ServicesTarget}}
 	// check the quota group slice
 	if opts.QuotaGroup != nil {
 		wrapperData.SliceUnit = opts.QuotaGroup.SliceFileName()
+		if opts.QuotaGroup.JournalLimit != nil {
+			wrapperData.LogNamespace = opts.QuotaGroup.JournalNamespaceName()
+		}
 	}
 
 	// Add extra "After" targets
@@ -1280,6 +1458,15 @@ WantedBy={{.SocketsTarget}}
 	return templateOut.Bytes()
 }
 
+func genJournalServiceFile(grp *quota.Group) []byte {
+	buf := bytes.Buffer{}
+	template := `[Service]
+LogsDirectory=
+`
+	fmt.Fprint(&buf, template)
+	return buf.Bytes()
+}
+
 func generateSnapSocketFiles(app *snap.AppInfo) (map[string][]byte, error) {
 	if err := snap.ValidateApp(app); err != nil {
 		return nil, err
@@ -1607,7 +1794,7 @@ type RestartServicesFlags struct {
 // TODO: change explicitServices format to be less unusual, more consistent
 // (introduce AppRef?)
 func RestartServices(svcs []*snap.AppInfo, explicitServices []string,
-	flags *RestartServicesFlags, inter interacter, tm timings.Measurer) error {
+	flags *RestartServicesFlags, inter Interacter, tm timings.Measurer) error {
 	sysd := systemd.New(systemd.SystemMode, inter)
 
 	unitNames := make([]string, 0, len(svcs))
@@ -1639,7 +1826,7 @@ func RestartServices(svcs []*snap.AppInfo, explicitServices []string,
 				err = sysd.ReloadOrRestart(unit.Name)
 			} else {
 				// note: stop followed by start, not just 'restart'
-				err = sysd.Restart([]string{unit.Name}, 5*time.Second)
+				err = sysd.Restart([]string{unit.Name})
 			}
 		})
 		if err != nil {
